/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_ber_rxh_
#define _oscl_ber_rxh_
#include "oscl/oid/api.h"

/** */
class BerExtendedUnsigned {
	private:
		/** */
		unsigned	_value;
		/** */
		unsigned	_length;
	public:
		/** */
		BerExtendedUnsigned(const unsigned char* p,unsigned pLen) throw();
		/** */
		unsigned	getLength() const throw();
		/** */
		unsigned	getValue() const throw();
	};

/** */
class BerOID {
	private:
		/** */
		unsigned				_index;
		/** */
		unsigned				_length;
		/** */
		const unsigned char*	_enc;
		/** */
		bool					_err;
		/** */
		unsigned				_offset;
		/** */
		unsigned long			_currentSubIdValue;
		/** */
		unsigned long			_currentSubIdLength;
	public:
		/** */
		BerOID(const void* content,unsigned len) throw();
		/** */
		void			decodeCurrentSubID() throw();
		/** */
		bool			error() const throw();
		/** */
		unsigned long	current() const throw();
		/** */
		void			first() throw();
		/** */
		void			next() throw();
		/** */
		bool			more() const throw();
	};

/** */
class BerRxEncoding {
	private:
		/** */
		const unsigned char*	_pdu;
		/** */
		const unsigned			_length;
		/** */
		unsigned				_offset;
		/** */
		unsigned long			_currentTag;
		/** */
		unsigned				_currentLength;
		/** */
		unsigned				_lengthOffset;
		/** */
		unsigned				_valueOffset;
		/** */

	public:
		/** */
		BerRxEncoding(const void* p,unsigned bufferLength) throw();

		/** */
		unsigned long	currentIdentifierTag() const throw();
		/** */
		unsigned long	currentContentLength() const throw();
		/** */
		const void*		currentContent() const throw();
		/** */
		const void*		currentEncoding() const throw();
		/** */
		unsigned		currentSize() const throw();
		/** */
		bool			nextEncoding() throw();
		/** */
		bool			currentTypeIsPrimitive() const throw();
		/** */
		bool			currentClassIsUniversal() const throw();
		/** */
		bool			currentClassIsApplication() const throw();
		/** */
		bool			currentClassIsContextSpecific() const throw();
		/** */
		bool			currentClassIsContextPrivate() const throw();
		/** */
		bool			currentIsINTEGER() const throw();
		/** */
		bool			currentIsOCTETSTRING() const throw();
		/** */
		bool			currentIsOID() const throw();
		/** */
		bool			currentIsSEQUENCE() const throw();
		/** */
		bool			currentIsEndOfContents() const throw();
		/** */
		signed long		decodeINTEGER() const throw();
		/** */
		signed long		decodeSINTEGER() const throw();
		/** */
		unsigned long	decodeUINTEGER() const throw();
		/** */
		unsigned		decodeOCTETSTRING(	void*		dest,
											unsigned	maxLength
											) const throw();
		/** */
		void			decodeOID(Oscl::ObjectID::Api& oid) const throw();
		/** */
		unsigned		decodeCurrentTag() throw();
		/** */
		unsigned		decodeCurrentLength() throw();
		/** */
		unsigned		lengthOfIndefinite() const throw();
		/** */
		bool			encodingIsValid() throw();
	};

#endif
