/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_ber_txh_
#define _oscl_ber_txh_
#include "oscl/oid/roapi.h"

typedef enum{
	berClassTypeUniversal		= 0,
	berClassTypeApplication		= 1,
	berClassTypeContextSpecific	= 2,
	berClassTypePrivate			= 3
	} BerClassType;

/** */
class BerTxEncoder {
	private:
		/** */
		unsigned char*		_pdu;
		/** */
		const unsigned		_length;
		/** */
		unsigned			_offset;
		/** */
		bool				_overflow;

	public:
		/** */
		BerTxEncoder(void* pdu,unsigned length) throw();
		/** */
		bool	tooBig() const throw(){return _overflow;}
		/** */
		void	encodeConstructedIdentifier(	BerClassType	type,
												unsigned char	tag
												) throw();
		/** */
		void	encodeSequenceIdentifier() throw();
		/** */
		void	encodeDefiniteTwoOctetLength(unsigned length) throw();
		/** */
		void	encodeINTEGER(signed long value) throw();
		/** */
		void	encodeSINTEGER(signed long value) throw();
		/** */
		void	encodeUINTEGER(unsigned long value) throw();
		/** */
		void	encodeOID(const Oscl::ObjectID::RO::Api& oid) throw();
		/** */
		void	encodeOCTETSTRING(const void* s,unsigned length) throw();
		/** */
		void	openConstructed(	BerClassType	type,
									unsigned char	tag
									) throw();
		/** */
		void	openSEQUENCE() throw();
		/** */
		void	closeConstructed() throw();

		/** */
		unsigned char*	currentBuffer() const throw();

		/** */
		unsigned		length() const throw();

		/** */
		unsigned		remaining() const throw();

		/** */
		void	advance(unsigned amount) throw();

		/** */
		void	append(const void* buffer,unsigned length) throw();

	private:
		void		encodeLength(unsigned long) throw();
		void		encodeValue(unsigned long) throw();
		unsigned	calcEncodedLength(unsigned long) throw();
	};

#endif
