/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_block_rw_deco_fsmh_
#define _oscl_block_rw_deco_fsmh_
#include "oscl/queue/queue.h"
#include "oscl/queue/queueitem.h"

/** */
namespace Oscl {
/** */
namespace Block {
/** */
namespace RW {
/** */
namespace Decorator {

/** */
class StateVar {
	public:
		/** */
		class ContextApi {
			public:	// request actions
				/** */
				virtual void	returnOpenReq() throw()=0;
				/** */
				virtual void	returnCloseReq() throw()=0;
				/** */
				virtual void	executeDeferredReq() throw()=0;
				/** */
				virtual void	deferReadReq() throw()=0;
				/** */
				virtual void	deferWriteReq() throw()=0;
				/** */
				virtual void	deferFlushReq() throw()=0;
				/** */
				virtual void	startRead() throw()=0;
				/** */
				virtual void	startWrite() throw()=0;
				/** */
				virtual void	startFlush() throw()=0;
				/** */
				virtual void	returnReadReq() throw()=0;
				/** */
				virtual void	returnWriteReq() throw()=0;
				/** */
				virtual void	returnFlushReq() throw()=0;
				/** */
				virtual void	returnIncommingReadCancelReq() throw()=0;
				/** */
				virtual void	returnIncommingWriteCancelReq() throw()=0;
				/** */
				virtual void	returnIncommingFlushCancelReq() throw()=0;
				/** */
				virtual void	returnReadCancelReq() throw()=0;
				/** */
				virtual void	returnWriteCancelReq() throw()=0;
				/** */
				virtual void	returnFlushCancelReq() throw()=0;
				/** */
				virtual void	cancelRead() throw()=0;
				/** */
				virtual void	cancelWrite() throw()=0;
				/** */
				virtual void	cancelFlush() throw()=0;
				/** */
				virtual void	finishRead() throw()=0;
				/** */
				virtual void	finishWrite() throw()=0;
				/** */
				virtual void	finishFlush() throw()=0;
				/** */
				virtual bool	deferQueueEmpty() throw()=0;
				/** */
				virtual bool	isIncommingCancelForCurrentRead() throw()=0;
				/** */
				virtual bool	isIncommingCancelForCurrentWrite() throw()=0;
				/** */
				virtual bool	isIncommingCancelForCurrentFlush() throw()=0;
			};
	private:
		/** */
		class State {
			public:	// Admin Events
				/** */
				virtual void	open(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				virtual void	close(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				virtual void	readCancelResp(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	writeCancelResp(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				virtual void	writeResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				virtual void	flushResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				virtual void	readResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				virtual void	readReq(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				virtual void	writeReq(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				virtual void	flushReq(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				virtual void	readCancelReq(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	writeCancelReq(	ContextApi&	context,
												StateVar&	sv
												) const throw();
			public:
				/** */
				virtual const char*	stateName() const throw()=0;
			};

	public:
		/** */
		class Closed : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Closed";}
			private:
				/** */
				void	open(	ContextApi&	context,
								StateVar&	sv
								) const throw();
			};
		/** */
		class Idle : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Resetting";}
			public:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	readReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	writeReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	flushReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	readCancelReq(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	writeCancelReq(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class DeferReq : public State {
			public:
				/** */
				void	readReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	writeReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	flushReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class Write : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Write";}
			public:
				/** */
				void	writeResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	writeCancelReq(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class Flush : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Flush";}
			public:
				/** */
				void	flushResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	writeCancelReq(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class Read : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Read";}
			public:
				/** */
				void	readResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	readCancelReq(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};

		/** */
		class ReadCanceling : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ReadCanceling";}
			public:
				/** */
				void	readCancelResp(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	readResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};

		/** */
		class WriteCanceling : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "WriteCanceling";}
			public:
				/** */
				void	writeCancelResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	writeResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class FlushCanceling : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "FlushCanceling";}
			public:
				/** */
				void	writeCancelResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	flushResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
	private:
		/** */
		struct TraceRec : public Oscl::QueueItem {
			public:
				/** */
				const State*	_state;
			};

	private:
		/** */
		ContextApi&		_context;
		/** */
		const State*	_state;

	private:
		/** */
		enum{nTraceRecs=20};
		/** */
		TraceRec				_traceRec[nTraceRecs];
		/** */
		Oscl::Queue<TraceRec>	_freeTraceRecs;
		/** */
		Oscl::Queue<TraceRec>	_trace;

	public:	// Constructors
		/** */
		StateVar(ContextApi& context) throw();

	public:	// events
		/** */
		void	open() throw();
		/** */
		void	close() throw();
		/** */
		void	readCancelResp() throw();
		/** */
		void	writeCancelResp() throw();
		/** */
		void	writeResp() throw();
		/** */
		void	flushResp() throw();
		/** */
		void	readResp() throw();
		/** */
		void	writeReq() throw();
		/** */
		void	flushReq() throw();
		/** */
		void	readReq() throw();
		/** */
		void	writeCancelReq() throw();
		/** */
		void	readCancelReq() throw();

	private: // global actions and state change operations
		/** */
		friend class State;
		friend class Closed;
		friend class Idle;
		friend class Read;
		friend class Write;
		friend class Flush;
		friend class ReadCanceling;
		friend class WriteCanceling;
		friend class FlushCanceling;
		/** */
		void	protocolError(const char* state) throw();
		/** */
		void	changeToClosed() throw();
		/** */
		void	changeToIdle() throw();
		/** */
		void	changeToRead() throw();
		/** */
		void	changeToWrite() throw();
		/** */
		void	changeToFlush() throw();
		/** */
		void	changeToReadCanceling() throw();
		/** */
		void	changeToWriteCanceling() throw();
		/** */
		void	changeToFlushCanceling() throw();

	private:
		/** */
		void	trace(const State& state) throw();
	};

}
}
}
}

#endif
