/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "pb24.h"
#include "oscl/error/fatal.h"

using namespace Oscl;
using namespace Oscl::Mot860::PortB;

PB24::PB24(	Bits::FieldApi<Mot8xx::Port::PBDAT::Reg>&	port,
			Bits::FieldApi<Mot8xx::Port::PBODR::Reg>&	pbdir,
			Bits::FieldApi<Mot8xx::Port::PBPAR::Reg>&	pbpar,
			Bits::FieldApi<Mot8xx::Port::PBODR::Reg>&	pbodr,
			Bits::FieldApi<Mot8xx::Port::PBDAT::Reg>&	reservation
			) throw():
		_port(port),
		_pbdir(pbdir),
		_pbpar(pbpar),
		_pbodr(pbodr),
		_reservation(reservation)
		{
	}

Bits::InputApi&		PB24::reserveInput() throw(){
	reserve();
	_pbpar.low();
	_pbdir.low();
	_pbodr.low();
	return _port;
	}

Bits::OutputApi&	PB24::reserveOutput() throw(){
	reserve();
	_pbpar.low();
	_pbdir.high();
	_pbodr.low();
	return _port;
	}

Bits::InOutApi&		PB24::reserveInOut() throw(){
	reserve();
	_pbpar.low();
	_pbdir.high();
	_pbodr.low();
	return _port;
	}

Bits::OutputApi&	PB24::reserveOpenDrainOutput() throw(){
	reserve();
	_pbpar.low();
	_pbdir.low();
	_pbodr.high();
	return _port;
	}

Bits::InOutApi&		PB24::reserveOpenDrainInOut() throw(){
	reserve();
	_pbpar.low();
	_pbdir.high();
	_pbodr.high();
	return _port;
	}

Bits::InputApi&	PB24::reserveSMRXD1() throw(){
	reserve();
	_pbpar.high();
	_pbdir.low();
	_pbodr.low();
	return _port;
	}

void	PB24::reserve() throw(){
	if(_reservation.testAndSet(_mask)) return;
	ErrorFatal::logAndExit("PB24 already reserved.");
	}

