/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "pc14.h"
#include "oscl/error/fatal.h"

using namespace Oscl;
using namespace Oscl::Mot860::PortC;

PC14::PC14(	Bits::FieldApi<Mot8xx::Port::PCDAT::Reg>&	pcdat,
			Bits::FieldApi<Mot8xx::Port::PCSO::Reg>&	pcdir,
			Bits::FieldApi<Mot8xx::Port::PCPAR::Reg>&	pcpar,
			Bits::FieldApi<Mot8xx::Port::PCSO::Reg>&	pcso,
			Bits::FieldApi<Mot8xx::Port::PCDAT::Reg>&	reservation
			) throw():
		_pcdat(pcdat),
		_pcdir(pcdir),
		_pcpar(pcpar),
		_pcso(pcso),
		_reservation(reservation)
		{
	}

Bits::InputApi&		PC14::reserveInput() throw(){
	reserve();
	_pcpar.low();
	_pcdir.low();
	_pcso.low();
	return _pcdat;
	}

Bits::OutputApi&	PC14::reserveOutput() throw(){
	reserve();
	_pcpar.low();
	_pcdir.high();
	_pcso.low();
	return _pcdat;
	}

Bits::InOutApi&		PC14::reserveInOut() throw(){
	reserve();
	_pcpar.low();
	_pcdir.high();
	_pcso.low();
	return _pcdat;
	}

Bits::InputApi&	PC14::reserveDREQ1() throw(){
	reserve();
	_pcpar.low();
	_pcdir.low();
	_pcso.high();
	return _pcdat;
	}

Bits::InputApi&	PC14::reserveRTS2() throw(){
	reserve();
	_pcpar.high();
	_pcdir.low();
	_pcso.low();
	return _pcdat;
	}

Bits::InputApi&	PC14::reserveL1ST2() throw(){
	reserve();
	_pcpar.high();
	_pcdir.high();
	_pcso.low();
	return _pcdat;
	}

void	PC14::reserve() throw(){
	if(_reservation.testAndSet(_mask)) return;
	ErrorFatal::logAndExit("PC14 already reserved.");
	}

