/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "l1table.h"

using namespace Oscl::Mot8xx::Mmu;

Level1Descriptor::Level1Descriptor() throw():
		_desc(Mot8xx::Mmu::Level1Desc::V::Value_Invalid)
		{
	}

void	Level1Descriptor::setLevel2Table(Level2Table& level2Table) throw(){
	_desc	&= ~(Level1Desc::L2BA::FieldMask);
	_desc	|= (((unsigned long)&level2Table) & Level1Desc::L2BA::FieldMask);
	}

Level2Table*	Level1Descriptor::getLevel2Table() throw(){
	return (Level2Table*)(_desc & Level1Desc::L2BA::FieldMask);
	}

void	Level1Descriptor::setAccessProtectionGroup(unsigned char apg) throw(){
	_desc	&= ~(Level1Desc::APG::FieldMask);
	_desc	|= ((apg<<Level1Desc::APG::Lsb) & Level1Desc::APG::FieldMask);
	}

void	Level1Descriptor::setGuarded() throw(){
	_desc	&= ~(Level1Desc::G::FieldMask);
	_desc	|= Level1Desc::G::ValueMask_Guarded;
	}

void	Level1Descriptor::setNonGuarded() throw(){
	_desc	&= ~(Level1Desc::G::FieldMask);
	_desc	|= Level1Desc::G::ValueMask_NonGuarded;
	}

void	Level1Descriptor::setPageSize4KByte() throw(){
	_desc	&= ~(Level1Desc::PS::FieldMask);
	_desc	|= Level1Desc::PS::ValueMask_Small;
	}

void	Level1Descriptor::setPageSize16KByte() throw(){
	_desc	&= ~(Level1Desc::PS::FieldMask);
	_desc	|= Level1Desc::PS::ValueMask_Small;
	}

void	Level1Descriptor::setPageSize512KByte() throw(){
	_desc	&= ~(Level1Desc::PS::FieldMask);
	_desc	|= Level1Desc::PS::ValueMask_512K;
	}

void	Level1Descriptor::setPageSize8MByte() throw(){
	_desc	&= ~(Level1Desc::PS::FieldMask);
	_desc	|= Level1Desc::PS::ValueMask_8M;
	}

void	Level1Descriptor::setCacheWritethrough() throw(){
	_desc	&= ~(Level1Desc::WT::FieldMask);
	_desc	|= Level1Desc::WT::ValueMask_Writethrough;
	}

void	Level1Descriptor::setCacheCopyback() throw(){
	_desc	&= ~(Level1Desc::WT::FieldMask);
	_desc	|= Level1Desc::WT::ValueMask_Copyback;
	}

void	Level1Descriptor::setDescriptorValid() throw(){
	_desc	&= ~(Level1Desc::V::FieldMask);
	_desc	|= Level1Desc::V::ValueMask_Valid;
	}

void	Level1Descriptor::setDescriptorInvalid() throw(){
	_desc	&= ~(Level1Desc::V::FieldMask);
	_desc	|= Level1Desc::V::ValueMask_Invalid;
	}

bool	Level1Descriptor::isValid() throw(){
	return	(_desc & Level1Desc::V::FieldMask)
			== Level1Desc::V::ValueMask_Valid;
	}

Level1Twam1Table::Level1Twam1Table() throw() {
	}

Level1Descriptor* Level1Twam1Table::getDesc(unsigned entry) throw(){
	if(entry > (nEntriesInLevel1Twam1Table-1)){
		return 0;
		}
	return &_table[entry];
	}

Level1Descriptor* Level1Twam1Table::getDesc(void* virtualAddr) throw(){
	return getDesc(	(unsigned)(
					(	((unsigned long)virtualAddr)
							& Mot8xx::Mmu::Twam1::EA::Level1Index::FieldMask)
							>> Mot8xx::Mmu::Twam1::EA::Level1Index::Lsb
						)
					);
	}

