/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_drv_mot_mpc8xx_siu_irqdrvh_
#define _oscl_drv_mot_mpc8xx_siu_irqdrvh_
#include "irqdrvapi.h"
#include "irqnum.h"

/** */
namespace Oscl {
/** */
namespace Mot8xx {
/** */
namespace Siu {

/** */
template <IrqEnum IrqNumber>
class IrqDriver : public IrqDriverApi {
	private:
		/** */
		volatile SIMASK::Reg&	_mask;
		/** */
		volatile SIPEND::Reg&	_pend;
		/** */
		volatile SIEL::Reg&		_el;
	public:
		/** */
		IrqDriver(	volatile SIMASK::Reg&	mask,
					volatile SIPEND::Reg&	pend,
					volatile SIEL::Reg&		el
					) throw():
				_mask(mask),
				_pend(pend),
				_el(el)
				{
			disable();
			}
		/** */
		void	enable() throw(){
			_mask	|= (		((Oscl::Mot8xx::Siu::SIMASK::Reg)0x80000000)
							>>	(IrqNumber*2)
							);
			}
		/** */
		void	disable() throw(){
			_mask	&= ~(		((Oscl::Mot8xx::Siu::SIMASK::Reg)0x80000000)
							>>	(IrqNumber*2)
							);
			}
		/** */
		void	edgeMode() throw(){
			_el	|= (((Oscl::Mot8xx::Siu::SIEL::Reg)0x80000000)>>(IrqNumber*2));
			}
		/** */
		void	levelMode() throw(){
			_el	&= ~(((Oscl::Mot8xx::Siu::SIEL::Reg)0x80000000)>>(IrqNumber*2));
			}
		/** */
		void	ack() throw(){
			_pend	|= (		((Oscl::Mot8xx::Siu::SIPEND::Reg)0x80000000)
							>>	(IrqNumber*2)
							);
			}
		/** */
		void	enableLowPowerWakeUp() throw(){
			_el	|= (((Oscl::Mot8xx::Siu::SIEL::Reg)0x40000000)>>(IrqNumber*2));
			}
		/** */
		void	disableLowPowerWakeUp() throw(){
			_el	&= ~(((Oscl::Mot8xx::Siu::SIEL::Reg)0x40000000)>>(IrqNumber*2));
			}
	};

}
}
}

#endif
