/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_drv_mot_quicc_sdma_rx_driverh_
#define _oscl_drv_mot_quicc_sdma_rx_driverh_
#include "api.h"
#include "oscl/frame/rx/return.h"
#include "rxbd.h"

/** */
namespace Oscl{
/** */
namespace Motorola {
/** */
namespace Quicc {
/** */
namespace Sdma {
/** */
namespace Rx {

/** */
class Driver : public Api {
	private:
		/** */
		typedef struct Buffer {
			/** */
			struct Buffer*	_link;
			} Buffer;
		/** */
		Buffer*							_freeBuffers;
		/** */
		const unsigned					_nDescs;
		/** */
		unsigned						_nextDescExpected;
		/** */
		unsigned						_firstDescSansBuffer;
		/** */
		unsigned						_nDescsSansBuffer;
		/** */
		unsigned						_nextDescInBlock;
		/** */
		unsigned						_bufferSize;
		/** */
		unsigned						_nBuffsInBlock;
		/** */
		const bool						_interruptEveryBuffer;
		/** */
		bool							_initialized;
		/** */
		bool							_blockCurrentlyOpen;

	protected:
		/** */
		BufferDesc*						const _descriptors;

	public:
		/** */
		Driver(	BufferDesc					descriptors[],
				unsigned					nDescriptors,
				bool						interruptEveryBuffer
				) throw();
		/** */
		void		initialize() throw();
		/** */
		bool		openBlock() throw();
		/** */
		unsigned	getNextBuffer(Octet** buffer) throw();
		/** */
		void		closeBlock() throw();
		/** */
		void		cancelBlock() throw();
		/** */
		unsigned	minimumBufferSize() const throw();
		/** */
		unsigned	maximumBufferSize() const throw();
		/** */
		bool		giveBuffer(	Octet*		buffer,
								Unsigned16	size
								) throw();
		/** */
		void		returnBuffer(Octet*	 buffer) throw();
		/** */
		void		reclaimBuffers(Oscl::Frame::Rx::Return& callback) throw();
		/** */
		unsigned	notEmptyDescIterate(	DescStatusMatch&	endConditon
											) throw();
		unsigned	currentDesc() const throw();

		/** */
		void		reuseBuffer(Octet* buffer) throw();

	private:
		/** */
		void		freeBuffer(Octet* buffer) throw();
		/** */
		Octet*		allocateBuffer() throw();
		/** */
		bool		attachToNeedyDesc(Octet* buffer) throw();
		/** */
		bool		descriptorHasNoBuffer(unsigned desc) throw();
		/** */
	};

}
}
}
}
}
#endif
