/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_driver_pci_funcapih_
#define _oscl_driver_pci_funcapih_
#include "config.h"
#include "oscl/hw/pci/config.h"

/** */
namespace Oscl {
/** */
namespace Pci {
/** */
namespace Config {
/** */
namespace Function {

/** */
typedef enum DevSelTiming {
	/** */
	Fast=0,
	/** */
	Medium=1,
	/** */
	Slow=2
	};

/** */
typedef enum HeaderType {
	/** */
	Standard=0,
	/** */
	PciToPciBridge=1,
	/** */
	PciToCardBusBridge=2
	};

/** */
class Api {
	public:
		/** */
		bool	exists() throw();
	public:
		/** Initiates transaction to read all PCI configuration
			registers for this function. Returns true if the
			function exits in PCI space and configuration was
			successfully read into cache.
		 */
		virtual bool			update() throw()=0;
	public:	// ID::Reg
		/** */
		virtual Unsigned16		deviceID() const throw()=0;
		/** */
		virtual Unsigned16		vendorID() const throw()=0;
	public:	// CmdStatus::CmdReg
		/** */
		virtual bool			busMasteringEnabled() const throw()=0;
		/** */
		virtual void			enableBusMastering() throw()=0;
		/** */
		virtual void			disableBusMastering() throw()=0;
		/** */
		virtual bool			monitoringSpecialCycles() const throw()=0;
		/** */
		virtual void			respondToSpecialCycles() throw()=0;
		/** */
		virtual void			ignoreSpecialCycles() throw()=0;
		/** */
		virtual bool			memoryWriteAndInvalidateEnabled() const throw()=0;
		/** */
		virtual void			enableMemoryWriteAndInvalidate() throw()=0;
		/** */
		virtual void			disableMemoryWriteAndInvalidate() throw()=0;
		/** */
		virtual bool			snoopingVgaPalette() const throw()=0;
		/** */
		virtual void			enableVgaPaletteSnoop() throw()=0;
		/** */
		virtual void			disableVgaPaletteSnoop() throw()=0;
		/** */
		virtual bool			parityErrorEnabled() const throw()=0;
		/** */
		virtual void			enableParityError() throw()=0;
		/** */
		virtual void			disableParityError() throw()=0;
		/** */
		virtual bool			steppingEnabled() const throw()=0;
		/** */
		virtual void			enableStepping() throw()=0;
		/** */
		virtual void			disableStepping() throw()=0;
		/** */
		virtual bool			systemErrorEnabled() const throw()=0;
		/** */
		virtual void			enableSystemError() throw()=0;
		/** */
		virtual void			disableSystemError() throw()=0;
		/** */
		virtual bool			fastBackToBackEnabled() const throw()=0;
		/** */
		virtual void			enableFastBackToBack() throw()=0;
		/** */
		virtual void			disableFastBackToBack() throw()=0;
	public:	// CmdStatus::StatusReg
		/** */
		virtual void			updateStatus() throw()=0;
		/** */
		virtual bool			hasCapabilitiesList() const throw()=0;
		/** */
		virtual bool			capableOf66MHz() const throw()=0;
		/** */
		virtual bool			udfSupported() const throw()=0;
		/** */
		virtual bool			fastBackToBackCapable() const throw()=0;
		/** */
		virtual bool			masterDataParityError() const throw()=0;
		/** */
		virtual void			clearMasterDataParityError() throw()=0;
		/** */
		virtual DevSelTiming	deviceSelectTiming() const throw()=0;
		/** */
		virtual bool			signaledTargetAbort() const throw()=0;
		/** */
		virtual void			clearSignaledTargetAbort() throw()=0;
		/** */
		virtual bool			receivedTargetAbort() const throw()=0;
		/** */
		virtual void			clearReceivedTargetAbort() throw()=0;
		/** */
		virtual bool			receivedMasterAbort() const throw()=0;
		/** */
		virtual void			clearReceivedMasterAbort() throw()=0;
		/** */
		virtual bool			signaledSystemError() const throw()=0;
		/** */
		virtual void			clearSignaledSystemError() throw()=0;
		/** */
		virtual bool			detectedParityError() const throw()=0;
		/** */
		virtual void			clearDetectedParityError() throw()=0;
	public:	// ClassRev::Reg
		/** */
		virtual unsigned char	baseClass() const throw()=0;
		/** */
		virtual unsigned char	subClass() const throw()=0;
		/** */
		virtual unsigned char	programmingInterface() const throw()=0;
		/** */
		virtual unsigned char	revisionID() const throw()=0;
	public:	// BistTypeLatCache::Reg
		/** */
		virtual bool			singleFunction() const throw()=0;
		/** */
		virtual HeaderType		headerType() const throw()=0;
		/** */
		virtual unsigned char	cacheLineSize() const throw()=0;
		/** */
		virtual void			setCacheLineSize(unsigned char dwords)throw()=0;
		/** */
		virtual unsigned char	latencyTimer() const throw()=0;
		/** */
		virtual void			setLatencyTimer(unsigned char pciClks)throw()=0;
		/** */
		virtual bool			bistCapable() const throw()=0;
		/** */
		virtual void			startBuiltInSelfTest() throw()=0;
		/** */
		virtual unsigned char	bistCompletionCode() const throw()=0;
	};
}
}
}
}

#endif
