/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "function.h"
#include "oscl/hw/pci/config.h"

using namespace Oscl::Pci::Config;
using namespace Oscl;

Function::Generic::Generic(	Oscl::Pci::Config::Api&	config,
							unsigned char			bus,
							unsigned char			device,
							unsigned char			function
							) throw():
		_config(config),
		_bus(bus),
		_device(device),
		_function(function)
		{
	}

bool			Function::Generic::update() throw(){
	readID();
	if(!exists()) return false;
	readCmdStatus();
	readClassRev();
	readBistTypeLatCache();
	return true;
	}

bool	Function::Generic::exists() const throw(){
	return vendorID() != ID::Vendor::Value_NoFunctionPresent;
	}

Unsigned16		Function::Generic::deviceID() const throw(){
	return (_id & ID::Device::FieldMask)>>ID::Device::Lsb;
	}

Unsigned16		Function::Generic::vendorID() const throw(){
	return (_id & ID::Vendor::FieldMask)>>ID::Vendor::Lsb;
	}

bool			Function::Generic::busMasteringEnabled() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::BusMaster::FieldMask)
			==	CmdStatus::CmdReg::BusMaster::ValueMask_Enable
			;
	}
void			Function::Generic::enableBusMastering() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::BusMaster::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::BusMaster::ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::Generic::disableBusMastering() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::BusMaster::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::BusMaster::ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::Generic::monitoringSpecialCycles() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::SpecialCycles::FieldMask)
			==	CmdStatus::CmdReg::SpecialCycles::ValueMask_Enable
			;
	}
void			Function::Generic::respondToSpecialCycles() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::SpecialCycles::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::SpecialCycles::ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::Generic::ignoreSpecialCycles() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::SpecialCycles::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::SpecialCycles::ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::Generic::memoryWriteAndInvalidateEnabled() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::MemoryWriteAndInvalidate::FieldMask)
			==	CmdStatus::CmdReg::MemoryWriteAndInvalidate::ValueMask_Enable
			;
	}
void			Function::Generic::enableMemoryWriteAndInvalidate() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::MemoryWriteAndInvalidate::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::MemoryWriteAndInvalidate::
					ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::Generic::disableMemoryWriteAndInvalidate() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::MemoryWriteAndInvalidate::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::MemoryWriteAndInvalidate::
					ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::Generic::snoopingVgaPalette() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::VgaPaletteSnoop::FieldMask)
			==	CmdStatus::CmdReg::VgaPaletteSnoop::ValueMask_Enable
			;
	}
void			Function::Generic::enableVgaPaletteSnoop() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::VgaPaletteSnoop::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::VgaPaletteSnoop::ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::Generic::disableVgaPaletteSnoop() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::VgaPaletteSnoop::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::VgaPaletteSnoop::ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::Generic::parityErrorEnabled() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::ParityErrorResponse::FieldMask)
			==	CmdStatus::CmdReg::ParityErrorResponse::ValueMask_Enable
			;
	}
void			Function::Generic::enableParityError() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::ParityErrorResponse::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::ParityErrorResponse::ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::Generic::disableParityError() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::ParityErrorResponse::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::ParityErrorResponse::ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::Generic::steppingEnabled() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::SteppingControl::FieldMask)
			==	CmdStatus::CmdReg::SteppingControl::ValueMask_Enable
			;
	}
void			Function::Generic::enableStepping() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::SteppingControl::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::SteppingControl::ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::Generic::disableStepping() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::SteppingControl::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::SteppingControl::ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::Generic::systemErrorEnabled() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::SystemError::FieldMask)
			==	CmdStatus::CmdReg::SystemError::ValueMask_Enable
			;
	}
void			Function::Generic::enableSystemError() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::SystemError::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::SystemError::ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::Generic::disableSystemError() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::SystemError::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::SystemError::ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::Generic::fastBackToBackEnabled() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::FastBackToBack::FieldMask)
			==	CmdStatus::CmdReg::FastBackToBack::ValueMask_Enable
			;
	}
void			Function::Generic::enableFastBackToBack() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::FastBackToBack::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::FastBackToBack::ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::Generic::disableFastBackToBack() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::FastBackToBack::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::FastBackToBack::ValueMask_Disable;
	flushCmdStatus();
	}

void			Function::Generic::updateStatus() throw(){
	readCmdStatus();
	}

bool			Function::Generic::hasCapabilitiesList() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::CapabilitiesList::FieldMask)
			==	CmdStatus::StatusReg::CapabilitiesList::ValueMask_Implemented
			;
	}

bool			Function::Generic::capableOf66MHz() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::CapableOf66MHz::FieldMask)
			==	CmdStatus::StatusReg::CapableOf66MHz::ValueMask_Capable
			;
	}

bool			Function::Generic::udfSupported() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::UdfSupport::FieldMask)
			==	CmdStatus::StatusReg::UdfSupport::ValueMask_Supported
			;
	}
bool			Function::Generic::fastBackToBackCapable() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::FastBackToBackCapable::FieldMask)
			==	CmdStatus::StatusReg::FastBackToBackCapable::ValueMask_Capable
			;
	}
bool			Function::Generic::masterDataParityError() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::MasterDataParityError::FieldMask)
			==	CmdStatus::StatusReg::MasterDataParityError::ValueMask_Error
			;
	}

void			Function::Generic::clearMasterDataParityError() throw(){
	clearStatus(CmdStatus::StatusReg::MasterDataParityError::ValueMask_Clear);
	}

Oscl::Pci::Config::Function::DevSelTiming
Function::Generic::deviceSelectTiming() const throw(){
	return  (Oscl::Pci::Config::Function::DevSelTiming)(
			(_cmdStatus & CmdStatus::StatusReg::DeviceSelectTiming::FieldMask)
			>>	CmdStatus::StatusReg::DeviceSelectTiming::Lsb
			);
	}
bool			Function::Generic::signaledTargetAbort() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::SignaledTargetAbort::FieldMask)
			==	CmdStatus::StatusReg::SignaledTargetAbort::ValueMask_Abort
			;
	}
void			Function::Generic::clearSignaledTargetAbort() throw(){
	clearStatus(CmdStatus::StatusReg::SignaledTargetAbort::ValueMask_Clear);
	}
bool			Function::Generic::receivedTargetAbort() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::ReceivedTargetAbort::FieldMask)
			==	CmdStatus::StatusReg::ReceivedTargetAbort::ValueMask_Abort
			;
	}
void			Function::Generic::clearReceivedTargetAbort() throw(){
	clearStatus(CmdStatus::StatusReg::ReceivedTargetAbort::ValueMask_Clear);
	}
bool			Function::Generic::receivedMasterAbort() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::ReceivedMasterAbort::FieldMask)
			==	CmdStatus::StatusReg::ReceivedMasterAbort::ValueMask_Abort
			;
	}
void			Function::Generic::clearReceivedMasterAbort() throw(){
	clearStatus(CmdStatus::StatusReg::ReceivedMasterAbort::ValueMask_Clear);
	}
bool			Function::Generic::signaledSystemError() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::SignaledSystemError::FieldMask)
			==	CmdStatus::StatusReg::SignaledSystemError::ValueMask_Error
			;
	}
void			Function::Generic::clearSignaledSystemError() throw(){
	clearStatus(CmdStatus::StatusReg::SignaledSystemError::ValueMask_Clear);
	}
bool			Function::Generic::detectedParityError() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::DetectedParityError::FieldMask)
			==	CmdStatus::StatusReg::DetectedParityError::ValueMask_Error
			;
	}
void			Function::Generic::clearDetectedParityError() throw(){
	clearStatus(CmdStatus::StatusReg::DetectedParityError::ValueMask_Clear);
	}
unsigned char	Function::Generic::baseClass() const throw(){
	return (_classRev & ClassRev::ClassCode::FieldMask)
			>>	ClassRev::ClassCode::Lsb
			;
	}
unsigned char	Function::Generic::subClass() const throw(){
	return (_classRev & ClassRev::SubClassCode::FieldMask)
			>>	ClassRev::SubClassCode::Lsb
			;
	}
unsigned char	Function::Generic::programmingInterface() const throw(){
	return (_classRev & ClassRev::ProgIf::FieldMask)
			>>	ClassRev::ProgIf::Lsb
			;
	}
unsigned char	Function::Generic::revisionID() const throw(){
	return (_classRev & ClassRev::Revision::FieldMask)
			>>	ClassRev::Revision::Lsb
			;
	}
bool			Function::Generic::singleFunction() const throw(){
	return (_bistTypeLatCache & BistTypeLatCache::MultiFunction::FieldMask)
			==	BistTypeLatCache::MultiFunction::ValueMask_SingleFunctionDevice
			;
	}
Oscl::Pci::Config::Function::HeaderType
Function::Generic::headerType() const throw(){
	return  (Oscl::Pci::Config::Function::HeaderType)(
			(_bistTypeLatCache & BistTypeLatCache::HeaderFormat::FieldMask)
			>>	BistTypeLatCache::HeaderFormat::Lsb
			);
	}
unsigned char	Function::Generic::cacheLineSize() const throw(){
	return (_bistTypeLatCache & BistTypeLatCache::CacheLineSize::FieldMask)
			>>	BistTypeLatCache::CacheLineSize::Lsb
			;
	}
void			Function::Generic::setCacheLineSize(unsigned char dwords)throw(){
	_bistTypeLatCache	&= ~BistTypeLatCache::CacheLineSize::FieldMask;
	_bistTypeLatCache	|= (	(dwords<<BistTypeLatCache::CacheLineSize::Lsb)
							&	BistTypeLatCache::CacheLineSize::FieldMask
							);
	flushBistTypeLatCache();
	}
unsigned char	Function::Generic::latencyTimer() const throw(){
	return (_bistTypeLatCache & BistTypeLatCache::LatencyTimer::FieldMask)
			>>	BistTypeLatCache::LatencyTimer::Lsb
			;
	}
void			Function::Generic::setLatencyTimer(unsigned char pciClks)throw(){
	_bistTypeLatCache	&= ~BistTypeLatCache::LatencyTimer::FieldMask;
	_bistTypeLatCache	|=		(pciClks << BistTypeLatCache::LatencyTimer::Lsb)
							&	BistTypeLatCache::LatencyTimer::FieldMask;
	flushBistTypeLatCache();
	readBistTypeLatCache();
	}
bool			Function::Generic::bistCapable() const throw(){
	return (_bistTypeLatCache & BistTypeLatCache::BistCapable::FieldMask)
			==	BistTypeLatCache::BistCapable::ValueMask_Capable
			;
	}

void			Function::Generic::startBuiltInSelfTest() throw(){
	BistTypeLatCache::Reg	mask	= _bistTypeLatCache;
	mask	&= (	BistTypeLatCache::LatencyTimer::FieldMask
				|	BistTypeLatCache::CacheLineSize::FieldMask
				);
	mask	|= BistTypeLatCache::BistStart::ValueMask_Start;
	_config.writeDword(	_bus,
						_device,
						_function,
						DwRegOffset::Bist,
						mask
						);
	}

unsigned char	Function::Generic::bistCompletionCode() const throw(){
	return (_bistTypeLatCache & BistTypeLatCache::CompletionCode::FieldMask)
			>>	BistTypeLatCache::CompletionCode::Lsb
			;
	}

void	Function::Generic::readID() throw(){
	_id	= _config.readDword(	_bus,
								_device,
								_function,
								DwRegOffset::ID
								);
	}

void	Function::Generic::readCmdStatus() throw(){
	_cmdStatus	= _config.readDword(	_bus,
										_device,
										_function,
										DwRegOffset::CmdStatus
										);
	}

void	Function::Generic::readClassRev() throw(){
	_classRev	= _config.readDword(	_bus,
										_device,
										_function,
										DwRegOffset::ClassRev
										);
	}

void	Function::Generic::readBistTypeLatCache() throw(){
	_bistTypeLatCache	= _config.readDword(	_bus,
												_device,
												_function,
												DwRegOffset::Bist
												);
	}

void	Function::Generic::flushCmdStatus() throw(){
	_config.writeDword(	_bus,
						_device,
						_function,
						DwRegOffset::CmdStatus,
						_cmdStatus & CmdStatus::CmdReg::FieldMask
						);
	}

void	Function::Generic::flushBistTypeLatCache() throw(){
	_config.writeDword(	_bus,
						_device,
						_function,
						DwRegOffset::Bist,
						_bistTypeLatCache & (
								BistTypeLatCache::LatencyTimer::FieldMask
							|	BistTypeLatCache::CacheLineSize::FieldMask
							)
						);
	}

void	Function::Generic::clearStatus(CmdStatus::Reg bitsToClear) throw(){
	CmdStatus::Reg	masked	= _cmdStatus;
	masked	&= CmdStatus::StatusReg::FieldMask;
	bitsToClear	&= ~CmdStatus::StatusReg::FieldMask;
	masked	|= bitsToClear;
	_config.writeDword(	_bus,
						_device,
						_function,
						DwRegOffset::CmdStatus,
						masked
						);
	}
