/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_driver_pci_functionh_
#define _oscl_driver_pci_functionh_
#include "funcapi.h"
#include "config.h"
#include "oscl/hw/pci/config.h"

/** */
namespace Oscl {
/** */
namespace Pci {
/** */
namespace Config {
/** */
namespace Function {

/** */
class Generic : public Api{
	private:
		/** */
		Oscl::Pci::Config::Api&			_config;
		/** */
		unsigned char					_bus;
		/** */
		unsigned char					_device;
		/** */
		unsigned char					_function;
		/** */
		ID::Reg							_id;
		/** */
		CmdStatus::Reg					_cmdStatus;
		/** */
		ClassRev::Reg					_classRev;
		/** */
		BistTypeLatCache::Reg			_bistTypeLatCache;
	public:
		/** */
		Generic(	Oscl::Pci::Config::Api&	config,
					unsigned char			bus,
					unsigned char			device,
					unsigned char			function
					) throw();
	public:
		/** */
		bool	exists() const throw();
	public:
		/** Initiates transaction to read all PCI configuration
			registers for this function. Returns true if the
			function exits in PCI space and configuration was
			successfully read into cache.
		 */
		bool			update() throw();
	public:	// ID::Reg
		/** */
		Unsigned16		deviceID() const throw();
		/** */
		Unsigned16		vendorID() const throw();
	public:	// CmdStatus::CmdReg
		/** */
		bool			ioDecodersEnabled() const throw();
		/** */
		void			enableIoDecoders() throw();
		/** */
		void			disableIoDecoders() throw();
		/** */
		bool			memoryDecodersEnabled() const throw();
		/** */
		void			enableMemoryDecoders() throw();
		/** */
		void			disableMemoryDecoders() throw();
		/** */
		bool			busMasteringEnabled() const throw();
		/** */
		void			enableBusMastering() throw();
		/** */
		void			disableBusMastering() throw();
		/** */
		bool			monitoringSpecialCycles() const throw();
		/** */
		void			respondToSpecialCycles() throw();
		/** */
		void			ignoreSpecialCycles() throw();
		/** */
		bool			memoryWriteAndInvalidateEnabled() const throw();
		/** */
		void			enableMemoryWriteAndInvalidate() throw();
		/** */
		void			disableMemoryWriteAndInvalidate() throw();
		/** */
		bool			snoopingVgaPalette() const throw();
		/** */
		void			enableVgaPaletteSnoop() throw();
		/** */
		void			disableVgaPaletteSnoop() throw();
		/** */
		bool			parityErrorEnabled() const throw();
		/** */
		void			enableParityError() throw();
		/** */
		void			disableParityError() throw();
		/** */
		bool			steppingEnabled() const throw();
		/** */
		void			enableStepping() throw();
		/** */
		void			disableStepping() throw();
		/** */
		bool			systemErrorEnabled() const throw();
		/** */
		void			enableSystemError() throw();
		/** */
		void			disableSystemError() throw();
		/** */
		bool			fastBackToBackEnabled() const throw();
		/** */
		void			enableFastBackToBack() throw();
		/** */
		void			disableFastBackToBack() throw();
	public:	// CmdStatus::StatusReg
		/** */
		void			updateStatus() throw();
		/** */
		bool			hasCapabilitiesList() const throw();
		/** */
		bool			capableOf66MHz() const throw();
		/** */
		bool			udfSupported() const throw();
		/** */
		bool			fastBackToBackCapable() const throw();
		/** */
		bool			masterDataParityError() const throw();
		/** */
		void			clearMasterDataParityError() throw();
		/** */
		DevSelTiming	deviceSelectTiming() const throw();
		/** */
		bool			signaledTargetAbort() const throw();
		/** */
		void			clearSignaledTargetAbort() throw();
		/** */
		bool			receivedTargetAbort() const throw();
		/** */
		void			clearReceivedTargetAbort() throw();
		/** */
		bool			receivedMasterAbort() const throw();
		/** */
		void			clearReceivedMasterAbort() throw();
		/** */
		bool			signaledSystemError() const throw();
		/** */
		void			clearSignaledSystemError() throw();
		/** */
		bool			detectedParityError() const throw();
		/** */
		void			clearDetectedParityError() throw();
	public:	// ClassRev::Reg
		/** */
		unsigned char	baseClass() const throw();
		/** */
		unsigned char	subClass() const throw();
		/** */
		unsigned char	programmingInterface() const throw();
		/** */
		unsigned char	revisionID() const throw();
	public:	// BistTypeLatCache::Reg
		/** */
		bool			singleFunction() const throw();
		/** */
		HeaderType		headerType() const throw();
		/** */
		unsigned char	cacheLineSize() const throw();
		/** */
		void			setCacheLineSize(unsigned char dwords)throw();
		/** */
		unsigned char	latencyTimer() const throw();
		/** */
		void			setLatencyTimer(unsigned char pciClks)throw();
		/** */
		bool			bistCapable() const throw();
		/** */
		void			startBuiltInSelfTest() throw();
		/** */
		unsigned char	bistCompletionCode() const throw();
	private:
		/** */
		void	readID() throw();
		/** */
		void	readCmdStatus() throw();
		/** */
		void	readClassRev() throw();
		/** */
		void	readBistTypeLatCache() throw();
	private:
		/** */
		void	flushCmdStatus() throw();
		/** */
		void	flushBistTypeLatCache() throw();
		/** */
		void	clearStatus(CmdStatus::Reg bitsToClear) throw();
	};

}
}
}
}

#endif
