/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "stdfunc.h"
#include "oscl/hw/pci/config.h"

using namespace Oscl::Pci::Config;
using namespace Oscl;

Function::HeaderTypeZero::HeaderTypeZero(	Oscl::Pci::Config::Api&	config,
						unsigned char			bus,
						unsigned char			device,
						unsigned char			function
						) throw():
		_config(config),
		_bus(bus),
		_device(device),
		_function(function)
		{
	}

bool			Function::HeaderTypeZero::update() throw(){
	using namespace Oscl::Pci::Config::HeaderTypeZero;
	readID();
	if(!exists()) return false;
	readCmdStatus();
	readClassRev();
	readBistTypeLatCache();
	switch(headerType()){
		case Standard:
			// Only deal with these for now
			break;
		case PciToPciBridge:
		case PciToCardBusBridge:
		default:
			return true;
		};
	for(unsigned reg=0;reg<MaxBaseAddressRegisters;++reg){
		readBaseAddress(reg);
		_baseAddress[reg]	= ~(Unsigned32)0;
		flushBaseAddress(reg);
		readBaseAddress(reg);
		}
	readSubsystem();
	readExpansionRomBaseAddr();
	readMaxLatMinGntIntPinIntLine();
	return true;
	}

bool	Function::HeaderTypeZero::exists() const throw(){
	return vendorID() != ID::Vendor::Value_NoFunctionPresent;
	}

Unsigned16		Function::HeaderTypeZero::deviceID() const throw(){
	return (_id & ID::Device::FieldMask)>>ID::Device::Lsb;
	}

Unsigned16		Function::HeaderTypeZero::vendorID() const throw(){
	return (_id & ID::Vendor::FieldMask)>>ID::Vendor::Lsb;
	}

bool			Function::HeaderTypeZero::ioDecodersEnabled() const throw(){
	using namespace Oscl::Pci::Config::HeaderTypeZero::CmdStatus;
	return (_cmdStatus & CmdReg::IoSpace::FieldMask)
			==	CmdReg::IoSpace::ValueMask_Enable
			;
	}

void			Function::HeaderTypeZero::enableIoDecoders() throw(){
	using namespace Oscl::Pci::Config::HeaderTypeZero::CmdStatus;
	_cmdStatus	&= ~CmdReg::IoSpace::FieldMask;
	_cmdStatus	|= CmdReg::IoSpace::ValueMask_Enable;
	flushCmdStatus();
	}

void			Function::HeaderTypeZero::disableIoDecoders() throw(){
	using namespace Oscl::Pci::Config::HeaderTypeZero::CmdStatus;
	_cmdStatus	&= ~CmdReg::IoSpace::FieldMask;
	_cmdStatus	|= CmdReg::IoSpace::ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::HeaderTypeZero::memoryDecodersEnabled() const throw(){
	using namespace Oscl::Pci::Config::HeaderTypeZero::CmdStatus;
	return (_cmdStatus & CmdReg::MemorySpace::FieldMask)
			==	CmdReg::MemorySpace::ValueMask_Enable
			;
	}
void			Function::HeaderTypeZero::enableMemoryDecoders() throw(){
	using namespace Oscl::Pci::Config::HeaderTypeZero::CmdStatus;
	_cmdStatus	&= ~CmdReg::MemorySpace::FieldMask;
	_cmdStatus	|= CmdReg::MemorySpace::ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::HeaderTypeZero::disableMemoryDecoders() throw(){
	using namespace Oscl::Pci::Config::HeaderTypeZero::CmdStatus;
	_cmdStatus	&= ~CmdReg::MemorySpace::FieldMask;
	_cmdStatus	|= CmdReg::MemorySpace::ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::HeaderTypeZero::busMasteringEnabled() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::BusMaster::FieldMask)
			==	CmdStatus::CmdReg::BusMaster::ValueMask_Enable
			;
	}
void			Function::HeaderTypeZero::enableBusMastering() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::BusMaster::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::BusMaster::ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::HeaderTypeZero::disableBusMastering() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::BusMaster::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::BusMaster::ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::HeaderTypeZero::monitoringSpecialCycles() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::SpecialCycles::FieldMask)
			==	CmdStatus::CmdReg::SpecialCycles::ValueMask_Enable
			;
	}
void			Function::HeaderTypeZero::respondToSpecialCycles() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::SpecialCycles::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::SpecialCycles::ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::HeaderTypeZero::ignoreSpecialCycles() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::SpecialCycles::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::SpecialCycles::ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::HeaderTypeZero::memoryWriteAndInvalidateEnabled() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::MemoryWriteAndInvalidate::FieldMask)
			==	CmdStatus::CmdReg::MemoryWriteAndInvalidate::ValueMask_Enable
			;
	}
void			Function::HeaderTypeZero::enableMemoryWriteAndInvalidate() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::MemoryWriteAndInvalidate::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::MemoryWriteAndInvalidate::
					ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::HeaderTypeZero::disableMemoryWriteAndInvalidate() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::MemoryWriteAndInvalidate::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::MemoryWriteAndInvalidate::
					ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::HeaderTypeZero::snoopingVgaPalette() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::VgaPaletteSnoop::FieldMask)
			==	CmdStatus::CmdReg::VgaPaletteSnoop::ValueMask_Enable
			;
	}
void			Function::HeaderTypeZero::enableVgaPaletteSnoop() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::VgaPaletteSnoop::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::VgaPaletteSnoop::ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::HeaderTypeZero::disableVgaPaletteSnoop() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::VgaPaletteSnoop::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::VgaPaletteSnoop::ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::HeaderTypeZero::parityErrorEnabled() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::ParityErrorResponse::FieldMask)
			==	CmdStatus::CmdReg::ParityErrorResponse::ValueMask_Enable
			;
	}
void			Function::HeaderTypeZero::enableParityError() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::ParityErrorResponse::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::ParityErrorResponse::ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::HeaderTypeZero::disableParityError() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::ParityErrorResponse::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::ParityErrorResponse::ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::HeaderTypeZero::steppingEnabled() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::SteppingControl::FieldMask)
			==	CmdStatus::CmdReg::SteppingControl::ValueMask_Enable
			;
	}
void			Function::HeaderTypeZero::enableStepping() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::SteppingControl::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::SteppingControl::ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::HeaderTypeZero::disableStepping() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::SteppingControl::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::SteppingControl::ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::HeaderTypeZero::systemErrorEnabled() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::SystemError::FieldMask)
			==	CmdStatus::CmdReg::SystemError::ValueMask_Enable
			;
	}
void			Function::HeaderTypeZero::enableSystemError() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::SystemError::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::SystemError::ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::HeaderTypeZero::disableSystemError() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::SystemError::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::SystemError::ValueMask_Disable;
	flushCmdStatus();
	}
bool			Function::HeaderTypeZero::fastBackToBackEnabled() const throw(){
	return (_cmdStatus & CmdStatus::CmdReg::FastBackToBack::FieldMask)
			==	CmdStatus::CmdReg::FastBackToBack::ValueMask_Enable
			;
	}
void			Function::HeaderTypeZero::enableFastBackToBack() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::FastBackToBack::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::FastBackToBack::ValueMask_Enable;
	flushCmdStatus();
	}
void			Function::HeaderTypeZero::disableFastBackToBack() throw(){
	_cmdStatus	&= ~CmdStatus::CmdReg::FastBackToBack::FieldMask;
	_cmdStatus	|= CmdStatus::CmdReg::FastBackToBack::ValueMask_Disable;
	flushCmdStatus();
	}

bool			Function::HeaderTypeZero::hasCapabilitiesList() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::CapabilitiesList::FieldMask)
			==	CmdStatus::StatusReg::CapabilitiesList::ValueMask_Implemented
			;
	}

bool			Function::HeaderTypeZero::capableOf66MHz() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::CapableOf66MHz::FieldMask)
			==	CmdStatus::StatusReg::CapableOf66MHz::ValueMask_Capable
			;
	}

bool			Function::HeaderTypeZero::udfSupported() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::UdfSupport::FieldMask)
			==	CmdStatus::StatusReg::UdfSupport::ValueMask_Supported
			;
	}
bool			Function::HeaderTypeZero::fastBackToBackCapable() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::FastBackToBackCapable::FieldMask)
			==	CmdStatus::StatusReg::FastBackToBackCapable::ValueMask_Capable
			;
	}
bool			Function::HeaderTypeZero::masterDataParityError() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::MasterDataParityError::FieldMask)
			==	CmdStatus::StatusReg::MasterDataParityError::ValueMask_Error
			;
	}

void			Function::HeaderTypeZero::clearMasterDataParityError() throw(){
	clearStatus(CmdStatus::StatusReg::MasterDataParityError::ValueMask_Clear);
	}

Oscl::Pci::Config::Function::DevSelTiming
Function::HeaderTypeZero::deviceSelectTiming() const throw(){
	return  (Oscl::Pci::Config::Function::DevSelTiming)(
			(_cmdStatus & CmdStatus::StatusReg::DeviceSelectTiming::FieldMask)
			>>	CmdStatus::StatusReg::DeviceSelectTiming::Lsb
			);
	}
bool			Function::HeaderTypeZero::signaledTargetAbort() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::SignaledTargetAbort::FieldMask)
			==	CmdStatus::StatusReg::SignaledTargetAbort::ValueMask_Abort
			;
	}
void			Function::HeaderTypeZero::clearSignaledTargetAbort() throw(){
	clearStatus(CmdStatus::StatusReg::SignaledTargetAbort::ValueMask_Clear);
	}
bool			Function::HeaderTypeZero::receivedTargetAbort() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::ReceivedTargetAbort::FieldMask)
			==	CmdStatus::StatusReg::ReceivedTargetAbort::ValueMask_Abort
			;
	}
void			Function::HeaderTypeZero::clearReceivedTargetAbort() throw(){
	clearStatus(CmdStatus::StatusReg::ReceivedTargetAbort::ValueMask_Clear);
	}
bool			Function::HeaderTypeZero::receivedMasterAbort() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::ReceivedMasterAbort::FieldMask)
			==	CmdStatus::StatusReg::ReceivedMasterAbort::ValueMask_Abort
			;
	}
void			Function::HeaderTypeZero::clearReceivedMasterAbort() throw(){
	clearStatus(CmdStatus::StatusReg::ReceivedMasterAbort::ValueMask_Clear);
	}
bool			Function::HeaderTypeZero::signaledSystemError() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::SignaledSystemError::FieldMask)
			==	CmdStatus::StatusReg::SignaledSystemError::ValueMask_Error
			;
	}
void			Function::HeaderTypeZero::clearSignaledSystemError() throw(){
	clearStatus(CmdStatus::StatusReg::SignaledSystemError::ValueMask_Clear);
	}
bool			Function::HeaderTypeZero::detectedParityError() const throw(){
	return (_cmdStatus & CmdStatus::StatusReg::DetectedParityError::FieldMask)
			==	CmdStatus::StatusReg::DetectedParityError::ValueMask_Error
			;
	}
void			Function::HeaderTypeZero::clearDetectedParityError() throw(){
	clearStatus(CmdStatus::StatusReg::DetectedParityError::ValueMask_Clear);
	}
unsigned char	Function::HeaderTypeZero::baseClass() const throw(){
	return (_classRev & ClassRev::ClassCode::FieldMask)
			>>	ClassRev::ClassCode::Lsb
			;
	}
unsigned char	Function::HeaderTypeZero::subClass() const throw(){
	return (_classRev & ClassRev::SubClassCode::FieldMask)
			>>	ClassRev::SubClassCode::Lsb
			;
	}
unsigned char	Function::HeaderTypeZero::programmingInterface() const throw(){
	return (_classRev & ClassRev::ProgIf::FieldMask)
			>>	ClassRev::ProgIf::Lsb
			;
	}
unsigned char	Function::HeaderTypeZero::revisionID() const throw(){
	return (_classRev & ClassRev::Revision::FieldMask)
			>>	ClassRev::Revision::Lsb
			;
	}
bool			Function::HeaderTypeZero::singleFunction() const throw(){
	return (_bistTypeLatCache & BistTypeLatCache::MultiFunction::FieldMask)
			==	BistTypeLatCache::MultiFunction::ValueMask_SingleFunctionDevice
			;
	}
Oscl::Pci::Config::Function::HeaderType
Function::HeaderTypeZero::headerType() const throw(){
	return  (Oscl::Pci::Config::Function::HeaderType)(
			(_bistTypeLatCache & BistTypeLatCache::HeaderFormat::FieldMask)
			>>	BistTypeLatCache::HeaderFormat::Lsb
			);
	}
unsigned char	Function::HeaderTypeZero::cacheLineSize() const throw(){
	return (_bistTypeLatCache & BistTypeLatCache::CacheLineSize::FieldMask)
			>>	BistTypeLatCache::CacheLineSize::Lsb
			;
	}
void			Function::HeaderTypeZero::setCacheLineSize(unsigned char dwords)throw(){
	_bistTypeLatCache	&= ~BistTypeLatCache::CacheLineSize::FieldMask;
	_bistTypeLatCache	|= (	(dwords<<BistTypeLatCache::CacheLineSize::Lsb)
							&	BistTypeLatCache::CacheLineSize::FieldMask
							);
	flushBistTypeLatCache();
	}
unsigned char	Function::HeaderTypeZero::latencyTimer() const throw(){
	return (_bistTypeLatCache & BistTypeLatCache::LatencyTimer::FieldMask)
			>>	BistTypeLatCache::LatencyTimer::Lsb
			;
	}
void			Function::HeaderTypeZero::setLatencyTimer(unsigned char pciClks)throw(){
	_bistTypeLatCache	&= ~BistTypeLatCache::LatencyTimer::FieldMask;
	_bistTypeLatCache	|=		(pciClks << BistTypeLatCache::LatencyTimer::Lsb)
							&	BistTypeLatCache::LatencyTimer::FieldMask;
	flushBistTypeLatCache();
	readBistTypeLatCache();
	}
bool			Function::HeaderTypeZero::bistCapable() const throw(){
	return (_bistTypeLatCache & BistTypeLatCache::BistCapable::FieldMask)
			==	BistTypeLatCache::BistCapable::ValueMask_Capable
			;
	}

void			Function::HeaderTypeZero::startBuiltInSelfTest() throw(){
	BistTypeLatCache::Reg	mask	= _bistTypeLatCache;
	mask	&= (	BistTypeLatCache::LatencyTimer::FieldMask
				|	BistTypeLatCache::CacheLineSize::FieldMask
				);
	mask	|= BistTypeLatCache::BistStart::ValueMask_Start;
	_config.writeDword(	_bus,
						_device,
						_function,
						DwRegOffset::Bist,
						mask
						);
	}

unsigned char	Function::HeaderTypeZero::bistCompletionCode() const throw(){
	return (_bistTypeLatCache & BistTypeLatCache::CompletionCode::FieldMask)
			>>	BistTypeLatCache::CompletionCode::Lsb
			;
	}

bool	Function::HeaderTypeZero::isImplemented(unsigned char bar) const throw(){
	return _baseAddress[bar];
	}

bool	Function::HeaderTypeZero::isIoAddress(unsigned char bar) const throw(){
	return _baseAddress[bar] & BaseAddress::SpaceType::FieldMask;
	}

bool	Function::HeaderTypeZero::prefetchable(unsigned char bar) const throw(){
	return _baseAddress[bar] & BaseAddress::Prefetchable::FieldMask;
	}

Oscl::Pci::Config::Function::DecoderConstraint
Function::HeaderTypeZero::barConstraint(unsigned char bar) const throw(){
	return (Oscl::Pci::Config::Function::DecoderConstraint)(
			(_baseAddress[bar] & BaseAddress::DecoderType::FieldMask)
			>>BaseAddress::DecoderType::Lsb
			);
	}

Unsigned32			Function::HeaderTypeZero::blockSizeMask(unsigned char bar) throw(){
	_baseAddress[bar]	= ~(Unsigned32)0;
	flushBaseAddress(bar);
	readBaseAddress(bar);
	return _baseAddress[bar] & BaseAddress::Address::FieldMask;
	}

void				Function::HeaderTypeZero::setAddress(	unsigned char	bar,
												Unsigned32		address
												) throw(){
	_baseAddress[bar]	&= ~BaseAddress::Address::FieldMask;
	if(address != (address & BaseAddress::Address::FieldMask)){
		while(true);
		}
	_baseAddress[bar]	|= address;
	flushBaseAddress(bar);
	}


void	Function::HeaderTypeZero::readID() throw(){
	_id	= _config.readDword(	_bus,
								_device,
								_function,
								DwRegOffset::ID
								);
	}

void	Function::HeaderTypeZero::readCmdStatus() throw(){
	_cmdStatus	= _config.readDword(	_bus,
										_device,
										_function,
										DwRegOffset::CmdStatus
										);
	}

void	Function::HeaderTypeZero::readClassRev() throw(){
	_classRev	= _config.readDword(	_bus,
										_device,
										_function,
										DwRegOffset::ClassRev
										);
	}

void	Function::HeaderTypeZero::readBistTypeLatCache() throw(){
	_bistTypeLatCache	= _config.readDword(	_bus,
												_device,
												_function,
												DwRegOffset::Bist
												);
	}

void	Function::HeaderTypeZero::readBaseAddress(unsigned baReg) throw(){
	using namespace Oscl::Pci::Config::HeaderTypeZero::DwRegOffset;
	_baseAddress[baReg]	= _config.readDword(	_bus,
												_device,
												_function,
												BaseAddr0+baReg
												);
	}

void	Function::HeaderTypeZero::readSubsystem() throw(){
	using namespace Oscl::Pci::Config::HeaderTypeZero::DwRegOffset;
	_subsystem	= _config.readDword(	_bus,
										_device,
										_function,
										SubsystemID
										);
	}

void	Function::HeaderTypeZero::readExpansionRomBaseAddr() throw(){
	using namespace Oscl::Pci::Config::HeaderTypeZero::DwRegOffset;
	_expansionRomBaseAddr	= _config.readDword(	_bus,
													_device,
													_function,
													ExpRomBaseAddr
													);
	}

void	Function::HeaderTypeZero::readMaxLatMinGntIntPinIntLine() throw(){
	using namespace Oscl::Pci::Config::HeaderTypeZero::DwRegOffset;
	_maxLatMinGntIntPinIntLine	= _config.readDword(	_bus,
														_device,
														_function,
														MaxLat
														);
	}

void	Function::HeaderTypeZero::flushCmdStatus() throw(){
	_config.writeDword(	_bus,
						_device,
						_function,
						DwRegOffset::CmdStatus,
						_cmdStatus & CmdStatus::CmdReg::FieldMask
						);
	}

void	Function::HeaderTypeZero::flushBistTypeLatCache() throw(){
	_config.writeDword(	_bus,
						_device,
						_function,
						DwRegOffset::Bist,
						_bistTypeLatCache & (
								BistTypeLatCache::LatencyTimer::FieldMask
							|	BistTypeLatCache::CacheLineSize::FieldMask
							)
						);
	}

void	Function::HeaderTypeZero::flushBaseAddress(unsigned baReg) throw(){
	using namespace Oscl::Pci::Config::HeaderTypeZero::DwRegOffset;
	_config.writeDword(	_bus,
						_device,
						_function,
						BaseAddr0+baReg,
						_baseAddress[baReg]
						);
	}

void	Function::HeaderTypeZero::flushSubsystem() throw(){
	using namespace Oscl::Pci::Config::HeaderTypeZero::DwRegOffset;
	_config.writeDword(	_bus,
						_device,
						_function,
						SubsystemID,
						_subsystem
						);
	}

void	Function::HeaderTypeZero::flushExpansionRomBaseAddr() throw(){
	using namespace Oscl::Pci::Config::HeaderTypeZero::DwRegOffset;
	_config.writeDword(	_bus,
						_device,
						_function,
						ExpRomBaseAddr,
						_expansionRomBaseAddr
						);
	}

void	Function::HeaderTypeZero::flushMaxLatMinGntIntPinIntLine() throw(){
	using namespace Oscl::Pci::Config::HeaderTypeZero::DwRegOffset;
	_config.writeDword(	_bus,
						_device,
						_function,
						MaxLat,
						_maxLatMinGntIntPinIntLine &
							(	Config::HeaderTypeZero::
									MaxLatMinGntIntPinIntLine::InterruptLine::
									FieldMask
								)
						);
	}

void	Function::HeaderTypeZero::clearStatus(CmdStatus::Reg bitsToClear) throw(){
	CmdStatus::Reg	masked	= _cmdStatus;
	masked	&= ~CmdStatus::StatusReg::FieldMask;
	bitsToClear	&= CmdStatus::StatusReg::FieldMask;
	masked	|= bitsToClear;
	_config.writeDword(	_bus,
						_device,
						_function,
						DwRegOffset::CmdStatus,
						masked
						);
	}

void			Function::HeaderTypeZero::updateStatus() throw(){
	readCmdStatus();
	}

