/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_driver_tundra_qspan_interrupt_dirh_
#define _oscl_driver_tundra_qspan_interrupt_dirh_
#include "oscl/hw/tundra/qspan/fields.h"

/** */
namespace Oscl {
/** */
namespace Tundra {
/** */
namespace Qspan {
/** */
namespace Irq {

/** */
class DirectionDriver {
	private:
		/** */
		volatile InterruptDirectionControl::Reg&	_control;
	public:
		/** */
		DirectionDriver(
			volatile InterruptDirectionControl::Reg& control
			) throw():_control(control){}
	public:
		/** */
		inline void	pciBusErrorLogInterruptToPciBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::PEL::FieldMask
							)
						| InterruptDirectionControl::PEL::ValueMask_ToPciBus;
			}
		/** */
		inline void	mapPciBusErrorLogInterruptToQBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::PEL::FieldMask
							)
						| InterruptDirectionControl::PEL::ValueMask_ToQBus;
			}
		/** */
		inline void	mapQBusErrorLogInterruptToPciBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::QEL::FieldMask
							)
						| InterruptDirectionControl::QEL::ValueMask_ToPciBus;
			}
		/** */
		inline void	mapQBusErrorLogInterruptToQBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::QEL::FieldMask
							)
						| InterruptDirectionControl::QEL::ValueMask_ToQBus;
			}
		/** */
		inline void	mapDataParityDetectedInterruptToPciBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::DPD::FieldMask
							)
						| InterruptDirectionControl::DPD::ValueMask_ToPciBus;
			}
		/** */
		inline void	mapDataParityDetectedInterruptToQBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::DPD::FieldMask
							)
						| InterruptDirectionControl::DPD::ValueMask_ToQBus;
			}
		/** */
		inline void	mapIdmaQBusErrorInterruptToPciBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::IQE::FieldMask
							)
						| InterruptDirectionControl::IQE::ValueMask_ToPciBus;
			}
		/** */
		inline void	mapIdmaQBusErrorInterruptToQBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::IQE::FieldMask
							)
						| InterruptDirectionControl::IQE::ValueMask_ToQBus;
			}
		/** */
		inline void	mapIdmaPciErrorInterruptToPciBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::IPE::FieldMask
							)
						| InterruptDirectionControl::IPE::ValueMask_ToPciBus;
			}
		/** */
		inline void	mapIdmaPciErrorInterruptToQBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::IPE::FieldMask
							)
						| InterruptDirectionControl::IPE::ValueMask_ToQBus;
			}
		/** */
		inline void	mapIdmaResetInterruptToPciBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::IRST::FieldMask
							)
						| InterruptDirectionControl::IRST::ValueMask_ToPciBus;
			}
		/** */
		inline void	mapIdmaResetInterruptToQBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::IRST::FieldMask
							)
						| InterruptDirectionControl::IRST::ValueMask_ToQBus;
			}
		/** */
		inline void	mapIdmaDoneInterruptToPciBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::DONE::FieldMask
							)
						| InterruptDirectionControl::DONE::ValueMask_ToPciBus;
			}
		/** */
		inline void	mapIdmaDoneInterruptToQBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::DONE::FieldMask
							)
						| InterruptDirectionControl::DONE::ValueMask_ToQBus;
			}
		/** */
		inline void	mapSoftware3InterruptToPciBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::SI3::FieldMask
							)
						| InterruptDirectionControl::SI3::ValueMask_ToPciBus;
			}
		/** */
		inline void	mapSoftware3InterruptToQBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::SI3::FieldMask
							)
						| InterruptDirectionControl::SI3::ValueMask_ToQBus;
			}
		/** */
		inline void	mapSoftware2InterruptToPciBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::SI2::FieldMask
							)
						| InterruptDirectionControl::SI2::ValueMask_ToPciBus;
			}
		/** */
		inline void	mapSoftware2InterruptToQBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::SI2::FieldMask
							)
						| InterruptDirectionControl::SI2::ValueMask_ToQBus;
			}
		/** */
		inline void	mapSoftware1InterruptToPciBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::SI1::FieldMask
							)
						| InterruptDirectionControl::SI1::ValueMask_ToPciBus;
			}
		/** */
		inline void	mapSoftware1InterruptToQBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::SI1::FieldMask
							)
						| InterruptDirectionControl::SI1::ValueMask_ToQBus;
			}
		/** */
		inline void	mapSoftware0InterruptToPciBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::SI0::FieldMask
							)
						| InterruptDirectionControl::SI0::ValueMask_ToPciBus;
			}
		/** */
		inline void	mapSoftware0InterruptToQBus() throw(){
			_control =	(		_control
							&	~InterruptDirectionControl::SI0::FieldMask
							)
						| InterruptDirectionControl::SI0::ValueMask_ToQBus;
			}
	};

}
}
}
}

#endif
