/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_driver_tundra_qspan_qspanh_
#define _oscl_driver_tundra_qspan_qspanh_
#include "oscl/hw/tundra/qspan/qspan.h"

/** */
namespace Oscl {
/** */
namespace Tundra {
/** */
namespace Qspan {

/** */
class Driver {
	private:
		/** */
		Oscl::Tundra::Qspan::RegMap&	_device;
		/** */
		const unsigned char				_busNumber;
	public:
		/** */
		Driver(	Oscl::Tundra::Qspan::RegMap&	device,
				unsigned char					busNumber
				) throw();
	public:
		/** */
		bool 	readConfigSpaceDW(	Unsigned32&		data,
									unsigned char	bus,
									unsigned char	device,
									unsigned char	function,
									unsigned char	reg
									) throw();
		/** */
		bool 	writeConfigSpaceDW(	Unsigned32		data,
									unsigned char	bus,
									unsigned char	device,
									unsigned char	function,
									unsigned char	reg
									) throw();
		/** */
		bool	readVendorID(	Unsigned16&		vendorID,
								unsigned char	bus,
								unsigned char	device,
								unsigned char	function
								) throw();
		/** */
		bool	readDeviceID(	Unsigned16&		deviceID,
								unsigned char	bus,
								unsigned char	device,
								unsigned char	function
								) throw();
		/** */
		bool	readStatusRegister(	Unsigned16&		statusRegister,
									unsigned char	bus,
									unsigned char	device,
									unsigned char	function
									) throw();
		/** */
		void	clearReceivedMasterAbort() throw();
		/** */
		void	clearReceivedTargetAbort() throw();

	public:
		/** */
		static void	catchBusError() throw();
		/** */
		static bool	caughtBusError() throw();
	};

}
}
}

#endif
