/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_drv_usb_device_dyndevh_
#define _oscl_drv_usb_device_dyndevh_
#include "oscl/mt/itc/dyn/adv/core.h"
#include "oscl/driver/usb/pipe/message/pipeapi.h"
#include "oscl/driver/usb/alloc/power/reqapi.h"
#include "oscl/driver/usb/alloc/power/syncapi.h"
#include "oscl/oid/fixed.h"
#include "oscl/driver/usb/desc/device.h"
#include "oscl/driver/usb/hcd/enum/monitor/api.h"
#include "oscl/driver/usb/hcd/enum/dcp/api.h"
#include "oscl/mt/itc/mbox/srvevt.h"

/** */
namespace Oscl {
/** */
namespace Usb {

/** */
class DynDevice :	public Oscl::Mt::Itc::Dyn::Adv::Core<DynDevice> {
	private:
		/** */
		class LockPayload {};
		/** */
		class UnlockPayload {};
		/** */
		typedef Oscl::Mt::Itc::SrvEvent<DynDevice,LockPayload>		LockReq;
		/** */
		typedef Oscl::Mt::Itc::SrvEvent<DynDevice,UnlockPayload>	UnlockReq;

	public:
		/** */
		Oscl::Mt::Itc::PostMsgApi&			_myPapi;
		/** */
		Oscl::Usb::Alloc::Power::
		Req::Api::SAP&						_powerSAP;
		/** */
		Oscl::Usb::Alloc::Power::
		SyncApi&							_powerSyncApi;
		/** */
		Oscl::Usb::HCD::Enum::
		Monitor::Api&						_hcdEnumMonitorApi;
		/** */
		Oscl::Usb::HCD::Enum::
		DefCntrlPipe::Api&					_hcdEnumDcpApi;
		/** */
		Oscl::Usb::Pipe::Message::PipeApi&	_ctrlPipe;
		/** */
		Oscl::Usb::Address					_usbAddress;
		/** */
		bool								_lowSpeedDevice;
		/** */
		enum{ObjectIDSize=32};	// FIXME
		/** */
		Oscl::ObjectID::Fixed<ObjectIDSize>	_location;
	public:	// Device descriptor data
		/** */
		const Unsigned16					_bcdUSB;
		/** */
		const Unsigned8						_bDeviceClass;
		/** */
		const Unsigned8						_bDeviceSubClass;
		/** */
		const Unsigned8						_bDeviceProtocol;
		/** */
		const Unsigned8						_bMaxPacketSize;
		/** */
		const Unsigned16					_idVendor;
		/** */
		const Unsigned16					_idProduct;
		/** */
		const Unsigned16					_bcdDevice;
		/** */
		const Unsigned8						_iManufacturer;
		/** */
		const Unsigned8						_iProduct;
		/** */
		const Unsigned8						_iSerialNumber;
		/** */
		const Unsigned8						_bNumConfigurations;
		/** */
		Oscl::Queue<Oscl::Mt::Itc::SrvMsg>	_pendingLocks;
		/** */
		bool								_locked;
		/** */
		bool								_reserved;

	public:
		/** */
		DynDevice(	Oscl::Mt::Itc::PostMsgApi&			myPapi,
					Oscl::Mt::Itc::Dyn::Adv::
					Core<DynDevice>::ReleaseApi&		advRapi,
					Oscl::Mt::Itc::PostMsgApi&			advPapi,
					Oscl::Usb::Pipe::Message::PipeApi&	ctrlPipe,
					Oscl::Usb::Address					myUsbAddress,
					bool								lowSpeedDevice,
					Oscl::Usb::Alloc::Power::
					Req::Api::SAP&						powerSAP,
					Oscl::Usb::Alloc::Power::
					SyncApi&							powerSyncApi,
					Oscl::Usb::HCD::Enum::
					Monitor::Api&						hcdEnumMonitorApi,
					Oscl::Usb::HCD::Enum::
					DefCntrlPipe::Api&					hcdEnumDcpApi,
					const Oscl::Usb::Desc::Device&		devDesc,
					const Oscl::ObjectID::RO::Api*		location=0
					) throw();
		/** */
		Oscl::Usb::Alloc::Power::
		Req::Api::SAP&				getPowerSAP() throw();
		/** */
		Oscl::Usb::Alloc::Power::
		SyncApi&					getPowerSyncApi() throw();

		/** */
		void						lock() throw();
		/** */
		void						unlock() throw();
		/** */
		bool						isReserved() throw();
		/** */
		void						reserve() throw();

	private: // Oscl::Mt::Itc::Dyn::Adv::Core
		/** */
		void	openDynamicService() throw();
		/** */
		void	closeDynamicService() throw();
		/** */
		DynDevice&	getDynSrv() throw();

	private:
		/** */
		friend class LockReq;
		/** */
		friend class UnlockReq;
		/** */
		void	request(LockReq& msg) throw();
		/** */
		void	request(UnlockReq& msg) throw();
	};

}
}

#endif
