/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "server.h"

using namespace Oscl::Usb::Device;

Server::Server(	InfoApi&						funcInfo,
				Oscl::Mt::Itc::PostMsgApi&		dynSrvPostApi,
				Oscl::Mt::Itc::PostMsgApi&		advPostApi,
				Oscl::Mt::Itc::Dyn::Adv::
				Core<Server>::ReleaseApi&		advertiser
				) throw():
		Oscl::Mt::Itc::Dyn::Adv::
		Core<Server>(dynSrvPostApi,advertiser,advPostApi),
		_funcInfo(funcInfo),
		_claimed(false)
		{
	}

bool	Server::isClaimed() const throw(){
	return _claimed;
	}

Unsigned16	Server::getUsbSpecRelease() const throw(){
	return _funcInfo.getUsbSpecRelease();
	}

Unsigned8	Server::getSubClassCode() const throw(){
	return _funcInfo.getSubClassCode();
	}

Unsigned8	Server::getProtocolCode() const throw(){
	return _funcInfo.getProtocolCode();
	}

Unsigned8	Server::getMaximumPacketSize() const throw(){
	return _funcInfo.getMaximumPacketSize();
	}

Unsigned16	Server::getVendorID() const throw(){
	return _funcInfo.getVendorID();
	}

Unsigned16	Server::getProductID() const throw(){
	return _funcInfo.getProductID();
	}

Unsigned16	Server::getDeviceReleaseNumber() const throw(){
	return _funcInfo.getDeviceReleaseNumber();
	}

Unsigned8	Server::getProductDescStrIndex() const throw(){
	return _funcInfo.getProductDescStrIndex();
	}

Unsigned8	Server::getSerialNumberStrIndex() const throw(){
	return _funcInfo.getSerialNumberStrIndex();
	}

Unsigned8	Server::getNumberOfConfigurations() const throw(){
	return _funcInfo.getNumberOfConfigurations();
	}

const Oscl::Usb::Device::LocationApi&	Server::getLocation() const throw(){
	return _funcInfo.getLocation();
	}

void	Server::openDynamicService() throw(){
	// Nothing to attach yet
	}

void	Server::closeDynamicService() throw(){
	// Nothing to detach yet
	}

Server&	Server::getDynSrv() throw(){
	return *this;
	}

