/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_drv_usb_enum_interface_storage_scsi_instanceh_
#define _oscl_drv_usb_enum_interface_storage_scsi_instanceh_
#include "oscl/driver/usb/enum/interface/api.h"
#include "oscl/mt/itc/dyn/monitor/unit/monitor/monitor.h"
#include "oscl/driver/usb/setup/mem.h"
#include "oscl/driver/usb/desc/config.h"
#include "oscl/driver/usb/desc/interface.h"
#include "oscl/driver/usb/desc/endpoint.h"
#include "oscl/mt/itc/srv/crespmem.h"
#include "oscl/oid/roapi.h"

#include "oscl/mt/itc/dyn/adv/creatorsync.h"

#include "oscl/driver/usb/tt/bo/driver.h"
#include "oscl/tt/lun/driver.h"

#include "oscl/block/rw/dyn/service.h"

/** */
namespace Oscl {
/** */
namespace Usb {
/** */
namespace Enum {
/** */
namespace Interface {
/** */
namespace Storage {
/** */
namespace SCSI {

/** This concrete class creates an instance of a USB SCSI
	serial device. 
 */
class Instance :	public	Oscl::Usb::Enum::Interface::Api,
					private	Oscl::Mt::Itc::Srv::Close::Resp::Api
					{
	public:
		/** */
		struct RawMem {
			/** SCSI Desc Mem */
			/** Standard Desc Mem */
			unsigned char	mem[		sizeof(Oscl::Usb::Desc::Interface)
									+	(3*sizeof(Oscl::Usb::Desc::Endpoint))
									];
			};
		/** */
		typedef Oscl::Memory::
				AlignedBlock<sizeof(RawMem)>		DescMem;
		/** */
		union PacketMem {
			/** */
			void*								__qitemlink;
			/** */
			DescMem								_descMem;
			/** */
			};

	private:	// Interface specific memory for created parts
		/** */
		Oscl::Memory::
		AlignedBlock<	sizeof(	Oscl::Usb::TT::BO::Driver
								)
							>					_driverMem;

		/** */
		Oscl::Memory::
		AlignedBlock<	sizeof(	Oscl::TT::LUN::Driver
								)
							>					_lunDriverMem;

	private:	// Interface specific constructor specified variables
		/** */
		Oscl::Memory::
		AlignedBlock<	sizeof(Oscl::Block::RW::Dyn::Service)
						>						_deviceMem;
		/** */
		Oscl::Usb::Setup::Mem&					_setupMem;
		/** */
		PacketMem&								_packetMem;
		/** */
		Oscl::Usb::TT::BO::
		Driver::SetupMem*						_driverSetupMem;
		/** */
		Oscl::Usb::TT::BO::
		Driver::PacketMem*						_driverPacketMem;
		/** */
		Oscl::TT::LUN::Driver::PacketMem*		_lunDriverPacketMem;
		/** */
		const unsigned							_oidType;
		/** */
		Oscl::Mt::Itc::PostMsgApi&				_driverPapi;
		/** */
		Oscl::Mt::Itc::PostMsgApi&				_lunDriverPapi;
		/** */
		Oscl::Mt::Itc::Dyn::Adv::Creator::
		SyncApi<	Oscl::Block::RW::
					Dyn::Service
					>&							_blockDevAdvCapi;
		/** */
		Oscl::Mt::Itc::PostMsgApi&      		_blockDevDynDevPapi;
		/** */
		Oscl::Mt::Itc::Dyn::Adv::
		Core<	Oscl::Block::RW::
				Dyn::Service
				>::ReleaseApi&					_blockDevDynAdvRapi;
		/** */
		Oscl::Mt::Itc::PostMsgApi&				_blockDevDynAdvPapi;

	private:	// Interface specific "created" variables
		/** */
		Oscl::Usb::Pipe::
		Bulk::IN::PipeApi*						_bulkInPipe;
		/** */
		Oscl::Usb::Pipe::
		Bulk::OUT::PipeApi*						_bulkOutPipe;
		/** */
		bool									_driverOpen;
		/** */
		Oscl::Usb::TT::BO::Driver*				_driver;
		/** */
		Oscl::TT::LUN::Driver*					_lunDriver;
		/** */
		Oscl::Block::RW::Dyn::Service*			_device;
		/** */
		unsigned char							_maxBulkInPacketSize;
		/** */
		unsigned char							_maxBulkOutPacketSize;
		/** */
		unsigned								_reservedPower;

	private:	// Generic "Instance" passed in through start()/stop()
		/** */
		Oscl::Mt::Itc::PostMsgApi*				_myPapi;
		/** */
		Oscl::Usb::HCD::Enum::Monitor::Api*		_hcdMonApi;
		/** */
		Oscl::Usb::Pipe::Message::PipeApi*		_ctrlPipe;
		/** */
		Oscl::Usb::Address						_usbAddress;
		/** */
		bool									_lowSpeedDevice;
		/** */
		Oscl::Mt::Itc::Srv::
		Close::Resp::CloseMem*					_closeMem;
		/** */
		StopDoneApi*							_sdapi;
		
	public:
		/** */
		Instance(	Oscl::Mt::Itc::PostMsgApi&				driverPapi,
					Oscl::Mt::Itc::PostMsgApi&				lunDriverPapi,
					Oscl::Mt::Itc::Dyn::Adv::Creator::
					SyncApi<	Oscl::Block::RW::
								Dyn::Service
								>&							blockDevAdvCapi,
					Oscl::Mt::Itc::PostMsgApi&      		blockDevDynDevPapi,
					Oscl::Mt::Itc::Dyn::Adv::
					Core<	Oscl::Block::RW::
							Dyn::Service
							>::ReleaseApi&					blockDevDynAdvRapi,
					Oscl::Mt::Itc::PostMsgApi&				blockDevDynAdvPapi,
					Oscl::Usb::Setup::Mem&					setupMem,
					PacketMem&								packetMem,
					Oscl::Usb::TT::BO::
					Driver::SetupMem*						driverSetupMem,
					Oscl::Usb::TT::BO::
					Driver::PacketMem*						driverPacketMem,
					Oscl::TT::LUN::Driver::PacketMem*		lunDriverPacketMem,
					unsigned								oidType,
					const Oscl::ObjectID::RO::Api*			location
					) throw();
	private:
		/** */
		void	releaseResources() throw();

	private: // helpers
		/** */
		Oscl::Usb::Desc::Interface*
			getInterface(	Oscl::Usb::Desc::
							Config*			config,
							unsigned char	interfaceNumber
							) throw();
		/** */
		bool	getEndpoints(	Oscl::Usb::Desc::Config*	config,
								Oscl::Usb::Desc::Endpoint**	bulkIn,
								Oscl::Usb::Desc::Endpoint**	bulkOut
								) throw();
		/** */
		bool	allocateBulkInPipe(	Oscl::Usb::Desc::
									Endpoint&			desc
									) throw();
		/** */
		bool	allocateBulkOutPipe(	Oscl::Usb::Desc::
										Endpoint&			desc
										) throw();
		/** */
		bool	setSCSIConfiguration(Oscl::Usb::Desc::Config& desc) throw();
		/** */
		bool	readEndpointStatus(unsigned endpointID) throw();
		/** */
		void	createDriver(	const Oscl::ObjectID::RO::Api&	location,
								unsigned char					interfaceNumber,
								unsigned char					bulkInEndpointID,
								unsigned char					bulkOutEndpointID
								) throw();
		/** */
		void	close(Oscl::Mt::Itc::Srv::Close::Req::Api::SAP& sap) throw();
		/** */
		bool    waitTestUnitReady(unsigned nRetries) throw();
		/** */
		bool	inquiry() throw();
		/** */
		bool	syncStart() throw();
		/** */
		bool	readCapacity() throw();
		/** */
		void	jungsoftSeq(const Oscl::ObjectID::RO::Api& location) throw();
		/** */
		void	genericSeq() throw();

	private:	// Oscl::Mt::Itc::Srv::Close::Resp::Api
		/** */
		void	response(	Oscl::Mt::Itc::Srv::Close::
							Resp::Api::CloseResp&		msg
							) throw();
		/** */
		void	response(	Oscl::Mt::Itc::Srv::Open::
							Resp::Api::OpenResp&		msg
							) throw();

	private: // Oscl::Usb::Enum::Interface::Api
		/** */
		void	start(	Oscl::Mt::Itc::PostMsgApi&		myPapi,
						Oscl::Usb::Pipe::
						Message::PipeApi&				epzPipe,
						Oscl::Usb::HCD::Enum::
						Monitor::Api&					hcdMonApi,
						Oscl::Usb::Desc::Config&		configDesc,
						Oscl::Usb::Address				usbAddress,
						bool							lowSpeedDevice,
						unsigned char					interfaceNumber,
						const Oscl::ObjectID::RO::Api*	location
						) throw();
		/** */
		void	stop(	StopDoneApi&				sdapi,
						Oscl::Mt::Itc::Srv::
						Close::Resp::CloseMem&		closeMem
						) throw();
		/** */
		bool	match(	Oscl::Usb::Pipe::
						Message::SyncApi&			epzSyncApi,
						Oscl::Usb::Desc::Config&	configDesc,
						Unsigned16					deviceIdVendor,
						Unsigned16					deviceIdProduct,
						Unsigned16					deviceBcdDevice,
						Unsigned8					deviceManufacturer,
						Unsigned8					deviceProduct,
						Unsigned8					deviceSerialNumber,
						unsigned char				interfaceNumber
						) throw();
	};

}
}
}
}
}
}

#endif

