/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_drv_usb_hub_port_adaptor_porth_
#define _oscl_drv_usb_hub_port_adaptor_porth_
#include "oscl/queue/queueitem.h"
#include "oscl/queue/queue.h"
#include "oscl/memory/block.h"
#include "oscl/driver/usb/hub/port/reqapi.h"
#include "oscl/driver/usb/setup/hub/clrportcconn.h"
#include "oscl/driver/usb/setup/hub/clrportcenable.h"
#include "oscl/driver/usb/setup/hub/clrportcoc.h"
#include "oscl/driver/usb/setup/hub/clrportconn.h"
#include "oscl/driver/usb/setup/hub/clrportcreset.h"
#include "oscl/driver/usb/setup/hub/clrportcsuspend.h"
#include "oscl/driver/usb/setup/hub/clrportenable.h"
#include "oscl/driver/usb/setup/hub/clrportoc.h"
#include "oscl/driver/usb/setup/hub/clrportpower.h"
#include "oscl/driver/usb/setup/hub/clrportreset.h"
#include "oscl/driver/usb/setup/hub/clrportsuspend.h"
#include "oscl/driver/usb/setup/hub/getportstatus.h"
#include "oscl/driver/usb/setup/hub/setportconn.h"
#include "oscl/driver/usb/setup/hub/setportenable.h"
#include "oscl/driver/usb/setup/hub/setportoc.h"
#include "oscl/driver/usb/setup/hub/setportpower.h"
#include "oscl/driver/usb/setup/hub/setportreset.h"
#include "oscl/driver/usb/setup/hub/setportsuspend.h"
#include "oscl/hw/usb/hub/port/packet.h"
#include "trans.h"
#include "oscl/driver/usb/hub/adaptor/portapi.h"

/** */
namespace Oscl {
/** */
namespace Usb {
/** */
namespace Hub {
/** */
namespace Port {
/** */
namespace Adaptor {

/** */
class Port :	public Oscl::Usb::Hub::Port::Req::Api,
				public Oscl::QueueItem
				{
	private:
		/** */
		typedef Oscl::Memory::AlignedBlock<	sizeof(GetStatusTrans)
											>	GetStatusTransMem;
		/** */
		union TransMem {
			/** */
			void*				__qitemlink;
			/** */
			GetStatusTransMem	getStatus;
			/** */
			Oscl::Memory::
			AlignedBlock<	sizeof(ClearConnChangeTrans)
							>	clrConnChange;
			/** */
			Oscl::Memory::
			AlignedBlock<	sizeof(ClearEnableChangeTrans)
							>	clrEnableChange;
			/** */
			Oscl::Memory::
			AlignedBlock<	sizeof(ClearOverCurrentChangeTrans)
							>	clrOverCurrentChange;
			/** */
			Oscl::Memory::
			AlignedBlock<	sizeof(ClearResetChangeTrans)
							>	clrResetChange;
			/** */
			Oscl::Memory::
			AlignedBlock<	sizeof(ClearSuspendChangeTrans)
							>	clrSuspendChange;
			/** */
			Oscl::Memory::
			AlignedBlock<	sizeof(ClearEnableTrans)
							>	clrEnable;
			/** */
			Oscl::Memory::
			AlignedBlock<	sizeof(ClearSuspendTrans)
							>	clrSuspend;
			/** */
			Oscl::Memory::
			AlignedBlock<	sizeof(ClearPowerTrans)
							>	clrPower;
			/** */
			Oscl::Memory::
			AlignedBlock<	sizeof(SetResetTrans)
							>	setReset;
			/** */
			Oscl::Memory::
			AlignedBlock<	sizeof(SetSuspendTrans)
							>	setSuspend;
			/** */
			Oscl::Memory::
			AlignedBlock<	sizeof(SetPowerTrans)
							>	setPower;
			};
		/** */
		union TransSetupMem {
			/** */
			GetStatusTrans::SetupMem				getStatus;
			/** */
			ClearConnChangeTrans::SetupMem			clrConnChange;
			/** */
			ClearEnableChangeTrans::SetupMem		clrEnableChange;
			/** */
			ClearOverCurrentChangeTrans::SetupMem	clrOverCurrentChange;
			/** */
			ClearResetChangeTrans::SetupMem			clrResetChange;
			/** */
			ClearSuspendChangeTrans::SetupMem		clrSuspendChange;
			/** */
			ClearEnableTrans::SetupMem				clrEnable;
			/** */
			ClearSuspendTrans::SetupMem				clrSuspend;
			/** */
			ClearPowerTrans::SetupMem				clrPower;
			/** */
			SetResetTrans::SetupMem					setReset;
			/** */
			SetSuspendTrans::SetupMem				setSuspend;
			/** */
			SetPowerTrans::SetupMem					setPower;
			};
		/** */
		union ContextSetupMem {
			/** */
			GetPortStatusTrans::SetupMem			getPortStatus;
			};

	public:
		/** */
		struct SetupMem {
			/** */
			TransSetupMem				trans;
			/** */
			ContextSetupMem				context;
			};
		/** */
		union TransPacketMem {
			/** */
			GetStatusTrans::PacketMem	getStatus;
			};
		/** */
		union ContextPacketMem {
			/** */
			GetPortStatusTrans::PacketMem	getPortStatus;
			};

	public:
		/** */
		struct PacketMem {
			/** */
			TransPacketMem				trans;
			/** */
			ContextPacketMem			context;
			};

		/** */
		union CloseMem {
			/** */
			Oscl::Usb::Pipe::Message::Resp::CancelMem	irp;
			};

	private:
		/** */
		GetStatusTransMem								_statusMem;
		/** */
		TransMem										_transMem;
		/** */
		Oscl::Usb::Pipe::Message::Resp::CancelMem		_cancelMem;
		/** */
		Oscl::Usb::Hub::Adaptor::PortApi&				_context;
		/** */
		Oscl::Usb::Pipe::Message::Req::Api::SAP&		_pipeSAP;
		/** */
		SetupMem&										_setupMem;
		/** */
		PacketMem&										_packetMem;
		/** */
		Oscl::Mt::Itc::PostMsgApi&						_myPapi;
		/** */
		Oscl::Usb::Hub::Port::Req::Api::ConcreteSAP		_hubPortSAP;
		/** */
		Oscl::Queue<NotifyPortChangeReq>				_notificationList;
		/** */
		Oscl::BitField<	Oscl::Usb::Hw::Hub::Port::
						wPortChange::Reg
						>								_portChangeStatus;
		/** */
		const unsigned									_portNum;
		/** */
		CancelReq*										_cancelReq;
		/** */
		TransApi*										_contextTrans;
		/** */
		TransApi*										_currentTrans;
		/** */
		CloseMem*										_closeMem;
		/** */
		bool											_contextNotifyEnabled;
		/** */
		bool											_open;
		/** */
		bool											_closing;

	public:
		/** */
		Port(	Oscl::Usb::Hub::
				Adaptor::PortApi&				context,
				Oscl::Usb::Pipe::Message::
				Req::Api::SAP&					pipeSAP,
				SetupMem&						setupMem,
				PacketMem&						packetMem,
				Oscl::Mt::Itc::PostMsgApi&		myPapi,
				unsigned						portNum
				) throw();

		/** */
		virtual ~Port(){}

		/** */
		void	open() throw();

		/** */
		void	close(CloseMem& mem) throw();

		/** */
		void	cancelNext() throw();

		/** */
		void	statusChange() throw();

		/** */
		Oscl::Usb::Hub::Port::Req::Api::SAP&	getHubPortSAP() throw();

	private:
		/** */
		void	clearPortPower() throw();

		/** */
		void		sendGetPortStatusIRP() throw();

		/** */
		void		notifyChanged() throw();

		/** */
		void		changeUpdate(	Oscl::Usb::Hw::Hub::Port::
									wPortChange::Reg			change
									) throw();

		/** */
		unsigned	getPortNum() throw();

	private:
		friend class GetPortStatusTrans;
		/** */
		void	canceled(GetPortStatusTrans& trans) throw();
		/** */
		void	failed(GetPortStatusTrans& trans) throw();
		/** */
		void	done(	GetPortStatusTrans&					trans,
						Oscl::Usb::Hw::Hub::Port::Packet&	packet
						) throw();

	private:
		friend class GetStatusTrans;
		/** */
		void	canceled(GetStatusTrans& trans) throw();
		/** */
		void	failed(GetStatusTrans& trans) throw();
		/** */
		void	done(	GetStatusTrans&						trans,
						Oscl::Usb::Hw::Hub::Port::Packet&	packet
						) throw();

	private:
		friend class ClearConnChangeTrans;
		/** */
		void	canceled(ClearConnChangeTrans& trans) throw();
		/** */
		void	failed(ClearConnChangeTrans& trans) throw();
		/** */
		void	done(ClearConnChangeTrans& trans) throw();

	private:
		friend class ClearEnableChangeTrans;
		/** */
		void	canceled(ClearEnableChangeTrans& trans) throw();
		/** */
		void	failed(ClearEnableChangeTrans& trans) throw();
		/** */
		void	done(ClearEnableChangeTrans& trans) throw();

	private:
		friend class ClearOverCurrentChangeTrans;
		/** */
		void	canceled(ClearOverCurrentChangeTrans& trans) throw();
		/** */
		void	failed(ClearOverCurrentChangeTrans& trans) throw();
		/** */
		void	done(ClearOverCurrentChangeTrans& trans) throw();

	private:
		friend class ClearResetChangeTrans;
		/** */
		void	canceled(ClearResetChangeTrans& trans) throw();
		/** */
		void	failed(ClearResetChangeTrans& trans) throw();
		/** */
		void	done(ClearResetChangeTrans& trans) throw();

	private:
		friend class ClearSuspendChangeTrans;
		/** */
		void	canceled(ClearSuspendChangeTrans& trans) throw();
		/** */
		void	failed(ClearSuspendChangeTrans& trans) throw();
		/** */
		void	done(ClearSuspendChangeTrans& trans) throw();

	private:
		friend class ClearEnableTrans;
		/** */
		void	canceled(ClearEnableTrans& trans) throw();
		/** */
		void	failed(ClearEnableTrans& trans) throw();
		/** */
		void	done(ClearEnableTrans& trans) throw();

	private:
		friend class ClearSuspendTrans;
		/** */
		void	canceled(ClearSuspendTrans& trans) throw();
		/** */
		void	failed(ClearSuspendTrans& trans) throw();
		/** */
		void	done(ClearSuspendTrans& trans) throw();

	private:
		friend class ClearPowerTrans;
		/** */
		void	canceled(ClearPowerTrans& trans) throw();
		/** */
		void	failed(ClearPowerTrans& trans) throw();
		/** */
		void	done(ClearPowerTrans& trans) throw();

	private:
		friend class SetResetTrans;
		/** */
		void	canceled(SetResetTrans& trans) throw();
		/** */
		void	failed(SetResetTrans& trans) throw();
		/** */
		void	done(SetResetTrans& trans) throw();

	private:
		friend class SetSuspendTrans;
		/** */
		void	canceled(SetSuspendTrans& trans) throw();
		/** */
		void	failed(SetSuspendTrans& trans) throw();
		/** */
		void	done(SetSuspendTrans& trans) throw();

	private:
		friend class SetPowerTrans;
		/** */
		void	canceled(SetPowerTrans& trans) throw();
		/** */
		void	failed(SetPowerTrans& trans) throw();
		/** */
		void	done(SetPowerTrans& trans) throw();

	private:	// Oscl::Usb::Hub::Port::Req::Api
		/** */
		void	request(SetResetReq& msg) throw();
		/** */
		void	request(SetSuspendReq& msg) throw();
		/** */
		void	request(SetPowerReq& msg) throw();
		/** */
		void	request(ClearEnableReq& msg) throw();
		/** */
		void	request(ClearSuspendReq& msg) throw();
		/** */
		void	request(ClearPowerReq& msg) throw();
		/** */
		void	request(ClearConnectChangeReq& msg) throw();
		/** */
		void	request(ClearResetChangeReq& msg) throw();
		/** */
		void	request(ClearEnableChangeReq& msg) throw();
		/** */
		void	request(ClearSuspendChangeReq& msg) throw();
		/** */
		void	request(ClearOverCurrentChangeReq& msg) throw();
		/** */
		void	request(CancelReq& msg) throw();
		/** */
		void	request(NotifyPortChangeReq& msg) throw();
		/** */
		void	request(CancelNotifyPortChangeReq& msg) throw();
		/** */
		void	request(GetPortStatusReq& msg) throw();
	};

}
}
}
}
}


#endif

