/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "fsm.h"
#include "oscl/error/fatal.h"

using namespace Oscl::Usb::Hub::Port;

static const StateVar::Closed						closed;
static const StateVar::Closing						closing;
static const StateVar::ClosingPccPc					closingPccPc;
static const StateVar::ClosingPncPn					closingPncPn;
static const StateVar::ClosingAllocUsbAddrHc		closingAllocUsbAddrHc;
static const StateVar::ClosingFreeUsbAddrHc			closingFreeUsbAddrHc;
static const StateVar::ClosingHczcHczUa				closingHczcHczUa;
static const StateVar::ClosingHczcUa				closingHczcUa;
static const StateVar::ClosingHczcAzUa				closingHczcAzUa;
static const StateVar::ClosingUnlockAddrZeroHcUa	closingUnlockAddrZeroHcUa;
static const StateVar::ClosingSetResetPccPcAzUa		closingSetResetPccPcAzUa;
static const StateVar::ClosingTimerCancelPncPnAzUa	closingTimerCancelPncPnAzUa;
static const StateVar::ClosingTimerCancelPncPnPaUa	closingTimerCancelPncPnPaUa;
static const StateVar::ClosingTimerCancelPaUa		closingTimerCancelPaUa;
static const StateVar::ClosingTimerCancelAzUa		closingTimerCancelAzUa;
static const StateVar::ClosingPncPnAzUa				closingPncPnAzUa;
static const StateVar::ClosingClrResetChangePccPcAzUa
				closingClrResetChangePccPcAzUa;
static const StateVar::ClosingResetWaitTwcTwAzUa	closingResetWaitTwcTwAzUa;
static const StateVar::ClosingUncUnPncPn			closingUncUnPncPn;
static const StateVar::ClosingUncUn					closingUncUn;
static const StateVar::ClosingReadDescIrpPncPnAzUa	closingReadDescIrpPncPnAzUa;
static const StateVar::ClosingReadDescIrpcIrpAzUa	closingReadDescIrpcIrpAzUa;
static const StateVar::ClosingSetAddrIrpPncPnAzUa	closingSetAddrIrpPncPnAzUa;
static const StateVar::ClosingSetAddrIrpcIrpAzUa	closingSetAddrIrpcIrpAzUa;
static const StateVar::ClosingUazPncPnUa			closingUazPncPnUa;
static const StateVar::ClosingUazUa					closingUazUa;
static const StateVar::ClosingPncPnUa				closingPncPnUa;
static const StateVar::ClosingPipeAllocHcPncPnUa	closingPipeAllocHcPncPnUa;
static const StateVar::ClosingPipeAllocHcUa			closingPipeAllocHcUa;
static const StateVar::ClosingFreePipeHcUa			closingFreePipeHcUa;
static const StateVar::ClosingFreePipeHcPncPnUa		closingFreePipeHcPncPnUa;
static const StateVar::ClosingAdvPncPnPaUa			closingAdvPncPnPaUa;
static const StateVar::ClosingPrepareHcPaUa			closingPrepareHcPaUa;
static const StateVar::ClosingDynSrvHcPaUa			closingDynSrvHcPaUa;
static const StateVar::ClosingRncRnUa				closingRncRnUa;
static const StateVar::ClosingRncRnPncPnUa			closingRncRnPncPnUa;
static const StateVar::DisabledPc					disabledPc;
static const StateVar::PowerWait					powerWait;
static const StateVar::DisabledPn					disabledPn;
static const StateVar::DisabledClrConnChangePc		disabledClrConnChangePc;
static const StateVar::DisabledGetStatusPc			disabledGetStatusPc;
static const StateVar::DisabledClrEnableChangePc	disabledClrEnableChangePc;
static const StateVar::DisabledClrResetChangePc		disabledClrResetChangePc;
static const StateVar::ConnAllocUsbAddrHc			connAllocUsbAddrHc;
static const StateVar::ConnLockAddrZeroUa			connLockAddrZeroUa;
static const StateVar::ConnSetResetPcAzUa			connSetResetPcAzUa;
static const StateVar::ConnWaitResetPnAzUa			connWaitResetPnAzUa;
static const StateVar::ConnClrResetChangePcAzUa		connClrResetChangePcAzUa;
static const StateVar::ConnResetWaitTwAzUa			connResetWaitTwAzUa;
static const StateVar::ConnWaitUsbAddrUnPn			connWaitUsbAddrUnPn;
static const StateVar::ConnReadDescIrpPnAzUa		connReadDescIrpPnAzUa;
static const StateVar::ConnReadDescErrorPnAzUa		connReadDescErrorPnAzUa;
static const StateVar::ConnSetAddrIrpPnAzUa			connSetAddrIrpPnAzUa;
static const StateVar::ConnSetAddrErrorPnAzUa		connSetAddrErrorPnAzUa;
static const StateVar::ConnUnlockAddrZeroHcPnUa		connUnlockAddrZeroHcPnUa;
static const StateVar::ConnPipeAllocHcPnUa			connPipeAllocHcPnUa;
static const StateVar::ReadyPnPaUa					readyPnPaUa;
static const StateVar::RezRnPnUa					rezRnPnUa;
static const StateVar::DiscUncUn					discUncUn;
static const StateVar::DiscUnlockAddrZeroHcUa		discUnlockAddrZeroHcUa;
static const StateVar::DiscFreeUsbAddrHc			discFreeUsbAddrHc;
static const StateVar::DiscTimerCancelAzUa			discTimerCancelAzUa;
static const StateVar::DiscTimerCancelPaUa			discTimerCancelPaUa;
static const StateVar::DiscReadDescIrpcIrpAzUa		discReadDescIrpcIrpAzUa;
static const StateVar::DiscSetAddrIrpcIrpAzUa		discSetAddrIrpcIrpAzUa;
static const StateVar::DiscPrepareHcPaUa			discPrepareHcPaUa;
static const StateVar::DiscDynSrvHcPaUa				discDynSrvHcPaUa;
static const StateVar::DiscFreePipeHcUa				discFreePipeHcUa;
static const StateVar::DiscPipeAllocHcUa			discPipeAllocHcUa;
static const StateVar::DiscRncRnUa					discRncRnUa;
static const StateVar::ClosingReadFullIrpPncPnPaUa	closingReadFullIrpPncPnPaUa;
static const StateVar::ClosingReadFullIrpcIrpPaUa	closingReadFullIrpcIrpPaUa;
static const StateVar::ClosingPncPnPaUa				closingPncPnPaUa;
static const StateVar::ConnReadFullIrpPnPaUa		connReadFullIrpPnPaUa;
static const StateVar::ConnReadFullErrorPnPaUa		connReadFullErrorPnPaUa;
static const StateVar::DiscReadFullIrpcIrpPaUa		discReadFullIrpcIrpPaUa;

// StateVar

StateVar::StateVar(ContextApi& context) throw():
		_context(context),
		_state(&closed)
		{
	for(unsigned i=0;i<nTraceRecs;++i){
		_freeTraceRecs.put(&_traceRec[i]);
		}
	}

void	StateVar::open() throw(){
	_state->open(_context,*this);
	}

void	StateVar::close() throw(){
	_state->close(_context,*this);
	}

void	StateVar::setResetDone() throw(){
	_state->setResetDone(_context,*this);
	}

void	StateVar::setSuspendDone() throw(){
	_state->setSuspendDone(_context,*this);
	}

void	StateVar::setPowerDone() throw(){
	_state->setPowerDone(_context,*this);
	}

void	StateVar::clrEnableDone() throw(){
	_state->clrEnableDone(_context,*this);
	}

void	StateVar::clrSuspendDone() throw(){
	_state->clrSuspendDone(_context,*this);
	}

void	StateVar::clrPowerDone() throw(){
	_state->clrPowerDone(_context,*this);
	}

void	StateVar::clrConnectChangeDone() throw(){
	_state->clrConnectChangeDone(_context,*this);
	}

void	StateVar::clrResetChangeDone() throw(){
	_state->clrResetChangeDone(_context,*this);
	}

void	StateVar::clrEnableChangeDone() throw(){
	_state->clrEnableChangeDone(_context,*this);
	}

void	StateVar::clrSuspendChangeDone() throw(){
	_state->clrSuspendChangeDone(_context,*this);
	}

void	StateVar::clrOverCurrentChangeDone() throw(){
	_state->clrOverCurrentChangeDone(_context,*this);
	}

void	StateVar::cancelReqDone() throw(){
	_state->cancelReqDone(_context,*this);
	}

void	StateVar::notifyPortChangeDone() throw(){
	_state->notifyPortChangeDone(_context,*this);
	}

void	StateVar::cancelNotifyPortChangeDone() throw(){
	_state->cancelNotifyPortChangeDone(_context,*this);
	}

void	StateVar::getPortStatusDone() throw(){
	_state->getPortStatusDone(_context,*this);
	}

void	StateVar::requestFailed() throw(){
	_state->requestFailed(_context,*this);
	}

void	StateVar::msgPipeIrpCancelDone() throw(){
	_state->msgPipeIrpCancelDone(_context,*this);
	}

void	StateVar::msgPipeNoDataIrpDone() throw(){
	_state->msgPipeNoDataIrpDone(_context,*this);
	}

void	StateVar::msgPipeReadIrpDone() throw(){
	_state->msgPipeReadIrpDone(_context,*this);
	}

void	StateVar::msgPipeWriteIrpDone() throw(){
	_state->msgPipeWriteIrpDone(_context,*this);
	}

void	StateVar::hcdMsgPipeAllocDone() throw(){
	_state->hcdMsgPipeAllocDone(_context,*this);
	}

void	StateVar::hcdMsgPipeFreeDone() throw(){
	_state->hcdMsgPipeFreeDone(_context,*this);
	}

void	StateVar::notifyMsgPipeResourceChangeDone() throw(){
	_state->notifyMsgPipeResourceChangeDone(_context,*this);
	}

void	StateVar::cancelNotifyMsgPipeResourceChangeDone() throw(){
	_state->cancelNotifyMsgPipeResourceChangeDone(_context,*this);
	}

void	StateVar::hcdLockAddressZeroDone() throw(){
	_state->hcdLockAddressZeroDone(_context,*this);
	}

void	StateVar::hcdUnlockAddressZeroDone() throw(){
	_state->hcdUnlockAddressZeroDone(_context,*this);
	}

void	StateVar::hcdCancelLockAddrZeroDone() throw(){
	_state->hcdCancelLockAddrZeroDone(_context,*this);
	}

void	StateVar::hcdAllocUsbAddrDone() throw(){
	_state->hcdAllocUsbAddrDone(_context,*this);
	}

void	StateVar::hcdFreeUsbAddrDone() throw(){
	_state->hcdFreeUsbAddrDone(_context,*this);
	}

void	StateVar::prepareToCloseDynSrvDone() throw(){
	_state->prepareToCloseDynSrvDone(_context,*this);
	}

void	StateVar::closeDynSrvDone() throw(){
	_state->closeDynSrvDone(_context,*this);
	}

void	StateVar::notifyUsbAddrResChangeDone() throw(){
	_state->notifyUsbAddrResChangeDone(_context,*this);
	}

void	StateVar::cancelNotifyUsbAddrResChangeDone() throw(){
	_state->cancelNotifyUsbAddrResChangeDone(_context,*this);
	}

void	StateVar::timerExpired() throw(){
	_state->timerExpired(_context,*this);
	}

void	StateVar::timerCanceled() throw(){
	_state->timerCanceled(_context,*this);
	}

void	StateVar::connectChanged() throw(){
	_state->connectChanged(_context,*this);
	}

void	StateVar::portDisabled() throw(){
	_state->portDisabled(_context,*this);
	}

void	StateVar::portResumed() throw(){
	_state->portResumed(_context,*this);
	}

void	StateVar::portOverCurrentChange() throw(){
	_state->portOverCurrentChange(_context,*this);
	}

void	StateVar::portResetDone() throw(){
	_state->portResetDone(_context,*this);
	}

// global actions
void	StateVar::protocolError(const char* state) throw(){
	ErrorFatal::logAndExit(state);
	}

// state change operations
void	StateVar::changeToClosed() throw(){
	trace(closed);
	_state	= &closed;
	}

void	StateVar::changeToClosing() throw(){
	trace(closing);
	_state	= &closing;
	}

void	StateVar::changeToClosingPccPc() throw(){
	trace(closingPccPc);
	_state	= &closingPccPc;
	}

void	StateVar::changeToClosingPncPn() throw(){
	trace(closingPncPn);
	_state	= &closingPncPn;
	}

void	StateVar::changeToClosingAllocUsbAddrHc() throw(){
	trace(closingAllocUsbAddrHc);
	_state	= &closingAllocUsbAddrHc;
	}

void	StateVar::changeToClosingFreeUsbAddrHc() throw(){
	trace(closingFreeUsbAddrHc);
	_state	= &closingFreeUsbAddrHc;
	}

void	StateVar::changeToClosingHczcHczUa() throw(){
	trace(closingHczcHczUa);
	_state	= &closingHczcHczUa;
	}

void	StateVar::changeToClosingHczcUa() throw(){
	trace(closingHczcUa);
	_state	= &closingHczcUa;
	}

void	StateVar::changeToClosingHczcAzUa() throw(){
	trace(closingHczcAzUa);
	_state	= &closingHczcAzUa;
	}

void	StateVar::changeToClosingUnlockAddrZeroHcUa() throw(){
	trace(closingUnlockAddrZeroHcUa);
	_state	= &closingUnlockAddrZeroHcUa;
	}

void	StateVar::changeToClosingSetResetPccPcAzUa() throw(){
	trace(closingSetResetPccPcAzUa);
	_state	= &closingSetResetPccPcAzUa;
	}

void	StateVar::changeToClosingTimerCancelPncPnAzUa() throw(){
	trace(closingTimerCancelPncPnAzUa);
	_state	= &closingTimerCancelPncPnAzUa;
	}

void	StateVar::changeToClosingTimerCancelPncPnPaUa() throw(){
	trace(closingTimerCancelPncPnPaUa);
	_state	= &closingTimerCancelPncPnPaUa;
	}

void	StateVar::changeToClosingTimerCancelPaUa() throw(){
	trace(closingTimerCancelPaUa);
	_state	= &closingTimerCancelPaUa;
	}

void	StateVar::changeToClosingTimerCancelAzUa() throw(){
	trace(closingTimerCancelAzUa);
	_state	= &closingTimerCancelAzUa;
	}

void	StateVar::changeToClosingPncPnAzUa() throw(){
	trace(closingPncPnAzUa);
	_state	= &closingPncPnAzUa;
	}

void	StateVar::changeToClosingClrResetChangePccPcAzUa() throw(){
	trace(closingClrResetChangePccPcAzUa);
	_state	= &closingClrResetChangePccPcAzUa;
	}

void	StateVar::changeToClosingResetWaitTwcTwAzUa() throw(){
	trace(closingResetWaitTwcTwAzUa);
	_state	= &closingResetWaitTwcTwAzUa;
	}

void	StateVar::changeToClosingUncUnPncPn() throw(){
	trace(closingUncUnPncPn);
	_state	= &closingUncUnPncPn;
	}

void	StateVar::changeToClosingUncUn() throw(){
	trace(closingUncUn);
	_state	= &closingUncUn;
	}

void	StateVar::changeToClosingReadDescIrpPncPnAzUa() throw(){
	trace(closingReadDescIrpPncPnAzUa);
	_state	= &closingReadDescIrpPncPnAzUa;
	}

void	StateVar::changeToClosingReadDescIrpcIrpAzUa() throw(){
	trace(closingReadDescIrpcIrpAzUa);
	_state	= &closingReadDescIrpcIrpAzUa;
	}

void	StateVar::changeToClosingSetAddrIrpPncPnAzUa() throw(){
	trace(closingSetAddrIrpPncPnAzUa);
	_state	= &closingSetAddrIrpPncPnAzUa;
	}

void	StateVar::changeToClosingSetAddrIrpcIrpAzUa() throw(){
	trace(closingSetAddrIrpcIrpAzUa);
	_state	= &closingSetAddrIrpcIrpAzUa;
	}

void	StateVar::changeToClosingUazPncPnUa() throw(){
	trace(closingUazPncPnUa);
	_state	= &closingUazPncPnUa;
	}

void	StateVar::changeToClosingUazUa() throw(){
	trace(closingUazUa);
	_state	= &closingUazUa;
	}

void	StateVar::changeToClosingPncPnUa() throw(){
	trace(closingPncPnUa);
	_state	= &closingPncPnUa;
	}

void	StateVar::changeToClosingPipeAllocHcPncPnUa() throw(){
	trace(closingPipeAllocHcPncPnUa);
	_state	= &closingPipeAllocHcPncPnUa;
	}

void	StateVar::changeToClosingPipeAllocHcUa() throw(){
	trace(closingPipeAllocHcUa);
	_state	= &closingPipeAllocHcUa;
	}

void	StateVar::changeToClosingFreePipeHcUa() throw(){
	trace(closingFreePipeHcUa);
	_state	= &closingFreePipeHcUa;
	}

void	StateVar::changeToClosingFreePipeHcPncPnUa() throw(){
	trace(closingFreePipeHcPncPnUa);
	_state	= &closingFreePipeHcPncPnUa;
	}

void	StateVar::changeToClosingAdvPncPnPaUa() throw(){
	trace(closingAdvPncPnPaUa);
	_state	= &closingAdvPncPnPaUa;
	}

void	StateVar::changeToClosingPrepareHcPaUa() throw(){
	trace(closingPrepareHcPaUa);
	_state	= &closingPrepareHcPaUa;
	}

void	StateVar::changeToClosingDynSrvHcPaUa() throw(){
	trace(closingDynSrvHcPaUa);
	_state	= &closingDynSrvHcPaUa;
	}

void	StateVar::changeToClosingRncRnUa() throw(){
	trace(closingRncRnUa);
	_state	= &closingRncRnUa;
	}

void	StateVar::changeToClosingRncRnPncPnUa() throw(){
	trace(closingRncRnPncPnUa);
	_state	= &closingRncRnPncPnUa;
	}

void	StateVar::changeToPowerWait() throw(){
	trace(powerWait);
	_state	= &powerWait;
	}

void	StateVar::changeToDisabledPc() throw(){
	trace(disabledPc);
	_state	= &disabledPc;
	}

void	StateVar::changeToDisabledPn() throw(){
	trace(disabledPn);
	_state	= &disabledPn;
	}

void	StateVar::changeToDisabledClrConnChangePc() throw(){
	trace(disabledClrConnChangePc);
	_state	= &disabledClrConnChangePc;
	}

void	StateVar::changeToDisabledGetStatusPc() throw(){
	trace(disabledGetStatusPc);
	_state	= &disabledGetStatusPc;
	}

void	StateVar::changeToDisabledClrEnableChangePc() throw(){
	trace(disabledClrEnableChangePc);
	_state	= &disabledClrEnableChangePc;
	}

void	StateVar::changeToDisabledClrResetChangePc() throw(){
	trace(disabledClrResetChangePc);
	_state	= &disabledClrResetChangePc;
	}

void	StateVar::changeToConnAllocUsbAddrHc() throw(){
	trace(connAllocUsbAddrHc);
	_state	= &connAllocUsbAddrHc;
	}

void	StateVar::changeToConnLockAddrZeroUa() throw(){
	trace(connLockAddrZeroUa);
	_state	= &connLockAddrZeroUa;
	}

void	StateVar::changeToConnSetResetPcAzUa() throw(){
	trace(connSetResetPcAzUa);
	_state	= &connSetResetPcAzUa;
	}

void	StateVar::changeToConnWaitResetPnAzUa() throw(){
	trace(connWaitResetPnAzUa);
	_state	= &connWaitResetPnAzUa;
	}

void	StateVar::changeToConnClrResetChangePcAzUa() throw(){
	trace(connClrResetChangePcAzUa);
	_state	= &connClrResetChangePcAzUa;
	}

void	StateVar::changeToConnResetWaitTwAzUa() throw(){
	trace(connResetWaitTwAzUa);
	_state	= &connResetWaitTwAzUa;
	}

void	StateVar::changeToConnWaitUsbAddrUnPn() throw(){
	trace(connWaitUsbAddrUnPn);
	_state	= &connWaitUsbAddrUnPn;
	}

void	StateVar::changeToConnReadDescIrpPnAzUa() throw(){
	trace(connReadDescIrpPnAzUa);
	_state	= &connReadDescIrpPnAzUa;
	}

void	StateVar::changeToConnReadDescErrorPnAzUa() throw(){
	trace(connReadDescErrorPnAzUa);
	_state	= &connReadDescErrorPnAzUa;
	}

void	StateVar::changeToConnSetAddrIrpPnAzUa() throw(){
	trace(connSetAddrIrpPnAzUa);
	_state	= &connSetAddrIrpPnAzUa;
	}

void	StateVar::changeToConnSetAddrErrorPnAzUa() throw(){
	trace(connSetAddrErrorPnAzUa);
	_state	= &connSetAddrErrorPnAzUa;
	}

void	StateVar::changeToConnUnlockAddrZeroHcPnUa() throw(){
	trace(connUnlockAddrZeroHcPnUa);
	_state	= &connUnlockAddrZeroHcPnUa;
	}

void	StateVar::changeToConnPipeAllocHcPnUa() throw(){
	trace(connPipeAllocHcPnUa);
	_state	= &connPipeAllocHcPnUa;
	}

void	StateVar::changeToReadyPnPaUa() throw(){
	trace(readyPnPaUa);
	_state	= &readyPnPaUa;
	}

void	StateVar::changeToRezRnPnUa() throw(){
	trace(rezRnPnUa);
	_state	= &rezRnPnUa;
	}

void	StateVar::changeToDiscUncUn() throw(){
	trace(discUncUn);
	_state	= &discUncUn;
	}

void	StateVar::changeToDiscUnlockAddrZeroHcUa() throw(){
	trace(discUnlockAddrZeroHcUa);
	_state	= &discUnlockAddrZeroHcUa;
	}

void	StateVar::changeToDiscFreeUsbAddrHc() throw(){
	trace(discFreeUsbAddrHc);
	_state	= &discFreeUsbAddrHc;
	}

void	StateVar::changeToDiscTimerCancelAzUa() throw(){
	trace(discTimerCancelAzUa);
	_state	= &discTimerCancelAzUa;
	}

void	StateVar::changeToDiscTimerCancelPaUa() throw(){
	trace(discTimerCancelPaUa);
	_state	= &discTimerCancelPaUa;
	}

void	StateVar::changeToDiscReadDescIrpcIrpAzUa() throw(){
	trace(discReadDescIrpcIrpAzUa);
	_state	= &discReadDescIrpcIrpAzUa;
	}

void	StateVar::changeToDiscSetAddrIrpcIrpAzUa() throw(){
	trace(discSetAddrIrpcIrpAzUa);
	_state	= &discSetAddrIrpcIrpAzUa;
	}

void	StateVar::changeToDiscPrepareHcPaUa() throw(){
	trace(discPrepareHcPaUa);
	_state	= &discPrepareHcPaUa;
	}

void	StateVar::changeToDiscDynSrvHcPaUa() throw(){
	trace(discDynSrvHcPaUa);
	_state	= &discDynSrvHcPaUa;
	}

void	StateVar::changeToDiscFreePipeHcUa() throw(){
	trace(discFreePipeHcUa);
	_state	= &discFreePipeHcUa;
	}

void	StateVar::changeToDiscPipeAllocHcUa() throw(){
	trace(discPipeAllocHcUa);
	_state	= &discPipeAllocHcUa;
	}

void	StateVar::changeToDiscRncRnUa() throw(){
	trace(discRncRnUa);
	_state	= &discRncRnUa;
	}

void	StateVar::changeToClosingReadFullIrpPncPnPaUa() throw(){
	trace(closingReadFullIrpPncPnPaUa);
	_state	= &closingReadFullIrpPncPnPaUa;
	}

void	StateVar::changeToClosingReadFullIrpcIrpPaUa() throw(){
	trace(closingReadFullIrpcIrpPaUa);
	_state	= &closingReadFullIrpcIrpPaUa;
	}

void	StateVar::changeToClosingPncPnPaUa() throw(){
	trace(closingPncPnPaUa);
	_state	= &closingPncPnPaUa;
	}

void	StateVar::changeToConnReadFullIrpPnPaUa() throw(){
	trace(connReadFullIrpPnPaUa);
	_state	= &connReadFullIrpPnPaUa;
	}

void	StateVar::changeToConnReadFullErrorPnPaUa() throw(){
	trace(connReadFullErrorPnPaUa);
	_state	= &connReadFullErrorPnPaUa;
	}

void	StateVar::changeToDiscReadFullIrpcIrpPaUa() throw(){
	trace(discReadFullIrpcIrpPaUa);
	_state	= &discReadFullIrpcIrpPaUa;
	}

unsigned long	_nTraces;

void	StateVar::trace(const State& state) throw(){
	TraceRec*	rec;
	++_nTraces;
	rec	= _freeTraceRecs.get();
	if(!rec){
		rec	= _trace.get();
		}
	rec->_state	= &state;
	_trace.put(rec);
	}

