/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_drv_usb_hub_port_fsm_fsmh_
#define _oscl_drv_usb_hub_port_fsm_fsmh_
#include "oscl/queue/queue.h"
#include "oscl/queue/queueitem.h"

/** */
namespace Oscl {
/** */
namespace Usb {
/** */
namespace Hub {
/** */
namespace Port {

/** */
class StateVar {
	public:
		/** */
		class ContextApi {
			public:	// request actions
				/** */
				virtual void	requestSetReset() throw()=0;
				/** */
				virtual void	requestSetSuspend() throw()=0;
				/** */
				virtual void	requestSetPower() throw()=0;
				/** */
				virtual void	requestClrEnable() throw()=0;
				/** */
				virtual void	requestClrSuspend() throw()=0;
				/** */
				virtual void	requestClrPower() throw()=0;
				/** */
				virtual void	requestClrConnectChange() throw()=0;
				/** */
				virtual void	requestClrResetChange() throw()=0;
				/** */
				virtual void	requestClrEnableChange() throw()=0;
				/** */
				virtual void	requestClrSuspendChange() throw()=0;
				/** */
				virtual void	requestClrOverCurrentChange() throw()=0;
				/** */
				virtual void	requestCancelReq() throw()=0;
				/** */
				virtual void	requestNotifyPortChange() throw()=0;
				/** */
				virtual void	requestCancelNotifyPortChange() throw()=0;
				/** */
				virtual void	requestGetPortStatus() throw()=0;
				/** */
				virtual void	requestMsgPipeIrpCancel() throw()=0;
				/** */
				virtual void	requestMsgPipeNoDataIrp() throw()=0;
				/** */
				virtual void	requestMsgPipeReadPartIrp() throw()=0;
				/** */
				virtual void	requestMsgPipeReadFullIrp() throw()=0;
				/** */
				virtual void	requestMsgPipeWriteIrp() throw()=0;
				/** */
				virtual void	requestHcdMsgPipeAlloc() throw()=0;
				/** */
				virtual void	requestHcdMsgPipeFree() throw()=0;
				/** */
				virtual void	requestNotifyMsgPipeResourceChange() throw()=0;
				/** */
				virtual void	requestCancelNotifyMsgPipeResChange() throw()=0;
				/** */
				virtual void	requestHcdLockAddressZero() throw()=0;
				/** */
				virtual void	requestHcdUnlockAddressZero() throw()=0;
				/** */
				virtual void	requestHcdCancelLockAddrZero() throw()=0;
				/** was: requestHcdAllocNewAddr */
				virtual void	requestHcdAllocUsbAddr() throw()=0;
				/** was: requestHcdFreeAddr */
				virtual void	requestHcdFreeUsbAddr() throw()=0;
				/** */
				virtual void	requestNotifyUsbAddrResourceChange() throw()=0;
				/** */
				virtual void	requestCancelNotifyUsbAddrResChange() throw()=0;
				/** */
				virtual void	prepareToCloseDynSrv() throw()=0;
				/** */
				virtual void	closeDynSrv() throw()=0;
				/** */
				virtual void	advertiseNewPipe() throw()=0;
				/** */
				virtual void	parseChange() throw()=0;
				/** */
				virtual void	startRetryTimer() throw()=0;
				/** */
				virtual void	startPowerTimer() throw()=0;
				/** */
				virtual void	startResetTimer() throw()=0;
				/** */
				virtual void	cancelTimer() throw()=0;
				/** */
				virtual bool	deviceConnected() throw()=0;
				/** */
				virtual bool	overCurrent() throw()=0;
				/**	returns true if requestHcdAllocUsbAddr successfully
					obtained a new USB address.
				 */
				virtual bool	usbAddrAllocated() throw()=0;
				/** returns true if the just completed
					requestHcdLockAddressZero indicates that the
					lock request was cancelled before the lock
					was obtained.
				 */
				virtual bool	lockAddressZeroCanceled() throw()=0;
				/**	returns true if requestHcdMsgPipeAlloc successfully
					allocated a new pipe.
				 */
				virtual bool	pipeAllocated() throw()=0;
				/** returns true if last Read IRP failed. */
				virtual bool	irpReadError() throw()=0;
				/** */
				/** returns true if last NoData IRP failed. */
				virtual bool	irpNoDataError() throw()=0;
			public:
				/** */
				virtual void	returnOpenReq() throw()=0;
				/** */
				virtual void	returnCloseReq() throw()=0;
			};
	private:
		/** */
		class State {
			public:	// Admin Events
				/** */
				virtual void	open(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				virtual void	close(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			public:	// Hub port events
				/** */
				virtual void	setResetDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	setSuspendDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	setPowerDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	clrEnableDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	clrSuspendDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	clrPowerDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	clrConnectChangeDone(	ContextApi&	context,
														StateVar&	sv
														) const throw();
				/** */
				virtual void	clrResetChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				virtual void	clrEnableChangeDone(	ContextApi&	context,
														StateVar&	sv
														) const throw();
				/** */
				virtual void	clrSuspendChangeDone(	ContextApi&	context,
														StateVar&	sv
														) const throw();
				/** */
				virtual void	clrOverCurrentChangeDone(	ContextApi&	context,
															StateVar&	sv
															) const throw();
				/** */
				virtual void	cancelReqDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	notifyPortChangeDone(	ContextApi&	context,
														StateVar&	sv
														) const throw();
				/** */
				virtual void	cancelNotifyPortChangeDone(	ContextApi&	context,
															StateVar&	sv
															) const throw();
				/** */
				virtual void	getPortStatusDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				virtual void	requestFailed(	ContextApi&	context,
												StateVar&	sv
												) const throw();

			public:	// Message Pipe Events
				/** */
				virtual void	msgPipeIrpCancelDone(	ContextApi&	context,
														StateVar&	sv
														) const throw();
				/** */
				virtual void	msgPipeNoDataIrpDone(	ContextApi&	context,
														StateVar&	sv
														) const throw();
				/** */
				virtual void	msgPipeReadIrpDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				virtual void	msgPipeWriteIrpDone(	ContextApi&	context,
														StateVar&	sv
														) const throw();
			public:	// HCD Message Pipe Allocation Events
				/** */
				virtual void	hcdMsgPipeAllocDone(	ContextApi&	context,
														StateVar&	sv
														) const throw();
				/** */
				virtual void	hcdMsgPipeFreeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				virtual void	notifyMsgPipeResourceChangeDone(
															ContextApi&	context,
															StateVar&	sv
															) const throw();
				/** */
				virtual void	cancelNotifyMsgPipeResourceChangeDone(
															ContextApi&	context,
															StateVar&	sv
															) const throw();
			public:	// HCD Hub Control
				/** */
				virtual void	hcdLockAddressZeroDone(	ContextApi&	context,
														StateVar&	sv
														) const throw();
				/** */
				virtual void	hcdUnlockAddressZeroDone(	ContextApi&	context,
															StateVar&	sv
															) const throw();
				/** */
				virtual void	hcdCancelLockAddrZeroDone(	ContextApi&	context,
															StateVar&	sv
															) const throw();
				/** was : hcdAllocNewAddrDone */
				virtual void	hcdAllocUsbAddrDone(	ContextApi&	context,
														StateVar&	sv
														) const throw();
				/** was : hcdFreeAddrDone */
				virtual void	hcdFreeUsbAddrDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				virtual void	prepareToCloseDynSrvDone(	ContextApi&	context,
															StateVar&	sv
															) const throw();
				/** */
				virtual void	closeDynSrvDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				virtual void	notifyUsbAddrResChangeDone(
															ContextApi&	context,
															StateVar&	sv
															) const throw();
				/** */
				virtual void	cancelNotifyUsbAddrResChangeDone(
															ContextApi&	context,
															StateVar&	sv
															) const throw();
			public:
				/** */
				virtual void	timerExpired(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	timerCanceled(	ContextApi&	context,
												StateVar&	sv
												) const throw();
			public:
				/** */
				virtual void	connectChanged(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	portDisabled(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	portResumed(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	portOverCurrentChange(	ContextApi&	context,
														StateVar&	sv
														) const throw();
				/** */
				virtual void	portResetDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
			public:
				/** */
				virtual const char*	stateName() const throw()=0;
			};

	public:
		/** */
		class Closed : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Closed";}
			private:
				/** */
				void	open(	ContextApi&	context,
								StateVar&	sv
								) const throw();
			};
		/** */
		class Closing : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Closing";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
			};
		/** */
		class ClosingPccPc : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingPccPc";}
			private:
				/** */
				void	setResetDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	setSuspendDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	setPowerDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	clrEnableDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	clrSuspendDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	clrPowerDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	clrConnectChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	clrResetChangeDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	clrEnableChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	clrSuspendChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	clrOverCurrentChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	cancelReqDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	getPortStatusDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	requestFailed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	timerExpired(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	timerCanceled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ClosingPncPn : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingPncPn";}
			private:
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	cancelNotifyPortChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
			};
		/** */
		class ClosingAllocUsbAddrHc : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingAllocUsbAddrHc";}
			private:
				/** */
				void	hcdAllocUsbAddrDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
			};
		/** */
		class ClosingFreeUsbAddrHc : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingFreeUsbAddrHc";}
			private:
				/** */
				void	hcdFreeUsbAddrDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		/** */
		class ClosingHczcHczUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingHczcHczUa";}
			private:
				/** */
				void	hcdLockAddressZeroDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
			};
		/** */
		class ClosingHczcUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingHczcUa";}
			private:
				/** */
				void	hcdCancelLockAddrZeroDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
			};
		/** */
		class ClosingHczcAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingHczcAzUa";}
			private:
				/** */
				void	hcdCancelLockAddrZeroDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
			};
		/** */
		class ClosingUnlockAddrZeroHcUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingUnlockAddrZeroHcUa";}
			private:
				/** */
				void	hcdUnlockAddressZeroDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
			};
		/** */
		class ClosingSetResetPccPcAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingSetResetPccPcAzUa";}
			private:
				/** */
				void	setResetDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	cancelReqDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	requestFailed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ClosingTimerCancelPncPnAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingTimerCancelPncPnAzUa";}
			private:
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	cancelNotifyPortChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	timerExpired(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	timerCanceled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ClosingTimerCancelPncPnPaUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingTimerCancelPncPnPaUa";}
			private:
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	cancelNotifyPortChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	timerExpired(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	timerCanceled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ClosingTimerCancelPaUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingTimerCancelPncPnPaUa";}
			private:
				/** */
				void	timerExpired(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	timerCanceled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ClosingTimerCancelAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingTimerCancelAzUa";}
			private:
				/** */
				void	timerExpired(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	timerCanceled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ClosingPncPnAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingPncPnAzUa";}
			private:
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	cancelNotifyPortChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
			};
		/** */
		class ClosingClrResetChangePccPcAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingClrResetChangePccPcAzUa";}
			private:
				/** */
				void	clrResetChangeDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	cancelReqDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	requestFailed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ClosingResetWaitTwcTwAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingResetWaitTwcTwAzUa";}
			private:
				/** */
				void	timerExpired(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	timerCanceled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ClosingUncUnPncPn : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingUncUnPncPn";}
			private:
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	cancelNotifyPortChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	notifyUsbAddrResChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	cancelNotifyUsbAddrResChangeDone(	ContextApi&	context,
															StateVar&	sv
															) const throw();
			};
		/** */
		class ClosingUncUn : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingUncUn";}
			private:
				/** */
				void	notifyUsbAddrResChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	cancelNotifyUsbAddrResChangeDone(	ContextApi&	context,
															StateVar&	sv
															) const throw();
			};
		/** */
		class ClosingReadDescIrpPncPnAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingReadDescIrpPncPnAzUa";}
			private:
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	cancelNotifyPortChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	msgPipeReadIrpDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		/** */
		class ClosingReadDescIrpcIrpAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingReadDescIrpcIrpAzUa";}
			private:
				/** */
				void	msgPipeIrpCancelDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	msgPipeReadIrpDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		/** */
		class ClosingSetAddrIrpPncPnAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingSetAddrIrpPncPnAzUa";}
			private:
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	cancelNotifyPortChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	msgPipeNoDataIrpDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
			};
		/** */
		class ClosingSetAddrIrpcIrpAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingSetAddrIrpcIrpAzUa";}
			private:
				/** */
				void	msgPipeIrpCancelDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	msgPipeNoDataIrpDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
			};
		/** */
		class ClosingUazPncPnUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingUazPncPnUa";}
			private:
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	cancelNotifyPortChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	hcdUnlockAddressZeroDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
			};
		/** */
		class ClosingUazUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingUazUa";}
			private:
				/** */
				void	hcdUnlockAddressZeroDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
			};
		/** */
		class ClosingPncPnUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingPncPnUa";}
			private:
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	cancelNotifyPortChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
			};
		/** */
		class ClosingPipeAllocHcPncPnUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingPipeAllocHcPncPnUa";}
			private:
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	cancelNotifyPortChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	hcdMsgPipeAllocDone(	ContextApi&	context,
														StateVar&	sv
														) const throw();
			};
		/** */
		class ClosingPipeAllocHcUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingPipeAllocHcUa";}
			private:
				/** */
				void	hcdMsgPipeAllocDone(	ContextApi&	context,
														StateVar&	sv
														) const throw();
			};
		/** */
		class ClosingFreePipeHcUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingFreePipeHcUa";}
			private:
				/** */
				void	hcdMsgPipeFreeDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		/** */
		class ClosingFreePipeHcPncPnUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingFreePipeHcPncPnUa";}
			private:
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	cancelNotifyPortChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	hcdMsgPipeFreeDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		/** */
		class ClosingAdvPncPnPaUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingAdvPncPnPaUa";}
			private:
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	cancelNotifyPortChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
			};
		/** */
		class ClosingPrepareHcPaUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingPrepareHcPaUa";}
			private:
				/** */
				void	prepareToCloseDynSrvDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
			};
		/** */
		class ClosingDynSrvHcPaUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingDynSrvHcPaUa";}
			private:
				/** */
				void	closeDynSrvDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		/** */
		class ClosingRncRnUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingRncRnUa";}
			private:
				/** */
				void	notifyMsgPipeResourceChangeDone(	ContextApi&	context,
															StateVar&	sv
															) const throw();
				/** */
				void	cancelNotifyMsgPipeResourceChangeDone(
														ContextApi&	context,
														StateVar&	sv
														) const throw();
			};
		/** */
		class ClosingRncRnPncPnUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingRncRnPncPnUa";}
			private:
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	cancelNotifyPortChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	notifyMsgPipeResourceChangeDone(	ContextApi&	context,
															StateVar&	sv
															) const throw();
				/** */
				void	cancelNotifyMsgPipeResourceChangeDone(
														ContextApi&	context,
														StateVar&	sv
														) const throw();
			};
		/** */
		class ClosingReadFullIrpPncPnPaUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingReadFullIrpPncPnPaUa";}
			private:
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	cancelNotifyPortChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	msgPipeReadIrpDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		/** */
		class ClosingReadFullIrpcIrpPaUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingReadFullIrpcIrpPaUa";}
			private:
				/** */
				void	msgPipeIrpCancelDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	msgPipeReadIrpDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		/** */
		class ClosingPncPnPaUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ClosingPncPnPaUa";}
			private:
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	cancelNotifyPortChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
			};
		/** */
		class PowerWait : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "PowerWait";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	timerExpired(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class DisabledPc : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DisabledPc";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	setPowerDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	requestFailed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class DisabledPn : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DisabledPn";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	connectChanged(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portDisabled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portResetDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portOverCurrentChange(	ContextApi&	context,
												StateVar&	sv
												) const throw();
			};
		/** */
		class DisabledClrEnableChangePc : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DisabledClrEnableChangePc";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	clrEnableChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	requestFailed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class DisabledClrResetChangePc : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DisabledClrResetChangePc";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	clrResetChangeDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	requestFailed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class DisabledClrConnChangePc : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DisabledClrConnChangePc";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	clrConnectChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	requestFailed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class DisabledGetStatusPc : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DisabledGetStatusPc";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	getPortStatusDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	requestFailed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ConnAllocUsbAddrHc : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ConnAllocUsbAddrHc";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	hcdAllocUsbAddrDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
			};
		/** */
		class ConnLockAddrZeroUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ConnLockAddrZeroUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	hcdLockAddressZeroDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
			};
		/** */
		class ConnSetResetPcAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ConnSetResetPcAzUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	setResetDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	requestFailed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ConnWaitResetPnAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ConnWaitResetPnAzUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	connectChanged(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portDisabled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portResetDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ConnClrResetChangePcAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ConnClrResetChangePcAzUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	clrResetChangeDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	requestFailed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ConnResetWaitTwAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ConnResetWaitTwAzUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	timerExpired(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ConnWaitUsbAddrUnPn : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ConnWaitUsbAddrUnPn";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	notifyUsbAddrResChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	connectChanged(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portDisabled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ConnReadDescIrpPnAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ConnReadDescIrpPnAzUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	msgPipeReadIrpDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	connectChanged(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portDisabled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portResumed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ConnReadDescErrorPnAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ConnReadDescErrorPnAzUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	connectChanged(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portDisabled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portResumed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	timerExpired(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ConnSetAddrIrpPnAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ConnSetAddrIrpPnAzUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	msgPipeNoDataIrpDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	connectChanged(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portDisabled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ConnSetAddrErrorPnAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ConnSetAddrErrorPnAzUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	connectChanged(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portDisabled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	timerExpired(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ConnUnlockAddrZeroHcPnUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ConnUnlockAddrZeroHcPnUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	hcdUnlockAddressZeroDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	connectChanged(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portDisabled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ConnPipeAllocHcPnUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ConnPipeAllocHcPnUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	hcdMsgPipeAllocDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	connectChanged(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portDisabled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ConnReadFullIrpPnPaUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ConnReadFullIrpPnPaUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	msgPipeReadIrpDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	connectChanged(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portDisabled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portResumed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ConnReadFullErrorPnPaUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ConnReadFullErrorPnPaUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	connectChanged(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portDisabled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portResumed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	timerExpired(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class ReadyPnPaUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ReadyPnPaUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	connectChanged(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portDisabled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portResumed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class RezRnPnUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "RezRnPnUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyPortChangeDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	notifyMsgPipeResourceChangeDone(	ContextApi&	context,
															StateVar&	sv
															) const throw();
				/** */
				void	connectChanged(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	portDisabled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class DiscUncUn : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DiscUncUn";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyUsbAddrResChangeDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				void	cancelNotifyUsbAddrResChangeDone(	ContextApi&	context,
															StateVar&	sv
															) const throw();
			};
		/** */
		class DiscUnlockAddrZeroHcUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DiscUnlockAddrZeroHcUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	hcdUnlockAddressZeroDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
			};
		/** */
		class DiscFreeUsbAddrHc : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DiscFreeUsbAddrHc";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	hcdFreeUsbAddrDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		/** */
		class DiscTimerCancelAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DiscTimerCancelAzUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	timerExpired(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	timerCanceled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class DiscTimerCancelPaUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DiscTimerCancelPaUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	timerExpired(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	timerCanceled(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class DiscReadDescIrpcIrpAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DiscReadDescIrpcIrpAzUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	msgPipeIrpCancelDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	msgPipeReadIrpDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		/** */
		class DiscSetAddrIrpcIrpAzUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DiscSetAddrIrpcIrpAzUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	msgPipeIrpCancelDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	msgPipeNoDataIrpDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
			};
		/** */
		class DiscPrepareHcPaUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DiscPrepareHcPaUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	prepareToCloseDynSrvDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
			};
		/** */
		class DiscDynSrvHcPaUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DiscDynSrvHcPaUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	closeDynSrvDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		class DiscFreePipeHcUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DiscFreePipeHcUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	hcdMsgPipeFreeDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		/** */
		class DiscPipeAllocHcUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DiscPipeAllocHcUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	hcdMsgPipeAllocDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
			};
		/** */
		class DiscRncRnUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DiscRncRnUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	notifyMsgPipeResourceChangeDone(	ContextApi&	context,
															StateVar&	sv
															) const throw();
				/** */
				void	cancelNotifyMsgPipeResourceChangeDone(
														ContextApi&	context,
														StateVar&	sv
														) const throw();
			};
		/** */
		class DiscReadFullIrpcIrpPaUa : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "DiscReadFullIrpcIrpPaUa";}
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	msgPipeIrpCancelDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				void	msgPipeReadIrpDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};

	private:
		/** */
		struct TraceRec : public Oscl::QueueItem {
			public:
				/** */
				const State*	_state;
			};

	private:
		/** */
		ContextApi&		_context;
		/** */
		const State*	_state;

	private:
		/** */
		enum{nTraceRecs=20};
		/** */
		TraceRec				_traceRec[nTraceRecs];
		/** */
		Oscl::Queue<TraceRec>	_freeTraceRecs;
		/** */
		Oscl::Queue<TraceRec>	_trace;

	public:	// Constructors
		/** */
		StateVar(ContextApi& context) throw();

	public:	// events
		/** */
		void	open() throw();
		/** */
		void	close() throw();
		/** */
		void	setResetDone() throw();
		/** */
		void	setSuspendDone() throw();
		/** */
		void	setPowerDone() throw();
		/** */
		void	clrEnableDone() throw();
		/** */
		void	clrSuspendDone() throw();
		/** */
		void	clrPowerDone() throw();
		/** */
		void	clrConnectChangeDone() throw();
		/** */
		void	clrResetChangeDone() throw();
		/** */
		void	clrEnableChangeDone() throw();
		/** */
		void	clrSuspendChangeDone() throw();
		/** */
		void	clrOverCurrentChangeDone() throw();
		/** */
		void	cancelReqDone() throw();
		/** */
		void	notifyPortChangeDone() throw();
		/** */
		void	cancelNotifyPortChangeDone() throw();
		/** */
		void	getPortStatusDone() throw();
		/** */
		void	requestFailed() throw();
		/** */
		void	msgPipeIrpCancelDone() throw();
		/** */
		void	msgPipeNoDataIrpDone() throw();
		/** */
		void	msgPipeReadIrpDone() throw();
		/** */
		void	msgPipeWriteIrpDone() throw();
		/** */
		void	hcdMsgPipeAllocDone() throw();
		/** */
		void	hcdMsgPipeFreeDone() throw();
		/** */
		void	notifyMsgPipeResourceChangeDone() throw();
		/** */
		void	cancelNotifyMsgPipeResourceChangeDone() throw();
		/** */
		void	hcdLockAddressZeroDone() throw();
		/** */
		void	hcdUnlockAddressZeroDone() throw();
		/** */
		void	hcdCancelLockAddrZeroDone() throw();
		/** */
		void	hcdAllocUsbAddrDone() throw();
		/** */
		void	hcdFreeUsbAddrDone() throw();
		/** */
		void	prepareToCloseDynSrvDone() throw();
		/** */
		void	closeDynSrvDone() throw();
		/** */
		void	notifyUsbAddrResChangeDone() throw();
		/** */
		void	cancelNotifyUsbAddrResChangeDone() throw();
		/** */
		void	timerExpired() throw();
		/** */
		void	timerCanceled() throw();
		/** */
		void	connectChanged() throw();
		/** */
		void	portDisabled() throw();
		/** */
		void	portResumed() throw();
		/** */
		void	portOverCurrentChange() throw();
		/** */
		void	portResetDone() throw();

	private: // global actions and state change operations
		/** */
		friend class State;
		friend class Closed;
		friend class Closing;
		friend class ClosingPccPc;
		friend class ClosingPncPn;
		friend class ClosingAllocUsbAddrHc;
		friend class ClosingFreeUsbAddrHc;
		friend class ClosingHczcHczUa;
		friend class ClosingHczcUa;
		friend class ClosingHczcAzUa;
		friend class ClosingUnlockAddrZeroHcUa;
		friend class ClosingSetResetPccPcAzUa;
		friend class ClosingTimerCancelPncPnAzUa;
		friend class ClosingTimerCancelPncPnPaUa;
		friend class ClosingTimerCancelPaUa;
		friend class ClosingTimerCancelAzUa;
		friend class ClosingPncPnAzUa;
		friend class ClosingPncPnPaUa;
		friend class ClosingClrResetChangePccPcAzUa;
		friend class ClosingResetWaitTwcTwAzUa;
		friend class ClosingUncUnPncPn;
		friend class ClosingUncUn;
		friend class ClosingReadDescIrpPncPnAzUa;
		friend class ClosingReadFullIrpPncPnPaUa;
		friend class ClosingReadDescIrpcIrpAzUa;
		friend class ClosingReadFullIrpcIrpPaUa;
		friend class ClosingSetAddrIrpPncPnAzUa;
		friend class ClosingSetAddrIrpcIrpAzUa;
		friend class ClosingUazPncPnUa;
		friend class ClosingUazUa;
		friend class ClosingPncPnUa;
		friend class ClosingPipeAllocHcPncPnUa;
		friend class ClosingPipeAllocHcUa;
		friend class ClosingFreePipeHcUa;
		friend class ClosingFreePipeHcPncPnUa;
		friend class ClosingAdvPncPnPaUa;
		friend class ClosingPrepareHcPaUa;
		friend class ClosingDynSrvHcPaUa;
		friend class ClosingRncRnUa;
		friend class ClosingRncRnPncPnUa;
		friend class PowerWait;
		friend class DisabledPc;
		friend class DisabledPn;
		friend class DisabledClrConnChangePc;
		friend class DisabledGetStatusPc;
		friend class DisabledClrEnableChangePc;
		friend class DisabledClrResetChangePc;
		friend class ConnAllocUsbAddrHc;
		friend class ConnLockAddrZeroUa;
		friend class ConnSetResetPcAzUa;
		friend class ConnWaitResetPnAzUa;
		friend class ConnClrResetChangePcAzUa;
		friend class ConnResetWaitTwAzUa;
		friend class ConnWaitUsbAddrUnPn;
		friend class ConnReadDescIrpPnAzUa;
		friend class ConnReadDescErrorPnAzUa;
		friend class ConnSetAddrIrpPnAzUa;
		friend class ConnSetAddrErrorPnAzUa;
		friend class ConnUnlockAddrZeroHcPnUa;
		friend class ConnPipeAllocHcPnUa;
		friend class ConnReadFullIrpPnPaUa;
		friend class ConnReadFullErrorPnPaUa;
		friend class ReadyPnPaUa;
		friend class RezRnPnUa;
		friend class DiscUncUn;
		friend class DiscUnlockAddrZeroHcUa;
		friend class DiscFreeUsbAddrHc;
		friend class DiscTimerCancelAzUa;
		friend class DiscTimerCancelPaUa;
		friend class DiscReadDescIrpcIrpAzUa;
		friend class DiscSetAddrIrpcIrpAzUa;
		friend class DiscPrepareHcPaUa;
		friend class DiscDynSrvHcPaUa;
		friend class DiscFreePipeHcUa;
		friend class DiscPipeAllocHcUa;
		friend class DiscRncRnUa;
		friend class DiscReadFullIrpcIrpPaUa;
		/** */
		void	protocolError(const char* state) throw();
		/** */
		void	changeToClosed() throw();
		/** */
		void	changeToClosing() throw();
		/** */
		void	changeToClosingPccPc() throw();
		/** */
		void	changeToClosingPncPn() throw();
		/** */
		void	changeToClosingAllocUsbAddrHc() throw();
		/** */
		void	changeToClosingFreeUsbAddrHc() throw();
		/** */
		void	changeToClosingHczcHczUa() throw();
		/** */
		void	changeToClosingHczcUa() throw();
		/** */
		void	changeToClosingHczcAzUa() throw();
		/** */
		void	changeToClosingUnlockAddrZeroHcUa() throw();
		/** */
		void	changeToClosingSetResetPccPcAzUa() throw();
		/** */
		void	changeToClosingPncPnAzUa() throw();
		/** */
		void	changeToClosingTimerCancelPncPnAzUa() throw();
		/** */
		void	changeToClosingTimerCancelPncPnPaUa() throw();
		/** */
		void	changeToClosingTimerCancelPaUa() throw();
		/** */
		void	changeToClosingTimerCancelAzUa() throw();
		/** */
		void	changeToClosingClrResetChangePccPcAzUa() throw();
		/** */
		void	changeToClosingResetWaitTwcTwAzUa() throw();
		/** */
		void	changeToClosingUncUnPncPn() throw();
		/** */
		void	changeToClosingUncUn() throw();
		/** */
		void	changeToClosingReadDescIrpPncPnAzUa() throw();
		/** */
		void	changeToClosingReadDescIrpcIrpAzUa() throw();
		/** */
		void	changeToClosingSetAddrIrpPncPnAzUa() throw();
		/** */
		void	changeToClosingSetAddrIrpcIrpAzUa() throw();
		/** */
		void	changeToClosingUazPncPnUa() throw();
		/** */
		void	changeToClosingUazUa() throw();
		/** */
		void	changeToClosingPncPnUa() throw();
		/** */
		void	changeToClosingPipeAllocHcPncPnUa() throw();
		/** */
		void	changeToClosingPipeAllocHcUa() throw();
		/** */
		void	changeToClosingFreePipeHcUa() throw();
		/** */
		void	changeToClosingFreePipeHcPncPnUa() throw();
		/** */
		void	changeToClosingAdvPncPnPaUa() throw();
		/** */
		void	changeToClosingPrepareHcPaUa() throw();
		/** */
		void	changeToClosingDynSrvHcPaUa() throw();
		/** */
		void	changeToClosingRncRnUa() throw();
		/** */
		void	changeToClosingRncRnPncPnUa() throw();
		/** */
		void	changeToClosingReadFullIrpPncPnPaUa() throw();
		/** */
		void	changeToClosingReadFullIrpcIrpPaUa() throw();
		/** */
		void	changeToClosingPncPnPaUa() throw();
		/** */
		void	changeToPowerWait() throw();
		/** */
		void	changeToDisabledPc() throw();
		/** */
		void	changeToDisabledPn() throw();
		/** */
		void	changeToDisabledClrConnChangePc() throw();
		/** */
		void	changeToDisabledGetStatusPc() throw();
		/** */
		void	changeToDisabledClrEnableChangePc() throw();
		/** */
		void	changeToDisabledClrResetChangePc() throw();
		/** */
		void	changeToConnAllocUsbAddrHc() throw();
		/** */
		void	changeToConnLockAddrZeroUa() throw();
		/** */
		void	changeToConnSetResetPcAzUa() throw();
		/** */
		void	changeToConnWaitResetPnAzUa() throw();
		/** */
		void	changeToConnClrResetChangePcAzUa() throw();
		/** */
		void	changeToConnResetWaitTwAzUa() throw();
		/** */
		void	changeToConnWaitUsbAddrUnPn() throw();
		/** */
		void	changeToConnReadDescIrpPnAzUa() throw();
		/** */
		void	changeToConnReadDescErrorPnAzUa() throw();
		/** */
		void	changeToConnSetAddrIrpPnAzUa() throw();
		/** */
		void	changeToConnSetAddrErrorPnAzUa() throw();
		/** */
		void	changeToConnUnlockAddrZeroHcPnUa() throw();
		/** */
		void	changeToConnPipeAllocHcPnUa() throw();
		/** */
		void	changeToConnReadFullIrpPnPaUa() throw();
		/** */
		void	changeToConnReadFullErrorPnPaUa() throw();
		/** */
		void	changeToReadyPnPaUa() throw();
		/** */
		void	changeToRezRnPnUa() throw();
		/** */
		void	changeToDiscUncUn() throw();
		/** */
		void	changeToDiscUnlockAddrZeroHcUa() throw();
		/** */
		void	changeToDiscFreeUsbAddrHc() throw();
		/** */
		void	changeToDiscTimerCancelAzUa() throw();
		/** */
		void	changeToDiscTimerCancelPaUa() throw();
		/** */
		void	changeToDiscReadDescIrpcIrpAzUa() throw();
		/** */
		void	changeToDiscSetAddrIrpcIrpAzUa() throw();
		/** */
		void	changeToDiscPrepareHcPaUa() throw();
		/** */
		void	changeToDiscDynSrvHcPaUa() throw();
		/** */
		void	changeToDiscFreePipeHcUa() throw();
		/** */
		void	changeToDiscPipeAllocHcUa() throw();
		/** */
		void	changeToDiscRncRnUa() throw();
		/** */
		void	changeToDiscReadFullIrpcIrpPaUa() throw();

	private:
		/** */
		void	trace(const State& state) throw();
	};

}
}
}
}

#endif
