/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_drv_usb_ohci_hcd_pipe_message_driverh_
#define _oscl_drv_usb_ohci_hcd_pipe_message_driverh_
#include "oscl/driver/usb/pipe/message/reqapi.h"
#include "oscl/driver/usb/ohci/ed/message.h"
#include "oscl/driver/usb/ohci/hcd/server/hcdapi.h"
#include "oscl/queue/queueitem.h"
#include "oscl/mt/itc/srv/close.h"

/** */
namespace Oscl {
/** */
namespace Usb {
/** */
namespace Ohci {
/** */
namespace HCD {
/** */
namespace Pipe {
/** */
namespace Message {

class Driver :
	public Oscl::Usb::Pipe::Message::Req::Api,
	public Oscl::Mt::Itc::Srv::Close::Req::Api,
	public Oscl::QueueItem
	{
	private:
		/** */
		class TransApi : public QueueItem {
			public:
				/** */
				virtual bool	irpMatch(Oscl::Mt::Itc::SrvMsg& irp) throw()=0;
				/** */
				virtual void	cancel(CancelReq& cancelReq) throw()=0;
				/** */
				virtual void	cancel() throw()=0;
			};

		/** */
		class NoDataTrans : public TransApi {
			private:
				/** This memory is used to perform exactly one
					deferred task at a time. The transaction
					must track the state of this memory to
					prevent problems.
				 */
				Oscl::Memory::AlignedBlock
					<sizeof(	Oscl::Usb::Ohci::HCD::
								DeferredTask<NoDataTrans>
								)
						>									_dtMem;
				/** */
				Oscl::Usb::Ohci::TD::Transfer<NoDataTrans>	_otherTransfer;
				/** */
				Oscl::Usb::Ohci::TD::Transfer<NoDataTrans>	_lastTransfer;
				/** */
				Driver&										_context;
				/** */
				Oscl::Usb::Ohci::HCD::HcdApi&				_hcdContext;
				/** */
				Oscl::Usb::Ohci::ED::Message&				_ed;
				/** */
				NoDataReq&									_irp;
				/** */
				CancelReq*									_cancelReq;
				/** */
				Oscl::Queue<TransactionDescMem>				_tdMem;
				/** */
				Oscl::Queue<	Oscl::Usb::Ohci::
								TD::Descriptor
								>							_pendingTdList;
				/** */
				unsigned									_nTDs;
				/** */
				bool										_canceling;
				/** */
				unsigned									_sofFrameNumber;
				/** */
				bool										_retryCancel;
				/** */
				enum{nRequiredTDs=1+1};
				/** */
				const Oscl::Usb::Pipe::Status::Result*	_result;

			public:
				/** */
				NoDataTrans(	Driver&							context,
								Oscl::Usb::Ohci::HCD::HcdApi&	hcdContext,
								Oscl::Usb::Ohci::ED::Message&	ed,
								NoDataReq&						irp
								) throw();

			private: // TransApi
				/** */
				bool	irpMatch(Oscl::Mt::Itc::SrvMsg& irp) throw();
				/** */
				void	cancel(CancelReq& cancelReq) throw();
				/** */
				void	cancel() throw();

			private:
				/** */
				friend class Oscl::Usb::Ohci::TD::Transfer<NoDataTrans>;
				/** */
				void	tdAlloc() throw();
				/** */
				void	cancelSof() throw();
				/** */
				void	executeIRP() throw();
				/** */
				void	lastTransferComplete(TD::Descriptor& desc) throw();
				/** */
				void	otherTransferComplete(TD::Descriptor& desc) throw();
				/** */
				Oscl::Usb::Ohci::HCD::DeferredTaskApi&	tdMemTask() throw();
				/** */
				Oscl::Usb::Ohci::HCD::
				DeferredTaskApi&		cancelSofTask() throw();
			};

		/** */
		class ReadDataTrans : public TransApi {
			private:
				/** This memory is used to perform exactly one
					deferred task at a time. The transaction
					must track the state of this memory to
					prevent problems.
				 */
				Oscl::Memory::AlignedBlock
					<sizeof(	Oscl::Usb::Ohci::HCD::
								DeferredTask<ReadDataTrans>
								)
						>									_dtMem;
				/** */
				Oscl::Usb::Ohci::TD::
				Transfer<ReadDataTrans>						_otherTransfer;
				/** */
				Oscl::Usb::Ohci::TD::
				Transfer<ReadDataTrans>						_lastTransfer;
				/** */
				Driver&										_context;
				/** */
				Oscl::Usb::Ohci::HCD::HcdApi&				_hcdContext;
				/** */
				Oscl::Usb::Ohci::ED::Message&				_ed;
				/** */
				ReadReq&									_irp;
				/** */
				CancelReq*									_cancelReq;
				/** */
				Oscl::Queue<TransactionDescMem>				_tdMem;
				/** */
				Oscl::Queue<	Oscl::Usb::Ohci::
								TD::Descriptor
								>							_pendingTdList;
				/** */
				unsigned									_nTDs;
				/** */
				unsigned									_nRequiredTDs;
				/** */
				unsigned									_dataLength;
				/** */
				bool										_canceling;
				/** */
				unsigned									_sofFrameNumber;
				/** */
				bool										_retryCancel;
				/** */
				const Oscl::Usb::Pipe::Status::Result*	_result;

			public:
				/** */
				ReadDataTrans(	Driver&							context,
								Oscl::Usb::Ohci::HCD::HcdApi&	hcdContext,
								Oscl::Usb::Ohci::ED::Message&	ed,
								ReadReq&						irp
								) throw();

			private: // TransApi
				/** */
				bool	irpMatch(Oscl::Mt::Itc::SrvMsg& irp) throw();
				/** */
				void	cancel(CancelReq& msg) throw();
				/** */
				void	cancel() throw();

			private:
				/** */
				friend class Oscl::Usb::Ohci::TD::Transfer<ReadDataTrans>;
				/** */
				void	tdAlloc() throw();
				/** */
				void	cancelSof() throw();
				/** */
				void	executeIRP() throw();
				/** */
				void	lastTransferComplete(TD::Descriptor& desc) throw();
				/** */
				void	otherTransferComplete(TD::Descriptor& desc) throw();
				/** */
				Oscl::Usb::Ohci::HCD::DeferredTaskApi&	tdMemTask() throw();
				/** */
				Oscl::Usb::Ohci::HCD::
				DeferredTaskApi&		cancelSofTask() throw();
			};

		/** */
		class WriteDataTrans : public TransApi {
			private:
				/** This memory is used to perform exactly one
					deferred task at a time. The transaction
					must track the state of this memory to
					prevent problems.
				 */
				Oscl::Memory::AlignedBlock
					<sizeof(	Oscl::Usb::Ohci::HCD::
								DeferredTask<WriteDataTrans>
								)
						>									_dtMem;
				/** */
				Oscl::Usb::Ohci::TD::
				Transfer<WriteDataTrans>					_otherTransfer;
				/** */
				Oscl::Usb::Ohci::TD::
				Transfer<WriteDataTrans>					_lastTransfer;
				/** */
				Driver&										_context;
				/** */
				Oscl::Usb::Ohci::HCD::HcdApi&				_hcdContext;
				/** */
				Oscl::Usb::Ohci::ED::Message&				_ed;
				/** */
				WriteReq&									_irp;
				/** */
				CancelReq*									_cancelReq;
				/** */
				Oscl::Queue<TransactionDescMem>				_tdMem;
				/** */
				Oscl::Queue<	Oscl::Usb::Ohci::
								TD::Descriptor
								>							_pendingTdList;
				/** */
				unsigned									_nTDs;
				/** */
				unsigned									_nRequiredTDs;
				/** */
				unsigned									_dataLength;
				/** */
				bool										_canceling;
				/** */
				unsigned									_sofFrameNumber;
				/** */
				bool										_retryCancel;
				/** */
				const Oscl::Usb::Pipe::Status::Result*	_result;

			public:
				/** */
				WriteDataTrans(	Driver&							context,
								Oscl::Usb::Ohci::HCD::HcdApi&	hcdContext,
								Oscl::Usb::Ohci::ED::Message&	ed,
								WriteReq&						irp
								) throw();

			private: // TransApi
				/** */
				bool	irpMatch(Oscl::Mt::Itc::SrvMsg& irp) throw();
				/** */
				void	cancel(CancelReq& msg) throw();
				/** */
				void	cancel() throw();

			private:
				/** */
				friend class Oscl::Usb::Ohci::TD::Transfer<WriteDataTrans>;
				/** */
				void	tdAlloc() throw();
				/** */
				void	cancelSof() throw();
				/** */
				void	executeIRP() throw();
				/** */
				void	lastTransferComplete(TD::Descriptor& desc) throw();
				/** */
				void	otherTransferComplete(TD::Descriptor& desc) throw();
				/** */
				Oscl::Usb::Ohci::HCD::DeferredTaskApi&	tdMemTask() throw();
				/** */
				Oscl::Usb::Ohci::HCD::
				DeferredTaskApi&		cancelSofTask() throw();
			};
		/** */
		typedef union TransMem {
			/** */
            void*	__qitemlink;
			/** */
			Oscl::Memory::AlignedBlock<sizeof(ReadDataTrans)>	_read;
			/** */
			Oscl::Memory::AlignedBlock<sizeof(WriteDataTrans)>	_write;
			/** */
			Oscl::Memory::AlignedBlock<sizeof(NoDataTrans)>		_none;
			} TransMem;

	private:
		/** */
		Oscl::Mt::Itc::Srv::CloseSync				_ocSync;

		/** */
		enum{maxTransfers = 2};

		/** */
		TransMem									_transMem[maxTransfers];

		/** */
		Oscl::Queue<TransMem>						_freeTransMem;

		/** */
		Oscl::Queue<TransApi>						_pendingTransactions;

		/** */
		Oscl::Usb::Pipe::
		Message::Req::Api::ConcreteSAP				_sap;

		/** */
		Oscl::Usb::Ohci::ED::Message&				_ed;

		/** */
		Oscl::Queue<Oscl::Mt::Itc::SrvMsg>			_deferredQ;
		/** */
		Oscl::Usb::Ohci::HCD::HcdApi&			_hcdContext;
		/** */
		unsigned									_skipCount;
		/** */
		const unsigned								_maxPacketSize;
		/** */
		Oscl::Mt::Itc::Srv::
		Close::Req::Api::CloseReq*					_closeReq;
		/** */
		unsigned									_nFreeTransMem;
		/** */
		bool										_opened;

	public:
		/** */
		Driver(	Oscl::Mt::Itc::PostMsgApi&			myPapi,
				Oscl::Usb::Ohci::ED::Message&		ed,
				Oscl::Usb::Ohci::HCD::HcdApi&		hcdContext
				) throw();

		/** */
		virtual ~Driver() {}

		/** */
		Oscl::Usb::Pipe::Message::
		Req::Api::SAP&					getSAP() throw();

		/** */
		Oscl::Mt::Itc::Srv::
		Close::Req::Api::SAP&			getCloseSAP() throw();

		/** */
		Oscl::Mt::Itc::Srv::
		OpenCloseSyncApi&				getOpenCloseSyncApi() throw();

		/** */
		Oscl::Usb::Ohci::ED::Message&	getED() throw();

		/** */
		unsigned	getMaxPacketSize() const throw();

		/** */
		void		cancelAllPendingTransactions() throw();

	private: // Transfer
		/** */
		friend class NoDataTrans;
		/** */
		friend class ReadDataTrans;
		/** */
		friend class WriteDataTrans;
		/** */
		void	free(NoDataTrans* xfer) throw();
		/** */
		void	free(ReadDataTrans* xfer) throw();
		/** */
		void	free(WriteDataTrans* xfer) throw();
		/** */
		void	free(TransMem* mem) throw();
		/** */
		TransMem*	allocTransMem() throw();
		/** */
		TransApi*	remove(TransApi* trans) throw();
		/** */
		void	pending(TransApi* trans) throw();
		/** */
		void	skip() throw();
		/** */
		void	dontSkip() throw();

	private:	// Oscl::Usb::Pipe::Message::Req::Api
		/** */
		void	request(CancelReq& msg) throw();
		/** */
		void	request(NoDataReq& msg) throw();
		/** */
		void	request(ReadReq& msg) throw();
		/** */
		void	request(WriteReq& msg) throw();

	private:	// Oscl::Mt::Itc::Srv::Close::Req::Api
		/** */
		void	request(OpenReq& msg) throw();
		/** */
		void	request(CloseReq& msg) throw();
	};

}
}
}
}
}
}

#endif
