/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_drv_usb_ohci_hcd_server_driverh_
#define _oscl_drv_usb_ohci_hcd_server_driverh_
#include "oscl/driver/usb/ohci/hcd/rhreqapi.h"
#include "oscl/interrupt/shandler.h"
#include "oscl/interrupt/isrdsr.h"
#include "oscl/mt/itc/mbox/srvapi.h"
#include "oscl/mt/itc/srv/close.h"
#include "oscl/mt/itc/srv/closeapi.h"
#include "oscl/hw/usb/ohci/reg.h"
#include "oscl/hw/usb/ohci/hcca.h"
#include "oscl/driver/usb/ohci/reg/control.h"
#include "oscl/driver/usb/ohci/reg/cmdstat.h"
#include "oscl/driver/usb/ohci/reg/irqstat.h"
#include "oscl/driver/usb/ohci/reg/irqenable.h"
#include "oscl/driver/usb/ohci/reg/irqdisable.h"
#include "oscl/queue/queue.h"
#include "oscl/bus/api.h"
#include "oscl/driver/usb/ohci/td/general/desc.h"
#include "oscl/driver/usb/ohci/td/isoch/desc.h"
#include "oscl/driver/usb/alloc/addr/reqapi.h"
#include "oscl/driver/usb/ohci/ed/message.h"
#include "oscl/memory/block.h"
#include "oscl/driver/usb/ohci/ed/in.h"
#include "oscl/driver/usb/ohci/ed/out.h"
#include "oscl/driver/usb/ohci/ed/isoin.h"
#include "oscl/driver/usb/ohci/ed/isoout.h"
#include "oscl/driver/usb/ohci/ed/message.h"
#include "oscl/driver/usb/alloc/pipe/message/reqapi.h"
#include "oscl/driver/usb/alloc/pipe/interrupt/in/reqapi.h"
#include "oscl/driver/usb/alloc/pipe/bulk/in/reqapi.h"
#include "oscl/driver/usb/alloc/pipe/bulk/out/reqapi.h"
#include "oscl/driver/usb/alloc/bw/reqapi.h"
#include "oscl/driver/usb/alloc/bw/interrupt/reqapi.h"
#include "oscl/driver/usb/hcd/enum/monitor/var.h"
#include "oscl/driver/usb/hcd/enum/dcp/var.h"
#include "oscl/driver/usb/ohci/bw/config.h"
#include "tdmem.h"
#include "edmem.h"
#include "hcdapi.h"
#include "defer.h"
#include "msgpipe.h"
#include "iinpipe.h"
#include "bulkinpipe.h"
#include "bulkoutpipe.h"

/** */
namespace Oscl {
/** */
namespace Usb {
/** */
namespace Ohci {
/** */
namespace HCD {

/** */
class Driver :	public Oscl::Interrupt::StatusHandlerApi,
				public Oscl::Interrupt::IsrDsrApi,
				public Oscl::Mt::Itc::Srv::CloseSync,
				public Oscl::Mt::Itc::Srv::Close::Req::Api,
				public Oscl::Usb::Alloc::Addr::Req::Api,
				public Oscl::Usb::Alloc::Pipe::Message::Req::Api,
				public Oscl::Usb::Alloc::Pipe::Interrupt::IN::Req::Api,
				public Oscl::Usb::Alloc::Pipe::Bulk::IN::Req::Api,
				public Oscl::Usb::Alloc::Pipe::Bulk::OUT::Req::Api,
				public Oscl::Usb::Alloc::BW::Req::Api,
				public Oscl::Usb::Alloc::BW::Interrupt::Req::Api,
				public Oscl::Mt::Itc::ServerApi::SignaledApi,
				public RootHub::Req::Api,
				public HcdApi
				{
	private:
		/** */
		class ControlListAttachDefTask {
			private:
				/** */
				Oscl::Memory::AlignedBlock
					<sizeof(DeferredTask<ControlListAttachDefTask>)>	_dtMem;
				/** */
				Driver&							_context;
				/** */
				Oscl::Usb::Alloc::Pipe::
				Message::Req::Api::NewReq&		_msg;
				/** */
				MessagePipeRec&					_pipe;
				/** */
				unsigned						_currentFrameNumber;
			public:
				/** */
				ControlListAttachDefTask(	Driver&				context,
											Oscl::Usb::Alloc::
											Pipe::Message::
											Req::Api::NewReq&	msg,
											MessagePipeRec&		pipe
											) throw();
				/** */
				void	queueSofTask() throw();
				/** */
				void	sofDone() throw();
			};
		/** */
		class ControlListDetachDefTask {
			private:
				/** */
				Oscl::Memory::AlignedBlock
					<sizeof(DeferredTask<ControlListDetachDefTask>)>	_dtMem;
				/** */
				Driver&							_context;
				/** */
				Oscl::Usb::Alloc::Pipe::
				Message::Req::Api::FreeReq&		_msg;
				/** */
				unsigned						_currentFrameNumber;
			public:
				/** */
				ControlListDetachDefTask(	Driver&				context,
											Oscl::Usb::Alloc::
											Pipe::Message::
											Req::Api::FreeReq&	msg
											) throw();
				/** */
				void	queueSofTask() throw();
				/** */
				void	sofDone() throw();
			};

		/** */
		class BulkInListAttachDefTask {
			private:
				/** */
				Oscl::Memory::AlignedBlock
					<sizeof(DeferredTask<BulkInListAttachDefTask>)>	_dtMem;
				/** */
				Driver&							_context;
				/** */
				Oscl::Usb::Alloc::Pipe::
				Bulk::IN::Req::Api::NewReq&		_msg;
				/** */
				BulkInPipeRec&					_pipe;
				/** */
				unsigned						_currentFrameNumber;
			public:
				/** */
				BulkInListAttachDefTask(	Driver&				context,
											Oscl::Usb::Alloc::
											Pipe::Bulk::IN::
											Req::Api::NewReq&	msg,
											BulkInPipeRec&		pipe
											) throw();
				/** */
				void	queueSofTask() throw();
				/** */
				void	sofDone() throw();
			};
		/** */
		class BulkInListDetachDefTask {
			private:
				/** */
				Oscl::Memory::AlignedBlock
					<sizeof(DeferredTask<BulkInListDetachDefTask>)>	_dtMem;
				/** */
				Driver&							_context;
				/** */
				Oscl::Usb::Alloc::Pipe::
				Bulk::IN::Req::Api::FreeReq&	_msg;
				/** */
				unsigned						_currentFrameNumber;
			public:
				/** */
				BulkInListDetachDefTask(	Driver&				context,
											Oscl::Usb::Alloc::
											Pipe::Bulk::IN::
											Req::Api::FreeReq&	msg
											) throw();
				/** */
				void	queueSofTask() throw();
				/** */
				void	sofDone() throw();
			};

		/** */
		class BulkOutListAttachDefTask {
			private:
				/** */
				Oscl::Memory::AlignedBlock
					<sizeof(DeferredTask<BulkOutListAttachDefTask>)>	_dtMem;
				/** */
				Driver&							_context;
				/** */
				Oscl::Usb::Alloc::Pipe::
				Bulk::OUT::Req::Api::NewReq&	_msg;
				/** */
				BulkOutPipeRec&					_pipe;
				/** */
				unsigned						_currentFrameNumber;
			public:
				/** */
				BulkOutListAttachDefTask(	Driver&				context,
											Oscl::Usb::Alloc::
											Pipe::Bulk::OUT::
											Req::Api::NewReq&	msg,
											BulkOutPipeRec&		pipe
											) throw();
				/** */
				void	queueSofTask() throw();
				/** */
				void	sofDone() throw();
			};
		/** */
		class BulkOutListDetachDefTask {
			private:
				/** */
				Oscl::Memory::AlignedBlock
					<sizeof(DeferredTask<BulkOutListDetachDefTask>)>	_dtMem;
				/** */
				Driver&							_context;
				/** */
				Oscl::Usb::Alloc::Pipe::
				Bulk::OUT::Req::Api::FreeReq&	_msg;
				/** */
				unsigned						_currentFrameNumber;
			public:
				/** */
				BulkOutListDetachDefTask(	Driver&				context,
											Oscl::Usb::Alloc::
											Pipe::Bulk::OUT::
											Req::Api::FreeReq&	msg
											) throw();
				/** */
				void	queueSofTask() throw();
				/** */
				void	sofDone() throw();
			};

		/** */
		class InterruptListAttachDefTask {
			private:
				/** */
				Oscl::Memory::AlignedBlock
					<sizeof(DeferredTask<InterruptListAttachDefTask>)>	_dtMem;
				/** */
				Driver&							_context;
				/** */
				Oscl::Usb::Alloc::BW::
				Interrupt::Req::Api::
				ReserveReq&						_msg;
				/** */
				Oscl::Usb::Ohci::
				BW::Allocator&					_allocator;
				/** */
				unsigned						_currentFrameNumber;
			public:
				/** */
				InterruptListAttachDefTask(	Driver&				context,
											Oscl::Usb::Alloc::
											BW::Interrupt::
											Req::Api::
											ReserveReq&			msg,
											Oscl::Usb::Ohci::
											BW::Allocator&		allocator
											) throw();
				/** */
				void	queueSofTask() throw();
				/** */
				void	sofDone() throw();
			};
		/** */
		class InterruptListDetachDefTask {
			private:
				/** */
				Oscl::Memory::AlignedBlock
					<sizeof(DeferredTask<InterruptListDetachDefTask>)>	_dtMem;
				/** */
				Driver&							_context;
				/** */
				Oscl::Usb::Alloc::
				BW::Interrupt::
				Req::Api::FreeReq&				_msg;
				/** */
				Oscl::Usb::Ohci::
				BW::Allocator&					_allocator;
				/** */
				unsigned						_currentFrameNumber;
			public:
				/** */
				InterruptListDetachDefTask(	Driver&				context,
											Oscl::Usb::Alloc::
											BW::Interrupt::
											Req::Api::
											FreeReq&			msg,
											Oscl::Usb::Ohci::
											BW::Allocator&		allocator
											) throw();
				/** */
				void	queueSofTask() throw();
				/** */
				void	sofDone() throw();
			};

		/** */
		union DeferredTaskMem {
			/** */
			void*	__qitemlink;
			/** */
			Oscl::Memory::AlignedBlock
				<sizeof(ControlListAttachDefTask)>	_cladt;
			/** */
			Oscl::Memory::AlignedBlock
				<sizeof(ControlListDetachDefTask)>	_clddt;
			/** */
			Oscl::Memory::AlignedBlock
				<sizeof(InterruptListAttachDefTask)>	_iladt;
			/** */
			Oscl::Memory::AlignedBlock
				<sizeof(InterruptListDetachDefTask)>	_ilddt;
			};
		/** */
		typedef Oscl::Usb::Ohci::TD::Transfer<Driver>	Transfer;

	public:
		/** */
		union PipeMem {
			/** */
			void*	__qitemlink;
			/** */
			Oscl::Memory::AlignedBlock<sizeof(MessagePipeRec)>		_message;
			/** */
			Oscl::Memory::AlignedBlock<sizeof(InterruptInPipeRec)>	_iin;
			/** */
			Oscl::Memory::AlignedBlock<sizeof(BulkInPipeRec)>		_bulkin;
			/** */
			Oscl::Memory::AlignedBlock<sizeof(BulkOutPipeRec)>		_bulkout;
			};

	private:
		/** */
		enum{maxUsbDeviceAddresses=128};
	private:
		/** */
		Oscl::Usb::HCD::Enum::Monitor::Var			_enumMonitorVar;
		/** */
		Oscl::Memory::
		AlignedBlock<	sizeof(	Oscl::Usb::
								HCD::Enum::
								DefCntrlPipe::Var
								)
						>							_enumDcpVarMem;
		/** */
		Oscl::Usb::Ohci::BW::Config<6>				_bwAlloc;
		/** */
		Oscl::Usb::HCD::Enum::DefCntrlPipe::Var*	_enumDcpVar;
		/** */
		bool						_allocatedUsbAddr[maxUsbDeviceAddresses];
		/** */
		Oscl::Queue<PipeMem>			_freePipeMem;
		/** */
		Oscl::Queue<	Oscl::Usb::Alloc::
						Addr::Req::Api::
						UsbAddrResChangeReq
						>					_usbAddrChangeList;
		/** */
		Oscl::Queue<DeferredTaskMem>		_freeDtMem;
		/** */
		Oscl::Queue<Oscl::Mt::Itc::SrvMsg>	_pendingDeferredTaskReqList;
		/** */
		enum{nDeferredTasks=2};
		/** */
		DeferredTaskMem						_dtMem[nDeferredTasks];
		/** */
		Oscl::Usb::Ohci::Map&				_reg;
		/** */
		Register::Control					_control;
		/** */
		Register::CmdStatus					_cmdStatus;
		/** */
		Register::InterruptStatus			_irqStatus;
		/** */
		Register::InterruptEnable			_irqEnable;
		/** */
		Register::InterruptDisable			_irqDisable;
		/** */
		Oscl::Usb::Ohci::Hcca&				_hcca;
		/** */
		Oscl::Bus::Api&						_dmaBus;
		/** */
		Oscl::Mt::Itc::ServerApi&			_server;
		/** */
		bool								_open;
		/** */
		RootHub::Req::Api::OpenReq*			_rhOpenReq;
		/** */
		RootHub::Req::Api::CloseReq*		_rhCloseReq;
		/** */
		RootHub::Req::Api::ChangeReq*		_rhChangeReq;
		/** */
		RootHub::Req::Api::ConcreteSAP		_rhSAP;
		/** */
		Oscl::Usb::Alloc::Addr::
		Req::Api::ConcreteSAP				_hubPortSAP;
		/** */
		Oscl::Usb::Alloc::Pipe::
		Message::Req::Api::ConcreteSAP		_msgPipeAllocSAP;
		/** */
		Oscl::Usb::Alloc::Pipe::
		Interrupt::IN::Req::Api::
		ConcreteSAP							_intInPipeAllocSAP;
		/** */
		Oscl::Usb::Alloc::Pipe::
		Bulk::IN::Req::Api::
		ConcreteSAP							_bulkInPipeAllocSAP;
		/** */
		Oscl::Usb::Alloc::Pipe::
		Bulk::OUT::Req::Api::
		ConcreteSAP							_bulkOutPipeAllocSAP;
		/** */
		Oscl::Queue<	Oscl::Usb::Alloc::
						Addr::
						Req::Api::
						LockAddressZeroReq
						>					_addressZeroLockPendingQ;
		/** */
		bool								_addressZeroLocked;

		/** Always contains high and low speed EP descriptors for
			default address.
		 */
		Oscl::Usb::Ohci::ED::Message*		_controlEpListHead;
		/** */
		Oscl::Usb::Ohci::ED::Descriptor*	_bulkEpListHead;
		/** */
		MessagePipeRec*						_lsDefaultPipe;
		/** */
		MessagePipeRec*						_hsDefaultPipe;
		/** */
		Oscl::Queue<TransactionDescMem>		_freeTDList;
		/** */
		Oscl::Queue<EndpointDescMem>		_freeEDList;
		/** */
		Oscl::Queue<DeferredTaskApi>		_sofDeferredTaskList;
		/** */
		Oscl::Queue<DeferredTaskApi>		_tdMemDeferredTaskList;
		/** */
		Transfer							_unexpectedTransfer;
		/** */
		Oscl::Queue<MessagePipeRec>			_activeMessageEpList;
		/** */
		Oscl::Queue<InterruptInPipeRec>		_activeInterruptInPipeList;
		/** */
		Oscl::Queue<BulkInPipeRec>			_activeBulkInPipeList;
		/** */
		Oscl::Queue<BulkOutPipeRec>			_activeBulkOutPipeList;
		/** */
		Oscl::Queue<	Oscl::Usb::Alloc::
						Pipe::Message::Req::
						Api::NotifyChangeReq
						>					_msgPipeResChangeList;
		/** */
		Oscl::Queue<	Oscl::Usb::Alloc::
						Pipe::Interrupt::IN::
						Req::Api::
						NotifyChangeReq
						>					_intInPipeResChangeList;
		/** */
		Oscl::Queue<	Oscl::Usb::Alloc::
						Pipe::Bulk::IN::
						Req::Api::
						NotifyChangeReq
						>					_bulkInPipeResChangeList;
		/** */
		Oscl::Queue<	Oscl::Usb::Alloc::
						Pipe::Bulk::OUT::
						Req::Api::
						NotifyChangeReq
						>					_bulkOutPipeResChangeList;
		/** */
		unsigned long						_controlListDisableCount;
		/** */
		unsigned long						_periodicListDisableCount;
		/** */
		unsigned long						_bulkListDisableCount;
		/** _pipeMem[maxEndpoints] */
		PipeMem*							_pipeMem;

	public:
		/** */
		Driver(	Oscl::Usb::Ohci::Map&			reg,
				Oscl::Usb::Ohci::Hcca&			hcca,
				TransactionDescMem*				tdArray,
				unsigned						tdArraySize,
				EndpointDescMem*				edArray,
				unsigned						edArraySize,
				Oscl::Bus::Api&					dmaBus,
				Oscl::Mt::Itc::ServerApi&		server,
				PipeMem*						pipeMem,
				unsigned						maxEndpoints
				) throw();

		/** */
		~Driver();
		/** */
		RootHub::Req::Api::SAP&			getRootHubSAP() throw();
		/** */
		Oscl::Usb::Pipe::Message::
		Req::Api::SAP&					getLsEpSAP() throw();
		/** */
		Oscl::Usb::Pipe::Message::
		Req::Api::SAP&					getHsEpSAP() throw();
		/** */
		Oscl::Usb::Pipe::Message::
		SyncApi&						getLsSyncApi() throw();
		/** */
		Oscl::Usb::Pipe::Message::
		SyncApi&						getHsSyncApi() throw();
		/** */
		Oscl::Usb::Alloc::Addr::
		Req::Api::SAP&					getHubPortSAP() throw();
		/** */
		Oscl::Usb::Alloc::Pipe::
		Message::Req::Api::SAP&			getMsgPipeAllocSAP() throw();
		/** */
		Oscl::Usb::Alloc::Pipe::
		Interrupt::IN::Req::Api::SAP&	getInterruptInPipeAllocSAP() throw();
		/** */
		Oscl::Usb::Alloc::Pipe::
		Bulk::IN::Req::Api::SAP&		getBulkInPipeAllocSAP() throw();
		/** */
		Oscl::Usb::Alloc::Pipe::
		Bulk::OUT::Req::Api::SAP&		getBulkOutPipeAllocSAP() throw();

		/** */
		Oscl::Usb::HCD::Enum::
		DefCntrlPipe::Api&				getEnumDefCntrlPipeApi() throw();

		/** */
		Oscl::Usb::HCD::Enum::
		Monitor::Api&					getEnumMonitorApi() throw();
	private:
		/** Return address zero if allocation fails */
		Oscl::Usb::Address	allocateAddress() throw();
		/** */
		void	freeAddress(Oscl::Usb::Address address) throw();
		/** */
		bool	usbAddressAvailable() throw();

	private:
		friend class ControlListAttachDefTask;
		friend class ControlListDetachDefTask;
		friend class InterruptListAttachDefTask;
		friend class InterruptListDetachDefTask;
		friend class BulkInListAttachDefTask;
		friend class BulkInListDetachDefTask;
		friend class BulkOutListAttachDefTask;
		friend class BulkOutListDetachDefTask;
		/** */
		void	insertToControlList(Oscl::Usb::Ohci::ED::Message& ed) throw();
		/** */
		void	removeFromControlList(Oscl::Usb::Ohci::ED::Message& ed) throw();
		/** */
		void	insertToBulkList(Oscl::Usb::Ohci::ED::Descriptor& ed) throw();
		/** */
		void	removeFromBulkList(Oscl::Usb::Ohci::ED::Descriptor& ed) throw();
		/** */
		DeferredTaskMem*	allocDeferredTaskMem() throw();
		/** */
		void				free(DeferredTaskMem* mem) throw();
		/** */
		void				release(ControlListAttachDefTask& dt) throw();
		/** */
		void				release(ControlListDetachDefTask& dt) throw();
		/** */
		void				release(InterruptListAttachDefTask& dt) throw();
		/** */
		void				release(InterruptListDetachDefTask& dt) throw();
		/** */
		void				release(BulkInListAttachDefTask& dt) throw();
		/** */
		void				release(BulkInListDetachDefTask& dt) throw();
		/** */
		void				release(BulkOutListAttachDefTask& dt) throw();
		/** */
		void				release(BulkOutListDetachDefTask& dt) throw();
		/** */
		MessagePipeRec*		removeActiveMessageEp(	Oscl::Usb::Pipe::
													Message::PipeApi&	pipe
													) throw();
		/** */
		InterruptInPipeRec*
							removeActiveInterruptInPipe(	Oscl::Usb::Pipe::
															Interrupt::IN::
															PipeApi&		pipe
															) throw();
		/** */
		BulkInPipeRec*
							removeActiveBulkInPipe(	Oscl::Usb::Pipe::
													Bulk::IN::
													PipeApi&		pipe
													) throw();
		/** */
		BulkOutPipeRec*
							removeActiveBulkOutPipe(	Oscl::Usb::Pipe::
														Bulk::OUT::
														PipeApi&		pipe
														) throw();
		/** */
		void				disableControlList() throw();
		/** */
		void				enableControlList() throw();
		/** */
		void				disablePeriodicList() throw();
		/** */
		void				enablePeriodicList() throw();
		/** */
		void				disableBulkList() throw();
		/** */
		void				enableBulkList() throw();

	private:
		/** */
		void	scheduleOverrun() throw();
		/** */
		void	writebackDoneHead(Unsigned32 doneHead) throw();
		/** */
		void	startOfFrame() throw();
		/** */
		void	resumeDetected() throw();
		/** */
		void	unrecoverableError() throw();
		/** */
		void	frameNumberOverflow() throw();
		/** */
		void	rootHubStatusChange() throw();
		/** */
		void	ownershipChange() throw();

	private:
		/** */
		MessagePipeRec*
				allocMessagePipe(	Address		usbAddress,
									EndpointID	endpointID,
									unsigned	maxPacketSize,
									bool		lowSpeedDevice
									) throw();
		/** */
		InterruptInPipeRec*
				allocInterruptInPipe(	ED::Descriptor&	ed,
										Address			usbAddress,
										EndpointID		endpointID,
										unsigned		maxPacketSize,
										bool			lowSpeedDevice
										) throw();
		/** */
		BulkInPipeRec*
				allocBulkInPipe(	Address			usbAddress,
									EndpointID		endpointID,
									unsigned		maxPacketSize,
									bool			lowSpeedDevice
									) throw();
		/** */
		BulkOutPipeRec*
				allocBulkOutPipe(	Address			usbAddress,
									EndpointID		endpointID,
									unsigned		maxPacketSize,
									bool			lowSpeedDevice
									) throw();
		/** */
		void	free(MessagePipeRec& pipe) throw();
		/** */
		void	free(InterruptInPipeRec& pipe) throw();
		/** */
		void	free(BulkInPipeRec& pipe) throw();
		/** */
		void	free(BulkOutPipeRec& pipe) throw();
		/** */
		void	notifyPipeResourceAvailable() throw();

	public: // Oscl::Mt::Itc::Srv::Close::Req::Api
		/** */
		void	request(	Oscl::Mt::Itc::Srv::
							Open::Req::Api::OpenReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Mt::Itc::Srv::
							Close::Req::Api::CloseReq&	msg
							) throw();

	public: // Oscl::Usb::Alloc::Addr::Req::Api
		/** */
		void	request(	Oscl::Usb::Alloc::Addr::
							Req::Api::LockAddressZeroReq&		msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Addr::
							Req::Api::UnlockAddressZeroReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Addr::
							Req::Api::CancelAddrZeroLockReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Addr::
							Req::Api::AllocateNewAddressReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Addr::
							Req::Api::FreeAddressReq&			msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Addr::
							Req::Api::UsbAddrResChangeReq&		msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Addr::
							Req::Api::CancelUsbAddrResChangeReq&	msg
							) throw();

	public:	//Oscl::Usb::Alloc::Pipe::Message::Req::Api
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Message::Req::Api::NewReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Message::Req::Api::FreeReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Message::Req::Api::NotifyChangeReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Message::Req::Api::CancelChangeReq&	msg
							) throw();

	public:	//Oscl::Usb::Alloc::Pipe::Interrupt::IN::Req::Api
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Interrupt::IN::Req::Api::NewReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Interrupt::IN::Req::Api::FreeReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Interrupt::IN::Req::Api::NotifyChangeReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Interrupt::IN::Req::Api::CancelChangeReq&	msg
							) throw();

	public:	//Oscl::Usb::Alloc::Pipe::Bulk::IN::Req::Api
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Bulk::IN::Req::Api::NewReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Bulk::IN::Req::Api::FreeReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Bulk::IN::Req::Api::NotifyChangeReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Bulk::IN::Req::Api::CancelChangeReq&	msg
							) throw();

	public:	//Oscl::Usb::Alloc::Pipe::Bulk::OUT::Req::Api
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Bulk::OUT::Req::Api::NewReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Bulk::OUT::Req::Api::FreeReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Bulk::OUT::Req::Api::NotifyChangeReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Usb::Alloc::Pipe::
							Bulk::OUT::Req::Api::CancelChangeReq&	msg
							) throw();

	private:	// Oscl::Interrupt::StatusHandlerApi
		/** */
		bool	interrupt() throw();

	private:	// Oscl::Interrupt::IsrDsrApi
		/** */
		bool	isr() throw();
		/** */
		void	dsr() throw();

	private:	// Oscl::Mt::Itc::Server
		/** */
		void	initialize() throw();

	private:	// RootHub::Req::Api
		/** */
		void	request(RootHub::Req::Api::OpenReq& msg) throw();
		/** */
		void	request(RootHub::Req::Api::CloseReq& msg) throw();
		/** */
		void	request(RootHub::Req::Api::ChangeReq& msg) throw();
		/** */
		void	request(RootHub::Req::Api::CancelAllReq& msg) throw();

	private:	// Oscl::Usb::Alloc::BW::Interrupt::Req::Api
		/** */
		void	request(	Alloc::BW::Interrupt::
							Req::Api::ReserveReq&	msg
							) throw();
		/** */
		void	request(	Alloc::BW::Interrupt::
							Req::Api::FreeReq&		msg
							) throw();

	private:	// Oscl::Usb::Alloc::BW::Req::Api
		/** */
		void	request(	Alloc::BW::
							Req::Api::ChangeReq&	msg
							) throw();
		/** */
		void	request(	Alloc::BW::
							Req::Api::CancelReq&	msg
							) throw();

	private:	// HcdApi
		/** */
		TransactionDescMem*	allocTdMem() throw();
		/** */
		void		free(TransactionDescMem* m) throw();
		/** */
		void		free(Oscl::Usb::Ohci::TD::Descriptor& d) throw();
		/** */
		void		queueSofTask(DeferredTaskApi& dt) throw();
		/** */
		void		cancelSofTask(DeferredTaskApi& dt) throw();
		/** */
		void		queueTdMemTask(DeferredTaskApi& dt) throw();
		/** */
		void		cancelTdMemTask(DeferredTaskApi& dt) throw();
		/** */
		void		controlListFilled() throw();
		/** */
		void		bulkListFilled() throw();
		/** */
		unsigned    getCurrentFrameNumber() throw();
		/** */
		Oscl::Bus::Api&	getDmaBusApi() throw();

	private:
		/** */
		Oscl::Usb::Ohci::
		TD::TransferApi&	getUnexpectedTransferVector() throw();
		/** */
		EndpointDescMem*	allocEdMem() throw();
		/** */
		void				free(EndpointDescMem* m) throw();
	private:
		/** */
		void	unexpectedDone(Oscl::Usb::Ohci::TD::Descriptor& desc) throw();
	private:	// ServerApi::SignaledApi
		/** */
		void	signaled() throw();
	};

}
}
}
}


#endif
