/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_drv_usb_ohci_hcd_server_epapih_
#define _oscl_drv_usb_ohci_hcd_server_epapih_
#include "oscl/queue/queue.h"
#include "defer.h"
#include "tdmem.h"
#include "oscl/bus/api.h"

/** */
namespace Oscl {
/** */
namespace Usb {
/** */
namespace Ohci {
/** */
namespace HCD {

/** This interface describes the operations availabe to
	endpoint services to the core HCD. The operations
	provided by this interface are serialized with all
	other HCD operations.
 */
class HcdApi {
	public:
		/** */
		virtual TransactionDescMem*	allocTdMem() throw()=0;
		/** */
		virtual void		free(TransactionDescMem*) throw()=0;
		/** */
		virtual void		free(Oscl::Usb::Ohci::TD::Descriptor& d) throw()=0;
		/** */
		virtual void		queueSofTask(DeferredTaskApi& dt) throw()=0;
		/** */
		virtual void		cancelSofTask(DeferredTaskApi& dt) throw()=0;
		/** */
		virtual void		queueTdMemTask(DeferredTaskApi& dt) throw()=0;
		/** */
		virtual void		cancelTdMemTask(DeferredTaskApi& dt) throw()=0;
		/** */
		virtual void		controlListFilled() throw()=0;
		/** */
		virtual void		bulkListFilled() throw()=0;
		/** */
		virtual unsigned	getCurrentFrameNumber() throw()=0;
		/** */
		virtual Oscl::Bus::Api&	getDmaBusApi() throw()=0;
	};

}
}
}
}


#endif
