/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_drv_usb_ohci_hub_root_porth_
#define _oscl_drv_usb_ohci_hub_root_porth_
#include "oscl/hw/usb/ohci/reg.h"
#include "oscl/driver/usb/ohci/reg/rhportstat.h"
#include "oscl/queue/queueitem.h"
#include "oscl/queue/queue.h"
#include "oscl/memory/block.h"
#include "oscl/driver/usb/hub/port/reqapi.h"

/** */
namespace Oscl {
/** */
namespace Usb {
/** */
namespace Ohci {
/** */
namespace Hub {
/** */
namespace Root {

/** */
class Port :	public Oscl::Usb::Hub::Port::Req::Api,
				public Oscl::QueueItem
				{
	private:
		/** */
		Oscl::Usb::Ohci::Register::RootHubPortStatus		_portStatus;
		/** */
		Oscl::Usb::Hub::Port::Req::Api::ConcreteSAP			_hubPortSAP;
		/** */
		Oscl::Queue<NotifyPortChangeReq>					_notificationList;
		/** */
		Oscl::BitField<	Oscl::Usb::Hw::Hub::Port::
						wPortChange::Reg
						>									_portChangeStatus;

	public:
		/** */
		Port(	volatile Oscl::Usb::Ohci::
				RhPortStatus::Reg&				reg,
				Oscl::Mt::Itc::PostMsgApi&		myPapi
				) throw();

		/** */
		virtual ~Port(){}

		/** */
		void	initialize() throw();

		/** */
		void	statusChange() throw();

		/** */
		Oscl::Usb::Hub::Port::Req::Api::SAP&	getHubPortSAP() throw();

	private:	// Oscl::Usb::Hub::Port::Req::Api
		/** */
		void	request(SetResetReq& msg) throw();
		/** */
		void	request(SetSuspendReq& msg) throw();
		/** */
		void	request(SetPowerReq& msg) throw();
		/** */
		void	request(ClearEnableReq& msg) throw();
		/** */
		void	request(ClearSuspendReq& msg) throw();
		/** */
		void	request(ClearPowerReq& msg) throw();
		/** */
		void	request(ClearConnectChangeReq& msg) throw();
		/** */
		void	request(ClearResetChangeReq& msg) throw();
		/** */
		void	request(ClearEnableChangeReq& msg) throw();
		/** */
		void	request(ClearSuspendChangeReq& msg) throw();
		/** */
		void	request(ClearOverCurrentChangeReq& msg) throw();
		/** */
		void	request(CancelReq& msg) throw();
		/** */
		void	request(NotifyPortChangeReq& msg) throw();
		/** */
		void	request(CancelNotifyPortChangeReq& msg) throw();
		/** */
		void	request(GetPortStatusReq& msg) throw();
	};

}
}
}
}
}


#endif
