/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_drv_usb_ohci_reg_controlh_
#define _oscl_drv_usb_ohci_reg_controlh_
#include "oscl/hw/usb/ohci/reg.h"
#include "oscl/bits/bitfield.h"

/** */
namespace Oscl {
/** */
namespace Usb {
/** */
namespace Ohci {
/** */
namespace Register {

/** */
class Control {
	public:
		typedef enum {
			Reset = Oscl::Usb::Ohci::Control::HCFS::Value_Reset,
			Resume = Oscl::Usb::Ohci::Control::HCFS::Value_Resume,
			Operational = Oscl::Usb::Ohci::Control::HCFS::Value_Operational,
			Suspend = Oscl::Usb::Ohci::Control::HCFS::Value_Suspend
			} UsbState;
		typedef enum {
			OneToOne = Oscl::Usb::Ohci::Control::CBSR::Value_OneToOne,
			TwoToOne = Oscl::Usb::Ohci::Control::CBSR::Value_TwoToOne,
			ThreeToOne = Oscl::Usb::Ohci::Control::CBSR::Value_ThreeToOne,
			FourToOne = Oscl::Usb::Ohci::Control::CBSR::Value_ThreeToOne
			} BulkServiceRatio;
	private:
		/** */
		Oscl::BitField<Oscl::Usb::Ohci::Control::Reg>&	_reg;

	public:
		/** */
		Control(volatile Oscl::Usb::Ohci::Control::Reg& reg) throw();
	public:	// CBSR
		/** */
		BulkServiceRatio	controlBulkServiceRatio() throw();
		/** */
		void		controlBulkServiceOneToOne() throw();
		/** */
		void		controlBulkServiceTwoToOne() throw();
		/** */
		void		controlBulkServiceThreeToOne() throw();
		/** */
		void		controlBulkServiceFourToOne() throw();

	public:	// PLE
		/** */
		bool		periodicListEnabled() throw();
		/** */
		void		enablePeriodicList() throw();
		/** */
		void		disablePeriodicList() throw();

	public:	// IE
		/** */
		bool		isochronousEnabled() throw();
		/** */
		void		enableIsochronous() throw();
		/** */
		void		disableIsochronous() throw();

	public:	// CLE
		/** */
		bool		controlListEnabled() throw();
		/** */
		void		enableControlList() throw();
		/** */
		void		disableControlList() throw();

	public:	// BLE
		/** */
		bool		bulkListEnabled() throw();
		/** */
		void		enableBulkList() throw();
		/** */
		void		disableBulkList() throw();

	public:	// HCFS
		/** */
		UsbState	state() throw();
		/** */
		void		usbReset() throw();
		/** */
		void		usbResume() throw();
		/** */
		void		usbOperational() throw();
		/** */
		void		usbSuspend() throw();

	public:	// IR
		/** */
		bool		interruptsRoutedToSystemManagement() throw();
		/** */
		void		routeInterruptsNormal() throw();
		/** */
		void		routeInterruptsToSystemManagement() throw();

	public:	// RWC
		/** */
		bool		remoteWakupConntected() throw();
		/** */
		void		indicateRemoteWakeupConnected() throw();
		/** */
		void		indicateRemoteWakeupNotConnected() throw();

	public:	// RWE
		/** */
		bool		remoteWakupEnabled() throw();
		/** */
		void		enableRemoteWakeup() throw();
		/** */
		void		disableRemoteWakeup() throw();
	};

}
}
}
}
#endif
