/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "reqapi.h"

using namespace Oscl::Usb::Pipe::Message;

Req::Api::CancelPayload::
CancelPayload(Oscl::Mt::Itc::SrvMsg& irpToCancel) throw():
		_irpToCancel(irpToCancel){
	}

Req::Api::NoDataPayload::
NoDataPayload(const Oscl::Usb::Setup::Packet& setupPkt) throw():
		_setupPkt(setupPkt),
		_failed(false)
		{
	}

const Oscl::Usb::Setup::Packet*
Req::Api::NoDataPayload::getSetupBuffer() const throw(){
	return &_setupPkt;
	}

Req::Api::ReadPayload::ReadPayload(	const Oscl::Usb::Setup::Packet&	setupPkt,
									void*							buffer
									) throw():
		_setupPkt(setupPkt),
		_buffer(buffer),
		_failed(false)
		{
	}

const Oscl::Usb::Setup::Packet*
Req::Api::ReadPayload::getSetupBuffer() const throw(){
	return &_setupPkt;
	}

void*	Req::Api::ReadPayload::getDataBuffer() throw(){
	return _buffer;
	}
	
Req::Api::WritePayload::
WritePayload(	const Oscl::Usb::Setup::Packet&	setupPkt,
				const void*				writeData
				) throw():
		_setupPkt(setupPkt),
		_writeData(writeData),
		_failed(false)
		{
	}

const Oscl::Usb::Setup::Packet*
Req::Api::WritePayload::getSetupBuffer() const throw(){
	return &_setupPkt;
	}

const void*	Req::Api::WritePayload::getWriteDataBuffer() const throw(){
	return _writeData;
	}

