/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_drv_usb_pipe_message_reqapih_
#define _oscl_drv_usb_pipe_message_reqapih_
#include "oscl/driver/usb/setup/packet.h"
#include "oscl/mt/itc/mbox/srvreq.h"
#include "oscl/mt/itc/mbox/sap.h"

/** */
namespace Oscl {
/** */
namespace Usb {
/** */
namespace Pipe {

/** */
namespace Status {
/** */
class Result;
}

/** */
namespace Message {

/** */
namespace Req {

using namespace Oscl::Mt::Itc;
using namespace Oscl::Usb::Hw::Setup;

/** */
class Api {
	public:
		/** */
		class CancelPayload {
			public:
				/** */
				Oscl::Mt::Itc::SrvMsg&	_irpToCancel;
			public:
				/** */
				CancelPayload(Oscl::Mt::Itc::SrvMsg& irpToCancel) throw();
			};
		/** */
		class NoDataPayload {
			private:
				/** Must be cache aligned and in a DMA memory space. */
				const Oscl::Usb::Setup::Packet&	_setupPkt;
			public:
				/** */
				const Status::Result*			_failed;
			public:
				/** */
				NoDataPayload(const Oscl::Usb::Setup::Packet& setupPkt) throw();
				/** */
				const Oscl::Usb::Setup::Packet*	getSetupBuffer() const throw();
			};
		/** */
		class ReadPayload {
			private:
				/** Must be cache aligned and in a DMA memory space. */
				const Oscl::Usb::Setup::Packet&	_setupPkt;

				/** Must be cache aligned and in a DMA memory space. */
				void*					_buffer;
			public:
				/** */
				const Status::Result*	_failed;
			public:
				/** */
				ReadPayload(	const Oscl::Usb::Setup::Packet&	setupPkt,
								void*					buffer
								) throw();
				/** */
				const Oscl::Usb::Setup::Packet*	getSetupBuffer() const throw();
				/** */
				void*	getDataBuffer() throw();
			};
		/** */
		class WritePayload {
			private:
				/** Must be cache aligned and in a DMA memory space. */
				const Oscl::Usb::Setup::Packet&	_setupPkt;
				/** Must be cache aligned and in a DMA memory space. */
				const void*				_writeData;
			public:
				/** */
				const Status::Result*	_failed;
			public:
				/** */
				WritePayload(	const Oscl::Usb::Setup::Packet&	setupPkt,
								const void*				writeData
								) throw();
				/** */
				const Oscl::Usb::Setup::Packet*	getSetupBuffer() const throw();
				/** */
				const void*	getWriteDataBuffer() const throw();
			};
	public:
		/** */
		typedef SrvRequest<Api,CancelPayload>	CancelReq;
		/** */
		typedef SrvRequest<Api,NoDataPayload>	NoDataReq;
		/** */
		typedef SrvRequest<Api,ReadPayload>		ReadReq;
		/** */
		typedef SrvRequest<Api,WritePayload>		WriteReq;

	public:
		/** */
		typedef SAP<Api>				SAP;
		/** */
		typedef ConcreteSAP<Api>		ConcreteSAP;

	public:
		/** */
		virtual void	request(CancelReq& msg) throw()=0;
		/** */
		virtual void	request(NoDataReq& msg) throw()=0;
		/** */
		virtual void	request(ReadReq& msg) throw()=0;
		/** */
		virtual void	request(WriteReq& msg) throw()=0;
	};

}
}
}
}
}

#endif
