/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "status.h"
#include "handler.h"

namespace Oscl {
namespace Usb {
namespace Pipe {
namespace Status {

class PipeClosed : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

class Canceled : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

class CRC : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

class BitStuffing : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

class DataToggleMismatch : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

class Stall : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

class DeviceNotResponding : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

class PidCheckFailure : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

class UnexpectedPID : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

class DataOverrun : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

class DataUnderrun : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

class BufferOverrun : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

class BufferUnderrun : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	Canceled::query(Handler& handler) const throw(){
	handler.canceled();
	}

void	PipeClosed::query(Handler& handler) const throw(){
	handler.pipeClosed();
	}

void	CRC::query(Handler& handler) const throw(){
	handler.crc();
	}

void	BitStuffing::query(Handler& handler) const throw(){
	handler.bitStuffing();
	}

void	DataToggleMismatch::query(Handler& handler) const throw(){
	handler.dataToggleMismatch();
	}

void	Stall::query(Handler& handler) const throw(){
	handler.stall();
	}

void	DeviceNotResponding::query(Handler& handler) const throw(){
	handler.deviceNotResponding();
	}

void	PidCheckFailure::query(Handler& handler) const throw(){
	handler.pidCheckFailure();
	}

void	UnexpectedPID::query(Handler& handler) const throw(){
	handler.unexpectedPID();
	}

void	DataOverrun::query(Handler& handler) const throw(){
	handler.dataOverrun();
	}

void	DataUnderrun::query(Handler& handler) const throw(){
	handler.dataUnderrun();
	}

void	BufferOverrun::query(Handler& handler) const throw(){
	handler.bufferOverrun();
	}

void	BufferUnderrun::query(Handler& handler) const throw(){
	handler.bufferUnderrun();
	}

const Result&	getCanceled() throw(){
	static const Canceled		canceled;
	return canceled;
	}

const Result&	getPipeClosed() throw(){
	static const PipeClosed		pipeClosed;
	return pipeClosed;
	}

const Result&	getCRC() throw(){
	static const CRC		crc;
	return crc;
	}

const Result&	getBitStuffing() throw(){
	static const BitStuffing	bitStuffing;
	return bitStuffing;
	}

const Result&	getDataToggleMismatch() throw(){
	static const DataToggleMismatch	dataToggleMismatch;
	return dataToggleMismatch;
	}

const Result&	getStall() throw(){
	static const Stall	stall;
	return stall;
	}

const Result&	getDeviceNotResponding() throw(){
	static const DeviceNotResponding	deviceNotResponding;
	return deviceNotResponding;
	}

const Result&	getPidCheckFailure() throw(){
	static const PidCheckFailure	pidCheckFailure;
	return pidCheckFailure;
	}

const Result&	getUnexpectedPID() throw(){
	static const UnexpectedPID	unexpectedPID;
	return unexpectedPID;
	}

const Result&	getDataOverrun() throw(){
	static const DataOverrun	dataOverrun;
	return dataOverrun;
	}

const Result&	getDataUnderrun() throw(){
	static const DataUnderrun	dataUnderrun;
	return dataUnderrun;
	}

const Result&	getBufferOverrun() throw(){
	static const BufferOverrun	bufferOverrun;
	return bufferOverrun;
	}

const Result&	getBufferUnderrun() throw(){
	static const BufferUnderrun	bufferUnderrun;
	return bufferUnderrun;
	}

}
}
}
}

