/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_drv_usb_tt_bo_driverh_
#define _oscl_drv_usb_tt_bo_driverh_
#include "oscl/memory/block.h"
#include "oscl/queue/queue.h"
#include "oscl/mt/itc/srv/close.h"
#include "oscl/driver/usb/pipe/message/pipeapi.h"
#include "oscl/driver/usb/pipe/bulk/in/respmem.h"
#include "oscl/driver/usb/pipe/bulk/out/respmem.h"
#include "oscl/driver/usb/pipe/message/respmem.h"
#include "oscl/driver/usb/setup/mem.h"
#include "oscl/tt/itc/sync.h"
#include "fsm.h"
#include "oscl/driver/usb/pipe/handler.h"

/** */
namespace Oscl {
/** */
namespace Usb {
/** */
namespace TT {
/** */
namespace BO {

/** This implementation of the Oscl::TT::ITC::Req::Api serializes
	all requests and adapts them as specified for USB Transparent
	Transport using the Bulk Only method.
 */
class Driver :	public Oscl::Mt::Itc::Srv::CloseSync,
				private Oscl::Mt::Itc::Srv::Close::Req::Api,
				private Oscl::Usb::Pipe::Bulk::OUT::Resp::Api,
				private Oscl::Usb::Pipe::Bulk::IN::Resp::Api,
				private Oscl::Usb::Pipe::Message::Resp::Api,
				private Oscl::TT::ITC::Req::Api,
				private Oscl::Usb::TT::BO::StateVar::ContextApi
				{
	private:
		/** */
		class GetMaxLunResult : public Oscl::Usb::Pipe::Status::Handler {
			public:
				/** */
				bool	_failed;
			public:
				/** */
				GetMaxLunResult() throw();
			private:
				/** */
				void	pipeClosed() throw();
				/** */
				void	canceled() throw();
				/** */
				void	crc() throw();
				/** */
				void	bitStuffing() throw();
				/** */
				void	dataToggleMismatch() throw();
				/** */
				void	stall() throw();
				/** */
				void	deviceNotResponding() throw();
				/** */
				void	pidCheckFailure() throw();
				/** */
				void	unexpectedPID() throw();
				/** */
				void	dataOverrun() throw();
				/** */
				void	dataUnderrun() throw();
				/** */
				void	bufferOverrun() throw();
				/** */
				void	bufferUnderrun() throw();
			};
		/** */
		class BulkInStatusResult : public Oscl::Usb::Pipe::Status::Handler {
			public:
				/** */
				bool	_failed;
			public:
				/** */
				BulkInStatusResult() throw();
			private:
				/** */
				void	pipeClosed() throw();
				/** */
				void	canceled() throw();
				/** */
				void	crc() throw();
				/** */
				void	bitStuffing() throw();
				/** */
				void	dataToggleMismatch() throw();
				/** */
				void	stall() throw();
				/** */
				void	deviceNotResponding() throw();
				/** */
				void	pidCheckFailure() throw();
				/** */
				void	unexpectedPID() throw();
				/** */
				void	dataOverrun() throw();
				/** */
				void	dataUnderrun() throw();
				/** */
				void	bufferOverrun() throw();
				/** */
				void	bufferUnderrun() throw();
			};
		/** */
		class BulkInReadResult : public Oscl::Usb::Pipe::Status::Handler {
			public:
				/** */
				bool	_failed;
			public:
				/** */
				BulkInReadResult() throw();
			private:
				/** */
				void	pipeClosed() throw();
				/** */
				void	canceled() throw();
				/** */
				void	crc() throw();
				/** */
				void	bitStuffing() throw();
				/** */
				void	dataToggleMismatch() throw();
				/** */
				void	stall() throw();
				/** */
				void	deviceNotResponding() throw();
				/** */
				void	pidCheckFailure() throw();
				/** */
				void	unexpectedPID() throw();
				/** */
				void	dataOverrun() throw();
				/** */
				void	dataUnderrun() throw();
				/** */
				void	bufferOverrun() throw();
				/** */
				void	bufferUnderrun() throw();
			};
		/** Returns _retry == true if the result is one
			that should be retried according to the USB
			Mass Storage spec.
		 */
		class BulkInStatusRetry : public Oscl::Usb::Pipe::Status::Handler {
			public:
				/** */
				bool	_retry;
			public:
				/** */
				BulkInStatusRetry() throw();
			private:
				/** */
				void	pipeClosed() throw();
				/** */
				void	canceled() throw();
				/** */
				void	crc() throw();
				/** */
				void	bitStuffing() throw();
				/** */
				void	dataToggleMismatch() throw();
				/** */
				void	stall() throw();
				/** */
				void	deviceNotResponding() throw();
				/** */
				void	pidCheckFailure() throw();
				/** */
				void	unexpectedPID() throw();
				/** */
				void	dataOverrun() throw();
				/** */
				void	dataUnderrun() throw();
				/** */
				void	bufferOverrun() throw();
				/** */
				void	bufferUnderrun() throw();
			};
	public:
		/** */
		struct CBW {
			/** */
			unsigned char	dCBWSignature[4];
			/** */
			unsigned char	dCBWTag[4];
			/** */
			unsigned char	dCBWDataTransferLength[4];
			/** */
			unsigned char	bCBWFlags;
			/** */
			unsigned char	bCBWLUN;
			/** */
			unsigned char	bCBWCBLength;
			/** */
			unsigned char	CBWCB[16];
			};
		/** */
		struct CSW {
			/** */
			unsigned char	dCSWSignature[4];
			/** */
			unsigned char	dCSWTag[4];
			/** */
			unsigned char	dCSWDataResidue[4];
			/** */
			unsigned char	dCSWStatus;
			};
		/** */
		struct SetupMem {
			/** */
			Oscl::Usb::Setup::Mem	controlSetup;
			};
		/** */
		struct ControlPacketMem {
			/** */
			Oscl::Memory::AlignedBlock<16>	controlBytes;
			};
		/** */
		union ContextPacketMem {
			/** */
			ControlPacketMem	control;
			};
		/** */
		enum{maxBulkRxPacketBytes=2*0x40};
		/** */
		struct BulkRxPacketMem {
			/** */
			Oscl::Memory::AlignedBlock<maxBulkRxPacketBytes>	mem;
			};
		/** */
		enum{maxBulkTxPacketBytes=1024};
		/** */
		union BulkTxPacketMem {
			/** */
			void*		__qitemlink;
			/** */
			Oscl::Memory::AlignedBlock<maxBulkTxPacketBytes>	mem;
			};
		/** */
		enum{nRxBuffers=2};
		/** */
		union PacketMem {
			/** */
			ControlPacketMem				control;
			/** */
			Oscl::Memory::AlignedBlock<31>	cbw;
			/** */
			Oscl::Memory::AlignedBlock<13>	csw;
			/** */
			Oscl::Memory::AlignedBlock<1>	getMaxLun;
			};
		/** */
		union CancelMem {
			/** */
			Oscl::Usb::Pipe::Bulk::
			IN::Resp::CancelMem					bulkInPipe;
			/** */
			Oscl::Usb::Pipe::Bulk::
			OUT::Resp::CancelMem				bulkOutPipe;
			/** */
			Oscl::Usb::Pipe::Message::
			Resp::CancelMem						messagePipe;
			};

	private:
		/** Message resources for bulk I/O. A union is
			used assuming that this implementation only
			has a single bulk transaction in progress
			at a time. Thats the plan since this is to
			be a simple implementation.
		 */
		union {
			/** */
			Oscl::Usb::Pipe::
			Bulk::IN::Resp::ReadMem				bulkIn;
			/** */
			Oscl::Usb::Pipe::
			Bulk::OUT::Resp::WriteMem			bulkOut;
			/** */
			Oscl::Usb::Pipe::
			Message::Resp::NoDataMem			controlNoData;
			/** */
			Oscl::Usb::Pipe::
			Message::Resp::ReadMem				controlRead;
			/** */
			Oscl::Usb::Pipe::
			Message::Resp::WriteMem				controlWrite;
			} _pipeMem;

		/** TT requests that are queued/serialized
			for execution while the pending bulk
			transaction is executing.
		 */
		Oscl::Queue<Oscl::Mt::Itc::SrvMsg>	_pendingReqs;

		/** Memory used to cancel USB bulk and control
			requests.
		 */
		CancelMem							_cancelMem;

		/** */
		Oscl::Usb::Pipe::
		Message::PipeApi&					_controlPipe;

		/** */
		const Unsigned8						_bInterfaceNumber;

		/** */
		const Unsigned8						_bulkInEndpointID;

		/** */
		const Unsigned8						_bulkOutEndpointID;

		/** */
		Oscl::Usb::Pipe::Bulk::
		IN::Req::Api::SAP&					_bulkInPipe;

		/** */
		Oscl::Usb::Pipe::Bulk::
		OUT::Req::Api::SAP&					_bulkOutPipe;

		/** */
		SetupMem&							_setupMem;

		/** */
		PacketMem&							_packetMem;

		/** */
		Oscl::Mt::Itc::PostMsgApi&			_myPapi;

		/** */
		Oscl::TT::ITC::Sync					_sync;

		/** */
		unsigned char						_maxBulkInPacketSize;

		/** */
		unsigned char						_maxBulkOutPacketSize;

		/** */
		Oscl::Mt::Itc::Srv::Open::
		Req::Api::OpenReq*					_openReq;

		/** */
		Oscl::Mt::Itc::Srv::Close::
		Req::Api::CloseReq*					_closeReq;

		/** */
		Oscl::Usb::Pipe::Bulk::
		OUT::Resp::Api::WriteResp*			_currentWrite;

		/** */
		Oscl::Usb::Pipe::Bulk::
		IN::Resp::Api::ReadResp*			_currentRead;

		/** */
		Oscl::TT::ITC::Req::Api::CancelReq*	_currentCancelReq;

		/** */
		Oscl::Usb::TT::BO::StateVar			_state;

		/** */
		Oscl::TT::ITC::
		Req::Api::CancelReq*				_cancelReq;

		/** */
		union Requests {
			/** */
			Oscl::Mt::Itc::SrvMsg*	generic;
			/** */
			Oscl::TT::ITC::Req::Api::ResetReq*				reset;
			/** */
			Oscl::TT::ITC::Req::Api::GetMaxLunReq*			getMaxLun;
			/** */
			Oscl::TT::ITC::Req::Api::ReadReq*				read;
			/** */
			Oscl::TT::ITC::Req::Api::WriteReq*				write;
			/** */
			Oscl::TT::ITC::Req::Api::CommandReq*			command;
			};

		/** The "just received" request */
		Requests							_incomming;

		/** The currently executing request */
		Requests							_current;

		/** */
		union {
			/** */
			Oscl::Mt::Itc::SrvMsg*							generic;
			/** */
			Oscl::Usb::Pipe::Bulk::
			OUT::Resp::Api::WriteResp*						bulkWrite;
			/** */
			Oscl::Usb::Pipe::Bulk::
			OUT::Resp::Api::CancelResp*						bulkOutCancel;
			/** */
			Oscl::Usb::Pipe::Bulk::
			IN::Resp::Api::ReadResp*						bulkRead;
			/** */
			Oscl::Usb::Pipe::Bulk::
			IN::Resp::Api::CancelResp*						bulkInCancel;
			/** */
			Oscl::Usb::Pipe::Message::
			Resp::Api::CancelResp*							messageCancel;
			/** */
			Oscl::Usb::Pipe::Message::
			Resp::Api::NoDataResp*							messageNoData;
			/** */
			Oscl::Usb::Pipe::Message::
			Resp::Api::ReadResp*							messageRead;
			/** */
			Oscl::Usb::Pipe::Message::
			Resp::Api::WriteResp*							messageWrite;
			} _currentPipe;

		/** */
		union{
			Unsigned8	array[4];
			Unsigned32	word;
			} _currentTag;

		/** */
		const Oscl::Usb::Pipe::Status::Result*		_result;

	public:
		/** */
		Driver(	Oscl::Usb::Pipe::
				Message::PipeApi&				controlPipe,
				Oscl::Usb::Pipe::Bulk::
				IN::Req::Api::SAP&				bulkInPipe,
				Oscl::Usb::Pipe::Bulk::
				OUT::Req::Api::SAP&				bulkOutPipe,
				Oscl::Mt::Itc::PostMsgApi&		myPapi,
				SetupMem&						setupMem,
				PacketMem&						packetMem,
				unsigned char					maxBulkInPacketSize,
				unsigned char					maxBulkOutPacketSize,
				Unsigned8						bInterfaceNumber,
				Unsigned8						bulkInEndpointID,
				Unsigned8						bulkOutEndpointID
				) throw();

		/** */
		Oscl::TT::Api::Api&	getTTApi() throw();

		/** */
		Oscl::TT::ITC::Req::Api::SAP&	getTTSAP() throw();

	private:	// Oscl::Mt::Itc::Srv::Close::Req::Api
		/** */
		void	request(	Oscl::Mt::Itc::Srv::
							Open::Req::Api::OpenReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Mt::Itc::Srv::
							Close::Req::Api::CloseReq&	msg
							) throw();

	public:	// Oscl::TT::ITC::Req::Api
		/** */
		void	request(ResetReq& msg) throw();
		/** */
		void	request(GetMaxLunReq& msg) throw();
		/** */
		void	request(CancelReq& msg) throw();
		/** */
		void	request(ReadReq& msg) throw();
		/** */
		void	request(WriteReq& msg) throw();
		/** */
		void	request(CommandReq& msg) throw();

	private:	// Oscl::Usb::Pipe::Bulk::OUT::Resp::Api
		/** */
		void	response(	Oscl::Usb::Pipe::Bulk::
							OUT::Resp::Api::WriteResp&		msg
							) throw();
		/** */
		void	response(	Oscl::Usb::Pipe::Bulk::
							OUT::Resp::Api::CancelResp&		msg
							) throw();

	private:	// Oscl::Usb::Pipe::Bulk::IN::Resp::Api
		/** */
		void	response(	Oscl::Usb::Pipe::Bulk::
							IN::Resp::Api::ReadResp&		msg
							) throw();
		/** */
		void	response(	Oscl::Usb::Pipe::Bulk::
							IN::Resp::Api::CancelResp&		msg
							) throw();

	public:	// Oscl::Usb::Pipe::Message::Resp::Api
		/** */
		void	response(	Oscl::Usb::Pipe::Message::
							Resp::Api::CancelResp&			msg
							) throw();
		/** */
		void	response(	Oscl::Usb::Pipe::Message::
							Resp::Api::NoDataResp&			msg
							) throw();
		/** */
		void	response(	Oscl::Usb::Pipe::Message::
							Resp::Api::ReadResp&			msg
							) throw();
		/** */
		void	response(	Oscl::Usb::Pipe::Message::
							Resp::Api::WriteResp&			msg
							) throw();

	private:	// Oscl::Usb::TT::BO::StateVar::ContextApi
		/** */
		void	returnOpenReq() throw();
		/** */
		void	returnCloseReq() throw();
		/** */
		void	executeDeferredReq() throw();
		/** */
		void	deferReq() throw();
		/** */
		void	flushDeferQueue() throw();
		/** */
		void	startReset() throw();
		/** */
		void	startGetMaxLun() throw();
		/** */
		void	startBulkInClearHalt() throw();
		/** */
		void	startCommandCB() throw();
		/** */
		void	startWriteCB() throw();
		/** */
		void	startWriteData() throw();
		/** */
		void	startReadCB() throw();
		/** */
		void	startReadData() throw();
		/** */
		void	startStatus() throw();
		/** */
		void	failReadReq() throw();
		/** */
		void	failWriteReq() throw();
		/** */
		void	failCommandReq() throw();
		/** */
		void	failResetReq() throw();
		/** */
		void	failGetMaxLunReq() throw();
		/** */
		void	finishWriteReq() throw();
		/** */
		void	finishCommandReq() throw();
		/** */
		void	finishReadReq() throw();
		/** */
		void	finishResetReq() throw();
		/** */
		void	finishGetMaxLunReq() throw();
		/** */
		void	returnResetReq() throw();
		/** */
		void	returnGetMaxLunReq() throw();
		/** */
		void	returnReadReq() throw();
		/** */
		void	returnWriteReq() throw();
		/** */
		void	returnCommandReq() throw();
		/** */
		void	returnCancelReq() throw();
		/** */
		void	returnCanceledReq() throw();
		/** */
		void	cancelClearFeatureReq() throw();
		/** */
		void	cancelResetReq() throw();
		/** */
		void	cancelGetMaxLunReq() throw();
		/** */
		void	cancelReadReq() throw();
		/** */
		void	cancelWriteReq() throw();
		/** */
		void	cancelCommandReq() throw();
		/** */
		bool	deferQueueEmpty() throw();
		/** */
		bool	writeFailed() throw();
		/** */
		bool	statusFailed() throw();
		/** */
		bool	readFailed() throw();
		/** */
		bool	cswIsValid() throw();
		/** */
		bool	statusRetryFailureType() throw();
	private:
		/** */
		const Oscl::TT::Status::Result*
		parseCswResult() throw();
		/** */
		const Oscl::TT::Status::Result*
		parseCSW(unsigned long& residue) throw();
		/** */
		void	cancelMessage(Oscl::Mt::Itc::SrvMsg& msg) throw();
		/** */
		void	cancelBulkIn(Oscl::Mt::Itc::SrvMsg& msg) throw();
		/** */
		void	cancelBulkOut(Oscl::Mt::Itc::SrvMsg& msg) throw();
	};

}
}
}
}


#endif
