/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_drv_usb_tt_bo_fsmh_
#define _oscl_drv_usb_tt_bo_fsmh_
#include "oscl/queue/queue.h"
#include "oscl/queue/queueitem.h"

/** */
namespace Oscl {
/** */
namespace Usb {
/** */
namespace TT {
/** */
namespace BO {

/** */
class StateVar {
	public:
		/** */
		class ContextApi {
			public:	// request actions
				/** */
				virtual void	returnOpenReq() throw()=0;
				/** */
				virtual void	returnCloseReq() throw()=0;
				/** */
				virtual void	executeDeferredReq() throw()=0;
				/** */
				virtual void	deferReq() throw()=0;
				/** */
				virtual void	flushDeferQueue() throw()=0;
				/** */
				virtual void	startReset() throw()=0;
				/** */
				virtual void	startGetMaxLun() throw()=0;
				/** */
				virtual void	startBulkInClearHalt()=0;
				/** */
				virtual void	startCommandCB() throw()=0;
				/** */
				virtual void	startWriteCB() throw()=0;
				/** */
				virtual void	startWriteData() throw()=0;
				/** */
				virtual void	startReadCB() throw()=0;
				/** */
				virtual void	startReadData() throw()=0;
				/** */
				virtual void	startStatus() throw()=0;
				/** */
				virtual void	failReadReq() throw()=0;
				/** */
				virtual void	failWriteReq() throw()=0;
				/** */
				virtual void	failCommandReq() throw()=0;
				/** */
				virtual void	failResetReq() throw()=0;
				/** */
				virtual void	failGetMaxLunReq() throw()=0;
				/** */
				virtual void	finishWriteReq() throw()=0;
				/** */
				virtual void	finishCommandReq() throw()=0;
				/** */
				virtual void	finishReadReq() throw()=0;
				/** */
				virtual void	finishResetReq() throw()=0;
				/** */
				virtual void	finishGetMaxLunReq() throw()=0;
				/** */
				virtual void	returnResetReq() throw()=0;
				/** */
				virtual void	returnGetMaxLunReq() throw()=0;
				/** */
				virtual void	returnReadReq() throw()=0;
				/** */
				virtual void	returnWriteReq() throw()=0;
				/** */
				virtual void	returnCommandReq() throw()=0;
				/** */
				virtual void	returnCancelReq() throw()=0;
				/** */
				virtual void	returnCanceledReq() throw()=0;
				/** */
				virtual void	cancelClearFeatureReq() throw()=0;
				/** */
				virtual void	cancelResetReq() throw()=0;
				/** */
				virtual void	cancelGetMaxLunReq() throw()=0;
				/** */
				virtual void	cancelReadReq() throw()=0;
				/** */
				virtual void	cancelWriteReq() throw()=0;
				/** */
				virtual void	cancelCommandReq() throw()=0;
				/** */
				virtual bool	deferQueueEmpty() throw()=0;
				/** */
				virtual bool	writeFailed() throw()=0;
				/** */
				virtual bool	readFailed() throw()=0;
				/** */
				virtual bool	statusFailed() throw()=0;
				/** */
				virtual bool	cswIsValid() throw()=0;
				/** */
				virtual bool	statusRetryFailureType() throw()=0;
			};
	private:
		/** */
		class State {
			public:	// Admin Events
				/** */
				virtual void	open(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				virtual void	close(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				virtual void	cancelResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				virtual void	controlNoDataResp(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				virtual void	controlReadResp(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				virtual void	controlWriteResp(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				virtual void	writeResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				virtual void	readResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				virtual void	readReq(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				virtual void	writeReq(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				virtual void	commandReq(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				virtual void	resetReq(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				virtual void	getMaxLunReq(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	cancelReq(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			public:
				/** */
				virtual const char*	stateName() const throw()=0;
			};

	public:
		/** */
		class Closed : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Closed";}
			private:
				/** */
				void	open(	ContextApi&	context,
								StateVar&	sv
								) const throw();
			};
		/** */
		class Idle : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Resetting";}
			public:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	readReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	writeReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	commandReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	resetReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	getMaxLunReq(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class Fail : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Resetting";}
			public:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	readReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	writeReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	commandReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	resetReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	getMaxLunReq(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class DeferReq : public State {
			public:
				/** */
				void	readReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	writeReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	commandReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	resetReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	getMaxLunReq(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class Resetting : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Resetting";}
			public:
				/** */
				void	controlNoDataResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class GettingMaxLun : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "GettingMaxLun";}
			public:
				/** */
				void	controlReadResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class CommandCB : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "CommandCB";}
			public:
				/** */
				void	writeResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class CommandStatus : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "CommandStatus";}
			public:
				/** */
				void	readResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class CommandClear : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "CommandClear";}
			public:
				/** */
				void	controlNoDataResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class CommandStatus2 : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "CommandStatus2";}
			public:
				/** */
				void	readResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class WriteCB : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "WriteCB";}
			public:
				/** */
				void	writeResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class WriteData : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "WriteData";}
			public:
				/** */
				void	writeResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class WriteStatus : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "WriteStatus";}
			public:
				/** */
				void	readResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class WriteClear : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "WriteClear";}
			public:
				/** */
				void	controlNoDataResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class WriteStatus2 : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "WriteStatus2";}
			public:
				/** */
				void	readResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class ReadCB : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ReadCB";}
			public:
				/** */
				void	writeResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class ReadData : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ReadData";}
			public:
				/** */
				void	readResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class ReadStatus : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ReadStatus";}
			public:
				/** */
				void	readResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class ReadClear : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ReadClear";}
			public:
				/** */
				void	controlNoDataResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class ReadStatus2 : public DeferReq {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "ReadStatus2";}
			public:
				/** */
				void	readResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	cancelReq(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class Canceling : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Canceling";}
			public:
				/** */
				void	cancelResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	controlNoDataResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	controlReadResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	controlWriteResp(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				void	writeResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	readResp(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};

	private:
		/** */
		struct TraceRec : public Oscl::QueueItem {
			public:
				/** */
				const State*	_state;
			};

	private:
		/** */
		ContextApi&		_context;
		/** */
		const State*	_state;

	private:
		/** */
		enum{nTraceRecs=20};
		/** */
		TraceRec				_traceRec[nTraceRecs];
		/** */
		Oscl::Queue<TraceRec>	_freeTraceRecs;
		/** */
		Oscl::Queue<TraceRec>	_trace;

	public:	// Constructors
		/** */
		StateVar(ContextApi& context) throw();

	public:	// events
		/** */
		void	open() throw();
		/** */
		void	close() throw();
		/** */
		void	cancelResp() throw();
		/** */
		void	controlNoDataResp() throw();
		/** */
		void	controlReadResp() throw();
		/** */
		void	controlWriteResp() throw();
		/** */
		void	writeResp() throw();
		/** */
		void	readResp() throw();
		/** */
		void	writeReq() throw();
		/** */
		void	commandReq() throw();
		/** */
		void	readReq() throw();
		/** */
		void	resetReq() throw();
		/** */
		void	getMaxLunReq() throw();
		/** */
		void	cancelReq() throw();

	private: // global actions and state change operations
		/** */
		friend class State;
		friend class Closed;
		friend class Resetting;
		friend class GettingMaxLun;
		friend class Idle;
		friend class ReadCB;
		friend class ReadData;
		friend class ReadStatus;
		friend class ReadClear;
		friend class ReadStatus2;
		friend class WriteCB;
		friend class CommandCB;
		friend class CommandStatus;
		friend class CommandClear;
		friend class CommandStatus2;
		friend class WriteData;
		friend class WriteStatus;
		friend class WriteClear;
		friend class WriteStatus2;
		friend class Canceling;
		/** */
		void	protocolError(const char* state) throw();
		/** */
		void	changeToClosed() throw();
		/** */
		void	changeToResetting() throw();
		/** */
		void	changeToGettingMaxLun() throw();
		/** */
		void	changeToFail() throw();
		/** */
		void	changeToIdle() throw();
		/** */
		void	changeToReadCB() throw();
		/** */
		void	changeToReadData() throw();
		/** */
		void	changeToReadStatus() throw();
		/** */
		void	changeToReadClear() throw();
		/** */
		void	changeToReadStatus2() throw();
		/** */
		void	changeToCommandCB() throw();
		/** */
		void	changeToWriteCB() throw();
		/** */
		void	changeToWriteData() throw();
		/** */
		void	changeToWriteStatus() throw();
		/** */
		void	changeToWriteClear() throw();
		/** */
		void	changeToWriteStatus2() throw();
		/** */
		void	changeToCommandStatus() throw();
		/** */
		void	changeToCommandClear() throw();
		/** */
		void	changeToCommandStatus2() throw();
		/** */
		void	changeToCanceling() throw();

	private:
		/** */
		void	trace(const State& state) throw();
		/** */
		void	executeDeferredReq() throw();
		/** */
		void	flushDeferQueue() throw();
	};

}
}
}
}

#endif
