/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_endian_littelh_
#define _oscl_endian_littelh_
#include "oscl/compiler/types.h"
/* This file should only be included by Little Endian machines.
	This file should only included indirectly by the
	"compiler/endian.h" file.
 */

/** */
namespace Oscl {
/** */
namespace Endian {

inline Unsigned16	fromLittleEndian(Unsigned16 le) throw(){
	return le;
	}

inline Unsigned16	fromBigEndian(Unsigned16 be) throw(){
	return (be<<8) | ((be>>8) & 0x00FF);
	}

inline Unsigned32	fromLittleEndian(Unsigned32 le) throw(){
	return le;
	}

inline Unsigned32	fromBigEndian(Unsigned32 be) throw(){
	Unsigned32	result	= ((be&0xFF)<<24);
	result	=  (be<<24)	& 0xFF000000;
	result	|= (be<<8)	& 0x00FF0000;
	result	|= (be>>8)	& 0x0000FF00;
	result	|= (be>>24)	& 0x000000FF;
	return result;
	}

inline Unsigned16	toLittleEndian(Unsigned16 native) throw(){
	return native;
	}

inline Unsigned16	toBigEndian(Unsigned16 native) throw(){
	return (native<<8) | ((native>>8) & 0x00FF);
	}

inline Unsigned32	toLittleEndian(Unsigned32 native) throw(){
	return native;
	}

inline Unsigned32	toBigEndian(Unsigned32 native) throw(){
	Unsigned32	result	= ((native&0xFF)<<24);
	result	=  (native<<24)	& 0xFF000000;
	result	|= (native<<8)	& 0x00FF0000;
	result	|= (native>>8)	& 0x0000FF00;
	result	|= (native>>24)	& 0x000000FF;
	return result;
	}

inline Signed16	fromLittleEndian(Signed16 le) throw(){
	return le;
	}

inline Signed16	fromBigEndian(Signed16 be) throw(){
	return (be<<8) | ((be>>8) & 0x00FF);
	}

inline Signed32	fromLittleEndian(Signed32 le) throw(){
	return le;
	}

inline Signed32	fromBigEndian(Signed32 be) throw(){
	Signed32	result	= ((be&0xFF)<<24);
	result	=  (be<<24)	& 0xFF000000;
	result	|= (be<<8)	& 0x00FF0000;
	result	|= (be>>8)	& 0x0000FF00;
	result	|= (be>>24)	& 0x000000FF;
	return result;
	}

inline Signed16	toLittleEndian(Signed16 native) throw(){
	return native;
	}

inline Signed16	toBigEndian(Signed16 native) throw(){
	return (native<<8) | ((native>>8) & 0x00FF);
	}

inline Signed32	toLittleEndian(Signed32 native) throw(){
	return native;
	}

inline Signed32	toBigEndian(Signed32 native) throw(){
	Signed32	result	= ((native&0xFF)<<24);
	result	=  (native<<24)	& 0xFF000000;
	result	|= (native<<8)	& 0x00FF0000;
	result	|= (native>>8)	& 0x0000FF00;
	result	|= (native>>24)	& 0x000000FF;
	return result;
	}

}
}

#endif
