/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "sv.h"
#include "oscl/error/fatal.h"

using namespace Oscl::Equip::Holder::FSM;

static const StateVar::Empty						empty;
static const StateVar::New							newState;
static const StateVar::Unrecognized					unrecognized;
static const StateVar::Unacceptable					unacceptable;
static const StateVar::Invalid						invalid;
static const StateVar::Missing						missing;
static const StateVar::Provisioning					provisioningState;
static const StateVar::Provisioned					provisionedState;
static const StateVar::Failed						failedState;

// StateVar

StateVar::StateVar(ContextApi& context) throw():
		_context(context),
		_state(&empty)
		{
	}

void	StateVar::insert() throw(){
	_state->insert(_context,*this);
	}

void	StateVar::remove() throw(){
	_state->remove(_context,*this);
	}

void	StateVar::enable() throw(){
	_state->enable(_context,*this);
	}

void	StateVar::disable() throw(){
	_state->disable(_context,*this);
	}

void	StateVar::criteriaChanged() throw(){
	_state->criteriaChanged(_context,*this);
	}

void	StateVar::reset() throw(){
	_state->reset(_context,*this);
	}

void	StateVar::failed() throw(){
	_state->failed(_context,*this);
	}

void	StateVar::provisioned() throw(){
	_state->provisioned(_context,*this);
	}

void	StateVar::provisioning() throw(){
	_state->provisioning(_context,*this);
	}

void	StateVar::query(Handler& h) const throw(){
	_state->query(h);
	}

// global actions
void	StateVar::protocolError(const char* state) throw(){
	ErrorFatal::logAndExit(state);
	}

// state change operations
void	StateVar::changeToEmpty() throw(){
	_state	= &empty;
	}

void	StateVar::changeToMissing() throw(){
	_state	= &missing;
	}

void	StateVar::changeToInvalid() throw(){
	_state	= &invalid;
	}

void	StateVar::changeToNew() throw(){
	_state	= &newState;
	}

void	StateVar::changeToUnacceptable() throw(){
	_state	= &unacceptable;
	}

void	StateVar::changeToUnrecognized() throw(){
	_state	= &unrecognized;
	}

void	StateVar::changeToFailed() throw(){
	_state	= &failedState;
	}

void	StateVar::changeToProvisioning() throw(){
	_state	= &provisioningState;
	}

void	StateVar::changeToProvisioned() throw(){
	_state	= &provisionedState;
	}

