/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_equipment_holder_fsm_svh_
#define _oscl_equipment_holder_fsm_svh_

/** */
namespace Oscl {
/** */
namespace Equip {
/** */
namespace Holder {
/** */
namespace FSM {

/** */
class StateVar {
	public:
		/** */
		class ContextApi {
			public:	// request actions
				/** */
				virtual	bool	holderEnabled() const throw()=0;
				/** */
				virtual	bool	isAcceptable() const throw()=0;
				/** */
				virtual	bool	isUnrecognized() const throw()=0;
			};
	public:
		/** */
		class Handler {
			public:
				/** */
				virtual	void	emptyState() throw()=0;
				/** */
				virtual	void	missingState() throw()=0;
				/** */
				virtual	void	invalidState() throw()=0;
				/** */
				virtual	void	newState() throw()=0;
				/** */
				virtual	void	provisioningState() throw()=0;
				/** */
				virtual	void	provisionedState() throw()=0;
				/** */
				virtual	void	failedState() throw()=0;
				/** */
				virtual	void	unrecognizedState() throw()=0;
				/** */
				virtual	void	unacceptableState() throw()=0;
			};
		/** */
		class State {
			public:	// Admin Events
				/** */
				virtual void	insert(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				virtual void	remove(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				virtual void	enable(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				virtual void	disable(	ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				virtual void	criteriaChanged(	ContextApi&	context,
													StateVar&	sv
													) const throw();
				/** */
				virtual void	reset(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				virtual void	failed(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				virtual void	provisioned(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	provisioning(	ContextApi&	context,
												StateVar&	sv
												) const throw();
			public:
				/** */
				virtual const char*	stateName() const throw()=0;
				/** This operation invokes the appropriate
					handler operation according to the concrete
					type of the State.
				 */
				virtual	void		query(Handler& h) const throw()=0;
			};

	public:
		/** */
		class Unequipped : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Unequipped";}
			private:
				/** */
				void	insert(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	criteriaChanged(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		/** */
		class Empty : public Unequipped {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Empty";}
				/** */
				void		query(Handler& h) const throw();
			public:
				/** */
				void	enable(	ContextApi&	context,
								StateVar&	sv
								) const throw();
			};
		/** */
		class Missing : public Unequipped {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Missing";}
				/** */
				void		query(Handler& h) const throw();
			public:
				/** */
				void	disable(	ContextApi&	context,
									StateVar&	sv
									) const throw();
			};
		/** */
		class Equipped : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Equipped";}
			public:
				/** */
				void	remove(	ContextApi&	context,
								StateVar&	sv
								) const throw();
			};
		/** */
		class Valid : public Equipped {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Valid";}
			public:
				/** */
				void	disable(	ContextApi&	context,
									StateVar&	sv
									) const throw();
				/** */
				void	reset(	ContextApi&	context,
								StateVar&	sv
								) const throw();
			};
		/** */
		class Invalid : public Equipped {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Invalid";}
				/** */
				void		query(Handler& h) const throw();
			public:
				/** */
				void	enable(	ContextApi&	context,
								StateVar&	sv
								) const throw();
			};

		/** */
		class Acceptable : public Valid {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Acceptable";}
			public:
				/** */
				void	criteriaChanged(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};

		/** */
		class Unacceptable : public Valid {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Unacceptable";}
				/** */
				void		query(Handler& h) const throw();
			public:
				/** */
				void	criteriaChanged(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		/** */
		class Unrecognized : public Valid {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Unrecognized";}
				/** */
				void		query(Handler& h) const throw();
			public:
				/** */
				void	criteriaChanged(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		/** */
		class New : public Acceptable {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "New";}
				/** */
				void		query(Handler& h) const throw();
			public:
				/** */
				void	provisioning(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class Provision : public Acceptable {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Provision";}
			public:
			};
		/** */
		class Failed : public Provision {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Failed";}
				/** */
				void		query(Handler& h) const throw();
			};
		/** */
		class Provisioning : public Provision {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Provisioning";}
				/** */
				void		query(Handler& h) const throw();
			public:
				/** */
				void	provisioned(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	failed(	ContextApi&	context,
								StateVar&	sv
								) const throw();
			};
		/** */
		class Provisioned : public Provision {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Provisioned";}
				/** */
				void		query(Handler& h) const throw();
			public:
			};
	private:
		/** */
		ContextApi&		_context;
		/** */
		const State*	_state;

	public:
		/** */
		StateVar(ContextApi& context) throw();

		/** */
		void	query(Handler& h) const throw();

		/** */
		inline const State&	state() const throw(){ return *_state;}

	public:	// events
		/** */
		void	insert() throw();
		/** */
		void	remove() throw();
		/** */
		void	enable() throw();
		/** */
		void	disable() throw();
		/** */
		void	criteriaChanged() throw();
		/** */
		void	reset() throw();
		/** */
		void	failed() throw();
		/** */
		void	provisioned() throw();
		/** */
		void	provisioning() throw();

	private: // global actions and state change operations
		/** */
		friend class Unequipped;
		friend class Empty;
		friend class Missing;
		friend class Equipped;
		friend class Valid;
		friend class Invalid;
		friend class Acceptable;
		friend class Unacceptable;
		friend class Unrecognized;
		friend class Provision;
		friend class Failed;
		friend class Provisioning;
		friend class Provisioned;
		/** */
		void	protocolError(const char* state) throw();
		/** */
		void	changeToEmpty() throw();
		/** */
		void	changeToMissing() throw();
		/** */
		void	changeToInvalid() throw();
		/** */
		void	changeToNew() throw();
		/** */
		void	changeToUnacceptable() throw();
		/** */
		void	changeToUnrecognized() throw();
		/** */
		void	changeToFailed() throw();
		/** */
		void	changeToProvisioning() throw();
		/** */
		void	changeToProvisioned() throw();
	};

}
}
}
}

#endif
