/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_equipment_holder_view_monitor_parth_
#define _oscl_equipment_holder_view_monitor_parth_
#include "base.h"

/** */
namespace Oscl {
/** */
namespace Equip {
/** */
namespace Holder {
/** */
namespace View {
/** */
namespace Monitor {

/** */
template <class Context>
class Part :	public Base,
				private Oscl::Equip::Holder::FSM::StateVar::Handler
				{
	public:
		/** */
		typedef void	(Context::* ActionFunc)();

	private:
		/** */
		Context&		_context;
		/** */
		ActionFunc		_emptyAction;
		/** */
		ActionFunc		_newAction;
		/** */
		ActionFunc		_unrecognizedAction;
		/** */
		ActionFunc		_unacceptableAction;
		/** */
		ActionFunc		_invalidAction;
		/** */
		ActionFunc		_missingAction;
		/** */
		ActionFunc		_provisioningAction;
		/** */
		ActionFunc		_provisionedAction;
		/** */
		ActionFunc		_failedAction;
		/** */
		ActionFunc		_stopCompleteAction;

	public:
		Part(	Oscl::Equip::Holder::View::
				ITC::Req::Api::SAP&				ehSAP,
				Oscl::Mt::Itc::PostMsgApi&		myPapi,
				Context&						context,
				ActionFunc						emptyAction,
				ActionFunc						newAction,
				ActionFunc						unrecognizedAction,
				ActionFunc						unacceptableAction,
				ActionFunc						invalidAction,
				ActionFunc						missingAction,
				ActionFunc						provisioningAction,
				ActionFunc						provisionedAction,
				ActionFunc						failedAction,
				ActionFunc						stopCompleteAction
				) throw();

	private:	// Base
		/** */
		void	stopComplete() throw();
		
	private:	// Oscl::Equip::Holder::FSM::StateVar::Handler
		/** */
		void	emptyState() throw();
		/** */
		void	missingState() throw();
		/** */
		void	invalidState() throw();
		/** */
		void	newState() throw();
		/** */
		void	provisioningState() throw();
		/** */
		void	provisionedState() throw();
		/** */
		void	failedState() throw();
		/** */
		void	unrecognizedState() throw();
		/** */
		void	unacceptableState() throw();
	};


/** */
template <class Context>
Part<Context>::Part(	Oscl::Equip::Holder::View::
						ITC::Req::Api::SAP&				ehSAP,
						Oscl::Mt::Itc::PostMsgApi&		myPapi,
						Context&						context,
						ActionFunc						emptyAction,
						ActionFunc						newAction,
						ActionFunc						unrecognizedAction,
						ActionFunc						unacceptableAction,
						ActionFunc						invalidAction,
						ActionFunc						missingAction,
						ActionFunc						provisioningAction,
						ActionFunc						provisionedAction,
						ActionFunc						failedAction,
						ActionFunc						stopCompleteAction
						) throw():
		Base(ehSAP,*this,myPapi),
		_context(context),
		_emptyAction(emptyAction),
		_newAction(newAction),
		_unrecognizedAction(unrecognizedAction),
		_unacceptableAction(unacceptableAction),
		_invalidAction(invalidAction),
		_missingAction(missingAction),
		_provisioningAction(provisioningAction),
		_provisionedAction(provisionedAction),
		_failedAction(failedAction),
		_stopCompleteAction(stopCompleteAction)
		{
	}

/** */
template <class Context>
void	Part<Context>::stopComplete() throw(){
	(_context.*_stopCompleteAction)();
	}

/** */
template <class Context>
void	Part<Context>::emptyState() throw(){
	(_context.*_emptyAction)();
	}

/** */
template <class Context>
void	Part<Context>::missingState() throw(){
	(_context.*_missingAction)();
	}

/** */
template <class Context>
void	Part<Context>::invalidState() throw(){
	(_context.*_invalidAction)();
	}

/** */
template <class Context>
void	Part<Context>::newState() throw(){
	(_context.*_newAction)();
	}

/** */
template <class Context>
void	Part<Context>::provisioningState() throw(){
	(_context.*_provisioningAction)();
	}

/** */
template <class Context>
void	Part<Context>::provisionedState() throw(){
	(_context.*_provisionedAction)();
	}

/** */
template <class Context>
void	Part<Context>::failedState() throw(){
	(_context.*_failedAction)();
	}

/** */
template <class Context>
void	Part<Context>::unrecognizedState() throw(){
	(_context.*_unrecognizedAction)();
	}

/** */
template <class Context>
void	Part<Context>::unacceptableState() throw(){
	(_context.*_unacceptableAction)();
	}

}
}
}
}
}

#endif
