/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_fp_fp16h_
#define _oscl_fp_fp16h_

/** */
namespace Oscl {

/** */
namespace FP {

/** */
class Frac16 {
	private:
		/** */
		Signed16	_value;
	public:
		inline Frac16(float value) throw(){
			_value	= (Signed16)(value * 256);
			}

		inline Frac16(const Frac16& value) throw(){
			_value	= value._value;
			}

		inline Frac16& operator *=(const Frac16& value) throw(){
			Signed32	temp	= value._value;
			temp	*= _value;
			temp	>>= 16;
			_value	= (Signed16)temp;
			return *this;
			}

		inline Frac16& operator /=(const Frac16& value) throw(){
			Signed32	dividend	= _value;
			Signed32	divisor		= value._value;
			dividend	<<= 16;
			dividend	/= divisor;
			_value	= (Signed16)dividend;
			return *this;
			}

		inline Frac16& operator +=(const Frac16& value) throw(){
			_value	+= value._value;
			return *this;
			}

		inline Frac16& operator -=(const Frac16& value) throw(){
			_value	-= value._value;
			return *this;
			}

		inline Frac16& operator *=(Unsigned8 value) throw(){
			_value	*= value;
			return *this;
			}

		inline operator float() const throw(){
			float	value	= _value;
			return (value / 256);
			}

		inline Signed16	raw() const throw(){
			return _value;
			}
	};

}
}

#endif
