/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_fs_dir_wo_handlerh_
#define _oscl_fs_dir_wo_handlerh_

/** */
namespace Oscl {
/** */
namespace FS {
/** */
namespace Dir {
/** */
namespace WO {

/** */
namespace Move {
/** */
namespace Status {
/** */
class Handler{
	public:
		/** */
		virtual void	noSuchVolume() throw()=0;
		/** */
		virtual void	noSuchSrcPath() throw()=0;
		/** */
		virtual void	invalidDestPath() throw()=0;
		/** */
		virtual void	destDirectoryFull() throw()=0;
		/** */
		virtual void	srcPathElementBusy() throw()=0;
		/** */
		virtual void	outOfSpace() throw()=0;
		/** */
		virtual void	transportError() throw()=0;
	};
}
}

/** */
namespace Copy {
/** */
namespace Status {
/** */
class Handler{
	public:
		/** */
		virtual void	noSuchSrcVolume() throw()=0;
		/** */
		virtual void	noSuchDestVolume() throw()=0;
		/** */
		virtual void	noSuchSrcPath() throw()=0;
		/** */
		virtual void	invalidDestPath() throw()=0;
		/** */
		virtual void	destDirectoryFull() throw()=0;
		/** */
		virtual void	destOutOfSpace() throw()=0;
		/** */
		virtual void	srcTransportError() throw()=0;
		/** */
		virtual void	destTransportError() throw()=0;
	};
}
}

/** */
namespace Create {
/** */
namespace Status {
/** */
class Handler{
	public:
		/** */
		virtual void	noSuchVolume() throw()=0;
		/** Path does not specify a directory.
			One or more elements of the directory
			portion of the path is not a directory.
		 */
		virtual void	invalidPath() throw()=0;
		/** Path already exists of the same type.
			E.g. the createDir() operation is invoked
			and a directory of the same name already
			exists. No change is made to the filesystem.
		 */
		virtual void	pathAlreadyExists() throw()=0;
		/** Path already exists and is not of the
			same type. E.g. the createFile() operation
			is specified, the same name already exists,
			but is a directory instead of a file.
		 */
		virtual void	otherTypePathExists() throw()=0;
		/** */
		virtual void	directoryFull() throw()=0;
		/** */
		virtual void	outOfSpace() throw()=0;
		/** */
		virtual void	transportError() throw()=0;
	};
}
}

/** */
namespace Remove {
/** */
namespace Status {
/** */
class Handler{
	public:
		/** */
		virtual void	noSuchVolume() throw()=0;
		/** */
		virtual void	noSuchPath() throw()=0;
		/** */
		virtual void	transportError() throw()=0;
	};
}
}


}
}
}
}

#endif
