/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "result.h"
#include "status.h"
#include "handler.h"

/** */
namespace Oscl {
/** */
namespace FS {
/** */
namespace Dir {
/** */
namespace WO {

/** */
namespace Move {
/** */
namespace Status {

class NoSuchVolume : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	NoSuchVolume::query(Handler& handler) const throw(){
	handler.noSuchVolume();
	}

const Result&	getNoSuchVolume() throw(){
	static const NoSuchVolume	err;
	return err;
	}

/** */
class NoSuchSrcPath : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	NoSuchSrcPath::query(Handler& handler) const throw(){
	handler.noSuchSrcPath();
	}

const Result&	getNoSuchSrcPath() throw(){
	static const NoSuchSrcPath	err;
	return err;
	}

/** */
class InvalidDestPath : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	InvalidDestPath::query(Handler& handler) const throw(){
	handler.invalidDestPath();
	}

const Result&	getInvalidDestPath() throw(){
	static const InvalidDestPath	err;
	return err;
	}

/** */
class DestDirectoryFull : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	DestDirectoryFull::query(Handler& handler) const throw(){
	handler.destDirectoryFull();
	}

const Result&	getDestDirectoryFull() throw(){
	static const DestDirectoryFull	err;
	return err;
	}

/** */
class SrcPathElementBusy : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	SrcPathElementBusy::query(Handler& handler) const throw(){
	handler.srcPathElementBusy();
	}

const Result&	getSrcPathElementBusy() throw(){
	static const SrcPathElementBusy	err;
	return err;
	}

/** */
class OutOfSpace : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	OutOfSpace::query(Handler& handler) const throw(){
	handler.outOfSpace();
	}

const Result&	getOutOfSpace() throw(){
	static const OutOfSpace	err;
	return err;
	}

/** */
class TransportError : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	TransportError::query(Handler& handler) const throw(){
	handler.transportError();
	}

const Result&	getTransportError() throw(){
	static const TransportError	err;
	return err;
	}

}
}

/** */
namespace Copy {
/** */
namespace Status {
/** */
class NoSuchSrcVolume : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	NoSuchSrcVolume::query(Handler& handler) const throw(){
	handler.noSuchSrcVolume();
	}

const Result&	getNoSuchSrcVolume() throw(){
	static const NoSuchSrcVolume	err;
	return err;
	}

/** */
class NoSuchDestVolume : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	NoSuchDestVolume::query(Handler& handler) const throw(){
	handler.noSuchDestVolume();
	}

const Result&	getNoSuchDestVolume() throw(){
	static const NoSuchDestVolume	err;
	return err;
	}

/** */
class NoSuchSrcPath : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	NoSuchSrcPath::query(Handler& handler) const throw(){
	handler.noSuchSrcPath();
	}

const Result&	getNoSuchSrcPath() throw(){
	static const NoSuchSrcPath	err;
	return err;
	}

/** */
class InvalidDestPath : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	InvalidDestPath::query(Handler& handler) const throw(){
	handler.invalidDestPath();
	}

const Result&	getInvalidDestPath() throw(){
	static const InvalidDestPath	err;
	return err;
	}

/** */
class DestDirectoryFull : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	DestDirectoryFull::query(Handler& handler) const throw(){
	handler.destDirectoryFull();
	}

const Result&	getDestDirectoryFull() throw(){
	static const DestDirectoryFull	err;
	return err;
	}

/** */
class DestOutOfSpace : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	DestOutOfSpace::query(Handler& handler) const throw(){
	handler.destOutOfSpace();
	}

const Result&	getDestOutOfSpace() throw(){
	static const DestOutOfSpace	err;
	return err;
	}

/** */
class SrcTransportError : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	SrcTransportError::query(Handler& handler) const throw(){
	handler.srcTransportError();
	}

const Result&	getSrcTransportError() throw(){
	static const SrcTransportError	err;
	return err;
	}

/** */
class DestTransportError : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	DestTransportError::query(Handler& handler) const throw(){
	handler.destTransportError();
	}

const Result&	getDestTransportError() throw(){
	static const DestTransportError	err;
	return err;
	}

}
}

/** */
namespace Create {
/** */
namespace Status {
/** */
class NoSuchVolume : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	NoSuchVolume::query(Handler& handler) const throw(){
	handler.noSuchVolume();
	}

const Result&	getNoSuchVolume() throw(){
	static const NoSuchVolume	err;
	return err;
	}

/** */
class InvalidPath : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	InvalidPath::query(Handler& handler) const throw(){
	handler.invalidPath();
	}

const Result&	getInvalidPath() throw(){
	static const InvalidPath	err;
	return err;
	}

/** */
class PathAlreadyExists : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	PathAlreadyExists::query(Handler& handler) const throw(){
	handler.pathAlreadyExists();
	}

const Result&	getPathAlreadyExists() throw(){
	static const PathAlreadyExists	err;
	return err;
	}

/** */
class OtherTypePathExists : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	OtherTypePathExists::query(Handler& handler) const throw(){
	handler.otherTypePathExists();
	}

const Result&	getOtherTypePathExists() throw(){
	static const OtherTypePathExists	err;
	return err;
	}

/** */
class DirectoryFull : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	DirectoryFull::query(Handler& handler) const throw(){
	handler.directoryFull();
	}

const Result&	getDirectoryFull() throw(){
	static const DirectoryFull	err;
	return err;
	}

/** */
class OutOfSpace : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	OutOfSpace::query(Handler& handler) const throw(){
	handler.outOfSpace();
	}

const Result&	getOutOfSpace() throw(){
	static const OutOfSpace	err;
	return err;
	}

/** */
class TransportError : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	TransportError::query(Handler& handler) const throw(){
	handler.transportError();
	}

const Result&	getTransportError() throw(){
	static const TransportError	err;
	return err;
	}
}
}

/** */
namespace Remove {
/** */
namespace Status {
/** */
class NoSuchVolume : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	NoSuchVolume::query(Handler& handler) const throw(){
	handler.noSuchVolume();
	}

const Result&	getNoSuchVolume() throw(){
	static const NoSuchVolume	err;
	return err;
	}
/** */
class NoSuchPath : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	NoSuchPath::query(Handler& handler) const throw(){
	handler.noSuchPath();
	}

const Result&	getNoSuchPath() throw(){
	static const NoSuchPath	err;
	return err;
	}

/** */
class TransportError : public Result {
	public:
		void	query(Handler& handler) const throw();
	};

void	TransportError::query(Handler& handler) const throw(){
	handler.transportError();
	}

const Result&	getTransportError() throw(){
	static const TransportError	err;
	return err;
	}

}
}


}
}
}
}

