/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_fs_fat_fath_
#define _oscl_fs_fat_fath_
#include "oscl/compiler/types.h"
#include "oscl/compiler/align.h"

/** */
namespace Oscl {
/** */
namespace FS {
/** */
namespace FAT {

/** */
bool	isValidShortFileName(const char* filename) throw();

/** */
struct Date {
	/** */
	Unsigned8	_value[2];
	/** */
	inline	Unsigned16	value() const throw(){
		Unsigned16
		v	=	_value[1];
		v	<<=	8;
		v	|=	_value[0];
		return v;
		}
	/** */
	inline Unsigned8	dayOfMonth() const throw(){
		Unsigned8
		v	=	_value[0];
		v	&=	0x1F;
		return v;
		}
	/** */
	inline Unsigned8	monthOfYear() const throw(){
		Unsigned16
		v	=	value();
		v	>>=	5;
		v	&=	0x000F;
		return (Unsigned8)v;
		}
	/** */
	inline Unsigned8	yearsSince1980() const throw(){
		Unsigned8
		v	=	_value[1];
		v	>>=	1;
		return (Unsigned8)v;
		}
	/** */
	inline void	value(Unsigned16 v) throw(){
		_value[0]	=	(Unsigned8)v;
		v			>>=	8;
		_value[1]	=	(Unsigned8)v;
		}
	/** */
	void	date(	unsigned		year,
					unsigned char	monthOfYear,
					unsigned char	dayOfMonth
					) throw();
	};
/** */
struct Time {
	/** */
	Unsigned8	_value[2];
	/** */
	inline	Unsigned16	value() const throw(){
		Unsigned16
		v	=	_value[1];
		v	<<=	8;
		v	|=	_value[0];
		return v;
		}
	/** */
	inline void	value(Unsigned16 v) throw(){
		_value[0]	=	(Unsigned8)v;
		v			>>=	8;
		_value[1]	=	(Unsigned8)v;
		}
	/** */
	inline Unsigned8	seconds() const throw(){
		Unsigned8
		v	=	_value[0];
		v	<<=	1;
		return v;
		}
	/** */
	inline Unsigned8	minutes() const throw(){
		Unsigned16
		v	=	value();
		v	>>=	5;
		v	&=	0x001F;
		return (Unsigned8)v;
		}
	/** */
	inline Unsigned8	hours() const throw(){
		Unsigned8
		v	=	_value[1];
		v	>>=	3;
		return (Unsigned8)v;
		}
	/** */
	void	time(	unsigned char	hours,
					unsigned char	minutes,
					unsigned char	seconds
					) throw();
	};

/** */
struct DirEntry {
	/** */
	enum{shortNameLen=11};
	/** */
	Unsigned8	_shortName[shortNameLen];
	/** */
	Unsigned8	_attributes;
	/** */
	Unsigned8	_ntReserved;
	/** */
	Unsigned8	_creationTimeTenths;
	/** */
	Time		_creationTime	OsclPackedPostMacro();
	/** */
	Date		_creationDate	OsclPackedPostMacro();
	/** */
	Date		_lastAccessDate	OsclPackedPostMacro();
	/** */
	Unsigned8	_firstClusterHigh[2];
	/** */
	Time		_writeTime	OsclPackedPostMacro();
	/** */
	Date		_writeDate	OsclPackedPostMacro();
	/** */
	Unsigned8	_firstClusterLow[2];
	/** */
	Unsigned8	_fileSize[4];
	/** */
	inline bool	longName() const throw(){
		return _attributes & 0x0F;
		}
	/** */
	inline bool	readOnly() const throw(){
		return (_attributes & 0x01) && !longName();
		}
	/** */
	inline bool	hidden() const throw(){
		return (_attributes & 0x02) && !longName();
		}
	/** */
	inline bool	system() const throw(){
		return (_attributes & 0x04) && !longName();
		}
	/** */
	inline bool	volumeID() const throw(){
		return (_attributes & 0x08) && !longName();
		}
	/** */
	inline bool	directory() const throw(){
		return _attributes & 0x10;
		}
	/** */
	inline bool	archive() const throw(){
		return _attributes & 0x20;
		}
	/** */
	inline bool	isFree() const throw(){
		return (_shortName[0] == 0xE5) || (_shortName[0] == 0x00);
		}
	/** */
	inline void	free() throw(){
		_shortName[0]	= 0xE5;
		}
	/** */
	inline bool	noMoreDirEntriesFollow() const throw(){
		return _shortName[0] == 0x00;
		}
	/** */
	inline void	freeNoMoreDirEntriesFollow() throw(){
		_shortName[0] = 0x00;
		}
	/** */
	inline Unsigned32	firstCluster() const throw(){
		Unsigned32
		value	=	_firstClusterHigh[1];
		value	<<=	8;
		value	|=	_firstClusterHigh[0];
		value	<<=	8;
		value	|=	_firstClusterLow[1];
		value	<<=	8;
		value	|=	_firstClusterLow[0];
		return value;
		}
	/** */
	inline void firstCluster(Unsigned32 value) throw(){
		_firstClusterLow[0]		= (Unsigned8)value;
		value	>>=	8;
		_firstClusterLow[1]		= (Unsigned8)value;
		value	>>=	8;
		_firstClusterHigh[0]	= (Unsigned8)value;
		value	>>=	8;
		_firstClusterHigh[1]	= (Unsigned8)value;
		}
	/** */
	inline Unsigned32	fileSize() const throw(){
		Unsigned32
		value	=	_fileSize[3];
		value	<<=	8;
		value	|=	_fileSize[2];
		value	<<=	8;
		value	|=	_fileSize[1];
		value	<<=	8;
		value	|=	_fileSize[0];
		return value;
		}
	/** */
	inline void		fileSize(Unsigned32 value) throw(){
		_fileSize[0]	= (Unsigned8)value;
		value	>>=	8;
		_fileSize[1]	= (Unsigned8)value;
		value	>>=	8;
		_fileSize[2]	= (Unsigned8)value;
		value	>>=	8;
		_fileSize[3]	= (Unsigned8)value;
		}
	/** */
	bool	nameMatch(	const char*	name,
						unsigned	nameLength
						) const throw();
	/** */
	bool	mainMatch(	const char*	main,
						unsigned	mainLength
						) const throw();
	/** */
	bool	extensionMatch(	const char*	extension,
							unsigned	extLength
							) const throw();
	/** Parses, validates and sets the _shortName
		using the name parameter. If the "name"
		parameter does not meet the 8.3 restriction
		or contains any invalid filename characters,
		then this operation will return "true".
	 */
	bool	name(const char* name) throw();
	};

/** */
class Partition {
	public:
		/** */
		const Unsigned32	_rootDirSectors;
		/** */
		const Unsigned32	_firstDataSector;
		/** */
		const Unsigned32	_totalSectors;
		/** */
		const Unsigned8		_sectorsPerCluster;
	public:
		/** */
		Partition(	Unsigned32	rootDirSectors,
					Unsigned32	firstDataSector,
					Unsigned32	totalSectors,
					Unsigned8	sectorsPerCluster
					) throw();
		/** */
		Unsigned32	firstSectorOfCluster(Unsigned32 cluster) const throw();
		/** */
		inline Unsigned8	sectorsPerCluster() const throw(){
			return _sectorsPerCluster;
			}
		/** */
		inline Unsigned32	totalSectors() const throw(){
			return _totalSectors;
			}
	};

/** */
namespace FAT16 {

/** */
struct Entry {
	/** */
	enum{EOC=0xFFF8};
	/** */
	Unsigned8	_value[2];
	/** */
	inline	Unsigned16	value() const throw(){
		Unsigned16
		v	=	_value[1];
		v	<<=	8;
		v	|=	_value[0];
		return v;
		}
	/** */
	inline	void	value(Unsigned16 v) throw(){
		_value[0]	= v;
		v			>>=	8;
		_value[1]	= v;
		}
	/** */
	inline void	markAsEOC() throw(){
		value(0xFFF8);
		}
	/** */
	inline void	markAsFree() throw(){
		value(0);
		}
	/** */
	inline	bool	isEOC() const throw(){
		return value() >= 0xFFF8;
		}
	/** */
	inline	bool	isFree() const throw(){
		return value() == 0;
		}
	};

/** Note: All sector addresses are relative to the
	beginning of this partition.
 */
class Partition : public Oscl::FS::FAT::Partition {
	public:
		/** */
		const Unsigned16	_reservedSectorCount;
		/** */
		const Unsigned16	_bytesPerSector;
		/** */
		const Unsigned32	_fatSize;
		/** */
		const Unsigned32	_firstRootDirSecNum;
	public:
		/** */
		Partition(	Unsigned32	rootDirSectors,
					Unsigned32	firstDataSector,
					Unsigned8	sectorsPerCluster,
					Unsigned16	reservedSectorCount,
					Unsigned16	bytesPerSector,
					Unsigned32	fatSize,
					Unsigned32	firstRootDirSecNum,
					Unsigned32	totalSectors
					) throw();
		/** */
		void	fatEntry(	Unsigned8		fatN,
							Unsigned32		cluster,
							Unsigned32&		fatSector,
							Unsigned32&		fatEntryOffset
							) const throw();
		/** */
		Unsigned32	firstRootDirSecNum() const throw();
	};

/** */
struct BS36 {
	/** */
	Unsigned8	_driveNumber;
	/** */
	Unsigned8	_reserved1;
	/** */
	Unsigned8	_bootSig;
	/** */
	Unsigned8	_volumeID[4];
	/** */
	Unsigned8	_volumeLabel[11];
	/** */
	Unsigned8	_fileSystemType[8];
	/** */
	inline Unsigned32	volumeID() const throw(){
		Unsigned32
		value	=	_volumeID[3];
		value	<<=	8;
		value	|=	_volumeID[2];
		value	<<=	8;
		value	|=	_volumeID[1];
		value	<<=	8;
		value	|=	_volumeID[0];
		return value;
		}
	};
}

/** */
namespace FAT32 {

/** */
struct BPB36 {
	/** */
	Unsigned8	_fatSize32[4];
	/** */
	Unsigned8	_extFlags[2];
	/** */
	Unsigned8	_fsVersion[2];
	/** */
	Unsigned8	_rootCluster[4];
	/** */
	Unsigned8	_fileSystemInfo[2];
	/** */
	Unsigned8	_bkBootSector[2];
	/** */
	Unsigned8	_reserved[12];
	/** */
	inline bool	fatsMirrored() const throw(){
		return !(_extFlags[0] & 0x80);
		}
	/** */
	inline unsigned	activeFAT() const throw(){
		return _extFlags[0] & 0x0F;
		}
	/** */
	inline Unsigned32	fatSize32() const throw(){
		Unsigned32
		value	=	_fatSize32[3];
		value	<<=	8;
		value	|=	_fatSize32[2];
		value	<<=	8;
		value	|=	_fatSize32[1];
		value	<<=	8;
		value	|=	_fatSize32[0];
		return value;
		}
	/** */
	inline Unsigned16	fsVersion() const throw(){
		Unsigned16
		value	=	_fsVersion[1];
		value	<<=	8;
		value	|=	_fsVersion[0];
		return value;
		}
	/** */
	inline Unsigned32	rootCluster() const throw(){
		Unsigned32
		value	=	_rootCluster[3];
		value	<<=	8;
		value	|=	_rootCluster[2];
		value	<<=	8;
		value	|=	_rootCluster[1];
		value	<<=	8;
		value	|=	_rootCluster[0];
		return value;
		}
	/** */
	inline Unsigned16	fileSystemInfo() const throw(){
		Unsigned16
		value	=	_fileSystemInfo[1];
		value	<<=	8;
		value	|=	_fileSystemInfo[0];
		return value;
		}
	/** */
	inline Unsigned16	bkBootSector() const throw(){
		Unsigned16
		value	=	_bkBootSector[1];
		value	<<=	8;
		value	|=	_bkBootSector[0];
		return value;
		}
	};

struct BS64 {
	/** */
	Unsigned8	_driveNumber;
	/** */
	Unsigned8	_reserved1;
	/** */
	Unsigned8	_bootSig;
	/** */
	Unsigned8	_volumeID[4];
	/** */
	Unsigned8	_volumeLabel[11];
	/** */
	Unsigned8	_fileSystemType[8];
	/** */
	inline Unsigned32	volumeID() const throw(){
		Unsigned32
		value	=	_volumeID[3];
		value	<<=	8;
		value	|=	_volumeID[2];
		value	<<=	8;
		value	|=	_volumeID[1];
		value	<<=	8;
		value	|=	_volumeID[0];
		return value;
		}
	};

}


/** */
struct BS {
	/** */
	Unsigned8	_jmpBoot[3];
	/** */
	Unsigned8	_oemName[8];
	/** */
	Unsigned8	_pad11[25];
	union {
		struct {
			FAT16::BS36		_bs36;		// offset 36,26 bytes
			} _fat16;
		struct {
			Unsigned8		_pad[28];	// offset 36, 28 bytes
			FAT32::BS64		_bs64;		// offset 64, 26 bytes
			} _fat32;
		} _type;
	};

/** */
struct BPB {
	/** */
	Unsigned8	_pad0[11];
	/** */
	Unsigned8	_bytesPerSector[2];
	/** */
	Unsigned8	_sectorsPerCluster;
	/** */
	Unsigned8	_reservedSectorCount[2];
	/** */
	Unsigned8	_numberOfFats;
	/** */
	Unsigned8	_rootEntryCount[2];
	/** */
	Unsigned8	_totalSectors16[2];
	/** */
	Unsigned8	_media;
	/** */
	Unsigned8	_fatSize16[2];
	/** */
	Unsigned8	_sectorsPerTrack[2];
	/** */
	Unsigned8	_numberOfHeads[2];
	/** */
	Unsigned8	_hiddenSectors[4];
	/** */
	Unsigned8	_totalSectors32[4];
	/** */
	FAT32::BPB36	_fat32;	// offset 36, 28 bytes long
	/** */
	inline Unsigned16	fatSize16() const throw(){
		Unsigned16
		value	=	_fatSize16[1];
		value	<<=	8;
		value	|=	_fatSize16[0];
		return value;
		}
	/** */
	inline Unsigned16	totalSectors16() const throw(){
		Unsigned16
		value	=	_totalSectors16[1];
		value	<<=	8;
		value	|=	_totalSectors16[0];
		return value;
		}
	/** */
	inline Unsigned16	bytesPerSector() const throw(){
		Unsigned16
		value	=	_bytesPerSector[1];
		value	<<=	8;
		value	|=	_bytesPerSector[0];
		return value;
		}
	/** */
	inline Unsigned16	reservedSectorCount() const throw(){
		Unsigned16
		value	=	_reservedSectorCount[1];
		value	<<=	8;
		value	|=	_reservedSectorCount[0];
		return value;
		}
	/** */
	inline Unsigned16	rootEntryCount() const throw(){
		Unsigned16
		value	=	_rootEntryCount[1];
		value	<<=	8;
		value	|=	_rootEntryCount[0];
		return value;
		}
	/** */
	inline Unsigned16	sectorsPerTrack() const throw(){
		Unsigned16
		value	=	_sectorsPerTrack[1];
		value	<<=	8;
		value	|=	_sectorsPerTrack[0];
		return value;
		}
	/** */
	inline Unsigned16	numberOfHeads() const throw(){
		Unsigned16
		value	=	_numberOfHeads[1];
		value	<<=	8;
		value	|=	_numberOfHeads[0];
		return value;
		}
	/** */
	inline Unsigned32	hiddenSectors() const throw(){
		Unsigned32
		value	=	_hiddenSectors[3];
		value	<<=	8;
		value	|=	_hiddenSectors[2];
		value	<<=	8;
		value	|=	_hiddenSectors[1];
		value	<<=	8;
		value	|=	_hiddenSectors[0];
		return value;
		}
	/** */
	inline Unsigned32	totalSectors32() const throw(){
		Unsigned32	value;
		value	=	_totalSectors32[3];
		value	<<=	8;
		value	|=	_totalSectors32[2];
		value	<<=	8;
		value	|=	_totalSectors32[1];
		value	<<=	8;
		value	|=	_totalSectors32[0];
		return value;
		}
	/** */
	inline Unsigned32	totalSectors() const throw(){
		Unsigned32	value;
		value	= totalSectors16();
		if(!value){
			value	=	totalSectors32();
			}
		return value;
		}
	/** */
	inline Unsigned32	fatSize() const throw(){
		Unsigned32	value;
		value	= fatSize16();
		if(!value){
			value	=	_fat32._fatSize32[3];
			value	<<=	8;
			value	|=	_fat32._fatSize32[2];
			value	<<=	8;
			value	|=	_fat32._fatSize32[1];
			value	<<=	8;
			value	|=	_fat32._fatSize32[0];
			}
		return value;
		}

	/** */
	Unsigned32	firstRootDirSecNum() const throw();

	/** */
	inline Unsigned32	rootDirSectors() const throw(){
		Unsigned32
		value	= (		(rootEntryCount()*32)
					+	(bytesPerSector()-1)
					) / bytesPerSector()
					;
		return value;
		}

	/** */
	inline Unsigned32	firstDataSector() const throw(){
		Unsigned32
		value	=		reservedSectorCount()
					+	(_numberOfFats * fatSize())
					+	rootDirSectors()
					;
		return value;
		}
	/** */
	inline Unsigned32	dataSectors() const throw(){
		Unsigned32
		value	=		totalSectors()
					-	(		reservedSectorCount()
							+	(_numberOfFats * fatSize())
							+	rootDirSectors()
							);
		return value;
		}
	/** */
	inline Unsigned32	countOfClusters() const throw(){
		Unsigned32
		value	= dataSectors() / _sectorsPerCluster;
		return value;
		}
	/** */
	inline bool	isFat12() const throw(){
		return (countOfClusters() < 4085);
		}
	/** */
	inline bool	isFat16() const throw(){
		Unsigned32	nClusters	= countOfClusters();
		return (nClusters >= 4085) && (nClusters < 65525);
		}
	/** */
	inline bool	isFat32() const throw(){
		return (countOfClusters() >= 65525);
		}
	};

/** */
union PartHeader {
	/** */
	BS	_bs;
	/** */
	BPB	_bpb;
	};

}
}
}

#endif
