/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "kernel.h"
#include "oscl/krux/thread/null.h"
#include "oscl/krux/timer/systimer.h"
#include "oscl/krux/thread/twstack.h"

using namespace Oscl::Krux;

Sema::BroadcastBinary		UKClockSema;
SchedPriList                Prioritizer;

static SchedPriListEntry	nullPriority(SchedPriListEntry::lowestPriority);
static SchedPriListEntry	highestPriority(SchedPriListEntry::highestPriority);
static SchedPriListEntry	isrHighPriority(6500);
static SchedPriListEntry	isrMiddlePriority(6000);
static SchedPriListEntry	isrLowPriority(5500);
static SchedPriListEntry	driverHighPriority(5000);
static SchedPriListEntry	driverMiddlePriority(4500);
static SchedPriListEntry	driverLowPriority(4000);
static SchedPriListEntry	coreHighPriority(3500);
static SchedPriListEntry	coreMiddlePriority(3000);
static SchedPriListEntry	coreLowPriority(2500);
static SchedPriListEntry	appHighPriority(2000);
static SchedPriListEntry	appMiddlePriority(1500);
static SchedPriListEntry	appLowPriority(1000);
static SchedPriListEntry	lowestPriority(500);

static 
NullThread<512>		nullThread(nullPriority.priorityBand);

SystemTimerThread	TheSystemTimerThread(UKClockSema);

static ThreadWithStack<1024>	sysTimerThread(	highestPriority.priorityBand,
												TheSystemTimerThread
												);

void	StartKernel() throw(){
	Prioritizer.insertByPriority(&highestPriority);
	Prioritizer.insertByPriority(&isrHighPriority);
	Prioritizer.insertByPriority(&isrMiddlePriority);
	Prioritizer.insertByPriority(&isrLowPriority);
	Prioritizer.insertByPriority(&driverHighPriority);
	Prioritizer.insertByPriority(&driverMiddlePriority);
	Prioritizer.insertByPriority(&driverLowPriority);
	Prioritizer.insertByPriority(&coreHighPriority);
	Prioritizer.insertByPriority(&coreMiddlePriority);
	Prioritizer.insertByPriority(&coreLowPriority);
	Prioritizer.insertByPriority(&appHighPriority);
	Prioritizer.insertByPriority(&appMiddlePriority);
	Prioritizer.insertByPriority(&appLowPriority);
	Prioritizer.insertByPriority(&lowestPriority);
	Prioritizer.insertByPriority(&nullPriority);
	Oscl::Krux::Scheduler::GetScheduler().start();
	}


Oscl::Krux::SchedPriBand&	lowestPriorityBand() throw(){
	return lowestPriority.priorityBand;
	}

Oscl::Krux::SchedPriBand&	highestPriorityBand() throw(){
	return highestPriority.priorityBand;
	}

Oscl::Krux::SchedPriBand&	isrHighPriorityBand() throw(){
	return isrHighPriority.priorityBand;
	}

Oscl::Krux::SchedPriBand&	isrLowPriorityBand() throw(){
	return isrLowPriority.priorityBand;
	}

Oscl::Krux::SchedPriBand&	isrMiddlePriorityBand() throw(){
	return isrMiddlePriority.priorityBand;
	}

Oscl::Krux::SchedPriBand&	driverHighPriorityBand() throw(){
	return driverHighPriority.priorityBand;
	}

Oscl::Krux::SchedPriBand&	driverLowPriorityBand() throw(){
	return driverLowPriority.priorityBand;
	}

Oscl::Krux::SchedPriBand&	driverMiddlePriorityBand() throw(){
	return driverMiddlePriority.priorityBand;
	}

Oscl::Krux::SchedPriBand&	coreHighPriorityBand() throw(){
	return coreHighPriority.priorityBand;
	}

Oscl::Krux::SchedPriBand&	coreLowPriorityBand() throw(){
	return coreLowPriority.priorityBand;
	}

Oscl::Krux::SchedPriBand&	coreMiddlePriorityBand() throw(){
	return coreMiddlePriority.priorityBand;
	}

Oscl::Krux::SchedPriBand&	appHighPriorityBand() throw(){
	return appHighPriority.priorityBand;
	}

Oscl::Krux::SchedPriBand&	appLowPriorityBand() throw(){
	return appLowPriority.priorityBand;
	}

Oscl::Krux::SchedPriBand&	appMiddlePriorityBand() throw(){
	return appMiddlePriority.priorityBand;
	}

