/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "sprg.h"

	.text
.=0x0100
	.globl	__ppcExceptionVectors
__ppcExceptionVectors:
	.globl	__ppc_Reset
__ppc_Reset:
	// First thing to do here is branch saving the
	// link register to an assembly language routine
	// that:
	//	a.	Operates without a stack
	//	b.	Determines the content SMP SPRGs:
	//			- SmpScratchSprg
	//			- SmpStackPtrSprg
	//	c.	Sets GPR1 to the SMP stack pointer.
	//	d.	Sets GPR3 to a non-zero value if this is
	//		the "master" processor that is responsible
	//		for initializing global resources.
	//	e.	Returns to the link register location.
	bl	__sprgInit
	// GPR3 contains "master" processor flag.
	// GPR1 (FramePtrGpr) contains valid stack pointer.
	
	// Create stack frame
	addi	StackPtrGpr,StackPtrGpr,-16
	lis		OsclPpcUisaGPR2,__ppc_Reset@h
	li		OsclPpcUisaGPR2,__ppc_Reset@l
	stw		OsclPpcUisaGPR2,4(StackPtrGpr)
	li		OsclPpcUisaGPR2,0
	stw		OsclPpcUisaGPR2,0(StackPtrGpr)

	// Prepare arguments to ppcReset():
	// GPR3 is already set for "isMasterProcessor"
	// GPR4 becomes "nip" (NIP at time of reset exception)
	mfspr	OsclPpcUisaGPR4,OsclPpcOeaSprSRR0
	// GPR5 becomes "srr1" (MSR before reset exception)
	mfspr	OsclPpcUisaGPR5,OsclPpcOeaSprSRR1
	// GPR6 becomes current "msr"
	mfmsr	OsclPpcUisaGPR6
	// OsclKruxFrame* ppcReset(isMasterProcessor,nip,srr1,msr)
	bl		ppcReset	// Returns OsclKruxFrame* in GPR3/FramePtrGpr
	// Returns OsclKruxFrame pointer in GPR3

	// Restore registers from OsclKruxFrame
	bl		__restoreFrame

	// Load link register from OsclKruxFrame context
	lwz		OsclPpcUisaGPR0,FrameLINK(FramePtrGpr)
	mtlr	OsclPpcUisaGPR0

	// Load GPR0 from OsclKruxFrame
	lwz		OsclPpcUisaGPR0,FrameGPR0(FramePtrGpr)

	// Save OsclKruxFrame pointer in SPRG
	mtspr	FrameSprg,FramePtrGpr

	// Save OsclKruxFrame pointer in SPRG
	mfspr	FramePtrGpr,ExceptScratchSprg
	rfi

.=0x0200
	.globl	__ppcMachineCheck
__ppcMachineCheck:
	mtspr	ExceptScratchSprg,FramePtrGpr
	mfspr	FramePtrGpr,FrameSprg

	stw		OsclPpcUisaGPR0,FrameGPR0(FramePtrGpr)
	mflr	OsclPpcUisaGPR0
	stw		OsclPpcUisaGPR0,FrameLINK(FramePtrGpr)

	bl		__saveFrame

	mfspr	StackPtrGpr,SmpStackPtrSprg

	// OsclKruxFrame* ppcMachineCheck(OsclKruxFrame* GPR3)
	bl		ppcMachineCheck	// Returns OsclKruxFrame* in GPR3/FramePtrGpr

	bl		__restoreFrame

	lwz		OsclPpcUisaGPR0,FrameLINK(FramePtrGpr)
	mtlr	OsclPpcUisaGPR0
	lwz		OsclPpcUisaGPR0,FrameGPR0(FramePtrGpr)

	mtspr	FrameSprg,FramePtrGpr
	mfspr	FramePtrGpr,ExceptScratchSprg
	rfi

.=0x0300
	.globl	__ppcDataAccess
__ppcDataAccess:
	b	.

.=0x0400
	.globl	__ppcInstructionAccess
__ppcInstructionAccess:
	b	.

.=0x0500
	.globl	__ppcExternalInterrupt
__ppcExternalInterrupt:
	mtspr	ExceptScratchSprg,FramePtrGpr
	mfspr	FramePtrGpr,FrameSprg

	stw		OsclPpcUisaGPR0,FrameGPR0(FramePtrGpr)
	mflr	OsclPpcUisaGPR0
	stw		OsclPpcUisaGPR0,FrameLINK(FramePtrGpr)

	bl		__saveFrame

	mfmsr	OsclPpcUisaGPR0		// Prepare to enable MMU
	ori		OsclPpcUisaGPR0,OsclPpcUisaGPR0,48		// Set MMU Enable Bits
	mtmsr	OsclPpcUisaGPR0		// Update MSR

	mfspr	StackPtrGpr,SmpStackPtrSprg

	// OsclKruxFrame* ppcExternalInterrupt(OsclKruxFrame* GPR3)
	bl		ppcExternalInterrupt	// Returns OsclKruxFrame* in GPR3/FramePtrGpr
	bl		Schedule

	mfmsr	OsclPpcUisaGPR0		// Prepare to disable MMU
	rlwinm	OsclPpcUisaGPR0,OsclPpcUisaGPR0,0,28,25	// Clear MMU Enable Bits
	mtmsr	OsclPpcUisaGPR0		// Update MSR

	bl		__restoreFrame

	lwz		OsclPpcUisaGPR0,FrameLINK(FramePtrGpr)
	mtlr	OsclPpcUisaGPR0
	lwz		OsclPpcUisaGPR0,FrameGPR0(FramePtrGpr)

	mtspr	FrameSprg,FramePtrGpr
	mfspr	FramePtrGpr,ExceptScratchSprg
	rfi


.=0x0600
	.globl	__ppcAlignment
__ppcAlignment:
	mfmsr	OsclPpcUisaGPR29
	mfspr	OsclPpcUisaGPR30,OsclPpcOeaSprSRR0
	mfspr	OsclPpcUisaGPR31,OsclPpcOeaSprSRR1
	b	.

.=0x0700
	.globl	__ppcProgramCheck
__ppcProgramCheck:
	b	.

.=0x0800
	.globl	__ppcFpuUnavailable
__ppcFpuUnavailable:
	b	.

.=0x0900
	.globl	__ppcDecrementer
__ppcDecrementer:
	mtspr	ExceptScratchSprg,FramePtrGpr
	mfspr	FramePtrGpr,FrameSprg

	stw		OsclPpcUisaGPR0,FrameGPR0(FramePtrGpr)
	mflr	OsclPpcUisaGPR0
	stw		OsclPpcUisaGPR0,FrameLINK(FramePtrGpr)

	bl		__saveFrame

	mfmsr	OsclPpcUisaGPR0		// Prepare to enable MMU
	ori		OsclPpcUisaGPR0,OsclPpcUisaGPR0,48		// Set MMU Enable Bits
	mtmsr	OsclPpcUisaGPR0		// Update MSR

	mfspr	StackPtrGpr,SmpStackPtrSprg

	// OsclKruxFrame* ppcDecrementer(OsclKruxFrame* GPR3)
	bl		ppcDecrementer	// Returns OsclKruxFrame* in GPR3/FramePtrGpr
	bl		Schedule

	mfmsr	OsclPpcUisaGPR0		// Prepare to disable MMU
	rlwinm	OsclPpcUisaGPR0,OsclPpcUisaGPR0,0,28,25	// Clear MMU Enable Bits
	mtmsr	OsclPpcUisaGPR0		// Update MSR

	bl		__restoreFrame

	lwz		OsclPpcUisaGPR0,FrameLINK(FramePtrGpr)
	mtlr	OsclPpcUisaGPR0
	lwz		OsclPpcUisaGPR0,FrameGPR0(FramePtrGpr)

	mtspr	FrameSprg,FramePtrGpr
	mfspr	FramePtrGpr,ExceptScratchSprg
	rfi

.=0x0A00
	.globl	__ppcTrap0A00
__ppcTrap0A00:
	b	.

.=0x0B00
	.globl	__ppcTrap0B00
__ppcTrap0B00:
	b	.

.=0x0C00
	.globl	__ppcSystemCall
__ppcSystemCall:
	mtspr	ExceptScratchSprg,FramePtrGpr
	mfspr	FramePtrGpr,FrameSprg

	stw		OsclPpcUisaGPR0,FrameGPR0(FramePtrGpr)
	mflr	OsclPpcUisaGPR0
	stw		OsclPpcUisaGPR0,FrameLINK(FramePtrGpr)

	bl		__saveFrame

	mfmsr	OsclPpcUisaGPR0		// Prepare to enable MMU
	ori		OsclPpcUisaGPR0,OsclPpcUisaGPR0,48		// Set MMU Enable Bits
	mtmsr	OsclPpcUisaGPR0		// Update MSR

	mfspr	StackPtrGpr,SmpStackPtrSprg

	// OsclKruxFrame* ppcSystemCall(OsclKruxFrame* GPR3)
	bl		ppcSystemCall	// Returns OsclKruxFrame* in GPR3/FramePtrGpr
	bl		Schedule

	mfmsr	OsclPpcUisaGPR0		// Prepare to disable MMU
	rlwinm	OsclPpcUisaGPR0,OsclPpcUisaGPR0,0,28,25	// Clear MMU Enable Bits
	mtmsr	OsclPpcUisaGPR0		// Update MSR

	bl		__restoreFrame

	lwz		OsclPpcUisaGPR0,FrameLINK(FramePtrGpr)
	mtlr	OsclPpcUisaGPR0
	lwz		OsclPpcUisaGPR0,FrameGPR0(FramePtrGpr)

	mtspr	FrameSprg,FramePtrGpr
	mfspr	FramePtrGpr,ExceptScratchSprg
	rfi

.=0x0D00
	.globl	__ppcTrace
__ppcTrace:
	b	.

.=0x0E00
	.globl	__ppcTrap0E00
__ppcTrap0E00:
	b	.

.=0x0F00
	.globl	__ppcTrap0F00
__ppcTrap0F00:
	b	.

.=0x1000
	.globl	__ppcImplementationSpecific
__ppcImplementationSpecific:
	.globl	__ppc8xxSoftwareEmulation
__ppc8xxSoftwareEmulation:
	mfmsr	OsclPpcUisaGPR29
	mfspr	OsclPpcUisaGPR30,OsclPpcOeaSprSRR0
	mfspr	OsclPpcUisaGPR31,OsclPpcOeaSprSRR1
	b	.

.=0x1100
	.globl	__ppc8xxInstructionTlbMiss
__ppc8xxInstructionTlbMiss:
	mtspr	OsclMot8xxSprM_TW,OsclPpcUisaGPR1
	mfspr	OsclPpcUisaGPR1,OsclPpcOeaSprSRR0
	mtspr	OsclMot8xxSprMD_EPN,OsclPpcUisaGPR1
	mfspr	OsclPpcUisaGPR1,OsclMot8xxSprM_TWB
	lwz		OsclPpcUisaGPR1,0(OsclPpcUisaGPR1)
	mtspr	OsclMot8xxSprMI_TWC,OsclPpcUisaGPR1
	mtspr	OsclMot8xxSprMD_TWC,OsclPpcUisaGPR1
	mfspr	OsclPpcUisaGPR1,OsclMot8xxSprMD_TWC
	lwz		OsclPpcUisaGPR1,0(OsclPpcUisaGPR1)
	mtspr	OsclMot8xxSprMI_RPN,OsclPpcUisaGPR1
	mfspr	OsclPpcUisaGPR1,OsclMot8xxSprM_TW
	rfi

.=0x1200
	.globl	__ppc8xxDataTlbMiss
__ppc8xxDataTlbMiss:
	mtspr	OsclMot8xxSprM_TW,OsclPpcUisaGPR1
//
#if 0
	mfspr	OsclPpcUisaGPR1,SmpScratchSprg
	sync
	stw		OsclPpcUisaGPR2,SmpScratchAreaGPR2(OsclPpcUisaGPR1)
	mfspr	OsclPpcUisaGPR2,OsclPpcOeaSprSRR0
	sync
	stw		OsclPpcUisaGPR2,SmpScratchAreaSRR0(OsclPpcUisaGPR1)
	mfspr	OsclPpcUisaGPR2,OsclPpcOeaSprSRR1
	sync
	stw		OsclPpcUisaGPR2,SmpScratchAreaSRR1(OsclPpcUisaGPR1)
	lwz		OsclPpcUisaGPR2,SmpScratchAreaGPR2(OsclPpcUisaGPR1)
#endif
//
	mfspr	OsclPpcUisaGPR1,OsclMot8xxSprM_TWB
	sync
	lwz		OsclPpcUisaGPR1,0(OsclPpcUisaGPR1)
	mtspr	OsclMot8xxSprMD_TWC,OsclPpcUisaGPR1
	mfspr	OsclPpcUisaGPR1,OsclMot8xxSprMD_TWC
	lwz		OsclPpcUisaGPR1,0(OsclPpcUisaGPR1)
	mtspr	OsclMot8xxSprMD_RPN,OsclPpcUisaGPR1
	mfspr	OsclPpcUisaGPR1,OsclMot8xxSprM_TW
	sync
	rfi

.=0x1300
	.globl	__ppc8xxInstructionTlbError
__ppc8xxInstructionTlbError:
	mtspr	ExceptScratchSprg,FramePtrGpr
	mfspr	FramePtrGpr,FrameSprg

	stw		OsclPpcUisaGPR0,FrameGPR0(FramePtrGpr)
	mflr	OsclPpcUisaGPR0
	stw		OsclPpcUisaGPR0,FrameLINK(FramePtrGpr)

	bl		__saveFrame

//	mfmsr	OsclPpcUisaGPR0		// Prepare to enable MMU
//	ori		OsclPpcUisaGPR0,OsclPpcUisaGPR0,48		// Set MMU Enable Bits
//	mtmsr	OsclPpcUisaGPR0		// Update MSR

	mfspr	StackPtrGpr,SmpStackPtrSprg

	// OsclKruxFrame* ppc8xxInstructionTlbError(OsclKruxFrame* GPR3)
	bl		ppc8xxInstructionTlbError	// Returns OsclKruxFrame* in GPR3/FramePtrGpr
	bl		Schedule

//	mfmsr	OsclPpcUisaGPR0		// Prepare to disable MMU
//	rlwinm	OsclPpcUisaGPR0,OsclPpcUisaGPR0,0,28,25	// Clear MMU Enable Bits
//	mtmsr	OsclPpcUisaGPR0		// Update MSR

	bl		__restoreFrame

	lwz		OsclPpcUisaGPR0,FrameLINK(FramePtrGpr)
	mtlr	OsclPpcUisaGPR0
	lwz		OsclPpcUisaGPR0,FrameGPR0(FramePtrGpr)

	mtspr	FrameSprg,FramePtrGpr
	mfspr	FramePtrGpr,ExceptScratchSprg
	rfi
//	b	.

.=0x1400
	.globl	__ppc8xxDataTlbError
__ppc8xxDataTlbError:
	mtspr	ExceptScratchSprg,FramePtrGpr
	mfspr	FramePtrGpr,FrameSprg

	stw		OsclPpcUisaGPR0,FrameGPR0(FramePtrGpr)
	mflr	OsclPpcUisaGPR0
	stw		OsclPpcUisaGPR0,FrameLINK(FramePtrGpr)

	bl		__saveFrame

//	mfmsr	OsclPpcUisaGPR0		// Prepare to enable MMU
//	ori		OsclPpcUisaGPR0,OsclPpcUisaGPR0,48		// Set MMU Enable Bits
//	mtmsr	OsclPpcUisaGPR0		// Update MSR

	mfspr	StackPtrGpr,SmpStackPtrSprg

	// OsclKruxFrame* ppc8xxDataTlbError(OsclKruxFrame* GPR3)
	bl		ppc8xxDataTlbError	// Returns OsclKruxFrame* in GPR3/FramePtrGpr
	bl		Schedule

//	mfmsr	OsclPpcUisaGPR0		// Prepare to disable MMU
//	rlwinm	OsclPpcUisaGPR0,OsclPpcUisaGPR0,0,28,25	// Clear MMU Enable Bits
//	mtmsr	OsclPpcUisaGPR0		// Update MSR

	bl		__restoreFrame

	lwz		OsclPpcUisaGPR0,FrameLINK(FramePtrGpr)
	mtlr	OsclPpcUisaGPR0
	lwz		OsclPpcUisaGPR0,FrameGPR0(FramePtrGpr)

	mtspr	FrameSprg,FramePtrGpr
	mfspr	FramePtrGpr,ExceptScratchSprg
	rfi
//	b	.

.=0x1500
	.globl	__ppcTrap1500
__ppcTrap1500:
	b	.

.=0x1600
	.globl	__ppcTrap1600
__ppcTrap1600:
	b	.

.=0x1700
	.globl	__ppcTrap1700
__ppcTrap1700:
	b	.

.=0x1800
	.globl	__ppcTrap1800
__ppcTrap1800:
	b	.

.=0x1900
	.globl	__ppcTrap1900
__ppcTrap1900:
	b	.

.=0x1A00
	.globl	__ppcTrap1A00
__ppcTrap1A00:
	b	.

.=0x1B00
	.globl	__ppcTrap1B00
__ppcTrap1B00:
	b	.

.=0x1C00
	.globl	__ppcDataBreakpoint
__ppcDataBreakpoint:
	b	.

.=0x1D00
	.globl	__ppcInstructionBreakpoint
__ppcInstructionBreakpoint:
	b	.

.=0x1E00
	.globl	__ppcPeripheralBreakpoint
__ppcPeripheralBreakpoint:
	b	.

.=0x1F00
	.globl	__ppcDevelopmentPort
__ppcDevelopmentPort:
	b	.

.=0x2000
	.globl	__ppcPageTable
__ppcPageTable:
