/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "sprg.h"

	.text
	.globl	StartOOOS
/*************************
 * StartOOOS
 * Input: Frame pointer in GPR3
 */
StartOOOS:

	// Restore registers from Frame
	bl		__restoreFrame

	// Load link register from Frame context
	lwz		OsclPpcUisaGPR0,FrameLINK(FramePtrGpr)
	mtlr	OsclPpcUisaGPR0

	// Load GPR0 from Frame
	lwz		OsclPpcUisaGPR0,FrameGPR0(FramePtrGpr)

	// Save Frame pointer in SPRG
	mtspr	FrameSprg,FramePtrGpr

	// Save Frame pointer in SPRG
	mfspr	FramePtrGpr,ExceptScratchSprg
	rfi

// This subroutine saves all GPRs into the Frame
// structure except FramePtrGpr and OsclPpcUisaGPR0.
// It also saves CR,XER,CTR,SRR0,and SRR1.
// Expects FramePtrGpr to contain a pointer to the Frame
// to be restored.
// This routine clobbers all GPRs except FramePtrGpr.
// On return:
//	ExceptScratchSprg contains the new frame pointer.
//  GPR0 is clobbered.
	.globl	__saveFrame
__saveFrame:
	stw		OsclPpcUisaGPR1,FrameGPR1(FramePtrGpr)
	stw		OsclPpcUisaGPR2,FrameGPR2(FramePtrGpr)
	stw		OsclPpcUisaGPR4,FrameGPR4(FramePtrGpr)
	stw		OsclPpcUisaGPR5,FrameGPR5(FramePtrGpr)
	stw		OsclPpcUisaGPR6,FrameGPR6(FramePtrGpr)
	stw		OsclPpcUisaGPR7,FrameGPR7(FramePtrGpr)
	stw		OsclPpcUisaGPR8,FrameGPR8(FramePtrGpr)
	stw		OsclPpcUisaGPR9,FrameGPR9(FramePtrGpr)
	stw		OsclPpcUisaGPR10,FrameGPR10(FramePtrGpr)
	stw		OsclPpcUisaGPR11,FrameGPR11(FramePtrGpr)
	stw		OsclPpcUisaGPR12,FrameGPR12(FramePtrGpr)
	stw		OsclPpcUisaGPR13,FrameGPR13(FramePtrGpr)
	stw		OsclPpcUisaGPR14,FrameGPR14(FramePtrGpr)
	stw		OsclPpcUisaGPR15,FrameGPR15(FramePtrGpr)
	stw		OsclPpcUisaGPR16,FrameGPR16(FramePtrGpr)
	stw		OsclPpcUisaGPR17,FrameGPR17(FramePtrGpr)
	stw		OsclPpcUisaGPR18,FrameGPR18(FramePtrGpr)
	stw		OsclPpcUisaGPR19,FrameGPR19(FramePtrGpr)
	stw		OsclPpcUisaGPR20,FrameGPR20(FramePtrGpr)
	stw		OsclPpcUisaGPR21,FrameGPR21(FramePtrGpr)
	stw		OsclPpcUisaGPR22,FrameGPR22(FramePtrGpr)
	stw		OsclPpcUisaGPR23,FrameGPR23(FramePtrGpr)
	stw		OsclPpcUisaGPR24,FrameGPR24(FramePtrGpr)
	stw		OsclPpcUisaGPR25,FrameGPR25(FramePtrGpr)
	stw		OsclPpcUisaGPR26,FrameGPR26(FramePtrGpr)
	stw		OsclPpcUisaGPR27,FrameGPR27(FramePtrGpr)
	stw		OsclPpcUisaGPR28,FrameGPR28(FramePtrGpr)
	stw		OsclPpcUisaGPR29,FrameGPR29(FramePtrGpr)
	stw		OsclPpcUisaGPR30,FrameGPR30(FramePtrGpr)
	stw		OsclPpcUisaGPR31,FrameGPR31(FramePtrGpr)

	mfspr	OsclPpcUisaGPR0,OsclPpcOeaSprSRR0
	stw		OsclPpcUisaGPR0,FrameNIP(FramePtrGpr)
	mfspr	OsclPpcUisaGPR0,OsclPpcOeaSprSRR1
	stw		OsclPpcUisaGPR0,FrameMSR(FramePtrGpr)
	mtspr	OsclMot8xxSprEID,OsclPpcUisaGPR0
	mfctr	OsclPpcUisaGPR0
	stw		OsclPpcUisaGPR0,FrameCTR(FramePtrGpr)
	mfxer	OsclPpcUisaGPR0
	stw		OsclPpcUisaGPR0,FrameXER(FramePtrGpr)
	mfcr	OsclPpcUisaGPR0
	stw		OsclPpcUisaGPR0,FrameCR(FramePtrGpr)

	mfspr	OsclPpcUisaGPR0,ExceptScratchSprg
	stw		OsclPpcUisaGPR0,FrameGPR3(FramePtrGpr)

	blr

// This subroutine restores all GPRs except FramePtrGpr
// and OsclPpcUisaGPR0. It also restores CR,XER,CTR,SRR0,and SRR1.
// Expects FramePtrGpr to contain a pointer to the Frame
// to be restored.
// This routine clobbers all GPRs except FramePtrGpr.
// On return:
//	ExceptScratchSprg contains the new frame pointer.
//  GPR0 is clobbered.
	.globl	__restoreFrame
__restoreFrame:
	lwz		OsclPpcUisaGPR0,FrameGPR3(FramePtrGpr)
	mtspr	ExceptScratchSprg,OsclPpcUisaGPR0

	lwz		OsclPpcUisaGPR0,FrameCR(FramePtrGpr)
	mtcr	OsclPpcUisaGPR0
	lwz		OsclPpcUisaGPR0,FrameXER(FramePtrGpr)
	mtxer	OsclPpcUisaGPR0
	lwz		OsclPpcUisaGPR0,FrameCTR(FramePtrGpr)
	mtctr	OsclPpcUisaGPR0
	lwz		OsclPpcUisaGPR0,FrameMSR(FramePtrGpr)
	mtspr	OsclMot8xxSprNRI,OsclPpcUisaGPR0
	mtspr	OsclPpcOeaSprSRR1,OsclPpcUisaGPR0
	lwz		OsclPpcUisaGPR0,FrameNIP(FramePtrGpr)
	mtspr	OsclPpcOeaSprSRR0,OsclPpcUisaGPR0

	lwz		OsclPpcUisaGPR31,FrameGPR31(FramePtrGpr)
	lwz		OsclPpcUisaGPR30,FrameGPR30(FramePtrGpr)
	lwz		OsclPpcUisaGPR29,FrameGPR29(FramePtrGpr)
	lwz		OsclPpcUisaGPR28,FrameGPR28(FramePtrGpr)
	lwz		OsclPpcUisaGPR27,FrameGPR27(FramePtrGpr)
	lwz		OsclPpcUisaGPR26,FrameGPR26(FramePtrGpr)
	lwz		OsclPpcUisaGPR25,FrameGPR25(FramePtrGpr)
	lwz		OsclPpcUisaGPR24,FrameGPR24(FramePtrGpr)
	lwz		OsclPpcUisaGPR23,FrameGPR23(FramePtrGpr)
	lwz		OsclPpcUisaGPR22,FrameGPR22(FramePtrGpr)
	lwz		OsclPpcUisaGPR21,FrameGPR21(FramePtrGpr)
	lwz		OsclPpcUisaGPR20,FrameGPR20(FramePtrGpr)
	lwz		OsclPpcUisaGPR19,FrameGPR19(FramePtrGpr)
	lwz		OsclPpcUisaGPR18,FrameGPR18(FramePtrGpr)
	lwz		OsclPpcUisaGPR17,FrameGPR17(FramePtrGpr)
	lwz		OsclPpcUisaGPR16,FrameGPR16(FramePtrGpr)
	lwz		OsclPpcUisaGPR15,FrameGPR15(FramePtrGpr)
	lwz		OsclPpcUisaGPR14,FrameGPR14(FramePtrGpr)
	lwz		OsclPpcUisaGPR13,FrameGPR13(FramePtrGpr)
	lwz		OsclPpcUisaGPR12,FrameGPR12(FramePtrGpr)
	lwz		OsclPpcUisaGPR11,FrameGPR11(FramePtrGpr)
	lwz		OsclPpcUisaGPR10,FrameGPR10(FramePtrGpr)
	lwz		OsclPpcUisaGPR9,FrameGPR9(FramePtrGpr)
	lwz		OsclPpcUisaGPR8,FrameGPR8(FramePtrGpr)
	lwz		OsclPpcUisaGPR7,FrameGPR7(FramePtrGpr)
	lwz		OsclPpcUisaGPR6,FrameGPR6(FramePtrGpr)
	lwz		OsclPpcUisaGPR5,FrameGPR5(FramePtrGpr)
	lwz		OsclPpcUisaGPR4,FrameGPR4(FramePtrGpr)
	lwz		OsclPpcUisaGPR2,FrameGPR2(FramePtrGpr)
	lwz		OsclPpcUisaGPR1,FrameGPR1(FramePtrGpr)
	blr

