/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_logic_lsi_integratorh_
#define _oscl_logic_lsi_integratorh_
#include "oscl/logic/seq/api.h"
#include "intapi.h"
#include "oscl/logic/output.h"

/** */
namespace Oscl {
/** */
namespace Logic {
/** */
namespace Lsi {

/** */
class Integ :	public IntegApi,
				protected Seq::InputApi,
				protected Seq::Api
				{
	private:
		/** */
		class Inhibit :	public Logic::InputApi,
						public Logic::StateApi
						{
			private:
				/** */
				Integ&		_context;
				/** */
				bool		_output;
				/** */
				unsigned	_assertedCount;
			public:
				/** */
				explicit Inhibit(Integ& context) throw();
				/** */
				const bool&	getOutput() throw();
			public:
				/** */
				void	attach(	Logic::OutputApi&	output,
								Logic::Wire&		wire)
								throw();
				/** */
				void	detach(	Logic::OutputApi&	output,
								Logic::Wire&		wire
								) throw();
			public:
				/** */
				void	asserted() throw();
				/** */
				void	negated() throw();
			};
		/** */
		class Set :	public Logic::InputApi,
					public Logic::StateApi
					{
			private:
				/** */
				Integ&	_context;
				/** */
				bool				_output;
				/** */
				unsigned			_assertedCount;
			public:
				/** */
				explicit Set(Integ& context) throw();
				/** */
				const bool&	getOutput() throw();
			public:
				/** */
				void	attach(	Logic::OutputApi&	output,
								Logic::Wire&		wire
								) throw();
				/** */
				void	detach(	Logic::OutputApi&	output,
								Logic::Wire&		wire
								) throw();
			public:
				/** */
				void	asserted() throw();
				/** */
				void	negated() throw();
			};

		/** */
		class Clear :	public Logic::InputApi,
						public Logic::StateApi
						{
			private:
				/** */
				Integ&		_context;
				/** */
				bool		_output;
				/** */
				unsigned	_assertedCount;
			public:
				/** */
				explicit Clear(Integ& context) throw();
				/** */
				const bool&	getOutput() throw();
			public:
				/** */
				void	attach(	Logic::OutputApi&	output,
								Logic::Wire&		wire
								) throw();
				/** */
				void	detach(	Logic::OutputApi&	output,
								Logic::Wire&		wire
								) throw();
			public:
				/** */
				void	asserted() throw();
				/** */
				void	negated() throw();
			};
	private:
		/** */
		signed int					_maxValue;
		/** */
		signed int					_currentCount;
		/** */
		signed int					_upCountValue;
		/** */
		signed int					_downCountValue;
		/** */
		static const signed int		_minValue	= 0;
	private:
		/** */
		Logic::Output	_output;
		/** */
		Inhibit			_inhibitInput;
		/** */
		Set				_setInput;
		/** */
		Clear			_clearInput;
		/** */
		const bool&		_inhibited;
		/** */
		const bool&		_set;
		/** */
		const bool&		_cleared;
		/** */
		bool			_integratorState;
	public:
		/** */
		explicit Integ(	const bool&	dataInput,
						unsigned	nTicks,
						unsigned	upToDownRatio
						) throw();
	public:
		/** */
		Seq::InputApi&		getClockInput() throw();
		/** */
		Logic::OutputApi&	getOutput() throw();
		/** */
		Logic::InputApi&	getInhibit() throw();
		/** */
		Logic::InputApi&	getSet() throw();
		/** */
		Logic::InputApi&	getClear() throw();
	private:
		/** */
		void	attach(Seq::SubjectApi& output,Seq::Wire& wire) throw();
		/** */
		void	detach(Seq::SubjectApi& output,Seq::Wire& wire) throw();
	private:
		/** */
		void	tick() throw();
	public:
		/** */
		void	clearAsserted() throw();
		/** */
		void	clearNegated() throw();
		/** */
		void	setAsserted() throw();
		/** */
		void	setNegated() throw();
		/** */
		void	evaluateOutput() throw();
	};
};
};
};

#endif
