/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_ms_fat_partitionh_
#define _oscl_ms_fat_partitionh_
#include "oscl/compiler/types.h"

/** */
namespace Oscl {
/** Mass Storage */
namespace MS {
/** */
namespace FAT {

/** */
class Geometry {
	private:
		/** */
		const unsigned long	_totalSectors;
		/** */
		const unsigned long	_totalHeads;
	public:
		/** */
		Geometry(	unsigned long	totalSectors,
					unsigned long	totalHeads
					) throw():
			_totalSectors(totalSectors),
			_totalHeads(totalHeads)
			{}
	public:
		inline Unsigned32 chsToLBA(	unsigned		cylinder,
									unsigned		head,
									unsigned long	sector
									) throw(){
			return		(sector -1)
					+	(head * _totalSectors)
					+	(cylinder * (_totalHeads+1) * _totalSectors);
			}
		inline void		lbaToCHS(	Unsigned32		lba,
									unsigned&		cylinder,
									unsigned&		head,
									unsigned long&	sector
									) throw(){
			const Unsigned32	cylhead	= lba/_totalSectors;
			sector		= (lba%_totalSectors)+1;
			head		= cylhead%(_totalHeads+1);
			cylinder	= cylhead/(_totalHeads+1);
			}
	};

/** */
namespace Partition {

/** */
struct CHS {
	/** */
	Unsigned8		_head;
	/** */
	Unsigned8		_cylinderSector[2];
	/** */
	inline	Unsigned32	chs() const throw(){
		Unsigned32	value	= _head;
		value <<=	8;
		value	|= _cylinderSector[1];
		value <<=	8;
		value	|= _cylinderSector[0];
		return value;
		}
	};

/** */
struct PartitionEntry {
	/** */
	Unsigned8		_state;
	/** */
	CHS				_start;
	/** */
	Unsigned8		_type;
	/** */
	CHS				_end;
	/** */
	Unsigned8		_startingLBA[4];
	/** */
	Unsigned8		_lengthInSectors[4];
	/** */
	enum{
			/** */
			typeUnknown					= 0x00,
			/** */
			typeFat12					= 0x01,
			/** */
			typeSmallFat16				= 0x04,
			/** */
			typeExtendedPartition		= 0x05,
			/** */
			typeLargeFat16				= 0x06,
			/** */
			typeFat32					= 0x0B,
			/** */
			typeFat32LBA				= 0x0C,
			/** */
			typeLargeFat16LBA			= 0x0E,
			/** */
			typeExtendedPartitionLBA	= 0x0F
			};

	/** */
	inline	bool	isActive() const throw(){
		return _state & 0x80;
		}

	/** */
	inline	Unsigned32	startingLBA() const throw(){
		Unsigned32
		value	=	_startingLBA[3];
		value	<<=	8;
		value	|=	_startingLBA[2];
		value	<<=	8;
		value	|=	_startingLBA[1];
		value	<<=	8;
		value	|=	_startingLBA[0];
		return value;
		}
	inline	Unsigned32	lengthInSectors() const throw(){
		Unsigned32
		value	=	_lengthInSectors[3];
		value	<<=	8;
		value	|=	_lengthInSectors[2];
		value	<<=	8;
		value	|=	_lengthInSectors[1];
		value	<<=	8;
		value	|=	_lengthInSectors[0];
		return value;
		}
	};

/** */
struct MBR {
	/** */
	Unsigned8		_code[446];
	/** */
	PartitionEntry	_pe[4];
	/** */
	Unsigned8		_marker[2];
	/** */
	inline bool	validMarker() const throw(){
		return (_marker[0] == 0x55) && (_marker[1] == 0xAA);
		}
	};

void print(const MBR& resp) throw();

}
}
}
}

#endif
