/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_mt_itc_dyn_adv_monitor_fsmh_
#define _oscl_mt_itc_dyn_adv_monitor_fsmh_

/** */
namespace Oscl {
/** */
namespace Mt {
/** */
namespace Itc {
/** */
namespace Dyn {
/** */
namespace Adv {
/** */
namespace Monitor {

/** */
class StateVar {
	public:
		/** */
		class ContextApi {
			public:
				/** */
				virtual ~ContextApi(){}
			public:
				/** */
				virtual void	sendCreateNotificationReq() throw()=0;
				/** */
				virtual void	openRequestDone() throw()=0;
				/** */
				virtual void	closeRequestDone() throw()=0;
				/** */
				virtual void	activateNewMonitors() throw()=0;
				/** */
				virtual void	queueDeactivateReq() throw()=0;
				/** */
				virtual void	deactivateNextMonitor() throw()=0;
				/** */
				virtual void	moveActiveToDeactivateList() throw()=0;
				/** */
				virtual void	cancelCreateNotificationReq() throw()=0;
			public:
				/** */
				virtual bool	newDynSrvListNotEmpty() throw()=0;
				/** */
				virtual bool	deactivateListIsEmpty() throw()=0;
				/** */
				virtual bool	activeListIsEmpty() throw()=0;
			};
	private:
		/** */
		class State {
			public:
				virtual ~State(){}
			public:
				/** */
				virtual void	open(	ContextApi&	context,
										StateVar&	internal
										) const throw();
				/** */
				virtual void	close(	ContextApi&	context,
										StateVar&	internal
										) const throw();

				/** */
				virtual void	create(	ContextApi&	context,
										StateVar&	internal
										) const throw();

				/** */
				virtual void	deactivate(	ContextApi&	context,
											StateVar&	internal
											) const throw();

				/** */
				virtual void	deactivated(	ContextApi&	context,
												StateVar&	internal
												) const throw();

				/** */
				virtual void	canceled(	ContextApi&	context,
											StateVar&	internal
											) const throw();
			};
		/** */
		class ClosedState : public State {
			private:
				/** */
				void	open(	ContextApi&	context,
								StateVar&	internal
								) const throw();
			};
		/** */
		class OpenedState : public State {
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	internal
								) const throw();
				/** */
				void	create(	ContextApi&	context,
								StateVar&	internal
								) const throw();
				/** */
				void	deactivate(	ContextApi&	context,
									StateVar&	internal
									) const throw();
			};
		/** */
		class DeactivatingState : public State {
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	internal
								) const throw();
				/** */
				void	create(	ContextApi&	context,
								StateVar&	internal
								) const throw();
				/** */
				void	deactivated(	ContextApi&	context,
										StateVar&	internal
										) const throw();
				/** */
				void	deactivate(	ContextApi&	context,
									StateVar&	internal
									) const throw();
			};
		/** */
		class ClosingState : public State {
			private:
				/** */
				void	create(	ContextApi&	context,
								StateVar&	internal
								) const throw();
				/** */
				void	deactivated(	ContextApi&	context,
										StateVar&	internal
										) const throw();
				/** */
				void	deactivate(	ContextApi&	context,
									StateVar&	internal
									) const throw();
			};
		/** */
		class CancelingState : public State {
			private:
				/** */
				void	create(	ContextApi&	context,
								StateVar&	internal
								) const throw();
				/** */
				void	canceled(	ContextApi&	context,
									StateVar&	internal
									) const throw();
			};
	private:
		/** */
		static const ClosedState		_closedState;
		/** */
		static const OpenedState		_openedState;
		/** */
		static const DeactivatingState	_deactivatingState;
		/** */
		static const ClosingState		_closingState;
		/** */
		static const CancelingState		_cancelingState;
	private:
		/** */
		const State*					_state;
		/** */
		ContextApi&						_context;
	public:
		/** */
		StateVar(ContextApi& context) throw();
		/** */
		void	open() throw();
		/** */
		void	close() throw();
		/** */
		void	create() throw();
		/** */
		void	deactivate() throw();
		/** */
		void	deactivated() throw();
		/** */
		void	canceled() throw();
	private:
		friend class State;
		friend class ClosedState;
		friend class OpenedState;
		friend class DeactivatingState;
		friend class ClosingState;
		friend class CancelingState;
	private:
		/** */
		void	protocolError() throw();
	private:
		/** */
		void	changeToOpenedState() throw();
		/** */
		void	changeToClosedState() throw();
		/** */
		void	changeToDeactivatingState() throw();
		/** */
		void	changeToClosingState() throw();
		/** */
		void	changeToCancelingState() throw();
	};

};
};
};
};
};
};

#endif
