/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_mt_itc_dyn_monitor_ds_fsmh_
#define _oscl_mt_itc_dyn_monitor_ds_fsmh_

/** */
namespace Oscl {
/** */
namespace Mt {
/** */
namespace Itc {
/** */
namespace Dyn {
/** */
namespace Monitor {

/** */
class StateVar {
	public:
		/** */
		class ContextApi {
			public:
				/** */
				virtual ~ContextApi(){}
			public:
				/** */
				virtual void	requestReleaseNotification() throw()=0;
				/** */
				virtual void	activateMonitors() throw()=0;
				/** */
				virtual void	deactivateMonitors() throw()=0;
				/** */
				virtual void	release() throw()=0;
				/** */
				virtual void	cancelReleaseNotification() throw()=0;
				/** */
				virtual void	monitorDeactivated() throw()=0;
			};
	private:
		/** */
		class State {
			public:
				virtual ~State(){}
			public:
				/** */
				virtual void	activate(	ContextApi&	context,
											StateVar&	statevar
											) const throw();
				/** */
				virtual void	releaseNotification(	ContextApi&	context,
														StateVar&	statevar
														) const throw();

				/** */
				virtual void	deactivate(	ContextApi&	context,
											StateVar&	statevar
											) const throw();

				/** */
				virtual void	deactivated(	ContextApi&	context,
												StateVar&	statevar
												) const throw();
				/** */
				virtual void	releaseNotificationCanceled(
												ContextApi&	context,
												StateVar&	statevar
												) const throw();
			};
		/** */
		class DeactivatedState : public State {
			private:
				/** */
				void	activate(	ContextApi&	context,
									StateVar&	statevar
									) const throw();
			};
		/** */
		class ActivatedState : public State {
			private:
				/** */
				void	releaseNotification(	ContextApi&	context,
												StateVar&	statevar
												) const throw();

				/** */
				void	deactivate(	ContextApi&	context,
									StateVar&	statevar
									) const throw();
			};
		/** */
		class DeactivatingState : public State {
			private:
				/** */
				void	deactivated(	ContextApi&	context,
										StateVar&	statevar
										) const throw();
				/** */
				void	deactivate(	ContextApi&	context,
									StateVar&	statevar
									) const throw();
			};
		/** */
		class CancelingState : public State {
			private:
				/** */
				void	releaseNotification(	ContextApi&	context,
												StateVar&	statevar
												) const throw();
				/** */
				void	releaseNotificationCanceled(	ContextApi&	context,
														StateVar&	statevar
														) const throw();
			};
	private:
		/** */
		static const DeactivatedState	_deactivatedState;
		/** */
		static const ActivatedState		_activatedState;
		/** */
		static const CancelingState		_cancelingState;
		/** */
		static const DeactivatingState	_deactivatingState;
	private:
		/** */
		const State*					_state;
		/** */
		ContextApi&						_context;
	public:
		/** */
		StateVar(ContextApi& context) throw();
		/** */
		void	activate() throw();
		/** */
		void	releaseNotification() throw();
		/** */
		void	deactivate() throw();
		/** */
		void	deactivated() throw();
		/** */
		void	releaseNotificationCanceled() throw();
	private:
		friend class State;
		friend class ActivatedState;
		friend class CancelingState;
		friend class DeactivatingState;
		friend class DeactivatedState;
	private:
		/** */
		void	protocolError() throw();
	private:
		/** */
		void	changeToActivatedState() throw();
		/** */
		void	changeToCancelingState() throw();
		/** */
		void	changeToDeactivatingState() throw();
		/** */
		void	changeToDeactivatedState() throw();
	};

};
};
};
};
};

#endif
