/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_mt_itc_dyn_monitor_ds_monh_
#define _oscl_mt_itc_dyn_monitor_ds_monh_
#include <new>
#include "fsm.h"
#include "oscl/memory/block.h"
#include "oscl/mt/itc/dyn/cli/respapi.h"
#include "oscl/mt/itc/dyn/core/handle.h"

/** */
namespace Oscl {
/** */
namespace Mt {
/** */
namespace Itc {
/** */
namespace Dyn {
/** */
namespace Cli {

/** */
template <class subClassType, class deactMemType, class dynSrvType>
class Monitor : public Mt::Itc::Dyn::Cli::Resp::Api<dynSrvType>,
				public Mt::Itc::Dyn::Monitor::StateVar::ContextApi
				{
	public:
		/** */
		class ContextApi {
			public:
				/** */
				virtual ~ContextApi(){}
				/** */
				virtual void	release(subClassType& thisSc) throw()=0;
				/** */
				virtual void	deactivated(subClassType& thisSc) throw()=0;
			};
	public:
		/** */
		typedef struct ReleaseMem {
			Memory::AlignedBlock<
				sizeof(	typename Mt::Itc::Dyn::Cli::Req::Api<dynSrvType>::
						CancelReleasePayload
						)
				> _payload;
			Memory::AlignedBlock<
				sizeof(	typename Mt::Itc::Dyn::Cli::Resp::Api<dynSrvType>::
						CancelReleaseResp
						)
				> _response;
			} ReleaseMem;
		/** */
		typedef union DeactivateMem {
			ReleaseMem		_release;
			deactMemType	_mpDeactivateMem;
			} DeactivateMem;
	private:
		/** */
		Mt::Itc::Dyn::Monitor::StateVar			_state;
		/** */
		ContextApi&								_context;
		/** */
		Mt::Itc::Dyn::Handle<dynSrvType>		_dynSrv;
		/** */
		Mt::Itc::PostMsgApi&					_myPapi;
		/** */
		typename Mt::Itc::Dyn::Cli::Req::Api<dynSrvType>::
		ReleasePayload							_releasePayload;
		/** */
		typename Mt::Itc::Dyn::Cli::Resp::Api<dynSrvType>::
		ReleaseResp						_releaseResp;
		/** */
		DeactivateMem*							_deactivateMem;
	protected:
		/** */
		Monitor(	ContextApi&				context,
					Mt::Itc::PostMsgApi&	myPapi,
					dynSrvType&				dynSrv
					) throw();
		/** */
		virtual ~Monitor(){}
	public:
		/** */
		void	activate() throw();
		/** */
		void	deactivate(DeactivateMem& mem) throw();
	private: // StateVar::ContextApi
		/** */
		void	requestReleaseNotification() throw();
		/** */
		void	activateMonitors() throw();
		/** */
		void	deactivateMonitors() throw();
		/** */
		void	release() throw();
		/** */
		void	cancelReleaseNotification() throw();
		/** */
		void	monitorDeactivated() throw();
	private:
		/** */
		void	response(	typename Mt::Itc::Dyn::Cli::Resp::Api<dynSrvType>::
							ReleaseResp&	msg
							) throw();
		/** */
		void	response(	typename Mt::Itc::Dyn::Cli::Resp::Api<dynSrvType>::
							CancelReleaseResp&	msg
							) throw();
	private:
		/** */
		virtual void		subClassActivate() throw()=0;
		/** */
		virtual void		subClassDeactivate(deactMemType& mem) throw()=0;
		/** */
		virtual subClassType&	getSubClass() throw()=0;
	protected:
		/** */
		void				subClassDeactivated() throw();
	};

template <class subClassType, class deactMemType, class dynSrvType>
Monitor<subClassType,deactMemType,dynSrvType>::
	Monitor(	ContextApi&				context,
				Mt::Itc::PostMsgApi&	myPapi,
				dynSrvType&				dynSrv
				) throw():
		_state(*this),
		_context(context),
		_dynSrv(dynSrv),
		_myPapi(myPapi),
		_releaseResp(	dynSrv.getCliSAP().getReqApi(),
						*this,
						myPapi,
						_releasePayload
						),
		_deactivateMem(0)
		{
	}

template <class subClassType, class deactMemType, class dynSrvType>
void	Monitor<subClassType,deactMemType,dynSrvType>::activate() throw() {
	_state.activate();
	}

template <class subClassType, class deactMemType, class dynSrvType>
void	Monitor<subClassType,deactMemType,dynSrvType>::
		deactivate(DeactivateMem& mem) throw(){
	_deactivateMem	= &mem;
	_state.deactivate();
	}

template <class subClassType, class deactMemType, class dynSrvType>
void	Monitor<subClassType,deactMemType,dynSrvType>::
		requestReleaseNotification() throw(){
	_dynSrv->getCliSAP().post(_releaseResp.getSrvMsg());
	}

template <class subClassType, class deactMemType, class dynSrvType>
void	Monitor<subClassType,deactMemType,dynSrvType>::
		activateMonitors() throw(){
	subClassActivate();
	}

template <class subClassType, class deactMemType, class dynSrvType>
void	Monitor<subClassType,deactMemType,dynSrvType>::
		deactivateMonitors() throw(){
	subClassDeactivate(_deactivateMem->_mpDeactivateMem);
	}

template <class subClassType, class deactMemType, class dynSrvType>
void	Monitor<subClassType,deactMemType,dynSrvType>::release() throw(){
	_context.release(getSubClass());
	}
template <class subClassType, class deactMemType, class dynSrvType>
void	Monitor<subClassType,deactMemType,dynSrvType>::
		cancelReleaseNotification() throw(){
	using namespace Mt::Itc::Dyn::Cli;
	typename Req::Api<dynSrvType>::CancelReleasePayload*	payload	=
		new (&_deactivateMem->_release._payload)
				typename Req::Api<dynSrvType>::CancelReleasePayload(
				_releaseResp.getSrvMsg()
				);
	typename Resp::Api<dynSrvType>::CancelReleaseResp*	response =
		new (&_deactivateMem->_release._response)
				typename Resp::Api<dynSrvType>::CancelReleaseResp(
					_dynSrv->getCliSAP().getReqApi(),
					*this,
					_myPapi,
					*payload
					);
	_dynSrv->getCliSAP().post(response->getSrvMsg());
	}

template <class subClassType, class deactMemType, class dynSrvType>
void	Monitor<subClassType,deactMemType,dynSrvType>::
		monitorDeactivated() throw(){
	_context.deactivated(getSubClass());
	}

template <class subClassType, class deactMemType, class dynSrvType>
void	Monitor<subClassType,deactMemType,dynSrvType>::
		response(	typename Mt::Itc::Dyn::Cli::Resp::Api<dynSrvType>::
					ReleaseResp&	msg
					) throw(){
	_state.releaseNotification();
	}

template <class subClassType, class deactMemType, class dynSrvType>
void	Monitor<subClassType,deactMemType,dynSrvType>::
		response(	typename Mt::Itc::Dyn::Cli::Resp::Api<dynSrvType>::
					CancelReleaseResp&	msg
					) throw(){
	typename Mt::Itc::Dyn::Cli::Req::Api<dynSrvType>::
	CancelReleasePayload& payload = msg.getPayload();
	msg.Mt::Itc::Dyn::Cli::Resp::Api<dynSrvType>::CancelReleaseResp::~CancelReleaseResp();
	payload.Mt::Itc::Dyn::Cli::Req::Api<dynSrvType>::CancelReleasePayload::~CancelReleasePayload();
	_state.releaseNotificationCanceled();
	}

template <class subClassType, class deactMemType, class dynSrvType>
void	Monitor<subClassType,deactMemType,dynSrvType>::
		subClassDeactivated() throw(){
	_state.deactivated();
	}

}
}
}
}
}

#endif

