/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_mt_itc_dyn_mp_fsmh_
#define _oscl_mt_itc_dyn_mp_fsmh_

/** */
namespace Oscl {
/** */
namespace Mt {
/** */
namespace Itc {
/** */
namespace Dyn {
/** */
namespace MpMonitor {

/** */
class StateVar {
	public:
		/** */
		class ContextApi {
			public:
				/** */
				virtual void	initNotificationReq() throw()=0;
				/** */
				virtual void	requestChangeNotification() throw()=0;
				/** */
				virtual void	processChangeNotification() throw()=0;
				/** */
				virtual void	cancelChangeNotification() throw()=0;
				/** */
				virtual void	completeDeactivation() throw()=0;
			};
	private:
		/** */
		class State {
			public:
				/** */
				virtual ~State(){}
			public:
				/** */
				virtual void	activate(	ContextApi&	context,
											StateVar&	statevar
											) const throw();
				/** */
				virtual void	deactivate(	ContextApi&	context,
											StateVar&	statevar
											) const throw();
				/** */
				virtual void	notified(	ContextApi&	context,
											StateVar&	statevar
											) const throw();
				/** */
				virtual void	canceled(	ContextApi&	context,
											StateVar&	statevar
											) const throw();
			};
		/** */
		class ClosedState : public State {
			public:
				/** */
				void	activate(	ContextApi&	context,
									StateVar&	statevar
									) const throw();
			};
		/** */
		class OpenedState : public State {
			public:
				/** */
				void	deactivate(	ContextApi&	context,
									StateVar&	statevar
									) const throw();
				/** */
				void	notified(	ContextApi&	context,
									StateVar&	statevar
									) const throw();
			};
		/** */
		class ClosingState : public State {
			public:
				/** */
				void	notified(	ContextApi&	context,
									StateVar&	statevar
									) const throw();
				/** */
				void	canceled(	ContextApi&	context,
									StateVar&	statevar
									) const throw();
			};
	private:
		/** */
		static const ClosingState	_closingState;
		/** */
		static const ClosedState	_closedState;
		/** */
		static const OpenedState	_openedState;
	private:
		/** */
		ContextApi&		_context;
		/** */
		const State*	_state;
	public:
		/** */
		StateVar(ContextApi& context) throw();
		/** */
		void	activate() throw();
		/** */
		void	deactivate() throw();
		/** */
		void	notified() throw();
		/** */
		void	canceled() throw();
	private:
		friend class State;
		friend class ClosingState;
		friend class ClosedState;
		friend class OpenedState;
		/** */
		void	changeToClosedState() throw();
		/** */
		void	changeToOpenedState() throw();
		/** */
		void	changeToClosingState() throw();
		/** */
		void	protocolError() throw();
	};

};
};
};
};
};

#endif
