/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_mt_itc_dyn_monitor_unit_fsm_fsmh_
#define _oscl_mt_itc_dyn_monitor_unit_fsm_fsmh_

/** */
namespace Oscl {
/** */
namespace Mt {
/** */
namespace Itc {
/** */
namespace Dyn {
/** */
namespace Unit {

/** */
class StateVar {
	public:
		/** */
		class ContextApi {
			public:
				/** */
				virtual ~ContextApi() {}

			public:
				/** The create notification request is initialized to
					before being sent.
				 */
				virtual void	reqNewCreateNotification() throw()=0;
				/** The create notification request is not re-initialized
					before being sent.
				 */
				virtual void	reqCreateNotification() throw()=0;
				/** */
				virtual void	cancelCreateNotification() throw()=0;
				/** */
				virtual void	reqReleaseNotification() throw()=0;
				/** */
				virtual void	cancelReleaseNotification() throw()=0;
				/** */
				virtual void	openComplete() throw()=0;
				/** */
				virtual void	closeComplete() throw()=0;
				/** */
				virtual void	activateService() throw()=0;
				/** */
				virtual void	deactivateService() throw()=0;
				/** */
				virtual bool	match() throw()=0;
			};
		/** */
		class State {
			public:
				/** */
				virtual void	open(	ContextApi&	context,
										StateVar&	state
										) const throw();
				/** */
				virtual void	close(	ContextApi&	context,
										StateVar&	state
										) const throw();
				/** */
				virtual void	create(	ContextApi&	context,
										StateVar&	state
										) const throw();
				/** */
				virtual void	release(	ContextApi&	context,
											StateVar&	state
											) const throw();
				/** */
				virtual void	createCanceled(	ContextApi&	context,
												StateVar&	state
												) const throw();
				/** */
				virtual void	releaseCanceled(	ContextApi&	context,
													StateVar&	state
													) const throw();
				/** */
				virtual void	deactivateDone(	ContextApi&	context,
												StateVar&	state
												) const throw();
			};
		/** */
		class ClosedState : public State {
			public:
				/** */
				void	open(	ContextApi&	context,
								StateVar&	state
								) const throw();
			};
		/** */
		class ListeningState : public State {
			public:
				void	close(	ContextApi&	context,
								StateVar&	state
								) const throw();
				/** */
				void	create(	ContextApi&	context,
								StateVar&	state
								) const throw();
			};
		/** */
		class ActiveState : public State {
			public:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	state
								) const throw();
				/** */
				void	release(	ContextApi&	context,
									StateVar&	state
									) const throw();
			};
		/** */
		class DeactivateCloseState : public State {
			private:
				/** */
				void	release(	ContextApi&	context,
									StateVar&	state
									) const throw();
				/** */
				void	deactivateDone(	ContextApi&	context,
										StateVar&	state
										) const throw();
			};
		/** */
		class DeactivateOpenState : public State {
			private:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	state
								) const throw();
				/** */
				void	deactivateDone(	ContextApi&	context,
										StateVar&	state
										) const throw();
			};
		/** */
		class OpeningState : public State {
			public:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	state
								) const throw();
				/** */
				void	create(	ContextApi&	context,
								StateVar&	state
								) const throw();
				/** */
				void	release(	ContextApi&	context,
									StateVar&	state
									) const throw();
				/** */
				void	createCanceled(	ContextApi&	context,
										StateVar&	state
										) const throw();
				/** */
				void	releaseCanceled(	ContextApi&	context,
											StateVar&	state
											) const throw();
			};
		/** */
		class ClosingState : public State {
			public:
				/** */
				void	open(	ContextApi&	context,
								StateVar&	state
								) const throw();
				/** */
				void	create(	ContextApi&	context,
								StateVar&	state
								) const throw();
				/** */
				void	release(	ContextApi&	context,
									StateVar&	state
									) const throw();
				/** */
				void	createCanceled(	ContextApi&	context,
										StateVar&	state
										) const throw();
				/** */
				void	releaseCanceled(	ContextApi&	context,
											StateVar&	state
											) const throw();
			};
	private:
		/** */
		ContextApi&		_context;
		/** */
		const State*	_state;
	public:
		/** */
		StateVar(ContextApi& context) throw();

		/** */
		void	open() throw();
		/** */
		void	close() throw();
		/** */
		void	create() throw();
		/** */
		void	release() throw();
		/** */
		void	createCanceled() throw();
		/** */
		void	releaseCanceled() throw();
		/** */
		void	deactivateDone() throw();
	private:
		friend class State;
		friend class ClosedState;
		friend class ListeningState;
		friend class ActiveState;
		friend class ClosingState;
		friend class OpeningState;
		friend class DeactivateOpenState;
		friend class DeactivateCloseState;
		/** */
		void	changeToClosedState() throw();
		/** */
		void	changeToListeningState() throw();
		/** */
		void	changeToActiveState() throw();
		/** */
		void	changeToClosingState() throw();
		/** */
		void	changeToOpeningState() throw();
		/** */
		void	changeToDeactivateCloseState() throw();
		/** */
		void	changeToDeactivateOpenState() throw();
		/** */
		void	protocolError() throw();
	};

}
}
}
}
}


#endif
