/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_mt_itc_dyn_monitor_unit_monitor_monitorh_
#define _oscl_mt_itc_dyn_monitor_unit_monitor_monitorh_
#include <new>
#include "part.h"
#if 0
#include "oscl/mt/itc/dyn/adv/respapi.h"
#include "oscl/mt/itc/dyn/adv/clisync.h"
#include "oscl/mt/itc/dyn/cli/respapi.h"
#include "oscl/memory/block.h"
#endif
#include "oscl/mt/itc/srv/close.h"
#include "oscl/done/operation.h"

/** */
namespace Oscl {
/** */
namespace Mt {
/** */
namespace Itc {
/** */
namespace Dyn {
/** */
namespace Unit {

template <class dynSrvType>
class Monitor :	public Oscl::Mt::Itc::Dyn::Unit::Part<dynSrvType>,
				public Oscl::Mt::Itc::Srv::CloseSync,
				public Oscl::Mt::Itc::Srv::Close::Req::Api
				{
	private:
		/** */
		Oscl::Done::Operation< Monitor<dynSrvType> >	_startDoneOp;
		/** */
		Oscl::Done::Operation< Monitor<dynSrvType> >	_stopDoneOp;
		/** */
		Oscl::Mt::Itc::Srv::Open::Req::Api::OpenReq*	_openReq;
		/** */
		Oscl::Mt::Itc::Srv::Close::Req::Api::CloseReq*	_closeReq;

	public:
		/** */
		Monitor(	typename Oscl::Mt::Itc::Dyn::Adv::Cli::
                    Req::Api<dynSrvType>::SAP&		advSAP,
					Oscl::Mt::Itc::Dyn::Adv::Cli::
					SyncApi<dynSrvType>&			advFind,
					Oscl::Mt::Itc::PostMsgApi&		myPapi,
					Oscl::Mt::Itc::Dyn::Adv::
					Iterator<dynSrvType>&			iterator
					) throw();

		/** */
		void	deactivateDone() throw();

	private:	// Oscl::Mt::Itc::Srv::Close::Req::Api
		/** */
		void	request(	Oscl::Mt::Itc::Srv::
							Open::Req::Api::OpenReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Mt::Itc::Srv::
							Close::Req::Api::CloseReq&	msg
							) throw();
	private:
		/** */
		void	startDone() throw();
		/** */
		void	stopDone() throw();
	};

template <class dynSrvType>
Monitor<dynSrvType>::Monitor(	typename Oscl::Mt::Itc::Dyn::Adv::Cli::
			                    Req::Api<dynSrvType>::SAP&		advSAP,
								Oscl::Mt::Itc::Dyn::Adv::Cli::
								SyncApi<dynSrvType>&			advFind,
								Oscl::Mt::Itc::PostMsgApi&		myPapi,
								Oscl::Mt::Itc::Dyn::Adv::
								Iterator<dynSrvType>&			iterator
								) throw():
		Oscl::Mt::Itc::Dyn::Unit::Part<dynSrvType>(advSAP,advFind,myPapi,iterator),
		Oscl::Mt::Itc::Srv::CloseSync(*this,myPapi),
		_startDoneOp(*this,&Monitor<dynSrvType>::startDone),
		_stopDoneOp(*this,&Monitor<dynSrvType>::stopDone),
		_openReq(0),
		_closeReq(0)
		{
	}

template <class dynSrvType>
void	Monitor<dynSrvType>::deactivateDone() throw(){
	Part<dynSrvType>::deactivateDone();
	}

template <class dynSrvType>
void	Monitor<dynSrvType>::request(	Oscl::Mt::Itc::Srv::
										Open::Req::Api::OpenReq&	msg
										) throw(){
	_openReq	= &msg;
	Part<dynSrvType>::start(_startDoneOp);
	}

template <class dynSrvType>
void	Monitor<dynSrvType>::request(	Oscl::Mt::Itc::Srv::
										Close::Req::Api::CloseReq&	msg
										) throw(){
	_closeReq	= &msg;
	Part<dynSrvType>::stop(_stopDoneOp);
	}

template <class dynSrvType>
void	Monitor<dynSrvType>::startDone() throw(){
	_openReq->returnToSender();
	_openReq	= 0;
	}

template <class dynSrvType>
void	Monitor<dynSrvType>::stopDone() throw(){
	_closeReq->returnToSender();
	_closeReq	= 0;
	}

}
}
}
}
}

#endif
