/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_mt_itc_dyn_monitor_unit_monitor_parth_
#define _oscl_mt_itc_dyn_monitor_unit_monitor_parth_
#include <new>
#include "oscl/mt/itc/dyn/monitor/unit/fsm/fsm.h"
#include "oscl/mt/itc/dyn/adv/respapi.h"
#include "oscl/mt/itc/dyn/adv/clisync.h"
#include "oscl/mt/itc/dyn/cli/respapi.h"
#include "oscl/mt/itc/srv/close.h"
#include "oscl/memory/block.h"
#include "oscl/done/api.h"

/** */
namespace Oscl {
/** */
namespace Mt {
/** */
namespace Itc {
/** */
namespace Dyn {
/** */
namespace Unit {

template <class dynSrvType>
class Part :	public Oscl::Mt::Itc::Dyn::Adv::Cli::Resp::Api<dynSrvType>,
				public Oscl::Mt::Itc::Dyn::Cli::Resp::Api<dynSrvType>,
				public Oscl::Mt::Itc::Dyn::Unit::StateVar::ContextApi
				{
	private:
		/** */
		class CreateMsg {
			private:
				/** */
				typename Oscl::Mt::Itc::Dyn::Adv::Cli::
				Req::Api<dynSrvType>::CreatePayload	_payload;
				/** */
				typename Oscl::Mt::Itc::Dyn::Adv::Cli::
				Resp::Api<dynSrvType>::CreateResp		_resp;
			public:
				/** */
				CreateMsg(	Oscl::Mt::Itc::Dyn::Adv::Cli::
							Req::Api<dynSrvType>&			srv,
							Oscl::Mt::Itc::Dyn::Adv::Cli::
							Resp::Api<dynSrvType>&			cli,
							Oscl::Mt::Itc::PostMsgApi&		cliPapi
							) throw();
				/** */
				void	reset() throw();

				/** */
				typename Oscl::Mt::Itc::Dyn::Adv::Cli::
				Req::Api<dynSrvType>::CreateReq&	getReq() throw();
			};

		/** */
		class CancelCreateMsg {
			private:
				/** */
				typename Oscl::Mt::Itc::Dyn::Adv::Cli::
				Req::Api<dynSrvType>::CancelCreatePayload	_payload;
				/** */
				typename Oscl::Mt::Itc::Dyn::Adv::Cli::
				Resp::Api<dynSrvType>::CancelResp			_resp;
			public:
				/** */
				CancelCreateMsg(	Oscl::Mt::Itc::Dyn::Adv::Cli::
									Req::Api<dynSrvType>&				srv,
									Oscl::Mt::Itc::Dyn::Adv::Cli::
									Resp::Api<dynSrvType>&				cli,
									Oscl::Mt::Itc::PostMsgApi&			cliPapi,
									typename Oscl::Mt::Itc::Dyn::Adv::Cli::
									Req::Api<dynSrvType>::CreateReq&	req
									) throw();
				/** */
				typename Oscl::Mt::Itc::Dyn::Adv::Cli::
				Req::Api<dynSrvType>::CancelCreateReq&	getReq() throw();
			};

		/** */
		class ReleaseMsg {
			private:
				/** */
				typename Oscl::Mt::Itc::Dyn::Cli::
				Req::Api<dynSrvType>::ReleasePayload	_payload;
				/** */
				typename Oscl::Mt::Itc::Dyn::Cli::
				Resp::Api<dynSrvType>::ReleaseResp	_resp;
			public:
				/** */
				ReleaseMsg(	Oscl::Mt::Itc::Dyn::Cli::
							Req::Api<dynSrvType>&			srv,
							Oscl::Mt::Itc::Dyn::Cli::
							Resp::Api<dynSrvType>&			cli,
							Oscl::Mt::Itc::PostMsgApi&		cliPapi
							) throw();
				/** */
				void	reset() throw();

				/** */
				typename Oscl::Mt::Itc::Dyn::Cli::
				Req::Api<dynSrvType>::ReleaseReq&	getReq() throw();
			};

		/** */
		class CancelReleaseMsg {
			private:
				/** */
				typename Oscl::Mt::Itc::Dyn::Cli::
				Req::Api<dynSrvType>::CancelReleasePayload	_payload;
				/** */
				typename Oscl::Mt::Itc::Dyn::Cli::
				Resp::Api<dynSrvType>::CancelReleaseResp		_resp;
			public:
				/** */
				CancelReleaseMsg(	Oscl::Mt::Itc::Dyn::Cli::
									Req::Api<dynSrvType>&				srv,
									Oscl::Mt::Itc::Dyn::Cli::
									Resp::Api<dynSrvType>&				cli,
									Oscl::Mt::Itc::PostMsgApi&			cliPapi,
									typename Oscl::Mt::Itc::Dyn::Cli::
									Req::Api<dynSrvType>::ReleaseReq&	req
									) throw();
				/** */
				typename Oscl::Mt::Itc::Dyn::Cli::
				Req::Api<dynSrvType>::CancelReleaseReq&	getReq() throw();
			};

		/** */
		union RequestMem {
			/** */
			Oscl::Memory::AlignedBlock<sizeof(CreateMsg)>	create;
			/** */
			Oscl::Memory::AlignedBlock<sizeof(ReleaseMsg)>	release;
			};

		/** */
		union CancelMem {
			/** */
			Oscl::Memory::AlignedBlock<sizeof(CancelCreateMsg)>		create;
			/** */
			Oscl::Memory::AlignedBlock<sizeof(CancelReleaseMsg)>	release;
			};

	private:
		/** */
		RequestMem			_requestMem;
		/** */
		CancelMem			_cancelMem;
		/** */
		typename Oscl::Mt::Itc::Dyn::Adv::Cli::
		Req::Api<dynSrvType>::SAP&					_advSAP;
		/** */
		Oscl::Mt::Itc::Dyn::Adv::Cli::
		SyncApi<dynSrvType>&						_advFind;
		/** */
		Oscl::Mt::Itc::Dyn::Adv::
		Iterator<dynSrvType>&						_iterator;
		/** */
		Oscl::Mt::Itc::Dyn::Unit::StateVar			_state;
		/** */
		Oscl::Done::Api*							_done;
		/** */
		CreateMsg*									_createMsg;
		/** */
		ReleaseMsg*									_releaseMsg;
		/** */
		Oscl::Mt::Itc::PostMsgApi&					_myPapi;

	public:
		/** */
		Oscl::Mt::Itc::Dyn::
		Handle<dynSrvType>							_handle;

	public:
		/** */
		Part(	typename
				Oscl::Mt::Itc::Dyn::Adv::Cli::
				Req::Api<dynSrvType>::SAP&		advSAP,
				Oscl::Mt::Itc::Dyn::Adv::Cli::
				SyncApi<dynSrvType>&			advFind,
				Oscl::Mt::Itc::PostMsgApi&		myPapi,
				Oscl::Mt::Itc::Dyn::Adv::
				Iterator<dynSrvType>&			iterator
				) throw();

		/** */
		void	start(Oscl::Done::Api& startDone) throw();
		/** */
		void	stop(Oscl::Done::Api& stopDone) throw();
		/** */
		void	deactivateDone() throw();

	private:	// Oscl::Mt::Itc::Dyn::Adv::Cli::Resp::Api
		/** */
		void	response(	typename Oscl::Mt::Itc::Dyn::Adv::Cli::
							Resp::Api<dynSrvType>::CreateResp&	msg
							) throw();
		/** */
		void	response(	typename Oscl::Mt::Itc::Dyn::Adv::Cli::
							Resp::Api<dynSrvType>::CancelResp&	msg
							) throw();

	private:	// Oscl::Mt::Itc::Dyn::Cli::Resp::Api
		/** */
		void	response(	typename Oscl::Mt::Itc::Dyn::Cli::
							Resp::Api<dynSrvType>::ReleaseResp&	msg
							) throw();
		/** */
		void	response(	typename Oscl::Mt::Itc::Dyn::Cli::
							Resp::Api<dynSrvType>::CancelReleaseResp&	msg
							) throw();

	private: // Oscl::Mt::Itc::Dyn::Unit::StateVar::ContextApi
		/** */
		void	reqNewCreateNotification() throw();
		/** */
		void	reqCreateNotification() throw();
		/** */
		void	cancelCreateNotification() throw();
		/** */
		void	reqReleaseNotification() throw();
		/** */
		void	cancelReleaseNotification() throw();
		/** */
		void	openComplete() throw();
		/** */
		void	closeComplete() throw();
		/** */
		void	activateService() throw();
		/** */
		void	deactivateService() throw();
		/** */
		bool	match() throw();

	protected:
		/** Synchronous activation */
		virtual void	activate() throw()=0;
		/** Client begins deactivation sequence and then invokes
			deactivateDone() when deactivation is complete.
		 */
		virtual void	deactivate() throw()=0;
	};

template <class dynSrvType>
Part<dynSrvType>::CreateMsg::
CreateMsg(	Oscl::Mt::Itc::Dyn::Adv::Cli::
			Req::Api<dynSrvType>&			srv,
			Oscl::Mt::Itc::Dyn::Adv::Cli::
			Resp::Api<dynSrvType>&			cli,
			Oscl::Mt::Itc::PostMsgApi&		cliPapi
			) throw():
		_payload(),
		_resp(srv,cli,cliPapi,_payload)
		{
	}

template <class dynSrvType>
void	Part<dynSrvType>::CreateMsg::reset() throw(){
	_payload.setLastId(0);
	}

template <class dynSrvType>
typename Oscl::Mt::Itc::Dyn::Adv::Cli::Req::Api<dynSrvType>::
CreateReq&	Part<dynSrvType>::CreateMsg::getReq() throw(){
	return _resp.getSrvMsg();
	}

template <class dynSrvType>
Part<dynSrvType>::CancelCreateMsg::
CancelCreateMsg(	Oscl::Mt::Itc::Dyn::Adv::Cli::
					Req::Api<dynSrvType>&				srv,
					Oscl::Mt::Itc::Dyn::Adv::Cli::
					Resp::Api<dynSrvType>&				cli,
					Oscl::Mt::Itc::PostMsgApi&			cliPapi,
					typename Oscl::Mt::Itc::Dyn::Adv::Cli::
					Req::Api<dynSrvType>::CreateReq&	req
					) throw():
		_payload(req),
		_resp(srv,cli,cliPapi,_payload)
		{
	}

template <class dynSrvType>
typename Oscl::Mt::Itc::Dyn::Adv::Cli::Req::Api<dynSrvType>::
CancelCreateReq&	Part<dynSrvType>::CancelCreateMsg::getReq() throw(){
	return _resp.getSrvMsg();
	}

template <class dynSrvType>
Part<dynSrvType>::ReleaseMsg::
ReleaseMsg(	Oscl::Mt::Itc::Dyn::Cli::
			Req::Api<dynSrvType>&			srv,
			Oscl::Mt::Itc::Dyn::Cli::
			Resp::Api<dynSrvType>&			cli,
			Oscl::Mt::Itc::PostMsgApi&		cliPapi
			) throw():
		_payload(),
		_resp(srv,cli,cliPapi,_payload)
		{
	}

template <class dynSrvType>
typename Oscl::Mt::Itc::Dyn::Cli::Req::Api<dynSrvType>::
ReleaseReq&	Part<dynSrvType>::ReleaseMsg::getReq() throw(){
	return _resp.getSrvMsg();
	}

template <class dynSrvType>
Part<dynSrvType>::CancelReleaseMsg::
CancelReleaseMsg(	Oscl::Mt::Itc::Dyn::Cli::
					Req::Api<dynSrvType>&				srv,
					Oscl::Mt::Itc::Dyn::Cli::
					Resp::Api<dynSrvType>&				cli,
					Oscl::Mt::Itc::PostMsgApi&			cliPapi,
					typename Oscl::Mt::Itc::Dyn::Cli::
					Req::Api<dynSrvType>::ReleaseReq&	req
					) throw():
		_payload(req),
		_resp(srv,cli,cliPapi,_payload)
		{
	}

template <class dynSrvType>
Part<dynSrvType>::Part(	typename
						Oscl::Mt::Itc::Dyn::Adv::Cli::
						Req::Api<dynSrvType>::SAP&		advSAP,
						Oscl::Mt::Itc::Dyn::Adv::Cli::
						SyncApi<dynSrvType>&			advFind,
						Oscl::Mt::Itc::PostMsgApi&		myPapi,
						Oscl::Mt::Itc::Dyn::Adv::
						Iterator<dynSrvType>&			iterator
						) throw():
		_advSAP(advSAP),
		_advFind(advFind),
		_iterator(iterator),
		_state(*this),
		_myPapi(myPapi)
		{
	}

template <class dynSrvType>
void	Part<dynSrvType>::start(Oscl::Done::Api& done) throw(){
	_done	= &done;
	_state.open();
	}

template <class dynSrvType>
void	Part<dynSrvType>::stop(Oscl::Done::Api& done) throw(){
	_done	= &done;
	_state.close();
	}

template <class dynSrvType>
void	Part<dynSrvType>::deactivateDone() throw(){
	_state.deactivateDone();
	}

template <class dynSrvType>
typename Oscl::Mt::Itc::Dyn::Cli::Req::Api<dynSrvType>::
CancelReleaseReq&	Part<dynSrvType>::CancelReleaseMsg::getReq() throw(){
	return _resp.getSrvMsg();
	}

template <class dynSrvType>
void	Part<dynSrvType>::response(	typename Oscl::Mt::Itc::Dyn::Adv::Cli::
										Resp::Api<dynSrvType>::CreateResp&	msg
										) throw(){
	_state.create();
	}

template <class dynSrvType>
void	Part<dynSrvType>::response(	typename Oscl::Mt::Itc::Dyn::Adv::Cli::
										Resp::Api<dynSrvType>::CancelResp&	msg
										) throw(){
	_state.createCanceled();
	}

template <class dynSrvType>
void	Part<dynSrvType>::response(	typename Oscl::Mt::Itc::Dyn::Cli::
										Resp::Api<dynSrvType>::ReleaseResp&	msg
										) throw(){
	_state.release();
	}

template <class dynSrvType>
void	Part<dynSrvType>::response(	typename Oscl::Mt::Itc::Dyn::Cli::
										Resp::Api<dynSrvType>::
										CancelReleaseResp&	msg
										) throw(){
	_state.releaseCanceled();
	}

template <class dynSrvType>
void	Part<dynSrvType>::reqNewCreateNotification() throw(){
	_createMsg	= new (&_requestMem)
						CreateMsg(	_advSAP.getReqApi(),
									*this,
									_myPapi
									);
	_advSAP.post(_createMsg->getReq());
	}

template <class dynSrvType>
void	Part<dynSrvType>::reqCreateNotification() throw(){
	_advSAP.post(_createMsg->getReq());
	}

template <class dynSrvType>
void	Part<dynSrvType>::cancelCreateNotification() throw(){
	CancelCreateMsg*	msg	= new (&_cancelMem)
								CancelCreateMsg(	_advSAP.getReqApi(),
													*this,
													_myPapi,
													_createMsg->getReq()
													);
	_advSAP.post(msg->getReq());
	}

template <class dynSrvType>
void	Part<dynSrvType>::reqReleaseNotification() throw(){
	_releaseMsg	= new (&_requestMem)
						ReleaseMsg(	_handle->getDynSrv(),
									*this,
									_myPapi
									);
	_handle->getCliSAP().post(_releaseMsg->getReq());
	}

template <class dynSrvType>
void	Part<dynSrvType>::cancelReleaseNotification() throw(){
	CancelReleaseMsg*	msg	= new (&_cancelMem)
								CancelReleaseMsg(	_handle->getDynSrv(),
													*this,
													_myPapi,
													_releaseMsg->getReq()
													);
	_handle->getCliSAP().post(msg->getReq());
	}

template <class dynSrvType>
void	Part<dynSrvType>::openComplete() throw(){
	_handle	= 0;
	_done->done();
	_done	= 0;
	}

template <class dynSrvType>
void	Part<dynSrvType>::closeComplete() throw(){
	_handle	= 0;
	_done->done();
	_done	= 0;
	}

template <class dynSrvType>
void	Part<dynSrvType>::activateService() throw(){
	activate();
	}

template <class dynSrvType>
void	Part<dynSrvType>::deactivateService() throw(){
	deactivate();
	}

template <class dynSrvType>
bool	Part<dynSrvType>::match() throw(){
	_handle	= _advFind.find(_iterator);
	return (_handle != 0);
	}

}
}
}
}
}

#endif
