/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_mt_sema_sigapih_
#define _oscl_mt_sema_sigapih_

/** */
namespace Oscl {
/** */
namespace Mt {
/** */
namespace Sema {

/** Abstractly represents an entity which can cause a thread
	to become unblocked through a signal. This is the basis
	for a semaphore.
 */
class SignalApi {
	public:
		/** */
		virtual ~SignalApi(){}

	public:
		/** Called by a thread to cause the subject to be notified.
			Typically, an operating system trap is envoked to make
			a blocked thread ready to run.
		 */
		virtual void signal(void) throw() =0;

		/** Called by the supervisor-mode trap handlers
			and interrupt service routines to set the
			cause the blocked subject to become ready to
			run. This should never be called by
			threads.
		 */
		virtual void suSignal(void) throw() = 0;

	};

}
}
}

#endif
