/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "server.h"

using namespace Oscl::PciSrv::Func;
using namespace Oscl;

Server::Server(	InfoApi&						funcInfo,
				Oscl::Mt::Itc::PostMsgApi&		dynSrvPostApi,
				Oscl::Mt::Itc::PostMsgApi&		advPostApi,
				Oscl::Mt::Itc::Dyn::Adv::
				Core<Server>::ReleaseApi&		advertiser
				) throw():
		Oscl::Mt::Itc::Dyn::Adv::
		Core<Server>(dynSrvPostApi,advertiser,advPostApi),
		_funcInfo(funcInfo),
		_claimed(false)
		{
	}

bool	Server::isClaimed() const throw(){
	return _claimed;
	}

Unsigned16	Server::getDeviceID() const throw(){
	return _funcInfo.getDeviceID();
	}

Unsigned16	Server::getVendorID() const throw(){
	return _funcInfo.getVendorID();
	}

Unsigned8	Server::getClassCode() const throw(){
	return _funcInfo.getClassCode();
	}

Unsigned8	Server::getSubClassCode() const throw(){
	return _funcInfo.getSubClassCode();
	}

Unsigned8	Server::getProgIF() const throw(){
	return _funcInfo.getProgIF();
	}

Unsigned8	Server::getRevisionID() const throw(){
	return _funcInfo.getRevisionID();
	}

const Oscl::PciSrv::LocationApi&	Server::getLocation() const throw(){
	return _funcInfo.getLocation();
	}

unsigned long	Server::getBaseAddress(unsigned char n) const throw(){
	return _funcInfo.getBaseAddress(n);
	}

Oscl::Interrupt::Shared::SourceApi&	Server::getInterruptSource() throw(){
	return _funcInfo.getInterruptSource();
	}

void	Server::openDynamicService() throw(){
	// Nothing to attach yet
	}

void	Server::closeDynamicService() throw(){
	// Nothing to detach yet
	}

Server&	Server::getDynSrv() throw(){
	return *this;
	}

