/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_pcisrv_func_serverh_
#define _oscl_pcisrv_func_serverh_
#include "drvapi.h"
#include "oscl/pcisrv/enum/location.h"
#include "oscl/pcisrv/enum/funcapi.h"
#include "oscl/mt/itc/dyn/adv/core.h"

/** */
namespace Oscl {
/** */
namespace PciSrv {
/** */
namespace Func {

class InfoApi {
	public:
		/** */
		virtual Unsigned16	getDeviceID() const throw()=0;
		/** */
		virtual Unsigned16	getVendorID() const throw()=0;
		/** */
		virtual Unsigned8	getClassCode() const throw()=0;
		/** */
		virtual Unsigned8	getSubClassCode() const throw()=0;
		/** */
		virtual Unsigned8	getProgIF() const throw()=0;
		/** */
		virtual Unsigned8	getRevisionID() const throw()=0;
	public:
		/** */
		virtual const Oscl::PciSrv::LocationApi& getLocation() const throw()=0;
	public:
		/** */
		virtual unsigned long	getBaseAddress(unsigned char n) const throw()=0;
		/** */
		virtual Oscl::Interrupt::Shared::
		SourceApi&	getInterruptSource() throw()=0;
	};

/** */
class Server :	public DriverApi,
				public Oscl::PciSrv::Enum::FuncApi,
				public Oscl::Mt::Itc::Dyn::Adv::Core<Server>
				{
	private:
		/** */
		InfoApi&						_funcInfo;
		/** */
		bool							_claimed;

	public:
		/** */
		Server(	InfoApi&									funcInfo,
				Oscl::Mt::Itc::PostMsgApi&					dynSrvPostApi,
				Oscl::Mt::Itc::PostMsgApi&					advPostApi,
				Oscl::Mt::Itc::Dyn::Adv::Core<Server>::
					ReleaseApi&								advertiser
				) throw();

	public: // FuncApi
		/** */
		bool	isClaimed() const throw();
	public:	// DriverApi
		/** */
		Unsigned16	getDeviceID() const throw();
		/** */
		Unsigned16	getVendorID() const throw();
		/** */
		Unsigned8	getClassCode() const throw();
		/** */
		Unsigned8	getSubClassCode() const throw();
		/** */
		Unsigned8	getProgIF() const throw();
		/** */
		Unsigned8	getRevisionID() const throw();
		/** */
		const Oscl::PciSrv::LocationApi&	getLocation() const throw();
		/** */
		unsigned long	getBaseAddress(unsigned char n) const throw();
		/** */
		Oscl::Interrupt::Shared::SourceApi&	getInterruptSource() throw();

	public: // Oscl::Mt::Itc::Dyn::Adv::Core
		/** Executes within the dynamic service thread. Connect
			to any lower layer services synchronously.
		 */
		void	openDynamicService() throw();
		/** Executes within the dynamic service thread. Disconnect
			from any lower layer services synchronously.
		 */
		void	closeDynamicService() throw();
		/** */
		Server&	getDynSrv() throw();
	};

}
}
}

#endif
