/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_pdu_rocompvisith_
#define _oscl_pdu_rocompvisith_
#include "visitor.h"
#include "oscl/buffer/api.h"

/** */
namespace Oscl {
/** */
namespace Pdu {

class Composite;
class ReadOnlyComposite;
class Leaf;
class Fragment;
class ReadOnlyFragment;

/** */
class CopyRW : public ReadOnlyVisitor {
	private:
		/** */
		Oscl::Buffer::ReadWrite&	_destination;

	public:
		/** */
		CopyRW(Oscl::Buffer::ReadWrite& destination) throw();

	public:
		/** This operation is called when a ReadOnlyComposite node
			is visited in a compound PDU tree. The position is the
			offset relative to the beginning of the client packet.
			The position may be unique to each client.
		 */
		bool /* stop */	accept(	const Composite&	p,
								unsigned			position
								) throw();

		/** This operation is called when a ReadOnlyComposite node
			is visited in a compound PDU tree. The position is the
			offset relative to the beginning of the client packet.
			The position may be unique to each client.
		 */
		bool /* stop */	accept(	const ReadOnlyComposite&	p,
								unsigned					position
								) throw();

		/** This operation is called when a Leaf node
			is visited in a compound PDU tree. The position is the
			offset relative to the beginning of the client packet.
			The position may be unique to each client. The offset indexes
			the first valid octet in the leaf buffer for the current client.
			The length is the the number of valid octets beginning from
			the offset argument.
		 */
		bool /* stop */ accept(	const Leaf&		p,
								unsigned		position,
								unsigned		offset,
								unsigned		length
								) throw();

		/** This operation is called when a Leaf node
			is visited in a compound PDU tree. The position is the
			offset relative to the beginning of the client packet.
			The position may be unique to each client.
		 */
		bool /* stop */ accept(	const Fragment&	p,
								unsigned		position
								) throw();

		/** This operation is called when a Leaf node
			is visited in a compound PDU tree. The position is the
			offset relative to the beginning of the client packet.
			The position may be unique to each client.
		 */
		bool /* stop */ accept(	const ReadOnlyFragment&	p,
								unsigned				position
								) throw();
	};

/** */
class EqualRO : public ReadOnlyVisitor {
	private:
		/** */
		const Oscl::Buffer::ReadOnly&	_source;

		/** */
		const unsigned					_length;

	public:
		/** */
		bool							_match;

	public:
		/** */
		EqualRO(const Oscl::Buffer::ReadOnly&	source) throw();

	public:
		/** This operation is called when a ReadOnlyComposite node
			is visited in a compound PDU tree. The position is the
			offset relative to the beginning of the client packet.
			The position may be unique to each client.
		 */
		bool /* stop */	accept(	const Composite&	p,
								unsigned			position
								) throw();

		/** This operation is called when a ReadOnlyComposite node
			is visited in a compound PDU tree. The position is the
			offset relative to the beginning of the client packet.
			The position may be unique to each client.
		 */
		bool /* stop */	accept(	const ReadOnlyComposite&	p,
								unsigned					position
								) throw();

		/** This operation is called when a Leaf node
			is visited in a compound PDU tree. The position is the
			offset relative to the beginning of the client packet.
			The position may be unique to each client. The offset indexes
			the first valid octet in the leaf buffer for the current client.
			The length is the the number of valid octets beginning from
			the offset argument.
		 */
		bool /* stop */ accept(	const Leaf&		p,
								unsigned		position,
								unsigned		offset,
								unsigned		length
								) throw();

		/** This operation is called when a Leaf node
			is visited in a compound PDU tree. The position is the
			offset relative to the beginning of the client packet.
			The position may be unique to each client.
		 */
		bool /* stop */ accept(	const Fragment&	p,
								unsigned		position
								) throw();

		/** This operation is called when a Leaf node
			is visited in a compound PDU tree. The position is the
			offset relative to the beginning of the client packet.
			The position may be unique to each client.
		 */
		bool /* stop */ accept(	const ReadOnlyFragment&	p,
								unsigned				position
								) throw();
	};

/** */
class EqualBuff : public ReadOnlyVisitor {
	private:
		/** */
		const unsigned char*			_source;

		/** */
		const unsigned					_length;

	public:
		/** */
		 bool							_match;

	public:
		/** */
		EqualBuff(	const void*	source,
					unsigned	length
					) throw();
	public:
		/** This operation is called when a ReadOnlyComposite node
			is visited in a compound PDU tree. The position is the
			offset relative to the beginning of the client packet.
			The position may be unique to each client.
		 */
		bool /* stop */	accept(	const Composite&	p,
								unsigned			position
								) throw();

		/** This operation is called when a ReadOnlyComposite node
			is visited in a compound PDU tree. The position is the
			offset relative to the beginning of the client packet.
			The position may be unique to each client.
		 */
		bool /* stop */	accept(	const ReadOnlyComposite&	p,
								unsigned					position
								) throw();

		/** This operation is called when a Leaf node
			is visited in a compound PDU tree. The position is the
			offset relative to the beginning of the client packet.
			The position may be unique to each client. The offset indexes
			the first valid octet in the leaf buffer for the current client.
			The length is the the number of valid octets beginning from
			the offset argument.
		 */
		bool /* stop */ accept(	const Leaf&		p,
								unsigned	position,
								unsigned	offset,
								unsigned	length
								) throw();

		/** This operation is called when a Leaf node
			is visited in a compound PDU tree. The position is the
			offset relative to the beginning of the client packet.
			The position may be unique to each client.
		 */
		bool /* stop */ accept(	const Fragment&	p,
								unsigned		position
								) throw();

		/** This operation is called when a Leaf node
			is visited in a compound PDU tree. The position is the
			offset relative to the beginning of the client packet.
			The position may be unique to each client.
		 */
		bool /* stop */ accept(	const ReadOnlyFragment&	p,
								unsigned				position
								) throw();
	};

}
}

#endif
