/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_persist_table_cache_recordh_
#define _oscl_persist_table_cache_recordh_
#include "oscl/compiler/types.h"
#include "oscl/stream/output/api.h"
#include "oscl/stream/input/api.h"

/** */
namespace Oscl {
/** */
namespace Persist {
/** */
namespace Table {
/** */
namespace Cache {

/** */
class Record : public Oscl::Persist::Record::RW::Api {
	public:
		/** */
		void*				__qitemlink;
		/** */
		Unsigned8*	const	_record;
		/** */
		const unsigned		_recordSize;
		/** */
		bool				_dirty;

	public:
		/** */
		Record(	Unsigned8*	record,
				unsigned	recordSize
				) throw();

		/** Writes the record to dest starting at offset zero
			only if the record is dirty. If the record is written
			the dirty indication is cleared.
		 */
		void	flush(Oscl::Persist::Record::RW::Api& dest) const throw();

		/** Unconditionally writes the record to dest starting at offset zero.
		 */
		void	copyOut(Oscl::Persist::Record::RW::Api& dest) const throw();

	private:	// Oscl::Persist::Record::RO::Api
		/** */
		void	write(	const void*	src,
						unsigned	length,
						unsigned	offset
						) const throw();
		/** */
		void	read(	void*		dest,
						unsigned	length,
						unsigned	offset
						) const throw();
	};

}
}
}
}

#endif
