/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_pid_int16_calch_
#define _oscl_pid_int16_calch_
#include "oscl/compiler/types.h"

/** */
namespace Oscl {
/** */
namespace PID {
/** */
namespace Int16 {

/** */
class Calc {
	private:
		/**	This member represents the desired
			value. This is where we want to be.
		 */
		Signed16		_setpoint;

		/** This member is the difference between
			the setpoint and the last sample.
		 */
		Signed16		_prevErr;

		/** This member is the sum of the errors.
		 */
		Signed16		_ierror;

		/** This member is the constant multiplier
			for the "proportional" term of the PID.
			Fixed point 8.8.
		 */
		const Signed16	_Kp;

		/** This member is the constant multiplier
			for the "derivative" term of the PID.
			Fixed point 8.8.
		 */
		const Signed16	_Kd;

		/** This member is the constant multiplier
			for the "integral" term of the PID.
			Fixed point 8.8.
		 */
		const Signed16	_Ki;

		/** This constant is the maximum
			positive output value of the PID.
		 */
		const Signed16	_maxOutput;

		/** This constant is the minimum
			negative output value of the PID.
		 */
		const Signed16	_minOutput;

	public:
		/** */
		Calc(	Signed16	Kp,
				Signed16	Kd,
				Signed16	Ki,
				Signed16	maxOutput,
				Signed16	minOutput,
				Signed16	setpoint
				) throw();

		/** */
		Signed16	tick(Signed16 sample) throw();

		/** */
		void	change(Signed16 setpoint) throw();
	};

}
}
}

#endif
