/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_pid_int16_calch_
#define _oscl_pid_int16_calch_
#include "oscl/compiler/types.h"

/** */
namespace Oscl {
/** */
namespace PID {
/** */
namespace Normal {

/** This class implements a PID (Proportional Integral Differential)
	closed loop control algorithm. The input and output of the algorithm
	are normalized fixed point numbers that may be scaled by the
	application as appropriate.
 */
class Calc {
	private:
		/**	This member is a positive binary fixed point number
			with a value between 0.000000000000000 and
			0.111111111111111 inclusive.
			Note that this number *must*
			always be positive. The idea is that it
			represents a "percentage" of full scale.
			Note that full scale cannot be represented,
			however, a very close approximation of full
			scale is possible.
		 */
		Signed16		_setpoint;

		/** This member is the difference between
			the setpoint and the last sample. It is
			a binary fixed point number with a value between
			0.111111111111111 and -0.111111111111111
			inclusive.
		 */
		Signed16		_prevErr;

		/** This member is the integral sum of the errors.
		 */
		Signed16		_ierror;

		/** This member is the constant multiplier
			for the "proportional" term of the PID.
		 */
		const Signed16	_Kp;

		/** This member is the constant multiplier
			for the "derivative" term of the PID.
		 */
		const Signed16	_Kd;

		/** This member is the constant multiplier
			for the "integral" term of the PID.
		 */
		const Signed16	_Ki;

	public:
		/** */
		Calc(	Signed16	Kp,
				Signed16	Kd,
				Signed16	Ki,
				Signed16	setpoint
				) throw();

		/** */
		Signed16	tick(Signed16 sample) throw();

		/** */
		void	change(Signed16 setpoint) throw();
	};

}
}
}

#endif
