/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include <new>
#include "part.h"

using namespace Oscl::PPP::Basic;

Part::Part(	Oscl::Mt::Itc::PostMsgApi&	pppPapi,
			Oscl::Frame::Pdu::Rx::
			ProtoMux::Req::Api::SAP&	protoMuxSAP,
			Oscl::Frame::Pdu::Rx::
			ProtoMux::SyncApi&			protoMuxSyncApi,
			Oscl::Frame::Pdu::
			Tx::Req::Api::SAP&			protoTxSAP
			) throw():
		_myPapi(pppPapi),
		_closeSAP(*this,pppPapi),
		_layerStateDist(	pppPapi,
							false
							),
		_lcp(	pppPapi,
				_layerStateDist.getSAP(),
				protoTxSAP,
				
				),
		_nextStep(0),
		_openReq(0),
		_closeReq(0)
		{
	}

Oscl::Mt::Itc::Srv::Close::Req::Api::SAP&	Part::getCloseSAP() throw(){
	return _closeSAP;
	}

Oscl::PPP::Layer::State::Req::Api::SAP&	Part::getIpLayerStateSAP() throw(){
	return _layerStateDist.getSAP();
	}

void	Part::openLCP() throw(){
#if 0
	Oscl::Mt::Itc::Srv::
	Open::Req::Api::OpenPayload*
	payload	= new (&_openCloseMem.open.payload)
				Oscl::Mt::Itc::Srv::
				Open::Req::Api::OpenPayload();
	Oscl::Mt::Itc::Srv::
	Open::Resp::Api::OpenResp*
	resp	= new (&_openCloseMem.open.resp)
				Oscl::Mt::Itc::Srv::
				Open::Resp::Api::
				OpenResp(	_lcp.getCloseSAP().getReqApi(),
							*this,
							_myPapi,
							*payload
							);
	_lcp.getCloseSAP().post(resp->getSrvMsg());
	_nextStep	= &Oscl::PPP::Stream::Basic::Part::openCHAP;
#else
	openComplete();
#endif
	}

void	Part::openCHAP() throw(){
	}

void	Part::openIPCP() throw(){
	}

void	Part::openComplete() throw(){
	_openReq->returnToSender();
	_openReq	= 0;
	}

void	Part::closeLCP() throw(){
	}

void	Part::closeCHAP() throw(){
	}

void	Part::closeIPCP() throw(){
#if 0
	Oscl::Mt::Itc::Srv::
	Close::Req::Api::ClosePayload*
	payload	= new (&_openCloseMem.open.payload)
				Oscl::Mt::Itc::Srv::
				Close::Req::Api::ClosePayload();
	Oscl::Mt::Itc::Srv::
	Close::Resp::Api::CloseResp*
	resp	= new (&_openCloseMem.open.resp)
				Oscl::Mt::Itc::Srv::
				Close::Resp::Api::
				CloseResp(	_ipcp.getCloseSAP().getReqApi(),
							*this,
							_myPapi,
							*payload
							);
	_ipcp.getCloseSAP().post(resp->getSrvMsg());
	_nextStep	= &Oscl::PPP::Stream::Basic::Part::closeCHAP;
#else
	closeComplete();
#endif
	}

void	Part::closeComplete() throw(){
	_closeReq->returnToSender();
	_closeReq	= 0;
	}


void	Part::request(	Oscl::Mt::Itc::Srv::
						Open::Req::Api::OpenReq&	msg
						) throw(){
	_openReq	= &msg;
	openLCP();
	}

void	Part::request(	Oscl::Mt::Itc::Srv::
						Close::Req::Api::CloseReq&	msg
						) throw(){
	_closeReq	= &msg;
	closeIPCP();
	}

void	Part::response(	Oscl::Mt::Itc::Srv::
						Open::Resp::Api::OpenResp&	msg
						) throw(){
	(this->*_nextStep)();
	}

void	Part::response(	Oscl::Mt::Itc::Srv::
						Close::Resp::Api::CloseResp&	msg
						) throw(){
	(this->*_nextStep)();
	}

