/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_ppp_fsm_opth_
#define _oscl_ppp_fsm_opth_
#include "oscl/pdu/tx/api.h"
#include "var.h"

/** */
namespace Oscl {
/** */
namespace PPP {
/** */
namespace FSM {
/** */
namespace OPT {

/** */
class Context {
	public:
		/** */
		virtual ~Context(){}
		/** */
		virtual void	up() throw()=0;
		/** */
		virtual void	down() throw()=0;
		/** */
		virtual void	start() throw()=0;
		/** */
		virtual void	finished() throw()=0;
		/** */
		virtual void	optionReq(	const unsigned char*	option,
									unsigned char			type,
									unsigned				optLen
									) throw()=0;
		/** */
		virtual void	buildOptions() throw()=0;
		/** */
		virtual void	defaultOptions() throw()=0;
		/** */
		virtual void	applyOptions() throw()=0;
	};

/** */
class Variable : private Oscl::PPP::FSM::Context {
	private:
		enum{maxOptSize=512};
		enum{maxAckRejOptSize=512};
	private:
		/** */
		Oscl::PPP::FSM::StateVar	_state;
		/** */
		OPT::Context&				_context;
		/** */
		Oscl::PDU::TX::Api&			_transmitter;
		/** */
		unsigned					_restartCounterValue;
		/** */
		unsigned					_timerValue;
		/** */
		unsigned char				_identifier;
		/** */
		unsigned char				_options[maxOptSize];
		/** */
		unsigned					_optionLen;
		/** */
		unsigned char				_ackRejOpt[maxOptSize];
		/** */
		unsigned					_ackRejOptLen;
		/** */
		bool						_reject;
		/** */
		bool						_unacceptable;
		/** */
		const unsigned char*		_pdu;
		/** */
		unsigned					_pduLength;
		/** */
		bool						_timerActive;
		/** */
		unsigned					_timer;
		/** */
		unsigned					_restartCounter;
		/** */
		bool						_invalidOptionFormat;
		/** */
		bool						_layerOpen;

	public:
		/** */
		Variable(	OPT::Context&		context,
					Oscl::PDU::TX::Api&	transmitter,
					unsigned			restartCountInSeconds=3,
					unsigned			timerValueInSeconds=2
					) throw();
		/** */
		void	up() throw();
		/** */
		void	down() throw();
		/** */
		void	open() throw();
		/** */
		void	close() throw();
		/** */
		void	oneSecondTick() throw();
		/** */
		void	rx(const void* pdu,unsigned length) throw();
		/** */
		void	appendOption(	unsigned char	type,
								const void*		option,
								unsigned char	length
								) throw();
		/** */
		void	appendConfigNAK(	unsigned char	type,
									unsigned		length,
									const void*		option
									) throw();
		/** */
		void	appendConfigREJ(	unsigned char	type,
									unsigned		length,
									const void*		option
									) throw();

	private:	// Oscl::PPP::FSM::Context
		/** */
		void	tlu() throw();
		/** */
		void	tld() throw();
		/** */
		void	tls() throw();
		/** */
		void	tlf() throw();
		/** */
		void	irc() throw();
		/** */
		void	zrc() throw();
		/** */
		void	scr() throw();
		/** */
		void	sca() throw();
		/** */
		void	scn() throw();
		/** */
		void	str() throw();
		/** */
		void	sta() throw();
		/** */
		void	scj() throw();
		/** */
		void	ser() throw();
		/** */
		void	timerEnable() throw();
		/** */
		void	timerDisable() throw();

	private:
		/** */
		void	sendProtocolReject() throw();
		/** */
		void	configureRequest(	const unsigned char*	pdu,
									unsigned				length
									) throw();
		/** */
		void	configureAck(	const unsigned char*	pdu,
								unsigned				length
								) throw();
		/** */
		void	configureNak(	const unsigned char*	pdu,
								unsigned				length
								) throw();
		/** */
		void	configureReject(	const unsigned char*	pdu,
									unsigned				length
									) throw();
		/** */
		void	terminateRequest(	const unsigned char*	pdu,
									unsigned				length
									) throw();
		/** */
		void	terminateAck(	const unsigned char*	pdu,
								unsigned				length
								) throw();
		/** */
		void	codeReject(	const unsigned char*	pdu,
							unsigned				length
							) throw();
		/** */
		void	protocolReject(	const unsigned char*	pdu,
								unsigned				length
								) throw();
		/** */
		void	echoRequest(	const unsigned char*	pdu,
								unsigned				length
								) throw();
		/** */
		void	echoReply(	const unsigned char*	pdu,
							unsigned				length
							) throw();
		/** */
		void	discardRequest(	const unsigned char*	pdu,
								unsigned				length
								) throw();
	};

}
}
}
}


#endif
