/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_ppp_fsm_varh_
#define _oscl_ppp_fsm_varh_

/** */
namespace Oscl {
/** */
namespace PPP {
/** */
namespace FSM {

/** */
class Context {
	public:
		/** */
		virtual ~Context() {}
	public:
		/** */
		virtual	void	tlu() throw()=0;
		/** */
		virtual	void	tld() throw()=0;
		/** */
		virtual	void	tls() throw()=0;
		/** */
		virtual	void	tlf() throw()=0;
		/** */
		virtual	void	irc() throw()=0;
		/** */
		virtual	void	zrc() throw()=0;
		/** */
		virtual	void	scr() throw()=0;
		/** */
		virtual	void	sca() throw()=0;
		/** */
		virtual	void	scn() throw()=0;
		/** */
		virtual	void	str() throw()=0;
		/** */
		virtual	void	sta() throw()=0;
		/** */
		virtual	void	scj() throw()=0;
		/** */
		virtual	void	ser() throw()=0;
		/** */
		virtual	void	timerEnable() throw()=0;
		/** */
		virtual	void	timerDisable() throw()=0;
	};

/** */
class StateVar;

/** */
class State {
	public:
		/** */
		virtual void	up(Context& context,StateVar& sv) const throw();
		/** */
		virtual void	down(Context& context,StateVar& sv) const throw();
		/** */
		virtual void	open(Context& context,StateVar& sv) const throw();
		/** */
		virtual void	close(Context& context,StateVar& sv) const throw();
		/** */
		virtual void	topos(Context& context,StateVar& sv) const throw();
		/** */
		virtual void	toneg(Context& context,StateVar& sv) const throw();
		/** */
		virtual void	rcrpos(Context& context,StateVar& sv) const throw();
		/** */
		virtual void	rcrneg(Context& context,StateVar& sv) const throw();
		/** */
		virtual void	rca(Context& context,StateVar& sv) const throw();
		/** */
		virtual void	rcn(Context& context,StateVar& sv) const throw();
		/** */
		virtual void	rtr(Context& context,StateVar& sv) const throw();
		/** */
		virtual void	rta(Context& context,StateVar& sv) const throw();
		/** */
		virtual void	ruc(Context& context,StateVar& sv) const throw();
		/** */
		virtual void	rxjpos(Context& context,StateVar& sv) const throw();
		/** */
		virtual void	rxjneg(Context& context,StateVar& sv) const throw();
		/** */
		virtual void	rxr(Context& context,StateVar& sv) const throw();
	};

/** */
class Initial;
/** */
class Starting;
/** */
class Closed;
/** */
class Stopped;
/** */
class Closing;
/** */
class Stopping;
/** */
class ReqSent;
/** */
class AckRcvd;
/** */
class AckSent;
/** */
class Opened;

/** */
class StateVar {
	private:
		/** */
		Context&		_context;
		/** */
		const State*	_state;

	public:
		/** */
		StateVar(Context& context) throw();
		/** */
		void	up() throw();
		/** */
		void	down() throw();
		/** */
		void	open() throw();
		/** */
		void	close() throw();
		/** */
		void	topos() throw();
		/** */
		void	toneg() throw();
		/** */
		void	rcrpos() throw();
		/** */
		void	rcrneg() throw();
		/** */
		void	rca() throw();
		/** */
		void	rcn() throw();
		/** */
		void	rtr() throw();
		/** */
		void	rta() throw();
		/** */
		void	ruc() throw();
		/** */
		void	rxjpos() throw();
		/** */
		void	rxjneg() throw();
		/** */
		void	rxr() throw();
	private:
		/** */
		friend class State;
		/** */
		friend class Initial;
		/** */
		friend class Starting;
		/** */
		friend class Closed;
		/** */
		friend class Stopped;
		/** */
		friend class Closing;
		/** */
		friend class Stopping;
		/** */
		friend class ReqSent;
		/** */
		friend class AckRcvd;
		/** */
		friend class AckSent;
		/** */
		friend class Opened;
		/** */
		void	changeToInitial() throw();
		/** */
		void	changeToStarting() throw();
		/** */
		void	changeToClosed() throw();
		/** */
		void	changeToStopped() throw();
		/** */
		void	changeToClosing() throw();
		/** */
		void	changeToStopping() throw();
		/** */
		void	changeToReqSent() throw();
		/** */
		void	changeToAckRcvd() throw();
		/** */
		void	changeToAckSent() throw();
		/** */
		void	changeToOpened() throw();
		/** */
		void	protocolError() throw();
	};

}
}
}

#endif
