/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_ppp_layer_lcp_parth_
#define _oscl_ppp_layer_lcp_parth_
#include "oscl/mt/itc/srv/closeapi.h"
#include "oscl/ppp/layer/state/respmem.h"
#include "oscl/ppp/layer/state/dist/part.h"
#include "oscl/ppp/fsm/opt.h"
#include "oscl/frame/pdu/tx/respapi.h"
#include "oscl/frame/pdu/rx/respmem.h"
#include "oscl/hdlc/llc/proto/tx/encoder.h"
#include "txtrans.h"
#include "rxtrans.h"

/** */
namespace Oscl {
/** */
namespace PPP {
/** */
namespace Layer {
/** */
namespace LCP {

/** */
class Part :	private Oscl::PPP::Layer::State::Resp::Api,
				private Oscl::Mt::Itc::Srv::Close::Req::Api,
				private Oscl::Frame::Pdu::Rx::Resp::Api,
				private Oscl::PPP::FSM::OPT::Context,
				private Oscl::PDU::TX::Api
				{
	private:
		/** */
		friend class TxTrans;
		/** */
		friend class RxTrans;
	private:
		/** */
		enum{	optionTypeMaximumReceiveUnit				= 1,
				optionTypeAsyncCtrlCharMap					= 2,
				optionTypeAuthenticationProtocol			= 3,
				optionTypeQualityProtocol					= 4,
				optionTypeMagicNumber						= 5,
				optionTypeProtocolFieldCompression			= 7,
				optionTypeAddressAndControlFieldCompression	= 8 
				};

	private:
		/** */
		union TxTransMem {
			/** */
			void*	__qitemlink;
			/** */
			Oscl::Memory::
			AlignedBlock< sizeof(TxTrans) >	trans;
			};
		/** */
		union RxTransMem {
			/** */
			void*	__qitemlink;
			/** */
			Oscl::Memory::
			AlignedBlock< sizeof(RxTrans) >	trans;
			};
		/** */
		union CancelMem {
			/** */
			Oscl::Frame::Pdu::Rx::Resp::CancelMem			rx;
			/** */
			Oscl::PPP::Layer::State::Resp::CancelRespMem	change;
			};
	private:
		/** */
		enum{restartCountInSeconds=3};
		/** */
		enum{timerValueInSeconds=2};
		/** */
		Oscl::PPP::Layer::
		State::Resp::Api::ChangeResp		_changeResp;
		/** */
		CancelMem							_cancelMem;
		/** */
		Oscl::Mt::Itc::PostMsgApi&			_myPapi;
		/** */
		Oscl::PPP::Layer::State::Dist::Part	_layerStateDist;
		/** */
		Oscl::PPP::Layer::
		State::Req::Api::SAP&				_lowerLayerSAP;
		/** */
		Oscl::Frame::Pdu::
		Tx::Req::Api::SAP&					_txProtocolSAP;
		/** */
		Oscl::Frame::Pdu::
		Rx::Req::Api::SAP&					_rxProtocolSAP;
		/** */
		Oscl::HDLC::LLC::Proto::TX::Encoder	_llcProtoEncoder;
		/** */
		Oscl::PPP::FSM::
		OPT::Variable						_optFSM;
		/** */
		Oscl::Queue<TxTransMem>				_freeTxTransMem;
		/** */
		Oscl::Queue<RxTransMem>				_freeRxTransMem;
		/** */
		TxTrans*							_currentTxTrans;
		/** Function pointer to next sub-server to asynchronously close
		 */
		void			(Part::*	_nextStep)();
		/** */
		Oscl::Mt::Itc::Srv::
		Open::Req::Api::OpenReq*			_openReq;
		/** */
		Oscl::Mt::Itc::Srv::
		Close::Req::Api::CloseReq*			_closeReq;
		/** */
		bool								_open;
		/** */
		bool								_layerActive;
	private:
		unsigned			_mru;
		unsigned			_rxAsyncCharMap;
		unsigned			_txAsyncCharacterMap;

	public:
		/** */
		Part(	Oscl::Mt::Itc::PostMsgApi&	myPapi,
				Oscl::PPP::Layer::
				State::Req::Api::SAP&		lowerLayerSAP,
				Oscl::Frame::Pdu::
				Tx::Req::Api::SAP&			txProtocolSAP,
				Oscl::Frame::Pdu::
				Rx::Req::Api::SAP&			rxProtocolSAP,
				unsigned					protocolType,
				RxTransMem					rxTransMem[],
				unsigned					nRxTransMem,
				TxTransMem					txTransMem[],
				unsigned					nTxTransMem
				) throw();

		/** */
		Oscl::PPP::Layer::
		State::Req::Api::SAP&	getLayerStateSAP() throw();

		/** This operation is to be invoked approximately
			once every second from the myPapi thread.
		 */
		void	oneSecondTick() throw();

	private:
		/** */
		void		sendAllRxTrans() throw();
		/** */
		void		free(RxTrans& trans) throw();
		/** */
		void		free(TxTrans& trans) throw();

	private:
		/** */
		void		cancelTxReqs() throw();
		/** */
		void		cancelRxReqs() throw();
		/** */
		void		cancelChangeReqs() throw();
		/** */
		void		closeComplete() throw();
	private:
		/** */
		void		done(TxTrans& trans) throw();
		/** */
		void		done(RxTrans& trans) throw();

	private:
		/** */
		void	maximumReceiveUnitOption(	const unsigned char*	option,
											unsigned				length
											) throw();
		/** */
		void	asyncControlCharMapOption(	const unsigned char*	option,
											unsigned				length
									) throw();
		/** */
		void	authenticationProtocolOption(	const unsigned char*	option,
												unsigned				length
												) throw();
		/** */
		void	qualityProtocolOption(	const unsigned char*	option,
										unsigned				length
										) throw();
		/** */
		void	magicNumberOption(	const unsigned char*	option,
									unsigned				length
									) throw();
		/** */
		void	protocolFieldCompressionOption(	const unsigned char*	option,
												unsigned				length
												) throw();
		/** */
		void	addrCtrlFieldCompOption(	const unsigned char*	option,
											unsigned				length
											) throw();
		/** */
		void	mruConfigNAK() throw();
		/** */
		void	asyncControlCharMapOptionNAK() throw();
		/** */
		void	authenticationProtocolConfigNAK() throw();

	private:	// Oscl::PDU::TX::Api
		/** */
		unsigned	open(unsigned length) throw();

		/** */
		unsigned	append(const void* pdu,unsigned length) throw();

		/** */
		void		close() throw();

	private:	// Oscl::PPP::FSM::OPT::Context
		/** */
		void	up() throw();
		/** */
		void	down() throw();
		/** */
		void	start() throw();
		/** */
		void	finished() throw();
		/** */
		void	optionReq(	const unsigned char*	option,
							unsigned char			type,
							unsigned				optLen
							) throw();
		/** */
		void	buildOptions() throw();
		/** */
		void	defaultOptions() throw();
		/** */
		void	applyOptions() throw();

	private:	// Oscl::Mt::Itc::Srv::Close::Req::Api
		/** */
		void	request(	Oscl::Mt::Itc::Srv::
							Open::Req::Api::OpenReq&	msg
							) throw();
		/** */
		void	request(	Oscl::Mt::Itc::Srv::
							Close::Req::Api::CloseReq&	msg
							) throw();

	private:	// Oscl::Frame::Pdu::Rx::Resp::Api
		/** */
		void	response(Oscl::Frame::Pdu::Rx::Resp::Api::RxResp& msg) throw();

		/** */
		void	response(	Oscl::Frame::Pdu::Rx::
							Resp::Api::CancelAllResp&	msg
							) throw();

	private:	// Oscl::PPP::Layer::State::Resp::Api
		/** */
		void	response(	Oscl::PPP::Layer::
							State::Resp::Api::ChangeResp& msg
							) throw();
		/** */
		void	response(	Oscl::PPP::Layer::
							State::Resp::Api::CancelResp& msg
							) throw();
	};

}
}
}
}

#endif
