/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_ppp_lcp_lcph_
#define _oscl_ppp_lcp_lcph_
#include "oscl/ppp/fsm/opt.h"
#include "oscl/pdu/rx/api.h"

/** */
namespace Oscl {
/** */
namespace PPP {
/** */
namespace LCP {

/** */
class Context {
	public:
		/** */
		virtual ~Context(){}
		/** */
		virtual void	up() throw()=0;
		/** */
		virtual void	down() throw()=0;
		/** */
		virtual void	start() throw()=0;
		/** */
		virtual void	finished() throw()=0;
		/** */
		virtual void	setAsyncCharMapTx(unsigned long map) throw()=0;
		/** */
		virtual void	setAsyncCharMapRx(unsigned long map) throw()=0;
	};

/** */
class Protocol :	public Oscl::PPP::FSM::OPT::Context,
					public Oscl::PDU::RX::Api
					{
	private:
		/** */
		Oscl::PPP::FSM::OPT::Variable	_optionFSM;
		/** */
		LCP::Context&					_context;
		/** */
		unsigned long					_asyncCharMapTx;
		/** */
		unsigned long					_asyncCharMapRx;
		/** */
		unsigned						_mru;
		/** */
		enum{maxPduSize=256};
		/** */
		unsigned char					_pduCopy[maxPduSize];

	public:
		/** */
		Protocol(	LCP::Context&		context,
					Oscl::PDU::TX::Api&	transmitter,
					unsigned			restartCountInSeconds=3,
					unsigned			timerValueInSeconds=2
					) throw();
		/** */
		void	lowerLayerUp() throw();
		/** */
		void	lowerLayerDown() throw();
		/** */
		void	open() throw();
		/** */
		void	close() throw();
		/** */
		void	oneSecondTick() throw();
		/** */
		void	rx(	const void*		pdu,
					unsigned		length
					) throw();

	private:	// Oscl::PDU::RX::Api
		/** */
		Oscl::PDU::Api*	forward(Oscl::PDU::Api& frame) throw();

	private:	// Oscl::PPP::FSM::OPT::Context
		/** */
		void	up() throw();
		/** */
		void	down() throw();
		/** */
		void	start() throw();
		/** */
		void	finished() throw();
		/** */
		void	optionReq(	const unsigned char*	option,
							unsigned char			type,
							unsigned				optLen
							) throw();
		/** */
		void	buildOptions() throw();
		/** */
		void	defaultOptions() throw();
		/** */
		void	applyOptions() throw();

	private:
		/** */
		void	mruOption(	const unsigned char*	pdu,
							unsigned				length
							) throw();
		/** */
		void	asyncCharMapOption(	const unsigned char*	pdu,
									unsigned				length
									) throw();
		/** */
		void	authProtocolOption(	const unsigned char*	pdu,
									unsigned				length
									) throw();
		/** */
		void	qualityProtocolOption(	const unsigned char*	pdu,
										unsigned				length
										) throw();
		/** */
		void	magicNumberOption(	const unsigned char*	pdu,
									unsigned				length
									) throw();
		/** */
		void	protoFieldCompOption(	const unsigned char*	pdu,
										unsigned				length
										) throw();
		/** */
		void	addrCtrlFieldCompOption(	const unsigned char*	pdu,
											unsigned				length
											) throw();
		/** */
		void	mruConfigNAK() throw();
		/** */
		void	asyncCharMapConfigNAK() throw();
		/** */
		void	authProtocolConfigNAK() throw();
	};

}
}
}

#endif
