/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_ppp_stream_basic_configh_
#define _oscl_ppp_stream_basic_configh_
#include "part.h"

/** */
namespace Oscl {
/** */
namespace PPP {
/** */
namespace Stream {
/** */
namespace Basic {

/** */
template <	nComposites,		// 4
			nFragments,			// 4
			pduBufferSize,		// 128
			nStreamReads,		// 4
			streamReadBuffSize	// 32
			>
class Config {
	private:
		/** */
		union StreamBuffMem {
			/** */
			Oscl::Memory::AlignedBlock<bufferSize>	mem;
			};
		/** */
		union PduBuffMem {
			/** */
			Oscl::Memory::AlignedBlock< pduBufferSize >	mem;
			};

	private:
		/** */
		ReadTransMem		readMem[nStreamReads];
		/** */
		StreamBuffMem		streamBufferMem[nStreamReads];

	private:
		/** */
		CompositeMem	compositeMem[nComposites];
		/** */
		FragmentMem		fragmentMem[nFragments];
		/** */
		FixedMem		fixedMem[nFragments];
		/** */
		PduBuffMem		pduBufferMem[nFragments];

	public:
		/** */
		Part			_part;

	public:
		/** */
		Config(	Oscl::Mt::Itc::PostMsgApi&			pppPapi,
				Oscl::Mt::Itc::PostMsgApi&			txFramerPapi,
				Oscl::Mt::Itc::PostMsgApi&			rxFramerPapi,
				Oscl::Stream::Input::Req::Api::SAP&	rxStreamSAP,
				Oscl::Stream::Output::Api&			txStream,
				Oscl::Stream::Input::Req::Api::SAP&	streamSAP
				) throw():
			_part(	pppPapi,
					txFramerPapi,
					rxFramerPapi,
					rxStreamSAP,
					txStream,
					readMem,
					nStreamReads,
					streamReadBuffSize,
					streamBufferMem,
					compositeMem,
					fragmentMem,
					fixedMem,
					pduBufferMem,
					nComposites,
					nFragments,
					pduBufferSize
					){}
	};

}
}
}
}

#endif
