/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_protocol_ieee_mach_
#define _oscl_protocol_ieee_mach_
#include "oscl/compiler/types.h"

/** */
namespace Oscl {
/** */
namespace Protocol {
/** */
namespace IEEE {

/**  Represents a 48bit IEEE Mac address. Octets of the
	address are ordered as they are transmitted on the wire.
	The LSB of each octet is transmitted first. The first
	octet is transmitted first and the sixth octet is
	transmitted last.
	No constructor allowed. This is to allow the type
	to be used in packet structures unions. The MacAddressInit
	type is a subclass that allows for initialized instantiation.
 */
class MacAddress {
	public:
		/** */
		enum{nOctets=6};
		/** */
		enum{nBits=8*nOctets};
	private:
		/** */
		Octet	_addr[nOctets];
	public:
		/** */
		inline Octet	firstOctet() const throw(){
			return _addr[0];
			}
		/** */
		inline Octet	secondOctet() const throw(){
			return _addr[1];
			}
		/** */
		inline Octet	thirdOctet() const throw(){
			return _addr[2];
			}
		/** */
		inline Octet	fourthOctet() const throw(){
			return _addr[3];
			}
		/** */
		inline Octet	fifthOctet() const throw(){
			return _addr[4];
			}
		/** */
		inline Octet	sixthOctet() const throw(){
			return _addr[5];
			}
	public:
		/** */
		inline void	firstOctet(Octet octet) throw(){
			_addr[0]	= octet;
			}
		/** */
		inline void	secondOctet(Octet octet) throw(){
			_addr[1]	= octet;
			}
		/** */
		inline void	thirdOctet(Octet octet) throw(){
			_addr[2]	= octet;
			}
		/** */
		inline void	fourthOctet(Octet octet) throw(){
			_addr[3]	= octet;
			}
		/** */
		inline void	fifthOctet(Octet octet) throw(){
			_addr[4]	= octet;
			}
		/** */
		inline void	sixthOctet(Octet octet) throw(){
			_addr[5]	= octet;
			}
		/** */
        inline MacAddress&    operator =(const MacAddress& other) throw(){
			_addr[0] 	= other._addr[0];
			_addr[1] 	= other._addr[1];
			_addr[2] 	= other._addr[2];
			_addr[3] 	= other._addr[3];
			_addr[4] 	= other._addr[4];
			_addr[5] 	= other._addr[5];
			return *this;
			}
        inline bool    operator ==(const MacAddress& other) throw(){
			return(		(_addr[0] == other._addr[0])
					&&	(_addr[1] == other._addr[1])
					&&	(_addr[2] == other._addr[2])
					&&	(_addr[3] == other._addr[3])
					&&	(_addr[4] == other._addr[4])
					&&	(_addr[5] == other._addr[5])
					&&	(_addr[6] == other._addr[6])
					);
			}
	};

/** Adds constructor to MacAddress class. */
class MacAddressInit : public MacAddress {
	public:
		/** */
		MacAddressInit(	Octet	first,
						Octet	second,
						Octet	third,
						Octet	fourth,
						Octet	fifth,
						Octet	sixth
						) throw();
	};
}
}
}

#endif
