/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "oscl/atomic/incdec.h"
#include "header.h"
using namespace Oscl::Protocol::IP;
using namespace Oscl;

Unsigned16	Oscl::Protocol::IP::getNewIpID() throw(){
	static Oscl::Atomic::Integer	id;
	return (Unsigned16)OsclAtomicIncrement(&id);
	}

Unsigned16	Header::getChecksum() const throw(){
	Unsigned32	t	= type;
	t	&=	TtlProtoCheck::Checksum::FieldMask;
	t	>>=	TtlProtoCheck::Checksum::Lsb;
	return (Unsigned16)t;
	}

unsigned char	Header::getHeaderLength() const throw(){
	Unsigned32	t	= length;
	t	&=	TypeLen::IHL::FieldMask;
	t	>>=	TypeLen::IHL::Lsb;
	return (unsigned char)t;
	}

unsigned	Header::getVersion() const throw(){
	Unsigned32	t	= length;
	t	&=	TypeLen::Version::FieldMask;
	t	>>=	TypeLen::Version::Lsb;
	return (unsigned)t;
	}

unsigned char	Header::getProtocolType() const throw(){
	Unsigned32	t	= type;
	t	&=	TtlProtoCheck::Protocol::FieldMask;
	t	>>=	TtlProtoCheck::Protocol::Lsb;
	return (unsigned char)t;
	}

unsigned	Header::getTotalLength() const throw(){
	Unsigned32	t	= length;
	t	&=	TypeLen::TotalLength::FieldMask;
	t	>>=	TypeLen::TotalLength::Lsb;
	return (unsigned)t;
	}

unsigned	Header::getPayloadLength() const throw(){
	Unsigned32	totalLen	= length;
	Unsigned32	hdrLen		= totalLen;
	totalLen	&=	TypeLen::TotalLength::FieldMask;
	totalLen	>>=	TypeLen::TotalLength::Lsb;
	hdrLen		&=	TypeLen::IHL::FieldMask;
	hdrLen		>>=	TypeLen::IHL::Lsb;
	return (unsigned)(totalLen - (hdrLen<<2));
	}

void	Header::updateChecksum(unsigned nOptionBytes) throw(){
	Unsigned16	csum	= computeChecksum(nOptionBytes);
	if(!csum){
		csum	= ~csum;
		}
	type.changeBits(	TtlProtoCheck::Checksum::FieldMask,
						(csum<<TtlProtoCheck::Checksum::Lsb)
						);
	}

Unsigned16	Header::computeChecksum(unsigned nOptionBytes) const throw(){
	Unsigned32	csum	= 0;
	Unsigned32	l;
	l	= length;
	csum	+=	(l>>16) & 0x0000FFFF;
	csum	+=	l & 0x0000FFFF;
	l	= frag;
	csum	+=	(l>>16) & 0x0000FFFF;
	csum	+=	l & 0x0000FFFF;
	l	= type;
	l	&= ~(Oscl::Protocol::IP::TtlProtoCheck::Checksum::FieldMask);
	csum	+=	(l>>16) & 0x0000FFFF;
	csum	+=	l & 0x0000FFFF;
	l	=	source;
	csum	+=	(l>>16) & 0x0000FFFF;
	csum	+=	l & 0x0000FFFF;
	l	=	destination;
	csum	+=	(l>>16) & 0x0000FFFF;
	csum	+=	l & 0x0000FFFF;
	for(unsigned i=0;i<nOptionBytes;i+=2){
		Unsigned16	w;
		w	= options[i];
		w	<<= 8;
		w	|=	options[i+1];
		csum	+= w;
		}
	csum	+= (csum >> 16);
	csum	= ((~csum) & 0x0000FFFF);
	return csum;
	}

bool	Header::checksumOK(unsigned nOptionBytes) const throw(){
	Unsigned16	check	= getChecksum();
	if(check == 0){
		// Checksum not used by sender.
		return true;
		}
	Unsigned16	csum	= computeChecksum(nOptionBytes);
	if(check == csum){
		return true;
		}
	return false;
	}

