/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_protocol_ip_headerh_
#define _oscl_protocol_ip_headerh_
#include "address.h"
#include "ipreg.h"
#include "oscl/endian/type.h"
//#include "oscl/bits/bitfield.h"

/** */
namespace Oscl {
/** */
namespace Protocol {
/** */
namespace IP {

Unsigned16	getNewIpID() throw();

/** */
//typedef Oscl::Endian::Big::U32	Option;

/** */
struct Header {
	/** */
	typedef Oscl::Endian::Big::U32	Length;
	/** */
	typedef Oscl::Endian::Big::U32	Frag;
	/** */
	typedef Oscl::Endian::Big::U32	Type;
	/** */
	enum{minHeaderBytes=(sizeof(Unsigned32)*TypeLen::IHL::Value_Minimum)};
	/** */
	enum{maxHeaderBytes=(sizeof(Unsigned32)*TypeLen::IHL::Value_Maximum)};
	/** */
	enum{optionWordSize=sizeof(Unsigned32)};
	/** */
//	enum{maxOptionBytes=maxHeaderBytes-sizeof(Header)};
	enum{	maxOptionBytes=		maxHeaderBytes
							-	(		sizeof(Unsigned32)
									*TypeLen::IHL::Value_Minimum
									)
			};
	/** */
	Length		length;
	/** */
	Frag		frag;
	/** */
	Type		type;
	/** */
	Address		source;
	/** */
	Address		destination;
	/** */
	Unsigned8	options[maxOptionBytes];

	/** */
	Unsigned16	getChecksum() const throw();

	/** */
	unsigned char	getHeaderLength() const throw();

	/** */
	unsigned	getVersion() const throw();

	/** */
	unsigned char	getProtocolType() const throw();

	/** */
	unsigned	getTotalLength() const throw();

	/** */
	unsigned	getPayloadLength() const throw();

	/** */
	bool	checksumOK(unsigned nOptionBytes) const throw();
	/** */
	Unsigned16	computeChecksum(unsigned nOptionBytes) const throw();
	/** */
	void	updateChecksum(unsigned nOptionBytes) throw();
	};

}
}
}

#endif
