/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_protocol_ip_rx_srv_if_reqapih_
#define _oscl_protocol_ip_rx_srv_if_reqapih_
#include "oscl/mt/itc/mbox/srvreq.h"
#include "oscl/mt/itc/mbox/sap.h"
#include "oscl/protocol/ip/rx/if/rxif.h"

/** */
namespace Oscl {
/** */
namespace Protocol {
/** */
namespace IP {
/** */
namespace RX {
/** */
namespace Srv {
/** */
namespace IF {
/** */
namespace Req {

/** */
class AttachPayload {
	public:
		/** This is a reference to the interface
			being attached. The IP receive subsystem
			stores and thus owns this instance until
			it is subsequently detached. 
		 */
		Oscl::Protocol::IP::RX::IF::RxInterface&		_interface;

	public:
		/** */
		AttachPayload(	Oscl::Protocol::IP::
						RX::IF::RxInterface&	interface
						) throw();
	};

/** */
class DetachPayload {
	public:
		/** This is a reference to a previously attached
			interface. The IP receive subsystem
			removes all references to this interface
			before returning the detach message.
		 */
		Oscl::Protocol::IP::RX::IF::RxInterface&		_interface;

	public:
		/** */
		DetachPayload(	Oscl::Protocol::IP::
						RX::IF::RxInterface&		interface
						) throw();
	};

/** */
class Api {
	public:
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<Api,AttachPayload>	AttachReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<Api,DetachPayload>	DetachReq;
		/** */
		typedef Oscl::Mt::Itc::SAP<Api>						SAP;
		/** */
		typedef Oscl::Mt::Itc::ConcreteSAP<Api>				ConcreteSAP;
	public:
		/** */
		virtual ~Api(){}
		/** */
		virtual void	request(AttachReq& msg) throw()=0;
		/** */
		virtual void	request(DetachReq& msg) throw()=0;
	};

}
}
}
}
}
}
}

#endif
