/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_protocol_ip_tcp_conn_fsm_contexth_
#define _oscl_protocol_ip_tcp_conn_fsm_contexth_
#include "oscl/modulo/u32.h"

/** */
namespace Oscl {
/** */
namespace Protocol {
/** */
namespace IP {
/** */
namespace TCP {
/** */
namespace Conn {
/** */
namespace FSM {

/** */
class Context {
	public:
		/** */
		virtual ~Context(){}

		/** */
		virtual void	sendRstAck(	unsigned long	seq,
									unsigned long	ack
									) throw()=0;
		virtual void	sendSyn(unsigned long 	seq) throw()=0;
		virtual void	sendFin() throw()=0;
		virtual void	sendSynAck(	unsigned long	seq,
									unsigned long	ack
									) throw()=0;
		virtual void	sendRst(unsigned long 	seq) throw()=0;
		virtual void	sendAck(	unsigned long	seq,
									unsigned long	ack
									) throw()=0;
		virtual void	sendPiggybackAck(	unsigned long	seq,
											unsigned long	ack
											) throw()=0;
		/** Send queued requests as resources and flow control allow,
			with piggyback acknowledgment etc. Context is responsible
			for returning completed send requests to user after each
			is segmentized transmitted and placed in the retransmission
			queue.
		 */
		virtual void	sendQueuedPiggyback() throw()=0;
		/** If there are any queued received segments and there
			are queued receive requests, satisfy as many receive
			requests with the queued segments as possible, and
			return completed requests.
		 */
		virtual void	processQueuedReceiveSegments() throw()=0;
		/** Return current rx request with a closing error. */
		virtual void	returnRxRequestWithClosingError() throw()=0;
		virtual void	queueSendRequest() throw()=0;
		virtual void	queueNormalReceiveRequest() throw()=0;
		virtual void	queueUrgentReceiveRequest() throw()=0;
		virtual void	queuePushRequest() throw()=0;
		virtual void	queueCloseRequest() throw()=0;
		virtual void	copyNewConnectionValuesFromSegment() throw()=0;
		virtual void	acknowledgeRetransmissionQueue() throw()=0;
		virtual void	trimSegmentToFitWindow() throw()=0; // See RFC793 page 69 paragraph 1
		virtual void	removeAllFromRetransmissionQueue() throw()=0;
		virtual void	resetOutstandingReceiveRequests() throw()=0;
		virtual void	resetOutstandingTransmitRequests() throw()=0;
		/** All pending receive requests are returned with
			closing error.
		 */
		virtual void	returnAllPendingReceives() throw()=0;
		virtual void	returnAllPendingSends() throw()=0;
		virtual void	flushAllSegmentQueues() throw()=0;
		virtual void	signalConnectionClosing() throw()=0;
		virtual void	signalConnectionReset() throw()=0;
		virtual void	signalUrgentData() throw()=0;
		virtual void	signalUrgentDataIfRequired() throw()=0;
		virtual void	adjustWindow(unsigned nBytesReceived) throw()=0;
		virtual void	startTimeWaitTimer() throw()=0;
		virtual void	stopTimeWaitTimer() throw()=0;
		virtual void	startUserTimer() throw()=0;
		virtual void	stopUserTimer() throw()=0;
		virtual void	startRetransmissionTimer() throw()=0;
		virtual void	stopRetransmissionTimer() throw()=0;
		virtual void	acknowledgeUserClose() throw()=0;
		virtual void	deleteTheTCB() throw()=0;
		virtual void	setActiveOpen() throw()=0;
		virtual void	returnSendReqWithNotOpenError() throw()=0;
		virtual unsigned	deliverTextToRxBuffers() throw()=0;
		virtual void	returnOpenReq() throw()=0;
		virtual void	listening() throw()=0;

		virtual bool	userTimerIsExpired() throw()=0;

		/** SND */
		virtual void	setSndUna(const Oscl::Modulo::U32& value) throw()=0;
		virtual void	setSndNxt(const Oscl::Modulo::U32& value) throw()=0;
		virtual void	setSndWnd(unsigned long value) throw()=0;
		virtual void	setSndUp(unsigned long value) throw()=0;
		virtual void	setSndWl1(const Oscl::Modulo::U32& value) throw()=0;
		virtual void	setSndWl2(const Oscl::Modulo::U32& value) throw()=0;
		virtual void	setISS(const Oscl::Modulo::U32& value) throw()=0;

		/** RCV */
		virtual void	setRcvNxt(const Oscl::Modulo::U32& value) throw()=0;
		virtual void	setRcvWnd(unsigned long value) throw()=0;
		virtual void	setRcvUp(unsigned long value) throw()=0;
		virtual void	setIRS(const Oscl::Modulo::U32& value) throw()=0;

		/** */
		virtual bool	isACK() const throw()=0;
		virtual bool	isRST() const throw()=0;
		virtual bool	isSYN() const throw()=0;
		virtual bool	isURG() const throw()=0;
		virtual bool	isFIN() const throw()=0;
		virtual bool	securityCompartmentMatches() const throw()=0;
		virtual bool	isPassiveOpen() const throw()=0;
		virtual bool	finIsAcknowledged() const throw()=0;
		/** Returns true if any segments have been transmitted. */
		virtual bool	segmentsHaveBeenSent() const throw()=0;
		virtual bool	sendRequestQueueIsEmpty() const throw()=0;
		virtual bool	closeRequestQueueIsEmpty() const throw()=0;
		virtual bool	pendingRxSegmentQueueIsEmpty() const throw()=0;
		virtual bool	retransmissionQueueIsEmpty() const throw()=0;

		/** */
		virtual Oscl::Modulo::U32	getSegSEQ() const throw()=0;
		/** */
		virtual unsigned long		getSegLEN() const throw()=0;
		/** */
		virtual Oscl::Modulo::U32	getSegACK() const throw()=0;
		/** */
		virtual unsigned long		getSegUP() const throw()=0;
		/** */
		virtual unsigned long		getSegWND() const throw()=0;
		/** */
		virtual Oscl::Modulo::U32	getNewISS() const throw()=0;

		/** */
		virtual Oscl::Modulo::U32	getSndUna() const throw()=0;
		/** */
		virtual Oscl::Modulo::U32	getSndNxt() const throw()=0;
		/** */
		virtual unsigned long		getSndWnd() const throw()=0;
		/** */
		virtual unsigned long		getSndUp() const throw()=0;
		/** */
		virtual Oscl::Modulo::U32	getSndWl1() const throw()=0;
		/** */
		virtual Oscl::Modulo::U32	getSndWl2() const throw()=0;
		/** */
		virtual Oscl::Modulo::U32	getISS() const throw()=0;
		/** */
		virtual Oscl::Modulo::U32	getRcvNxt() const throw()=0;
		/** */
		virtual unsigned long		getRcvWnd() const throw()=0;
		/** */
		virtual unsigned long		getRcvUp() const throw()=0;
		/** */
		virtual Oscl::Modulo::U32	getIRS() const throw()=0;
	};

}
}
}
}
}
}

#endif
