/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "var.h"
#include "closed.h"
#include "listen.h"
#include "synrcvd.h"
#include "synsent.h"
#include "estab.h"
#include "finwait1.h"
#include "closewait.h"
#include "finwait2.h"
#include "closing.h"
#include "lastack.h"
#include "timewait.h"
#include "oscl/error/fatal.h"

using namespace Oscl::Protocol::IP::TCP::Conn::FSM;

const static Closed		stateClosed;
const static Listen		stateListen;
const static SynRcvd	stateSynRcvd;
const static SynSent	stateSynSent;
const static Estab		stateEstab;
const static FinWait1	stateFinWait1;
const static CloseWait	stateCloseWait;
const static FinWait2	stateFinWait2;
const static Closing	stateClosing;
const static LastAck	stateLastAck;
const static TimeWait	stateTimeWait;

Var::Var(Context& context) throw():
		_context(context),
		_state(&stateClosed)
		{
	}

void	Var::open() throw(){
	_state->open(_context,*this);
	}

void	Var::close() throw(){
	_state->close(_context,*this);
	}

void	Var::send() throw(){
	_state->send(_context,*this);
	}

void	Var::push() throw(){
	_state->push(_context,*this);
	}

void	Var::normalRx() throw(){
	_state->normalRx(_context,*this);
	}

void	Var::urgentRx() throw(){
	_state->urgentRx(_context,*this);
	}

void	Var::segmentArrival() throw(){
	NextStep	nextStep	= _state->segStage1(_context,*this);
	if(nextStep == Done) return;
	while(true){
		NextStep	prevStep	= nextStep;
		switch(nextStep){
			case Step1:
				nextStep	= _state->segStage2Step1(_context,*this);
				break;
			case Step2:
				nextStep	= _state->segStage2Step2(_context,*this);
				break;
			case Step3:
				nextStep	= _state->segStage2Step3(_context,*this);
				break;
			case Step4:
				nextStep	= _state->segStage2Step4(_context,*this);
				break;
			case Step5:
				nextStep	= _state->segStage2Step5(_context,*this);
				break;
			case Step6:
				nextStep	= _state->segStage2Step6(_context,*this);
				break;
			case Step7:
				nextStep	= _state->segStage2Step7(_context,*this);
				break;
			case Step8:
				nextStep	= _state->segStage2Step8(_context,*this);
				break;
			default:
				// Illegal STEP!
				// This should NEVER happen.
				for(;;);	// FIXME: use standard fatal error
				break;
			}
		if(nextStep == Done) break;
		if(nextStep <= prevStep){
			// Illegal! Cannot go back a step.
			// This should NEVER happen
			for(;;);	// FIXME: use standard fatal error
			}
		}
	}

void	Var::userTimeout() throw(){
	_state->userTimeout(_context,*this);
	}

void	Var::retransmissionTimeout() throw(){
	_state->retransmissionTimeout(_context,*this);
	}

void	Var::timeWaitTimeout() throw(){
	_state->timeWaitTimeout(_context,*this);
	}

void	Var::tick() throw(){
	_state->tick(_context,*this);
	}

void	Var::protocolError(	const char*	stateName,
							const char*	eventName
							){
	Oscl::ErrorFatal::logAndExit("");
	}

void	Var::changeToClosed() throw(){
	_state	= &stateClosed;
	}

void	Var::changeToListen() throw(){
	_state	= &stateListen;
	}

void	Var::changeToSynRcvd() throw(){
	_state	= &stateSynRcvd;
	}

void	Var::changeToSynSent() throw(){
	_state	= &stateSynSent;
	}

void	Var::changeToEstab() throw(){
	_state	= &stateEstab;
	}

void	Var::changeToFinWait1() throw(){
	_state	= &stateFinWait1;
	}

void	Var::changeToCloseWait() throw(){
	_state	= &stateCloseWait;
	}

void	Var::changeToFinWait2() throw(){
	_state	= &stateFinWait2;
	}

void	Var::changeToClosing() throw(){
	_state	= &stateClosing;
	}

void	Var::changeToLastAck() throw(){
	_state	= &stateLastAck;
	}

void	Var::changeToTimeWait() throw(){
	_state	= &stateTimeWait;
	}

