/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_scsi_cmd_reqsenseh_
#define _oscl_scsi_cmd_reqsenseh_
#include "oscl/compiler/types.h"

/** */
namespace Oscl {
/** */
namespace SCSI {
/** */
namespace CMD {

/** NOT in RBC */
namespace RequestSense {
/** */
struct Request {
	/** */
	const Unsigned8	_opcode;
	/** */
	Unsigned8		_reserved1;
	/** */
	Unsigned8		_reserved2;
	/** */
	Unsigned8		_reserved3;
	/** Maximum number of bytes that the client
		has allocated for returned data.
	 */
	Unsigned8		_allocationLength;
	/** */
	Unsigned8		_control;
	/** */
	Request() throw():
		_opcode(0x03),
		_reserved1(0),
		_reserved2(0),
		_reserved3(0),
		_control(0)
		{}
	};

/** */
struct Standard : public Request {
	/** */
	Standard(Unsigned8 allocationLength) throw(){
		_allocationLength	= allocationLength;
		}
	};

/** */
struct Response {
	/** */
	enum{size=18};
	/** */
	enum{minSize=8};
	/** */
	Unsigned8	_responseCode;
	/** */
	Unsigned8	_obsolete;
	/** */
	Unsigned8	_flags;
	/** */
	Unsigned8	_information[4];
	/** (n-7)*/
	Unsigned8	_additionalSenseLength;
	/** */
	Unsigned8	_cmdSpecificInfo[4];
	/** */
	Unsigned8	_additionalSenseCode;
	/** */
	Unsigned8	_additionalSeseCodeQualifier;
	/** */
	Unsigned8	_fieldReplaceableUnitCode;
	/** */
	Unsigned8	_senseKeySpecific1;
	/** */
	Unsigned8	_senseKeySpecific2;
	/** */
	inline bool	valid() const throw(){
		return _responseCode & 0x80;
		}
	/** */
	inline Unsigned8	responseCode() const throw(){
		return _responseCode & 0x7F;
		}
	/** */
	inline Unsigned8	senseKey() const throw(){
		return _flags & 0x0F;
		}
	};
void print(const Response& resp) throw();
}

}
}
}

#endif
