/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_tt_lun_scsih_
#define _oscl_tt_lun_scsih_
#include "oscl/compiler/types.h"
#include "read10.h"
#include "write10.h"
#include "reqsense.h"
#include "synccache.h"

/** */
namespace Oscl {
/** */
namespace SCSI {
/** */
namespace CMD {
/** */
namespace Inquiry {

/** */
struct Request {
	/** */
	const Unsigned8	_opcode;
	/** */
	Unsigned8		_flags;
	/** */
	Unsigned8		_pageOrOperationCode;
	/** */
	Unsigned8		_reserved;
	/** Maximum number of bytes that the client
		has allocated for returned data.
	 */
	Unsigned8		_allocationLength;
	/** */
	Unsigned8		_control;
	/** */
	Request() throw():
		_opcode(0x12),
		_reserved(0)
		{}
	};

/** */
struct Standard : public Request {
	/** */
	Standard(	Unsigned8	allocationLength) throw(){
		_flags	= 0;
		_pageOrOperationCode	= 0;
		_allocationLength		= allocationLength;
		_control				= 0;
		}
	};

/** */
struct Response {
	/** */
	enum{size=36};
	/** */
	Unsigned8	_peripheralQualifierAndDeviceType;
	/** */
	Unsigned8	_rmb;
	/** */
	Unsigned8	_version;
	/** */
	Unsigned8	_aercNacaHisupRdf;
	/** (n-4) */
	Unsigned8	_additionalLength;
	/** */
	Unsigned8	_sccs;
	/** */
	Unsigned8	_bqEsVsMpMcAddr16;
	/** */
	Unsigned8	_raWbus16SyncLinkedCqVs;
	/** */
	Unsigned8	_vendorIdentification[8];
	/** */
	Unsigned8	_productIdentification[16];
	/** */
	Unsigned8	_productRevisionLevel[4];
	};

void print(const Response& resp) throw();

}
/** */
namespace ModeSense6 {

/** */
struct Request {
	/** */
	const Unsigned8	_opcode;
	/** */
	Unsigned8		_flags;
	/** */
	Unsigned8		_pcPageCode;
	/** */
	Unsigned8		_reserved;
	/** Maximum number of bytes that the client
		has allocated for returned data.
	 */
	Unsigned8		_allocationLength;
	/** */
	Unsigned8		_control;
	/** */
	Request() throw():
		_opcode(0x1A),
		_reserved(0)
		{}
	};

/** */
struct RBC : public Request {
	/** */
	RBC(	Unsigned8	allocationLength,
			Unsigned8	pcPageCode = 0x86,
			bool		dbd	= true
			) throw(){
		if(dbd){
			_flags	= 0x08;
			}
		else {
			_flags	= 0x00;
			}
		_pcPageCode	= 	pcPageCode;
		_allocationLength		= allocationLength;
		_control				= 0;
		}
	};
/** */
struct Response {
	/** */
	enum{size=13};
	/** */
	Unsigned8	_psPageCode;
	/** */
	Unsigned8	_pageLength;
	/** */
	Unsigned8	_wcd;
	/** */
	Unsigned8	_logicalBlockSize[2];
	/** */
	Unsigned8	_numberOfLogicalBlocks[5];
	/** */
	Unsigned8	_powerPerformance;
	/** */
	Unsigned8	_readWriteFormatLock;
	/** */
	Unsigned8	_reserved;
	};

void print(const Response& resp) throw();

}

/** */
namespace ModeSense10 {

/** */
struct Request {
	/** */
	const Unsigned8	_opcode;
	/** */
	Unsigned8		_flags;
	/** */
	Unsigned8		_pcPageCode;
	/** */
	Unsigned8		_reserved3;
	/** */
	Unsigned8		_reserved4;
	/** */
	Unsigned8		_reserved5;
	/** */
	Unsigned8		_reserved6;
	/** Maximum number of bytes that the client
		has allocated for returned data.
	 */
	Unsigned8		_allocationLength[2];
	/** */
	Unsigned8		_control;
	/** */
	Request(	Unsigned16	allocationLength,
				Unsigned8	control
				) throw():
		_opcode(0x5A),
		_reserved3(0),
		_reserved4(0),
		_reserved5(0),
		_reserved6(0),
		_control(control)
		{
		_allocationLength[0]	= (Unsigned8)(allocationLength >> 8);
		_allocationLength[1]	= (Unsigned8)(allocationLength >> 0);
		}
	};

/** */
struct Standard : public Request {
	/** */
	Standard(	Unsigned16	allocationLength,
				Unsigned8	pageCode = 0
				) throw():
		Request(	allocationLength,
					0
					)
		{
		_flags	= 0x08;	// Disable Block Descriptors
		_pcPageCode	=		0x80		// Default values Page Control
										// Only Saved and Default are valid for RBC
						|	pageCode	// Specific device types Page Code
						;
		}
	};
/** */
struct Response {
	/** */
	enum{size=13};
	/** */
	Unsigned8	_psPageCode;
	/** */
	Unsigned8	_pageLength;
	/** */
	Unsigned8	_wcd;
	/** */
	Unsigned8	_logicalBlockSize[2];
	/** */
	Unsigned8	_numberOfLogicalBlocks[5];
	/** */
	Unsigned8	_powerPerformance;
	/** */
	Unsigned8	_readWriteFormatLock;
	/** */
	Unsigned8	_reserved;
	};

void print(const Response& resp) throw();

}


/** */
namespace FormatUnit {
/** */
struct Request {
	/** */
	const Unsigned8	_opcode;
	/** */
	Unsigned8		_vendorSpecific;
	/** */
	Unsigned8		_vsFlags;
	/** */
	Unsigned8		_reserved1;
	/** */
	Unsigned8		_reserved2;
	/** */
	const Unsigned8	_control;
	/** */
	Request() throw():
		_opcode(0x04),
		_reserved1(0),
		_reserved2(0),
		_control(0)
		{}
	};

/** */
struct Standard : public Request {
	/** */
	Standard() throw(){
		_vendorSpecific	= 0x00;
		_vsFlags	=		0x00	// Vendor specific
						|	0x00	// Progress
						|	0x00	// Percent/Time
						|	0x00	// Increment
						;
		}
	};

}
/** */
namespace ModeSelect {
/** */
struct Request {
	/** */
	const Unsigned8	_opcode;
	/** */
	Unsigned8		_flags;
	/** */
	Unsigned8		_reserved1;
	/** */
	Unsigned8		_reserved2;
	/** */
	Unsigned8		_parameterListLength;
	/** */
	Unsigned8		_control;
	/** */
	Request() throw():
		_opcode(0x15),
		_reserved1(0),
		_reserved2(0)
		{}
	};
/** */
struct RBC : public Request {
	/** */
	RBC() throw(){
		_flags	=			0x01	// Save Pages
						|	0x10	// Page Format
						;
		_parameterListLength	=		0x00;	// FIXME
		_control				= 0;	// FIXME
		}
	};

}
/** */
namespace Read6 {
/** */
struct Request {
	/** */
	const Unsigned8	_opcode;
	/** */
	Unsigned8		_lba[3];
	/** */
	Unsigned8		_transferLength;
	/** */
	Unsigned8		_control;
	/** */
	Request(	Unsigned32	lba,
				Unsigned8	transferLength,
				Unsigned8	control
				) throw():
		_opcode(0x08),
		_transferLength(transferLength),
		_control(control) {
		_lba[2]	= (Unsigned8)(lba >> 0);
		_lba[1]	= (Unsigned8)(lba >> 8);
		_lba[0]	= (Unsigned8)(lba >> 16);
		}
	};
/** */
struct Standard : public Request {
	/** */
	Standard(	Unsigned32	lba,
				Unsigned8	transferLength
				) throw():
		Request(lba,transferLength,0)
		{ }
	};
void print(const void* resp) throw();
}
/** */
namespace Read12 {
/** */
struct Request {
	/** */
	const Unsigned8	_opcode;
	/** */
	Unsigned8		_flags;
	/** */
	Unsigned8		_lba[4];
	/** */
	Unsigned8		_transferLength[4];
	/** */
	Unsigned8		_reserved10;
	/** */
	Unsigned8		_control;
	/** */
	Request(	Unsigned8	flags,
				Unsigned32	lba,
				Unsigned32	transferLength,
				Unsigned8	control
				) throw():
		_opcode(0xA8),
		_flags(flags),
		_reserved10(0),
		_control(control) {
		_transferLength[3]	= (Unsigned8)(transferLength >> 0);
		_transferLength[2]	= (Unsigned8)(transferLength >> 8);
		_transferLength[1]	= (Unsigned8)(transferLength >> 16);
		_transferLength[0]	= (Unsigned8)(transferLength >> 24);
		_lba[3]	= (Unsigned8)(lba >> 0);
		_lba[2]	= (Unsigned8)(lba >> 8);
		_lba[1]	= (Unsigned8)(lba >> 16);
		_lba[0]	= (Unsigned8)(lba >> 24);
		}
	};
/** */
struct Standard : public Request {
	/** */
	Standard(	Unsigned32	lba,
				Unsigned32	transferLength
				) throw():
		Request(0,lba,transferLength,0)
		{ }
	};
}
/** */
namespace ReadCapacity {
/** */
struct Request {
	/** */
	const Unsigned8	_opcode;
	/** */
	Unsigned8		_reserved1;
	/** */
	Unsigned8		_reserved2;
	/** */
	Unsigned8		_reserved3;
	/** */
	Unsigned8		_reserved4;
	/** */
	Unsigned8		_reserved5;
	/** */
	Unsigned8		_reserved6;
	/** */
	Unsigned8		_reserved7;
	/** */
	Unsigned8		_reserved8;
	/** */
	Unsigned8		_control;
	/** */
	Request() throw():
		_opcode(0x25),
		_reserved1(0),
		_reserved2(0),
		_reserved3(0),
		_reserved4(0),
		_reserved5(0),
		_reserved6(0),
		_reserved7(0),
		_reserved8(0)
		{}
	};
/** */
struct RBC : public Request {
	/** */
	RBC() throw(){
		_control	= 0;
		}
	};
/** */
struct Response {
	/** */
	enum{size=8};
	/** */
	Unsigned8	_logicalBlockAddress[4];
	/** */
	Unsigned8	_blockLengthInBytes[4];
	/** */
	inline Unsigned32	logicalBlockAddress() const throw(){
		Unsigned32
		value	=	_logicalBlockAddress[0];
		value	<<=	8;
		value	|=	_logicalBlockAddress[1];
		value	<<=	8;
		value	|=	_logicalBlockAddress[2];
		value	<<=	8;
		value	|=	_logicalBlockAddress[3];
		return value;
		}
	/** */
	inline Unsigned32	blockLengthInBytes() const throw(){
		Unsigned32
		value	=	_blockLengthInBytes[0];
		value	<<=	8;
		value	|=	_blockLengthInBytes[1];
		value	<<=	8;
		value	|=	_blockLengthInBytes[2];
		value	<<=	8;
		value	|=	_blockLengthInBytes[3];
		return value;
		}
	};

void print(const Response& resp) throw();

}
/** Prevent/Allow Medium Removal*/
namespace MediumRemoval {
}
/** */
namespace StartStopUnit {
/** */
struct Request {
	/** */
	const Unsigned8	_opcode;
	/** */
	Unsigned8		_immed;
	/** */
	Unsigned8		_reserved2;
	/** */
	Unsigned8		_reserved3;
	/** */
	Unsigned8		_flags;
	/** */
	Unsigned8		_control;
	/** */
	Request(	bool		immed,
				Unsigned8	flags,
				Unsigned8	control=0
				) throw():
		_opcode(0x1B),
		_immed(immed?0x01:0x00),
		_reserved2(0),
		_reserved3(0),
		_flags(flags),
		_control(control)
		{}
	};

/** */
struct StartSync : public Request {
	/** */
	StartSync() throw():
		Request(true,0x01,0x00)
		{}
	};
/** */
struct StopSync : public Request {
	/** */
	StopSync() throw():
		Request(true,0x00,0x00)
		{}
	};
/** */
struct GoActive : public Request {
	/** */
	GoActive() throw():
		Request(true,0x10,0x00)
		{}
	};
/** */
struct LoadMedium : public Request {
	/** */
	LoadMedium() throw():
		Request(true,0x03,0x00)
		{}
	};
}
/** */
namespace TestUnitReady {
/** */
struct Request {
	/** */
	const Unsigned8	_opcode;
	/** */
	Unsigned8		_reserved1;
	/** */
	Unsigned8		_reserved2;
	/** */
	Unsigned8		_reserved3;
	/** */
	Unsigned8		_reserved4;
	/** */
	Unsigned8		_control;
	/** */
	Request(Unsigned8 control) throw():
		_opcode(0x00),
		_reserved1(0),
		_reserved2(0),
		_reserved3(0),
		_reserved4(0),
		_control(0)
		{}
	};
/** */
struct Standard : public Request {
	/** */
	Standard() throw():
		Request(0)
		{ }
	};
}
/** */
namespace Verify {
}
/** */
namespace Write {
}
/** */
namespace WriteBuffer {
}

}
}
}

#endif
