/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_serial_itc_tx_driverh_
#define _oscl_serial_itc_tx_driverh_
#include "oscl/mt/itc/mbox/server.h"
#include "oscl/serial/driver/tx/api.h"
#include "oscl/stream/output/itc/reqapi.h"
#include "oscl/stream/output/itc/sync.h"
#include "oscl/memory/block.h"
#include "oscl/mt/itc/srv/closeapi.h"
#include "oscl/mt/itc/srv/close.h"

/** */
namespace Oscl {
/** */
namespace Serial {
/** */
namespace Itc {
/** */
namespace TX {

class Driver;

/** */
class Done : public Oscl::Done::Api {
	private:
		/** */
		Oscl::Stream::Output::Req::Api::WriteReq&	_req;
		/** */
		Driver&										_context;
	public:
		/** */
		Done(	Driver&										context,
				Oscl::Stream::Output::Req::Api::WriteReq&	req
				) throw();
	private:
		/** */
		void	done() throw();
	};

/** */
typedef Oscl::Memory::AlignedBlock< sizeof(Done) >	DoneMem;

/** */
class Driver :
	public Oscl::Mt::Itc::Server,
	public Oscl::Stream::Output::Req::Api,
	public Oscl::Mt::Itc::Srv::CloseSync,
	private Oscl::Mt::Itc::Srv::Close::Req::Api
	{
	private:
		/** */
		Oscl::Serial::Driver::TX::Api&							_driver;
		/** */
		Oscl::Stream::Output::Sync								_sync;
		/** */
		Oscl::Queue<Oscl::Stream::Output::Req::Api::WriteReq>	_pending;
		/** */
		Oscl::Queue<DoneMem>									_freeDoneList;

	public:
		/** */
		Driver(	Oscl::Serial::Driver::TX::Api&	driver,
				DoneMem							doneMem[],
				unsigned						nDoneMem
				) throw();

		/** */
		Oscl::Stream::Output::Api&				getSyncApi() throw();
		/** */
		Oscl::Stream::Output::Req::Api::SAP&	getSAP() throw();

	private:
		/** */
		void	mboxSignaled() throw();

	private:	// Oscl::Mt::Itc::Srv::Close::Req::Api
		/** */
		void    request(CloseReq& msg) throw();

	private:	// Oscl::Mt::Itc::Srv::Open::Req::Api
		/** */
		void    request(OpenReq& msg) throw();

	private: // Oscl::Stream::Output::Req::Api
		/** */
		void	request(Oscl::Stream::Output::Req::Api::WriteReq& msg) throw();
		/** */
		void	request(Oscl::Stream::Output::Req::Api::FlushReq& msg) throw();

	private:
		/** */
		friend class Done;
		/** */
		void	txPending() throw();
		/** */
		bool	tryTx(Oscl::Stream::Output::Req::Api::WriteReq& req) throw();
		/** */
		void	free(Done& done) throw();
	};

}
}
}
}

#endif
