/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "sv.h"
#include "oscl/error/fatal.h"

using namespace Oscl::Session::Creator::FSM;

static const StateVar::Closed						closed;
static const StateVar::Listening					listening;
static const StateVar::Active						active;
static const StateVar::WaitListenCancel				waitListenCancel;
static const StateVar::Closing						closing;

// StateVar

StateVar::StateVar(ContextApi& context) throw():
		_context(context),
		_state(&closed)
		{
	}

void	StateVar::open() throw(){
	_state->open(_context,*this);
	}

void	StateVar::close() throw(){
	_state->close(_context,*this);
	}

void	StateVar::listenReqDone() throw(){
	_state->listenReqDone(_context,*this);
	}

void	StateVar::sessionCloseNotificationDone() throw(){
	_state->sessionCloseNotificationDone(_context,*this);
	}

void	StateVar::listenCancelDone() throw(){
	_state->listenCancelDone(_context,*this);
	}

// global actions
void	StateVar::protocolError(const char* state) throw(){
	ErrorFatal::logAndExit(state);
	}

// state change operations
void	StateVar::changeToClosed() throw(){
	_state	= &closed;
	}

void	StateVar::changeToListening() throw(){
	_state	= &listening;
	}

void	StateVar::changeToActive() throw(){
	_state	= &active;
	}

void	StateVar::changeToWaitListenCancel() throw(){
	_state	= &waitListenCancel;
	}

void	StateVar::changeToClosing() throw(){
	_state	= &closing;
	}

