/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_session_creator_fsm_svh_
#define _oscl_session_creator_fsm_svh_

/** */
namespace Oscl {
/** */
namespace Session {
/** */
namespace Creator {
/** */
namespace FSM {

/** */
class StateVar {
	public:
		/** */
		class ContextApi {
			public:	// request actions
				/** */
				virtual	void	sendAsyncListenReq() throw()=0;
				/** */
				virtual	void	sendAsyncListenCancelReq() throw()=0;
				/** */
				virtual	void	syncCreateSession() throw()=0;
				/** */
				virtual	void	syncOpenSession() throw()=0;
				/** */
				virtual	void	syncCloseConnection() throw()=0;
				/** */
				virtual	void	syncCloseSession() throw()=0;
				/** */
				virtual	void	syncDestroySession() throw()=0;
				/** */
				virtual	void	sendAsyncSessionCloseNotificationReq() throw()=0;
				/** */
				virtual	void	returnOpenReq() throw()=0;
				/** */
				virtual	void	returnCloseReq() throw()=0;
			};
	public:
		/** */
		class State {
			public:	// Admin Events
				/** */
				virtual void	open(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				virtual void	close(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				virtual void	listenReqDone(	ContextApi&	context,
												StateVar&	sv
												) const throw();
				/** */
				virtual void	sessionCloseNotificationDone(
											ContextApi&	context,
											StateVar&	sv
											) const throw();
				/** */
				virtual void	listenCancelDone(	ContextApi&	context,
													StateVar&	sv
													) const throw();
			public:
				/** */
				virtual const char*	stateName() const throw()=0;
			};

	public:
		/** */
		class Closed : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Closed";}
			private:
				/** */
				void	open(	ContextApi&	context,
								StateVar&	sv
								) const throw();
			};
		/** */
		class Listening : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Listening";}
			public:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	listenReqDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
			};
		/** */
		class Active : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Active";}
			public:
				/** */
				void	close(	ContextApi&	context,
								StateVar&	sv
								) const throw();
				/** */
				void	sessionCloseNotificationDone(	ContextApi&	context,
														StateVar&	sv
														) const throw();
			};
		/** */
		class WaitListenCancel : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "WaitListenCancel";}
			public:
				/** */
				void	listenReqDone(	ContextApi&	context,
										StateVar&	sv
										) const throw();
				/** */
				void	listenCancelDone(	ContextApi&	context,
											StateVar&	sv
											) const throw();
			};
		/** */
		class Closing : public State {
			private:
				/** */
				const char*	stateName() const throw()
					{ return "Closing";}
			public:
				/** */
				void	sessionCloseNotificationDone(	ContextApi&	context,
														StateVar&	sv
														) const throw();
			};
	private:
		/** */
		ContextApi&		_context;
		/** */
		const State*	_state;

	public:
		/** */
		StateVar(ContextApi& context) throw();

	public:	// events
		/** */
		void	open() throw();
		/** */
		void	close() throw();
		/** */
		void	listenReqDone() throw();
		/** */
		void	sessionCloseNotificationDone() throw();
		/** */
		void	listenCancelDone() throw();

	private: // global actions and state change operations
		/** */
		friend class Closed;
		friend class Listening;
		friend class Active;
		friend class WaitListenCancel;
		friend class Closing;
		/** */
		void	protocolError(const char* state) throw();
		/** */
		void	changeToClosed() throw();
		/** */
		void	changeToListening() throw();
		/** */
		void	changeToActive() throw();
		/** */
		void	changeToWaitListenCancel() throw();
		/** */
		void	changeToClosing() throw();
	};

}
}
}
}

#endif
