/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_shell_line_stateh_
#define _oscl_shell_line_stateh_
#include "oscl/stream/output/api.h"

/** */
namespace Oscl {
/** */
namespace Shell {
/** */
namespace Line {

/** */
struct StateHeader {
	/** This output stream is to be used by the command
		to send ASCII data to the "user".
	 */
	Oscl::Stream::Output::Api&	_output;

	/** This is a pointer to the memory that may be
		used by a command to store state information
		during a multi-part statefull command. The
		memory point-to will NOT change between
		command invocations until the command indicates
		it has completed.
	 */
	void*	const		_stateMemory;

	/** This member gives the maximum size of the
		stateMemory in bytes.
	 */
	const unsigned		_maxStateMemorySize;

	/** */
	StateHeader(	Oscl::Stream::Output::Api&	output,
					void*						stateMemory,
					unsigned					maxStateMemorySize
					) throw():
			_output(output),
			_stateMemory(stateMemory),
			_maxStateMemorySize(maxStateMemorySize)
			{}
	};

}
}
}

#endif
