/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_stream_hdlc_fcs32h_
#define _oscl_stream_hdlc_fcs32h_
#include "api.h"
#include "oscl/compiler/types.h"

/** */
namespace Oscl {
/** */
namespace Stream {
/** */
namespace HDLC {
/** */
namespace FCS32 {

/** The initial FCS value */
enum{initialFCS=0xFFFFFFFF};

/** The final GOOD FCS value */
enum{goodFCS=0xDEBB20E3};

/** This is a lookup table used to calculate a 32 bit
	CRC on an octet basis. The table is the one listed
	in IETF RFC 1662.
 */
extern const Unsigned32	Table[256];

/** */
class Accumulator : public Oscl::Stream::HDLC::FCS::Api {
	private:
		/** */
		Unsigned32	_fcs;

	public:
		/** */
		Accumulator() throw();

	private:
		/** This operation is invoked to initialize the
			FCS for accumulation.
		 */
		void	initialize() throw();

		/** This operation is invoked for each octet to
			accumulate the CRC.
		 */
		void	accumulate(unsigned char octet) throw();

		/** This operation is invoked by a receiver after all
			octets in the frame have been received. It returns
			true if the accumulated CRC indicates that the frame
			was received error free.
		 */
		bool	fcsGood() throw();

		/** Returns the number of octets in the CRC. E.g. a
			CRC16 will return 2.
		  */
		unsigned		length() throw();

		/** This transmitter operation prepares the accumlated
			CRC for transmission. After invoking this operation
			the transmitter will invoke the "getNextFcsOctet()"
			operation for the number of times indicated by the
			value returned from this operation.
		 */
		unsigned		prepareToGetCRC() throw();

		/** Returns the next octet of the CRC. This operation
			is used by a transmitter to retrieve the accumulated
			CRC in the order in which the CRC is to be append
			to the frame. This operation should be invoked
			the number of times returned by the prepareToGetCRC()
			operation.
		 */
		unsigned char	getNextFcsOctet() throw();
	};

}
}
}
}

#endif
