/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_stream_hdlc_rx_fwd_forwardh_
#define _oscl_stream_hdlc_rx_fwd_forwardh_
#include "oscl/pdu/rx/api.h"
#include "oscl/pdu/buffer/allocator/itc/fixed.h"
#include "oscl/mt/itc/srv/close.h"
#include "oscl/mt/itc/srv/closeapi.h"
#include "oscl/stream/hdlc/rx/framer.h"
#include "oscl/stream/hdlc/fcs/fcs16.h"
#include "oscl/stream/input/itc/reqapi.h"

/** */
namespace Oscl {
/** */
namespace Stream {
/** */
namespace HDLC {
/** */
namespace RX {

/** */
class Forward :	public Oscl::PDU::Buffer::Allocator::Req::Api,
				public Oscl::Mt::Itc::Srv::Close::Req::Api,
				public Oscl::Frame::Stream::RX::Api,
				public Oscl::Mt::Itc::Srv::CloseSync
				{
	private:
		/** */
		enum{bufferSize=2048};
		/** */
		union FixedMem {
			void*	__qitemlink;
			Oscl::Memory::
			AlignedBlock<	sizeof(	Oscl::PDU::Buffer::
									Allocator::Fixed<bufferSize>
									)
							>	mem;
			};

		/** */
		enum{nBuffers=2};
		/** */
		FixedMem						_buffMem[nBuffers];
		/** */
		Oscl::Queue<FixedMem>			_freeBuffers;

		/** */
		Oscl::Mt::Itc::PostMsgApi&		_myPapi;
		/** */
		Oscl::PDU::Buffer::Allocator::Req::Api::ConcreteSAP	_allocSAP;
		/** */
		Oscl::PDU::RX::Api&				_receiver;

		/** */
		Oscl::PDU::Buffer::Allocator::Fixed<bufferSize>*	_current;

		/** */
		unsigned		_length;
		/** */
		unsigned char*	_buffer;

	public:
		/** */
		Forward(	Oscl::Mt::Itc::PostMsgApi&		papi,
					Oscl::PDU::RX::Api&				receiver
					) throw();

	private:	// Oscl::Mt::Itc::Srv::Open::Req::Api
		/** */
		void	request(	Oscl::Mt::Itc::Srv::
							Open::Req::Api::OpenReq&	msg
							) throw();

	private:	// Oscl::Mt::Itc::Srv::Close::Req::Api
		/** */
		void	request(	Oscl::Mt::Itc::Srv::
							Close::Req::Api::CloseReq&	msg
							) throw();

	private:	// Oscl::PDU::Buffer::Allocator::Req::Api
		/** */
		void	request(	Oscl::PDU::Buffer::
							Allocator::Req::FreeReq&	msg
							) throw();

	private:	// Oscl::Frame::Stream::RX::Api
		/** */
		void		open() throw();

		/** */
		unsigned	append(unsigned char octet) throw();

		/** */
		unsigned	append(const void* octets,unsigned nOctets) throw();

		/** */
		void		close(unsigned nFcsOctets) throw();

		/** */
		void		abort() throw();

	private:
		/** */
		void		free() throw();
		/** */
		FixedMem*	alloc() throw();
	};

}
}
}
}

#endif
