/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include <unistd.h>
#include "fd.h"

using namespace Oscl::Stream::Posix;

FD::FD(int inputFD,int outputFD) throw():
		_inputFD(inputFD),
		_outputFD(outputFD)
		{
	}

Oscl::Stream::Input::Api&	FD::input() throw(){
	return *this;
	}

Oscl::Stream::Output::Api&	FD::output() throw(){
	return *this;
	}

unsigned long	FD::read(	void*			dest,
							unsigned long	maxSize
							) throw(){
	return ::read(_inputFD,dest,maxSize);
	}

unsigned long	FD::read(Buffer::Base& dest) throw(){
	unsigned char	buffer[32];
	unsigned long	len	= read(buffer,32);
	dest.append(buffer,len);
	return len;
	}

unsigned long	FD::write(	const void*			source,
							unsigned long		count
							) throw(){
	return ::write(_outputFD,source,count);
	}

unsigned long	FD::write(const Buffer::Base& source) throw(){
	return write(source.startOfData(),source.remaining());
	}

void	FD::flush() throw(){
	// FD not buffered.
	}

