/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_strings_apih_
#define _oscl_strings_apih_

/** */
namespace Oscl {

/** */
namespace Strings {

/** */
class Api {
	protected:
		/** */
		bool	_truncated;
	public:
		/** */
		Api() throw();
		/** */
		virtual ~Api(){}
		/** */
		virtual Api&	operator += (signed long num) throw();
		/** */
		virtual Api&	operator += (signed num) throw();
		/** */
		virtual Api&	operator += (unsigned long num) throw();
		/** */
		virtual Api&	operator += (unsigned num) throw();
		/** */
		virtual Api&	operator += (const Api& s) throw();
		/** */
		virtual Api&	operator += (const char* s) throw();
		/** */
		virtual Api&	operator = (const char* s) throw();
		/** */
		virtual Api&	operator = (const Api& s) throw();
		/** */
		virtual bool	operator == (const char* s) const throw();
		/** */
		virtual bool	operator == (const Api& s) const throw();
		/** */
		virtual bool	operator != (const char* s) const throw();
		/** */
		virtual bool	operator != (const Api& s) const throw();
		/** */
		virtual bool	operator < (const Api& s) const throw();
		/** */
		virtual void	fillFromBuffer(const void* buffer,unsigned long nChars) throw();
		/** */
		virtual operator const char* () const throw();
		/** */
		virtual unsigned long length() const throw();

		/** Returns true if the string was truncated during a
			modification operation. The flag is cleared by the next
			successful non-truncating operation.
		 */
		inline bool truncated() const throw(){ return _truncated;}
	public:
		/** */
		virtual const char*	getString() const throw()=0;
		/** */
		virtual unsigned long maxLength() const throw()=0;
	protected:
		/** */
		virtual char*	getBuffer() throw()=0;
		/** */
		void	append(const char* src) throw();
		/** */
		void	assign(const char* src) throw();
	};

}
}

#endif
