/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include <stdlib.h>
#include <string.h>
#include "dynamic.h"

using namespace Oscl::Strings;

/*inline*/ void	Dynamic::alloc(unsigned long stringLength){
	unsigned long length;
	length		= stringLength+1+_blockSize;
	_maxLength	= (length/_blockSize);
	_maxLength	*= _blockSize;
	_string		= (char*)malloc(_maxLength);
	if(!_string){
		exit(1);
		}
	_maxLength	-= 1;
	}

/*inline*/ void	Dynamic::alloc(const char* string){
	_length		= strlen(string);
	alloc(_length);
	strcpy(_string,string);
	}

Dynamic::Dynamic(const char* string,unsigned long blockSize) throw():
		_blockSize(blockSize)
		{
	if(!string){
		string="";
		}
	alloc(string);
	}

Dynamic::Dynamic(const Dynamic& s) throw(){
	Api::operator=(s);
	if(truncated()){
		free(_string);
		alloc(s);
		}
	}

Dynamic::~Dynamic() throw(){
	if(_string) free(_string);
	}

Dynamic&	Dynamic::operator = (const Dynamic& s) throw(){
	const Api&	sapi	= s;
	Api::operator=(sapi);
	if(truncated()){
		free(_string);
		alloc(s);
		}
	return *this;
	}

Api&	Dynamic::operator += (signed long num) throw(){
	Api::operator+=(num);
	if(truncated()){
		char*	s	= _string;
		alloc(_maxLength+_blockSize);
		Api::operator=(s);
		Api::operator+=(num);
		free(s);
		}
	_length	= strlen(_string);
	return *this;
	}

Api&	Dynamic::operator += (signed num) throw(){
	Api::operator+=(num);
	if(truncated()){
		char*	s	= _string;
		alloc(_maxLength+_blockSize);
		Api::operator=(s);
		Api::operator+=(num);
		free(s);
		}
	_length	= strlen(_string);
	return *this;
	}

Api&	Dynamic::operator += (unsigned long num) throw(){
	Api::operator+=(num);
	if(truncated()){
		char*	s	= _string;
		alloc(_maxLength+_blockSize);
		Api::operator=(s);
		Api::operator+=(num);
		free(s);
		}
	_length	= strlen(_string);
	return *this;
	}

Api&	Dynamic::operator += (unsigned num) throw(){
	Api::operator+=(num);
	if(truncated()){
		char*	s	= _string;
		alloc(_maxLength+_blockSize);
		Api::operator=(s);
		Api::operator+=(num);
		free(s);
		}
	_length	= strlen(_string);
	return *this;
	}

Api&	Dynamic::operator += (const Api& scat) throw(){
	Api::operator+=(scat);
	if(truncated()){
		char*	s	= _string;
		alloc(_maxLength+_blockSize);
		Api::operator=(s);
		Api::operator+=(scat);
		free(s);
		}
	_length	= strlen(_string);
	return *this;
	}

Api&	Dynamic::operator += (const char* scat) throw(){
	Api::operator+=(scat);
	if(truncated()){
		char*	s	= _string;
		alloc(_maxLength+_blockSize);
		Api::operator=(s);
		Api::operator+=(scat);
		free(s);
		}
	_length	= strlen(_string);
	return *this;
	}

Api&	Dynamic::operator = (const char* s) throw(){
	Api::operator=(s);
	if(truncated()){
		free(_string);
		alloc(s);
		}
	return *this;
	}

Api&	Dynamic::operator = (const Api& s) throw(){
	Api::operator=(s);
	if(truncated()){
		free(_string);
		alloc(s);
		}
	return *this;
	}

void	Dynamic::fillFromBuffer(const void* buffer,unsigned long nChars) throw(){
	if(nChars > _maxLength){
		free(_string);
		_string	= (char*)malloc(nChars);
		strncpy(_string,(const char*)buffer,nChars);
		_string[nChars]	= '\0';
		_length	= strlen(_string);
		}
	}

unsigned long Dynamic::length() const throw(){
	return _length;
	}

const char*	Dynamic::getString() const throw(){
	return _string;
	}

unsigned long Dynamic::maxLength() const throw(){
	return _maxLength;
	}

char*	Dynamic::getBuffer() throw(){
	return _string;
	}

