/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_strings_dynamich_
#define _oscl_strings_dynamich_
#include "api.h"

/** */
namespace Oscl {

/** */
namespace Strings {

/** */
class Dynamic : public Strings::Api {
	private:
		/** */
		char*			_string;
		/** */
		unsigned long	_maxLength;
		/** */
		unsigned long	_blockSize;
		/** */
		unsigned long	_length;
	public:
		/** */
		Dynamic(const char* string = 0,unsigned long blockSize=128) throw();
		/** */
		Dynamic(const Dynamic& d) throw();
		/** */
		~Dynamic() throw();
		/** */
		Dynamic&	operator = (const Dynamic& s) throw();

	public: // Strings::Api overrides
		/** */
		Strings::Api&	operator += (signed long num) throw();
		/** */
		Strings::Api&	operator += (signed num) throw();
		/** */
		Strings::Api&	operator += (unsigned long num) throw();
		/** */
		Strings::Api&	operator += (unsigned num) throw();
		/** */
		Strings::Api&	operator += (const Strings::Api& s) throw();
		/** */
		Strings::Api&	operator += (const char* s) throw();
		/** */
		Strings::Api&	operator = (const char* s) throw();
		/** */
		Strings::Api&	operator = (const Strings::Api& s) throw();
		/** */
		void	fillFromBuffer(const void* buffer,unsigned long nChars) throw();
		/** */
		unsigned long length() const throw();

	public: // Strings::Api
		/** */
		const char*	getString() const throw();
		/** */
		unsigned long maxLength() const throw();

	protected:	// Strings::Api
		/** */
		char*	getBuffer() throw();

	private:
		/** */
		void	alloc(unsigned long stringLength);
		/** */
		void	alloc(const char* s);
	};

}
}

#endif
