/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_tl1_cmd_req_login_cmdh_
#define _oscl_tl1_cmd_req_login_cmdh_
#include "oscl/tl1/cmd/api.h"
#include "oscl/tl1/parser/tokenizer.h"
#include "oscl/mt/itc/atomic/respapi.h"
#include "oscl/tl1/output/api.h"
#include "oscl/tl1/session/output/api.h"
#include "oscl/tl1/session/state/api.h"

/** */
namespace Oscl {
/** */
namespace TL1 {
/** */
namespace Cmd {
/** */
namespace Req {
/** */
namespace Login {

/** This command returns an ASCII string that represents a
	key to be used in a one way encryption of the
	user's user name and password. After receiving
	the key, the user responds using the ACT-USER command
	with the ASCII version of the hashed user name key
	and password along with the user name. The TL1 agent
	uses the password associated with the user name and
	the same key, performs the one way hash and compares
	the result with the value specified in the ACT-USER
	command.
	If the values match, the session is authenticated
	and the priviledge level associated with the USER
	is established for all subsequent commands.
	The KEY value generated by this command is stored
	in the context for this stream session. The same
	KEY value is retrieved from the context by the
	ACT-USER command and is subsequently made invalid
	as it is only good for a single ACT-USER attempt.
	o Should the session be disconnected if the ACT-USER
	  attempt fails?

	oscl/checksum/md5/calculator.h
	Oscl::Checksum::MD5::Calculator
 */
class Command :
	public Oscl::TL1::Cmd::Api,
	private Oscl::Mt::Itc::Atomic::OperationApi<Oscl::TL1::Output::Api>,
	private Oscl::Mt::Itc::Atomic::Resp::Api<Oscl::TL1::Output::Api>
	{
	private:
		/** */
		Oscl::TL1::Session::Output::Api&				_output;
		/** */
		Oscl::TL1::Parser::Tokenizer&					_tokens;
		/** */
		Oscl::Mt::Itc::
		Atomic::Req::Api<	Oscl::TL1::
							Output::Api
							>::AtomicPayload			_payload;
		/** */
		Oscl::Mt::Itc::
		Atomic::Resp::Api<	Oscl::TL1::
							Output::Api
							>::AtomicResp				_resp;
		/** */
		Oscl::TL1::Session::State::Api&					_sessionApi;
		/** */
		Oscl::TL1::Cmd::DoneApi*						_done;

	public:
		/** */
		Command(	Oscl::TL1::Session::Output::Api&	output,
					Oscl::Mt::Itc::PostMsgApi&			myPapi,
					Oscl::TL1::Parser::Tokenizer&		tokens,
					Oscl::TL1::Session::State::Api&		sessionApi
					) throw();
	private:
		/** */
		void	execute( Oscl::TL1::Cmd::DoneApi& done ) throw();

	private:
		/** */
		void	execute(Oscl::TL1::Output::Api& api) throw();

	private: // Oscl::Mt::Itc::Atomic::Resp::Api<Oscl::TL1::Output::Api>
		/** */
		void	response(	Oscl::Mt::Itc::Atomic::
							Resp::Api<	Oscl::TL1::
										Output::Api
										>::AtomicResp&	msg
							) throw();
	};

}
}
}
}
}

#endif
