/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_tristate_itc_client_baseh_
#define _oscl_tristate_itc_client_baseh_
#include "respapi.h"
#include "oscl/memory/block.h"
#include "oscl/mt/itc/dyn/monitor/mp/fsm.h"

/** */
namespace Oscl {
/** */
namespace TriState {

/** */
class MonitorBase :	public Resp::Api,
					public Mt::Itc::Dyn::MpMonitor::StateVar::ContextApi
					{
	public:
		/** */
		typedef struct CancelMemory {
			/** */
			Memory::AlignedBlock<sizeof(Req::Api::CancelPayload)>	payload;
			/** */
			Memory::AlignedBlock<sizeof(Resp::Api::CancelResp)>	response;
			} CancelMemory;

	private:
		/** */
		Req::Api::SAP&						_monitorPointSAP;
		/** */
		Mt::Itc::PostMsgApi&				_myPapi;
		/** */
		Req::Api::RequestPayload			_payload;
		/** */
		Resp::Api::NotificationResp			_response;
		/** */
		Mt::Itc::Dyn::MpMonitor::StateVar	_state;
		/** */
		Var::Query&							_query;
		/** */
		CancelMemory*						_cancelMem;
	protected:
		/** */
		MonitorBase(	Req::Api::SAP&			monitorPointSAP,
						Mt::Itc::PostMsgApi&	myPapi,
						Var::Query&				query
						) throw();
		/** */
		void	activate() throw();
		/** */
		void	activate(	const Oscl::TriState::Var::
							State&		assumedInitialState
							) throw();
		/** */
		void	deactivate(CancelMemory& mem) throw();
	private:
		/** */
		void	response(Resp::Api::NotificationResp& msg) throw();
		/** */
		void	response(Resp::Api::CancelResp& msg) throw();
	private:
		/** */
		void	initNotificationReq() throw();
		/** */
		void	requestChangeNotification() throw();
		/** */
		void	processChangeNotification() throw();
		/** */
		void	cancelChangeNotification() throw();
		/** */
		void	completeDeactivation() throw();
	private:
		virtual void	deactivated() throw()=0;
	};

};
};

#endif
