/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "output.h"

using namespace Oscl;

TriState::Output::Output() throw()
		{
	}

void	TriState::Output::accept(TriState::Var::Query& query) throw(){
	_state.accept(query);
	}

void	TriState::Output::notify(const TriState::Var::State& newState) throw(){
	TriState::Req::Api::ChangeRequest* note;
	while((note=_noteQ.get())){
		note->getPayload()	= newState;
		note->returnToSender();
		}
	}

void TriState::Output::request(TriState::Req::Api::ChangeRequest& msg) throw(){
	if(_state == msg.getPayload()){
		_noteQ.put(&msg);
		}
	else{
		TriState::Var&	payloadState	= msg.getPayload();
		payloadState	= _state;
		msg.returnToSender();
		}
	}

void TriState::Output::request(TriState::Req::Api::CancelRequest& msg) throw(){
	for(	TriState::Req::Api::ChangeRequest* next=_noteQ.first();
			next;
			next=_noteQ.next(next)
			){
		if(next == &msg.getPayload()._reqToCancel){
			_noteQ.remove(next);
			next->returnToSender();
			break;
			}
		}
	msg.returnToSender();
	}

void	TriState::Output::asserted() throw(){
	assert();
	}

void	TriState::Output::negated() throw(){
	negate();
	}

void	TriState::Output::inhibited() throw(){
	inhibit();
	}

void	TriState::Output::assert() throw(){
	const TriState::Var::State&	assertedState	= TriState::Var::getAsserted();
	if(_state != assertedState){
		_state.assert();
		notify(assertedState);
		}
	}

void	TriState::Output::negate() throw(){
	const TriState::Var::State&	negatedState	= TriState::Var::getNegated();
	if(_state != negatedState){
		_state.negate();
		notify(negatedState);
		}
	}

void	TriState::Output::inhibit() throw(){
	const TriState::Var::State&	inhibitedState	= TriState::Var::getInhibited();
	if(_state != inhibitedState){
		_state.inhibit();
		notify(inhibitedState);
		}
	}

