/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "subject.h"

using namespace Oscl;

void	TriState::Subject::notify() throw(){
	TriState::Observer*	obsvr;
	for(obsvr=_observers.first();obsvr;obsvr=_observers.next(obsvr)){
		_state.accept(*obsvr);
		}
	}

void	TriState::Subject::attach(TriState::Observer& o,const TriState::Var::State& assumed) throw(){
	_observers.put(&o);
	if(_state != assumed) _state.accept(o);
	}

void	TriState::Subject::detach(TriState::Observer& o) throw(){
	_observers.remove(&o);
	}

void	TriState::Subject::assert() throw(){
	if(_state != TriState::Var::getAsserted()){
		_state.assert();
		notify();
		}
	}

void	TriState::Subject::negate() throw(){
	if(_state != TriState::Var::getNegated()){
		_state.negate();
		notify();
		}
	}

void	TriState::Subject::inhibit() throw(){
	if(_state != TriState::Var::getInhibited()){
		_state.inhibit();
		notify();
		}
	}

