/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_tristate_varh_
#define _oscl_tristate_varh_

/** */
namespace Oscl {
/** */
namespace TriState {

/** */
class Var {
	public:
		/** */
		class Query {
			public:
				/** */
				virtual void asserted() throw()=0;
				/** */
				virtual void negated() throw()=0;
				/** */
				virtual void inhibited() throw()=0;
			};
		/** */
		 class State {
			public:
				/** */
				virtual ~State(){}
				/** */
				virtual bool operator ==(const State& other) const throw()=0;
				/** */
				virtual bool operator !=(const State& other) const throw()=0;
				/** */
				virtual void accept(Query& q) const throw()=0;
			};
	public:
		/** */
		const State*		_state;
	public:
		/** */
		Var(const State& initial=getNegated()) throw();
		/** */
		void			accept(Query& q) const throw();
		/** */
		bool			operator == (const TriState::Var& other) throw();
		/** */
		bool			operator != (const TriState::Var& other) throw();
	public:
		/** */
		void			assert() throw();
		/** */
		void			negate() throw();
		/** */
		void			inhibit() throw();
		/** */
		void			operator=(const TriState::Var& newState) throw();
	public:
		/** */
		void			operator=(const State& newState) throw();
		/** */
		const State&	getCurrentState() const throw();
	public:
		/** */
		static const State&	getAsserted() throw();
		/** */
		static const State&	getNegated() throw();
		/** */
		static const State&	getInhibited() throw();
	};

};
};

#endif

