/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "fsm.h"
#include "oscl/error/fatal.h"

using namespace Oscl::TT::LUN;

static const StateVar::Closed						closed;
static const StateVar::Idle							idle;
static const StateVar::Read							read;
static const StateVar::Write						write;
static const StateVar::Flush						flush;
static const StateVar::FlushReqSense				flushReqSense;
static const StateVar::ReadCanceling				readCanceling;
static const StateVar::WriteCanceling				writeCanceling;
static const StateVar::FlushCanceling				flushCanceling;

// StateVar

StateVar::StateVar(ContextApi& context) throw():
		_context(context),
		_state(&closed)
		{
	for(unsigned i=0;i<nTraceRecs;++i){
		_freeTraceRecs.put(&_traceRec[i]);
		}
	}

void	StateVar::open() throw(){
	_state->open(_context,*this);
	}

void	StateVar::close() throw(){
	_state->close(_context,*this);
	}

void	StateVar::cancelResp() throw(){
	_state->cancelResp(_context,*this);
	}

void	StateVar::writeResp() throw(){
	_state->writeResp(_context,*this);
	}

void	StateVar::commandResp() throw(){
	_state->commandResp(_context,*this);
	}

void	StateVar::readResp() throw(){
	_state->readResp(_context,*this);
	}

void	StateVar::writeReq() throw(){
	_state->writeReq(_context,*this);
	}

void	StateVar::flushReq() throw(){
	_state->flushReq(_context,*this);
	}

void	StateVar::readReq() throw(){
	_state->readReq(_context,*this);
	}

void	StateVar::readCancelReq() throw(){
	_state->readCancelReq(_context,*this);
	}

void	StateVar::writeCancelReq() throw(){
	_state->writeCancelReq(_context,*this);
	}

// global actions
void	StateVar::protocolError(const char* state) throw(){
	ErrorFatal::logAndExit(state);
	}

// state change operations
void	StateVar::changeToClosed() throw(){
	trace(closed);
	_state	= &closed;
	}

void	StateVar::changeToIdle() throw(){
	trace(idle);
	_state	= &idle;
	_context.executeDeferredReq();
	}

void	StateVar::changeToRead() throw(){
	trace(read);
	_state	= &read;
	}

void	StateVar::changeToWrite() throw(){
	trace(write);
	_state	= &write;
	}

void	StateVar::changeToFlush() throw(){
	trace(flush);
	_state	= &flush;
	}

void	StateVar::changeToFlushReqSense() throw(){
	trace(flushReqSense);
	_state	= &flushReqSense;
	}

void	StateVar::changeToReadCanceling() throw(){
	trace(readCanceling);
	_state	= &readCanceling;
	}

void	StateVar::changeToWriteCanceling() throw(){
	trace(writeCanceling);
	_state	= &writeCanceling;
	}

void	StateVar::changeToFlushCanceling() throw(){
	trace(flushCanceling);
	_state	= &flushCanceling;
	}

static unsigned long	_nTraces;

void	StateVar::trace(const State& state) throw(){
	TraceRec*	rec;
	++_nTraces;
	rec	= _freeTraceRecs.get();
	if(!rec){
		rec	= _trace.get();
		}
	rec->_state	= &state;
	_trace.put(rec);
	}

