/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include "fsm.h"
#include "oscl/mt/thread.h"
using namespace Oscl::TT::LUN;

void	StateVar::State::
open(	ContextApi&	context,
		StateVar&	sv
		) const throw(){
	sv.protocolError(stateName());
	}

void	StateVar::State::
close(	ContextApi&	context,
		StateVar&	sv
		) const throw(){
	sv.protocolError(stateName());
	}

void	StateVar::State::
cancelResp(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	sv.protocolError(stateName());
	}

void	StateVar::State::
writeResp(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	sv.protocolError(stateName());
	}

void	StateVar::State::
commandResp(	ContextApi&	context,
				StateVar&	sv
				) const throw(){
	sv.protocolError(stateName());
	}

void	StateVar::State::
readResp(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	sv.protocolError(stateName());
	}

void	StateVar::State::
writeReq(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	sv.protocolError(stateName());
	}

void	StateVar::State::
flushReq(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	sv.protocolError(stateName());
	}

void	StateVar::State::
readReq(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	sv.protocolError(stateName());
	}

void	StateVar::State::
readCancelReq(	ContextApi&	context,
				StateVar&	sv
				) const throw(){
	sv.protocolError(stateName());
	}

void	StateVar::State::
writeCancelReq(	ContextApi&	context,
				StateVar&	sv
				) const throw(){
	sv.protocolError(stateName());
	}

///////////////////////////////////

void	StateVar::Closed::
open(	ContextApi&	context,
		StateVar&	sv
		) const throw(){
	context.returnOpenReq();
	sv.changeToIdle();
	}

///////////////////////////////////

void	StateVar::DeferReq::
writeReq(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	context.deferWriteReq();
	}

void	StateVar::DeferReq::
flushReq(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	context.deferFlushReq();
	}

void	StateVar::DeferReq::
readReq(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	context.deferReadReq();
	}

///////////////////////////////////
void	StateVar::Idle::
close(	ContextApi&	context,
		StateVar&	sv
		) const throw(){
	context.returnCloseReq();
	sv.changeToClosed();
	}

void	StateVar::Idle::
writeReq(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	context.startWrite();
	sv.changeToWrite();
	}

void	StateVar::Idle::
flushReq(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	if(context.isFlushSupportedByDevice()){
		context.startFlush();
		sv.changeToFlush();
		return;
		}
	context.returnIncommingFlushReq();
	}

void	StateVar::Idle::
readReq(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	context.startRead();
	sv.changeToRead();
	}

void	StateVar::Idle::
readCancelReq(	ContextApi&	context,
				StateVar&	sv
				) const throw(){
	context.returnIncommingReadCancelReq();
	}

void	StateVar::Idle::
writeCancelReq(	ContextApi&	context,
				StateVar&	sv
				) const throw(){
	context.returnIncommingWriteCancelReq();
	}

///////////////////////////////////

void	StateVar::Write::
writeResp(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	context.finishWrite();
	context.returnWriteReq();
	sv.changeToIdle();
	}

void	StateVar::Write::
writeCancelReq(	ContextApi&	context,
				StateVar&	sv
				) const throw(){
	if(context.isIncommingCancelForCurrentWrite()){
		context.cancelWrite();
		sv.changeToWriteCanceling();
		return;
		}
	context.returnIncommingWriteCancelReq();
	}

///////////////////////////////////

void	StateVar::Flush::
commandResp(	ContextApi&	context,
				StateVar&	sv
				) const throw(){
	context.finishCommand();
	if(context.isCommandCheckCondition()){
		context.startRequestSense();
		sv.changeToFlushReqSense();
		return;
		}
	context.returnFlushReq();
	sv.changeToIdle();
	}

void	StateVar::Flush::
writeCancelReq(	ContextApi&	context,
				StateVar&	sv
				) const throw(){
	if(context.isIncommingCancelForCurrentFlush()){
		context.cancelCommand();
		sv.changeToFlushCanceling();
		return;
		}
	context.returnIncommingWriteCancelReq();
	}

///////////////////////////////////

void	StateVar::FlushReqSense::
readResp(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	context.finishFlushReqSense();
	context.returnFlushReq();
	sv.changeToIdle();
	}

void	StateVar::FlushReqSense::
writeCancelReq(	ContextApi&	context,
				StateVar&	sv
				) const throw(){
	if(context.isIncommingCancelForCurrentFlush()){
		context.cancelRead();
		sv.changeToFlushCanceling();
		return;
		}
	context.returnIncommingWriteCancelReq();
	}

///////////////////////////////////

void	StateVar::Read::
readResp(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	context.finishRead();
	context.returnReadReq();
	sv.changeToIdle();
	}

void	StateVar::Read::
readCancelReq(	ContextApi&	context,
				StateVar&	sv
				) const throw(){
	if(context.isIncommingCancelForCurrentRead()){
		context.cancelRead();
		sv.changeToReadCanceling();
		return;
		}
	context.returnIncommingReadCancelReq();
	}

///////////////////////////////////

void	StateVar::ReadCanceling::
readResp(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	context.returnReadReq();
	}

void	StateVar::ReadCanceling::
cancelResp(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	context.returnReadCancelReq();
	sv.changeToIdle();
	}

///////////////////////////////////

void	StateVar::WriteCanceling::
writeResp(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	context.returnWriteReq();
	}

void	StateVar::WriteCanceling::
cancelResp(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	context.returnWriteCancelReq();
	sv.changeToIdle();
	}

///////////////////////////////////

void	StateVar::FlushCanceling::
commandResp(	ContextApi&	context,
				StateVar&	sv
				) const throw(){
	context.returnFlushReq();
	}

void	StateVar::FlushCanceling::
readResp(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	context.returnFlushReq();
	}

void	StateVar::FlushCanceling::
cancelResp(	ContextApi&	context,
			StateVar&	sv
			) const throw(){
	context.returnWriteCancelReq();
	sv.changeToIdle();
	}

