/*
   Copyright (C) 2004 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#ifndef _oscl_uart_srv_reqapih_
#define _oscl_uart_srv_reqapih_
#include "oscl/mt/itc/mbox/srvreq.h"
#include "oscl/mt/itc/mbox/sap.h"

/** */
namespace Oscl {
/** */
namespace UART {
/** */
namespace Req {

/** */
class SetBaudRatePayload {
	public:
		/** */
		const unsigned long	_bitsPerSecond;
		/** */
		bool				_success;
	public:
		/** */
		SetBaudRatePayload(unsigned long bitsPerSecond) throw();
	};

/** */
class NoParityPayload {
	public:
		/** */
		bool				_success;
	public:
		/** */
		NoParityPayload() throw();
	};

/** */
class EvenParityPayload {
	public:
		/** */
		bool				_success;
	public:
		/** */
		EvenParityPayload() throw();
	};

/** */
class OddParityPayload {
	public:
		/** */
		bool				_success;
	public:
		/** */
		OddParityPayload() throw();
	};

/** */
class MarkParityPayload {
	public:
		/** */
		bool				_success;
	public:
		/** */
		MarkParityPayload() throw();
	};

/** */
class SpaceParityPayload {
	public:
		/** */
		bool				_success;
	public:
		/** */
		SpaceParityPayload() throw();
	};

/** */
class SetStartBitsPayload {
	public:
		/** */
		const unsigned char	_nHalfBits;
		/** */
		bool				_success;
	public:
		/** */
		SetStartBitsPayload(unsigned char nHalfBits) throw();
	};

/** */
class SetStopBitsPayload {
	public:
		/** */
		const unsigned char	_nHalfBits;
		/** */
		bool				_success;
	public:
		/** */
		SetStopBitsPayload(unsigned char nHalfBits) throw();
	};

/** */
class RtsCtsFlowControlPayload {
	public:
		/** */
		bool				_success;
	public:
		/** */
		RtsCtsFlowControlPayload() throw();
	};

/** */
class ManualFlowControlPayload {
	public:
		/** */
		bool				_success;
	public:
		/** */
		ManualFlowControlPayload() throw();
	};

/** */
class DisableFlowControlPayload {
	public:
		/** */
		bool				_success;
	public:
		/** */
		DisableFlowControlPayload() throw();
	};

/** */
class AssertRtsPayload {
	public:
		/** */
		bool				_success;
	public:
		/** */
		AssertRtsPayload() throw();
	};

/** */
class NegateRtsPayload {
	public:
		/** */
		bool				_success;
	public:
		/** */
		NegateRtsPayload() throw();
	};

/** */
class AssertCtsPayload {
	public:
		/** */
		bool				_success;
	public:
		/** */
		AssertCtsPayload() throw();
	};

/** */
class NegateCtsPayload {
	public:
		/** */
		bool				_success;
	public:
		/** */
		NegateCtsPayload() throw();
	};

/** */
class DoThisPayload {
	public:
		/** */
		const char*			_cmd;
		/** */
		bool				_success;
	public:
		/** */
		DoThisPayload(const char* cmd) throw();
	};

/** */
class Api {
	public:
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											SetBaudRatePayload
											>	SetBaudRateReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											NoParityPayload
											>		NoParityReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											EvenParityPayload
											>	EvenParityReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											OddParityPayload
											>		OddParityReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											MarkParityPayload
											>	MarkParityReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											SpaceParityPayload
											>	SpaceParityReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											SetStartBitsPayload
											>	SetStartBitsReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											SetStopBitsPayload
											>	SetStopBitsReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											RtsCtsFlowControlPayload
											>	RtsCtsFlowControlReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											ManualFlowControlPayload
											>	ManualFlowControlReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											DisableFlowControlPayload
											>	DisableFlowControlReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											AssertRtsPayload
											>				AssertRtsReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											NegateRtsPayload
											>				NegateRtsReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											AssertCtsPayload
											>				AssertCtsReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											NegateCtsPayload
											>				NegateCtsReq;
		/** */
		typedef Oscl::Mt::Itc::SrvRequest<	Api,
											DoThisPayload
											>				DoThisReq;
		/** */
		typedef Oscl::Mt::Itc::SAP<Api>				SAP;
		/** */
		typedef Oscl::Mt::Itc::ConcreteSAP<Api>		ConcreteSAP;
	public:
		/** */
		virtual void	request(SetBaudRateReq& msg) throw()=0;
		/** */
		virtual void	request(NoParityReq& msg) throw()=0;
		/** */
		virtual void	request(EvenParityReq& msg) throw()=0;
		/** */
		virtual void	request(OddParityReq& msg) throw()=0;
		/** */
		virtual void	request(MarkParityReq& msg) throw()=0;
		/** */
		virtual void	request(SpaceParityReq& msg) throw()=0;
		/** */
		virtual void	request(SetStartBitsReq& msg) throw()=0;
		/** */
		virtual void	request(SetStopBitsReq& msg) throw()=0;
		/** */
		virtual void	request(RtsCtsFlowControlReq& msg) throw()=0;
		/** */
		virtual void	request(ManualFlowControlReq& msg) throw()=0;
		/** */
		virtual void	request(DisableFlowControlReq& msg) throw()=0;
		/** */
		virtual void	request(AssertRtsReq& msg) throw()=0;
		/** */
		virtual void	request(NegateRtsReq& msg) throw()=0;
		/** */
		virtual void	request(AssertCtsReq& msg) throw()=0;
		/** */
		virtual void	request(NegateCtsReq& msg) throw()=0;
		/** */
		virtual void	request(DoThisReq& msg) throw()=0;
	};
}
}
}

#endif
